/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include "xdx_device.h"
#include "xdx_irq.h"
#include "xdxacpu_irq.h"
#include "xdxacpu.h"
#include "../audio/xdxaudio_hal.h"
#define ACPU_AUDIODB           (((uint64_t)(1))<<40)
#define ACPU_XDMADB           (((uint64_t)(1))<<35)
static int xdxacpu_irq_set(struct xdx_device *xdev, struct xdx_irq_src *src,
			   unsigned type, enum xdx_interrupt_state state)
{
	// TODO: enable/disable IRQ
	return -1;
}

static int xdxacpu_irq_process(struct xdx_device *xdev,
			       struct xdx_irq_src *src,
			       struct xdx_irq_entry *entry)
{
	uint64_t int_status;
	struct xdx_acpu *acpu = xdev_to_acpu(xdev);
	acpudb_pool_t *pool = xdev->acpu.acpu_db_pool;
	int_status =
	    cmcu_doorbell_readq(acpu, CMCU_DOORBELLS_A2H0_INT_REGS_OFFSET);
	cmcu_doorbell_writeq(acpu, CMCU_DOORBELLS_A2H0_CLR_REGS_OFFSET,
			     int_status);

#if defined(SUPPORT_AUDIO)
	if (int_status & ACPU_AUDIODB)
		pangu_a0_audiof0_db_interrupt(xdev->xdx_hdmi_audio_hotplug_ptr);
#endif

	if (int_status & ACPU_XDMADB) {
		if (pool) {
			(void)queue_work(pool->wq, &pool->work);
		}
	}
	return 0;
}

static struct xdx_irq_src_funcs xdxacpu_src_funcs = {
	.set = xdxacpu_irq_set,
	.process = xdxacpu_irq_process,
};

int xdxacpu_isr_install(struct xdx_device *xdev)
{
	struct xdx_irq_src *irq_src;
	int ret;
	irq_src = kmalloc(sizeof(*irq_src), GFP_KERNEL);
	if (!irq_src)
		return -ENOMEM;
	irq_src->id = XDX_IRQ_CLIENTID_ACPU;
	irq_src->funcs = &xdxacpu_src_funcs;
	ret = xdx_irq_register_src(xdev, irq_src);
	if (ret) {
		dev_err(xdev->dev, "failed to register irq source\n");
		return ret;
	}
	xdev->acpu.irq_src = irq_src;
	return 0;
}

int xdxacpu_isr_uninstall(struct xdx_device *xdev)
{
	int ret;
	ret = xdx_irq_unregister_src(xdev, xdev->acpu.irq_src);
	if (ret) {
		dev_err(xdev->dev, "failed to unregister irq source\n");
		return ret;
	}
	kfree(xdev->acpu.irq_src);
	xdev->acpu.irq_src = NULL;
	return 0;
}
