/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/clk.h>
#include <linux/debugfs.h>

#include <drm/drm_atomic_helper.h>
#include <drm/drm_crtc.h>
#include <drm/drm_modeset_helper_vtables.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 13, 0))
#include <drm/drm_vblank.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0))
#include <drm/drm_irq.h>
#else
#include <drm/drmP.h>
#endif
#include <drm/drm_atomic.h>
#include <linux/pm_runtime.h>

#include <drm/xdxgpu_drm.h>

#include "dc8200.h"
#include "dc8200_mmu.h"

#ifdef CONFIG_VERISILICON_DEC
#include "dec400.h"
#endif

#include "dm_dc.h"
#include "xdx_device.h"
//#include "../dp/xdx_dp.h"
#include "kernel_compatibility.h"


static int dc8200_cursor_commit(struct dm_crtc *dm_crtc);
/*
static u32 dc8200_mmu_reg_read(struct dm_dc *dc, u32 reg)
{
	struct xdx_dm *dm = dc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev, xdev->hw_info.dc_mmu_regs.base + reg -
					    SE_MMU_REG_BASE);
}
*/
static void dc8200_mmu_reg_write(struct dm_dc *dc, u32 reg, u32 value)
{
	struct xdx_dm *dm = dc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);
	xdx_mmio_writel(xdev,
			xdev->hw_info.dc_mmu_regs[dc->id].base + reg - SE_MMU_REG_BASE,
			value);
}
/*
static void dc8200_mmu_set_clear(struct dm_dc *dc, u32 reg, u32 set, u32 clear)
{
	u32 value = dc8200_mmu_reg_read(dc, reg);

	value &= ~clear;
	value |= set;
	dc8200_mmu_reg_write(dc, reg, value);
}*/

static int dc8200_mmu_init(struct dc8200_dc *hw, struct dc8200_mmu *mmu)
{
	struct dm_dc *dc = &hw->base;
	u32 ext_mtlb = 0;
	u32 config = 0;

	ext_mtlb = (u32)((mmu->mtlb_physical >> 32) & 0xff);

	if (ext_mtlb & 0xFFFFFF00) {
		pr_err("Mtlb address out of range.\n");
		return -EFAULT;
	}

	config = (((mmu->mtlb_physical >> 8) & 0xfffffff0) | mmu->mode);

	dc8200_mmu_reg_write(dc, SE_MMU_REG_CONTROL, 0x20);
	dc8200_mmu_reg_write(dc, SE_MMU_REG_CONTEXT_PD, config);
	dc8200_mmu_reg_write(dc, SE_MMU_REG_CONTROL, 0x21);

	dc8200_mmu_reg_write(dc, SE_MMU_REG_SAFE_NON_SECUR, mmu->safe_page_physical & 0xFFFFFFFF);
	dc8200_mmu_reg_write(dc, SE_MMU_REG_SAFE_SECURE, mmu->safe_page_physical & 0xFFFFFFFF);
	dc8200_mmu_reg_write(dc, SE_MMU_REG_SAFE_EXT_ADDRESS, ((mmu->safe_page_physical >> 32) & 0xFF) << 16 \
						 | ((mmu->safe_page_physical >> 32) & 0xFF));
	return 0;
}

static void dc8200_enable_mmu_prefetch(struct dc8200_dc *hw, bool enable)
{
	struct dm_dc *dc = &hw->base;

	if (!hw->info->mmu_prefetch)
		return;

	if (enable)
		DC_WREG32(DC_MMU_PREFETCH, BIT(0));
	else
		DC_WREG32(DC_MMU_PREFETCH, 0);
}

static const struct dc8200_mmu_reg dc_mmu_reg = {
	.mmu_config = SE_MMU_REG_CONFIG,
	.mmu_control = SE_MMU_REG_CONTROL,
	.table_array_size = SE_MMU_REG_TABLE_ARRAY_SIZE,
	.safe_non_secure = SE_MMU_REG_SAFE_NON_SECUR,
	.safe_secure = SE_MMU_REG_SAFE_SECURE,
	.safe_ex = SE_MMU_REG_SAFE_EXT_ADDRESS,
	.context_pd_entry = SE_MMU_REG_CONTEXT_PD,
};
/*
static void dc8200_mmu_flush(struct dc8200_dc *hw)
{
	const struct dc8200_mmu_reg *reg = &dc_mmu_reg;
	struct dm_dc *dc = &hw->base;
	u32 value;

	value = dc8200_mmu_reg_read(dc, reg->mmu_config);

	dc8200_mmu_reg_write(dc, reg->mmu_config, value | BIT(4));
	dc8200_mmu_reg_write(dc, reg->mmu_config, value);
}
*/
static void dc_reg_write(struct dm_dc *dc, u32 reg, u32 value)
{
	struct xdx_dm *dm = dc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);

	xdx_mmio_writel(xdev, xdev->hw_info.dc_regs[dc->id].base + reg, value);
}

static u32 dc_reg_read(struct dm_dc *dc, u32 reg)
{
	struct xdx_dm *dm = dc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);

	return xdx_mmio_readl(xdev, xdev->hw_info.dc_regs[dc->id].base + reg);
}

static const u32 horKernel[] = {
	0x00000000, 0x20000000, 0x00002000, 0x00000000, 0x00000000, 0x00000000,
	0x23fd1c03, 0x00000000, 0x00000000, 0x00000000, 0x181f0000, 0x000027e1,
	0x00000000, 0x00000000, 0x00000000, 0x2b981468, 0x00000000, 0x00000000,
	0x00000000, 0x10f00000, 0x00002f10, 0x00000000, 0x00000000, 0x00000000,
	0x32390dc7, 0x00000000, 0x00000000, 0x00000000, 0x0af50000, 0x0000350b,
	0x00000000, 0x00000000, 0x00000000, 0x3781087f, 0x00000000, 0x00000000,
	0x00000000, 0x06660000, 0x0000399a, 0x00000000, 0x00000000, 0x00000000,
	0x3b5904a7, 0x00000000, 0x00000000, 0x00000000, 0x033c0000, 0x00003cc4,
	0x00000000, 0x00000000, 0x00000000, 0x3de1021f, 0x00000000, 0x00000000,
	0x00000000, 0x01470000, 0x00003eb9, 0x00000000, 0x00000000, 0x00000000,
	0x3f5300ad, 0x00000000, 0x00000000, 0x00000000, 0x00480000, 0x00003fb8,
	0x00000000, 0x00000000, 0x00000000, 0x3fef0011, 0x00000000, 0x00000000,
	0x00000000, 0x00000000, 0x00004000, 0x00000000, 0x00000000, 0x00000000,
	0x20002000, 0x00000000, 0x00000000, 0x00000000, 0x1c030000, 0x000023fd,
	0x00000000, 0x00000000, 0x00000000, 0x27e1181f, 0x00000000, 0x00000000,
	0x00000000, 0x14680000, 0x00002b98, 0x00000000, 0x00000000, 0x00000000,
	0x2f1010f0, 0x00000000, 0x00000000, 0x00000000, 0x0dc70000, 0x00003239,
	0x00000000, 0x00000000, 0x00000000, 0x350b0af5, 0x00000000, 0x00000000,
	0x00000000, 0x087f0000, 0x00003781, 0x00000000, 0x00000000, 0x00000000,
	0x399a0666, 0x00000000, 0x00000000, 0x00000000, 0x04a70000, 0x00003b59,
	0x00000000, 0x00000000, 0x00000000, 0x3cc4033c, 0x00000000, 0x00000000,
	0x00000000, 0x021f0000,
};
#define H_COEF_SIZE (sizeof(horKernel) / sizeof(u32))

static const u32 verKernel[] = {
	0x00000000, 0x20000000, 0x00002000, 0x00000000, 0x00000000, 0x00000000,
	0x23fd1c03, 0x00000000, 0x00000000, 0x00000000, 0x181f0000, 0x000027e1,
	0x00000000, 0x00000000, 0x00000000, 0x2b981468, 0x00000000, 0x00000000,
	0x00000000, 0x10f00000, 0x00002f10, 0x00000000, 0x00000000, 0x00000000,
	0x32390dc7, 0x00000000, 0x00000000, 0x00000000, 0x0af50000, 0x0000350b,
	0x00000000, 0x00000000, 0x00000000, 0x3781087f, 0x00000000, 0x00000000,
	0x00000000, 0x06660000, 0x0000399a, 0x00000000, 0x00000000, 0x00000000,
	0x3b5904a7, 0x00000000, 0x00000000, 0x00000000, 0x033c0000, 0x00003cc4,
	0x00000000, 0x00000000, 0x00000000, 0x3de1021f, 0x00000000, 0x00000000,
	0x00000000, 0x01470000, 0x00003eb9, 0x00000000, 0x00000000, 0x00000000,
	0x3f5300ad, 0x00000000, 0x00000000, 0x00000000, 0x00480000, 0x00003fb8,
	0x00000000, 0x00000000, 0x00000000, 0x3fef0011, 0x00000000, 0x00000000,
	0x00000000, 0x00000000, 0x00004000, 0x00000000, 0xcdcd0000, 0xfdfdfdfd,
	0xabababab, 0xabababab, 0x00000000, 0x00000000, 0x5ff5f456, 0x000f5f58,
	0x02cc6c78, 0x02cc0c28, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee, 0xfeeefeee,
	0xfeeefeee, 0xfeeefeee,
};
#define V_COEF_SIZE (sizeof(verKernel) / sizeof(u32))

/*
 * RGB 709->2020 conversion parameters
 */
static u16 RGB2RGB[RGB_TO_RGB_TABLE_SIZE] = { 10279, 5395, 709,	 1132, 15065,
					      187,   269,  1442, 14674 };

/*
 * YUV601 to RGB conversion parameters
 * YUV2RGB[0]  - [8] : C0 - C8;
 * YUV2RGB[9]  - [11]: D0 - D2;
 * YUV2RGB[12] - [13]: Y clamp min & max calue;
 * YUV2RGB[14] - [15]: UV clamp min & max calue;
 */
static s32 YUV601_2RGB[YUV_TO_RGB_TABLE_SIZE] = {
	1196, 0,       1640,   1196,	 -404, -836, 1196, 2076,
	0,    -916224, 558336, -1202944, 64,   940,  64,   960
};

/*
 * YUV709 to RGB conversion parameters
 * YUV2RGB[0]  - [8] : C0 - C8;
 * YUV2RGB[9]  - [11]: D0 - D2;
 * YUV2RGB[12] - [13]: Y clamp min & max calue;
 * YUV2RGB[14] - [15]: UV clamp min & max calue;
 */
static s32 YUV709_2RGB[YUV_TO_RGB_TABLE_SIZE] = {
	1196, 0,	1844,	1196,	  -220, -548, 1196, 2172,
	0,    -1020672, 316672, -1188608, 64,	940,  64,   960
};

/*
 * YUV2020 to RGB conversion parameters
 * YUV2RGB[0]  - [8] : C0 - C8;
 * YUV2RGB[9]  - [11]: D0 - D2;
 * YUV2RGB[12] - [13]: Y clamp min & max calue;
 * YUV2RGB[14] - [15]: UV clamp min & max calue;
 */
static s32 YUV2020_2RGB[YUV_TO_RGB_TABLE_SIZE] = {
	1196, 0,       1724,   1196,	 -192, -668, 1196, 2200,
	0,    -959232, 363776, -1202944, 64,   940,  64,   960
};

/*
 * RGB to YUV2020 conversion parameters
 * RGB2YUV[0] - [8] : C0 - C8;
 * RGB2YUV[9] - [11]: D0 - D2;
 */
static s16 RGB2YUV[RGB_TO_YUV_TABLE_SIZE] = { 230, 594,	 52,  -125, -323, 448,
					      448, -412, -36, 64,   512,  512 };

/*
 * Degamma table for 709 color space data.
 */
static u16 DEGAMMA_709[DEGAMMA_SIZE] = {
	0x0000, 0x0000, 0x0000, 0x0000, 0x0001, 0x0002, 0x0004, 0x0005, 0x0007,
	0x000a, 0x000d, 0x0011, 0x0015, 0x0019, 0x001e, 0x0024, 0x002a, 0x0030,
	0x0038, 0x003f, 0x0048, 0x0051, 0x005a, 0x0064, 0x006f, 0x007b, 0x0087,
	0x0094, 0x00a1, 0x00af, 0x00be, 0x00ce, 0x00de, 0x00ef, 0x0101, 0x0114,
	0x0127, 0x013b, 0x0150, 0x0166, 0x017c, 0x0193, 0x01ac, 0x01c4, 0x01de,
	0x01f9, 0x0214, 0x0230, 0x024d, 0x026b, 0x028a, 0x02aa, 0x02ca, 0x02ec,
	0x030e, 0x0331, 0x0355, 0x037a, 0x03a0, 0x03c7, 0x03ef, 0x0418, 0x0441,
	0x046c, 0x0498, 0x04c4, 0x04f2, 0x0520, 0x0550, 0x0581, 0x05b2, 0x05e5,
	0x0618, 0x064d, 0x0682, 0x06b9, 0x06f0, 0x0729, 0x0763, 0x079d, 0x07d9,
	0x0816, 0x0854, 0x0893, 0x08d3, 0x0914, 0x0956, 0x0999, 0x09dd, 0x0a23,
	0x0a69, 0x0ab1, 0x0afa, 0x0b44, 0x0b8f, 0x0bdb, 0x0c28, 0x0c76, 0x0cc6,
	0x0d17, 0x0d69, 0x0dbb, 0x0e10, 0x0e65, 0x0ebb, 0x0f13, 0x0f6c, 0x0fc6,
	0x1021, 0x107d, 0x10db, 0x113a, 0x119a, 0x11fb, 0x125d, 0x12c1, 0x1325,
	0x138c, 0x13f3, 0x145b, 0x14c5, 0x1530, 0x159c, 0x160a, 0x1678, 0x16e8,
	0x175a, 0x17cc, 0x1840, 0x18b5, 0x192b, 0x19a3, 0x1a1c, 0x1a96, 0x1b11,
	0x1b8e, 0x1c0c, 0x1c8c, 0x1d0c, 0x1d8e, 0x1e12, 0x1e96, 0x1f1c, 0x1fa3,
	0x202c, 0x20b6, 0x2141, 0x21ce, 0x225c, 0x22eb, 0x237c, 0x240e, 0x24a1,
	0x2536, 0x25cc, 0x2664, 0x26fc, 0x2797, 0x2832, 0x28cf, 0x296e, 0x2a0e,
	0x2aaf, 0x2b51, 0x2bf5, 0x2c9b, 0x2d41, 0x2dea, 0x2e93, 0x2f3e, 0x2feb,
	0x3099, 0x3148, 0x31f9, 0x32ab, 0x335f, 0x3414, 0x34ca, 0x3582, 0x363c,
	0x36f7, 0x37b3, 0x3871, 0x3930, 0x39f1, 0x3ab3, 0x3b77, 0x3c3c, 0x3d02,
	0x3dcb, 0x3e94, 0x3f5f, 0x402c, 0x40fa, 0x41ca, 0x429b, 0x436d, 0x4442,
	0x4517, 0x45ee, 0x46c7, 0x47a1, 0x487d, 0x495a, 0x4a39, 0x4b19, 0x4bfb,
	0x4cde, 0x4dc3, 0x4eaa, 0x4f92, 0x507c, 0x5167, 0x5253, 0x5342, 0x5431,
	0x5523, 0x5616, 0x570a, 0x5800, 0x58f8, 0x59f1, 0x5aec, 0x5be9, 0x5ce7,
	0x5de6, 0x5ee7, 0x5fea, 0x60ef, 0x61f5, 0x62fc, 0x6406, 0x6510, 0x661d,
	0x672b, 0x683b, 0x694c, 0x6a5f, 0x6b73, 0x6c8a, 0x6da2, 0x6ebb, 0x6fd6,
	0x70f3, 0x7211, 0x7331, 0x7453, 0x7576, 0x769b, 0x77c2, 0x78ea, 0x7a14,
	0x7b40, 0x7c6d, 0x7d9c, 0x7ecd, 0x3f65, 0x3f8c, 0x3fb2, 0x3fd8
};

/*
 * Degamma table for 2020 color space data.
 */
static u16 DEGAMMA_2020[DEGAMMA_SIZE] = {
	0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0001, 0x0001,
	0x0001, 0x0001, 0x0001, 0x0001, 0x0001, 0x0001, 0x0002, 0x0002, 0x0002,
	0x0002, 0x0002, 0x0003, 0x0003, 0x0003, 0x0003, 0x0004, 0x0004, 0x0004,
	0x0005, 0x0005, 0x0006, 0x0006, 0x0006, 0x0007, 0x0007, 0x0008, 0x0008,
	0x0009, 0x000a, 0x000a, 0x000b, 0x000c, 0x000c, 0x000d, 0x000e, 0x000f,
	0x000f, 0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0016, 0x0017, 0x0018,
	0x0019, 0x001b, 0x001c, 0x001e, 0x001f, 0x0021, 0x0022, 0x0024, 0x0026,
	0x0028, 0x002a, 0x002c, 0x002e, 0x0030, 0x0033, 0x0035, 0x0038, 0x003a,
	0x003d, 0x0040, 0x0043, 0x0046, 0x0049, 0x004d, 0x0050, 0x0054, 0x0057,
	0x005b, 0x005f, 0x0064, 0x0068, 0x006d, 0x0071, 0x0076, 0x007c, 0x0081,
	0x0086, 0x008c, 0x0092, 0x0098, 0x009f, 0x00a5, 0x00ac, 0x00b4, 0x00bb,
	0x00c3, 0x00cb, 0x00d3, 0x00dc, 0x00e5, 0x00ee, 0x00f8, 0x0102, 0x010c,
	0x0117, 0x0123, 0x012e, 0x013a, 0x0147, 0x0154, 0x0161, 0x016f, 0x017e,
	0x018d, 0x019c, 0x01ac, 0x01bd, 0x01ce, 0x01e0, 0x01f3, 0x0206, 0x021a,
	0x022f, 0x0244, 0x025a, 0x0272, 0x0289, 0x02a2, 0x02bc, 0x02d6, 0x02f2,
	0x030f, 0x032c, 0x034b, 0x036b, 0x038b, 0x03ae, 0x03d1, 0x03f5, 0x041b,
	0x0443, 0x046b, 0x0495, 0x04c1, 0x04ee, 0x051d, 0x054e, 0x0580, 0x05b4,
	0x05ea, 0x0622, 0x065c, 0x0698, 0x06d6, 0x0717, 0x075a, 0x079f, 0x07e7,
	0x0831, 0x087e, 0x08cd, 0x0920, 0x0976, 0x09ce, 0x0a2a, 0x0a89, 0x0aec,
	0x0b52, 0x0bbc, 0x0c2a, 0x0c9b, 0x0d11, 0x0d8b, 0x0e0a, 0x0e8d, 0x0f15,
	0x0fa1, 0x1033, 0x10ca, 0x1167, 0x120a, 0x12b2, 0x1360, 0x1415, 0x14d1,
	0x1593, 0x165d, 0x172e, 0x1806, 0x18e7, 0x19d0, 0x1ac1, 0x1bbb, 0x1cbf,
	0x1dcc, 0x1ee3, 0x2005, 0x2131, 0x2268, 0x23ab, 0x24fa, 0x2656, 0x27be,
	0x2934, 0x2ab8, 0x2c4a, 0x2dec, 0x2f9d, 0x315f, 0x3332, 0x3516, 0x370d,
	0x3916, 0x3b34, 0x3d66, 0x3fad, 0x420b, 0x4480, 0x470d, 0x49b3, 0x4c73,
	0x4f4e, 0x5246, 0x555a, 0x588e, 0x5be1, 0x5f55, 0x62eb, 0x66a6, 0x6a86,
	0x6e8c, 0x72bb, 0x7714, 0x7b99, 0x3dcb, 0x3e60, 0x3ef5, 0x3f8c
};

static const u16 LUT3D_COMMON[LUT3D_COMMON_SIZE] = {
	0x0000, 0x003f, 0x007f, 0x00bf, 0x00ff, 0x013f, 0x017f, 0x01bf, 0x01ff,
	0x023f, 0x027f, 0x02bf, 0x02ff, 0x033f, 0x037f, 0x03bf, 0x03ff
};

/* one is for primary plane and the other is for all overlay planes */
static const struct dc8200_plane_reg dc_plane_reg[] = {
	{
		.y_address = DC_FRAMEBUFFER_ADDRESS,
		.u_address = DC_FRAMEBUFFER_U_ADDRESS,
		.v_address = DC_FRAMEBUFFER_V_ADDRESS,
		.y_stride = DC_FRAMEBUFFER_STRIDE,
		.u_stride = DC_FRAMEBUFFER_U_STRIDE,
		.v_stride = DC_FRAMEBUFFER_V_STRIDE,
		.size = DC_FRAMEBUFFER_SIZE,
		.top_left = DC_FRAMEBUFFER_TOP_LEFT,
		.bottom_right = DC_FRAMEBUFFER_BOTTOM_RIGHT,
		.scale_factor_x = DC_FRAMEBUFFER_SCALE_FACTOR_X,
		.scale_factor_y = DC_FRAMEBUFFER_SCALE_FACTOR_Y,
		.h_filter_coef_index = DC_FRAMEBUFFER_H_FILTER_COEF_INDEX,
		.h_filter_coef_data = DC_FRAMEBUFFER_H_FILTER_COEF_DATA,
		.v_filter_coef_index = DC_FRAMEBUFFER_V_FILTER_COEF_INDEX,
		.v_filter_coef_data = DC_FRAMEBUFFER_V_FILTER_COEF_DATA,
		.init_offset = DC_FRAMEBUFFER_INIT_OFFSET,
		.color_key = DC_FRAMEBUFFER_COLOR_KEY,
		.color_key_high = DC_FRAMEBUFFER_COLOR_KEY_HIGH,
		.clear_value = DC_FRAMEBUFFER_CLEAR_VALUE,
		.color_table_index = DC_FRAMEBUFFER_COLOR_TABLE_INDEX,
		.color_table_data = DC_FRAMEBUFFER_COLOR_TABLE_DATA,
		.scale_config = DC_FRAMEBUFFER_SCALE_CONFIG,
		.water_mark = DC_FRAMEBUFFER_WATER_MARK,
		.degamma_index = DC_FRAMEBUFFER_DEGAMMA_INDEX,
		.degamma_data = DC_FRAMEBUFFER_DEGAMMA_DATA,
		.degamma_ex_data = DC_FRAMEBUFFER_DEGAMMA_EX_DATA,
		.src_global_color = DC_FRAMEBUFFER_SRC_GLOBAL_COLOR,
		.dst_global_color = DC_FRAMEBUFFER_DST_GLOBAL_COLOR,
		.blend_config = DC_FRAMEBUFFER_BLEND_CONFIG,
		.roi_origin = DC_FRAMEBUFFER_ROI_ORIGIN,
		.roi_size = DC_FRAMEBUFFER_ROI_SIZE,
		.YUVToRGBCoef0 = DC_FRAMEBUFFER_YUVTORGB_COEF0,
		.YUVToRGBCoef1 = DC_FRAMEBUFFER_YUVTORGB_COEF1,
		.YUVToRGBCoef2 = DC_FRAMEBUFFER_YUVTORGB_COEF2,
		.YUVToRGBCoef3 = DC_FRAMEBUFFER_YUVTORGB_COEF3,
		.YUVToRGBCoef4 = DC_FRAMEBUFFER_YUVTORGB_COEF4,
		.YUVToRGBCoefD0 = DC_FRAMEBUFFER_YUVTORGB_COEFD0,
		.YUVToRGBCoefD1 = DC_FRAMEBUFFER_YUVTORGB_COEFD1,
		.YUVToRGBCoefD2 = DC_FRAMEBUFFER_YUVTORGB_COEFD2,
		.YClampBound = DC_FRAMEBUFFER_Y_CLAMP_BOUND,
		.UVClampBound = DC_FRAMEBUFFER_UV_CLAMP_BOUND,
		.RGBToRGBCoef0 = DC_FRAMEBUFFER_RGBTORGB_COEF0,
		.RGBToRGBCoef1 = DC_FRAMEBUFFER_RGBTORGB_COEF1,
		.RGBToRGBCoef2 = DC_FRAMEBUFFER_RGBTORGB_COEF2,
		.RGBToRGBCoef3 = DC_FRAMEBUFFER_RGBTORGB_COEF3,
		.RGBToRGBCoef4 = DC_FRAMEBUFFER_RGBTORGB_COEF4,
	},
	{
		.y_address = DC_OVERLAY_ADDRESS,
		.u_address = DC_OVERLAY_U_ADDRESS,
		.v_address = DC_OVERLAY_V_ADDRESS,
		.y_stride = DC_OVERLAY_STRIDE,
		.u_stride = DC_OVERLAY_U_STRIDE,
		.v_stride = DC_OVERLAY_V_STRIDE,
		.size = DC_OVERLAY_SIZE,
		.top_left = DC_OVERLAY_TOP_LEFT,
		.bottom_right = DC_OVERLAY_BOTTOM_RIGHT,
		.scale_factor_x = DC_OVERLAY_SCALE_FACTOR_X,
		.scale_factor_y = DC_OVERLAY_SCALE_FACTOR_Y,
		.h_filter_coef_index = DC_OVERLAY_H_FILTER_COEF_INDEX,
		.h_filter_coef_data = DC_OVERLAY_H_FILTER_COEF_DATA,
		.v_filter_coef_index = DC_OVERLAY_V_FILTER_COEF_INDEX,
		.v_filter_coef_data = DC_OVERLAY_V_FILTER_COEF_DATA,
		.init_offset = DC_OVERLAY_INIT_OFFSET,
		.color_key = DC_OVERLAY_COLOR_KEY,
		.color_key_high = DC_OVERLAY_COLOR_KEY_HIGH,
		.clear_value = DC_OVERLAY_CLEAR_VALUE,
		.color_table_index = DC_OVERLAY_COLOR_TABLE_INDEX,
		.color_table_data = DC_OVERLAY_COLOR_TABLE_DATA,
		.scale_config = DC_OVERLAY_SCALE_CONFIG,
		.water_mark = DC_OVERLAY_WATER_MARK,
		.degamma_index = DC_OVERLAY_DEGAMMA_INDEX,
		.degamma_data = DC_OVERLAY_DEGAMMA_DATA,
		.degamma_ex_data = DC_OVERLAY_DEGAMMA_EX_DATA,
		.src_global_color = DC_OVERLAY_SRC_GLOBAL_COLOR,
		.dst_global_color = DC_OVERLAY_DST_GLOBAL_COLOR,
		.blend_config = DC_OVERLAY_BLEND_CONFIG,
		.roi_origin = DC_OVERLAY_ROI_ORIGIN,
		.roi_size = DC_OVERLAY_ROI_SIZE,
		.YUVToRGBCoef0 = DC_OVERLAY_YUVTORGB_COEF0,
		.YUVToRGBCoef1 = DC_OVERLAY_YUVTORGB_COEF1,
		.YUVToRGBCoef2 = DC_OVERLAY_YUVTORGB_COEF2,
		.YUVToRGBCoef3 = DC_OVERLAY_YUVTORGB_COEF3,
		.YUVToRGBCoef4 = DC_OVERLAY_YUVTORGB_COEF4,
		.YUVToRGBCoefD0 = DC_OVERLAY_YUVTORGB_COEFD0,
		.YUVToRGBCoefD1 = DC_OVERLAY_YUVTORGB_COEFD1,
		.YUVToRGBCoefD2 = DC_OVERLAY_YUVTORGB_COEFD2,
		.YClampBound = DC_OVERLAY_Y_CLAMP_BOUND,
		.UVClampBound = DC_OVERLAY_UV_CLAMP_BOUND,
		.RGBToRGBCoef0 = DC_OVERLAY_RGBTORGB_COEF0,
		.RGBToRGBCoef1 = DC_OVERLAY_RGBTORGB_COEF1,
		.RGBToRGBCoef2 = DC_OVERLAY_RGBTORGB_COEF2,
		.RGBToRGBCoef3 = DC_OVERLAY_RGBTORGB_COEF3,
		.RGBToRGBCoef4 = DC_OVERLAY_RGBTORGB_COEF4,
	},
};

static const struct drm_prop_enum_list sync_mode_enum_list[] = {
	{ XDX_SINGLE_DC, "single dc mode" },
	{ XDX_MULTI_DC_PRIMARY, "primary dc for multi dc mode" },
	{ XDX_MULTI_DC_SECONDARY, "secondary dc for multi dc mode" },
};

#ifdef CONFIG_VERISILICON_MMU
static const struct drm_prop_enum_list mmu_prefetch_enum_list[] = {
	{ XDX_MMU_PREFETCH_DISABLE, "disable mmu prefetch" },
	{ XDX_MMU_PREFETCH_ENABLE, "enable mmu prefetch" },
};
#endif

static inline void dc_set_clear(struct dm_dc *dc, u32 reg, u32 set, u32 clear)
{
	u32 value = DC_RREG32(reg);

	value &= ~clear;
	value |= set;

	DC_WREG32(reg, value);
}

static void load_default_filter(struct dm_dc *dc,
				const struct dc8200_plane_reg *reg, u32 offset)
{
	u8 i;

	DC_WREG32(reg->scale_config + offset, 0x33);
	DC_WREG32(reg->init_offset + offset, 0x80008000);
	DC_WREG32(reg->h_filter_coef_index + offset, 0x00);
	for (i = 0; i < H_COEF_SIZE; i++)
		DC_WREG32(reg->h_filter_coef_data + offset, horKernel[i]);

	DC_WREG32(reg->v_filter_coef_index + offset, 0x00);
	for (i = 0; i < V_COEF_SIZE; i++)
		DC_WREG32(reg->v_filter_coef_data + offset, verKernel[i]);
}

static void load_rgb_to_rgb(struct dm_dc *dc,
			    const struct dc8200_plane_reg *reg, u32 offset,
			    u16 *table)
{
	DC_WREG32(reg->RGBToRGBCoef0 + offset, table[0] | (table[1] << 16));
	DC_WREG32(reg->RGBToRGBCoef1 + offset, table[2] | (table[3] << 16));
	DC_WREG32(reg->RGBToRGBCoef2 + offset, table[4] | (table[5] << 16));
	DC_WREG32(reg->RGBToRGBCoef3 + offset, table[6] | (table[7] << 16));
	DC_WREG32(reg->RGBToRGBCoef4 + offset, table[8]);
}

static void load_yuv_to_rgb(struct dm_dc *dc,
			    const struct dc8200_plane_reg *reg, u32 offset,
			    s32 *table)
{
	DC_WREG32(reg->YUVToRGBCoef0 + offset,
		  (0xFFFF & table[0]) | (table[1] << 16));
	DC_WREG32(reg->YUVToRGBCoef1 + offset,
		  (0xFFFF & table[2]) | (table[3] << 16));
	DC_WREG32(reg->YUVToRGBCoef2 + offset,
		  (0xFFFF & table[4]) | (table[5] << 16));
	DC_WREG32(reg->YUVToRGBCoef3 + offset,
		  (0xFFFF & table[6]) | (table[7] << 16));
	DC_WREG32(reg->YUVToRGBCoef4 + offset, table[8]);
	DC_WREG32(reg->YUVToRGBCoefD0 + offset, table[9]);
	DC_WREG32(reg->YUVToRGBCoefD1 + offset, table[10]);
	DC_WREG32(reg->YUVToRGBCoefD2 + offset, table[11]);
	DC_WREG32(reg->YClampBound + offset, table[12] | (table[13] << 16));
	DC_WREG32(reg->UVClampBound + offset, table[14] | (table[15] << 16));
}

static int load_3dlut_table(struct dm_dc *dc, u32 reg)
{
	u8 r, g, b;
	u32 rgb;
	for(r = 0; r < LUT3D_COMMON_SIZE; r++) {
		for(g = 0; g < LUT3D_COMMON_SIZE; g++) {
			for (b = 0; b < LUT3D_COMMON_SIZE; b++) {
				rgb = ((LUT3D_COMMON[r] & 0x3FF) << 20)
					| ((LUT3D_COMMON[g] & 0x3FF) << 10)
					| (LUT3D_COMMON[b] & 0x3FF);
				DC_WREG32(reg, rgb);
			}
		}
	}
	return 0;
}

static void load_rgb_to_yuv(struct dm_dc *dc, u32 offset, s16 *table)
{
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEF0 + offset,
		  table[0] | (table[1] << 16));
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEF1 + offset,
		  table[2] | (table[3] << 16));
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEF2 + offset,
		  table[4] | (table[5] << 16));
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEF3 + offset,
		  table[6] | (table[7] << 16));
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEF4 + offset, table[8]);
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEFD0 + offset, table[9]);
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEFD1 + offset, table[10]);
	DC_WREG32(DC_DISPLAY_RGBTOYUV_COEFD2 + offset, table[11]);
}

static bool is_rgb(enum dc_color_format format)
{
	switch (format) {
	case DC_FORMAT_X4R4G4B4:
	case DC_FORMAT_A4R4G4B4:
	case DC_FORMAT_X1R5G5B5:
	case DC_FORMAT_A1R5G5B5:
	case DC_FORMAT_R5G6B5:
	case DC_FORMAT_X8R8G8B8:
	case DC_FORMAT_A8R8G8B8:
	case DC_FORMAT_A2R10G10B10:
		return true;
	default:
		return false;
	}
}

static void load_degamma_table(struct dm_dc *dc,
			       const struct dc8200_plane_reg *reg, u32 offset,
			       u16 *table)
{
	u16 i;
	u32 value;

	DC_WREG32(reg->degamma_index + offset, 0);

	for (i = 0; i < DEGAMMA_SIZE; i++) {
		value = table[i] | (table[i] << 16);
		DC_WREG32(reg->degamma_data + offset, value);
		DC_WREG32(reg->degamma_ex_data + offset, table[i]);
	}
}

static u32 get_addr_offset(u32 id)
{
	u32 offset = 0;

	switch (id) {
	case PRIMARY_PLANE_1:
	case OVERLAY_PLANE_1:
		offset = 0x04;
		break;
	case OVERLAY_PLANE_2:
		offset = 0x08;
		break;
	case OVERLAY_PLANE_3:
		offset = 0x0C;
		break;
	default:
		break;
	}

	return offset;
}

static const u32 primary_overlay_format0[] = {
	DRM_FORMAT_XRGB4444,	DRM_FORMAT_XBGR4444,	DRM_FORMAT_RGBX4444,
	DRM_FORMAT_BGRX4444,	DRM_FORMAT_ARGB4444,	DRM_FORMAT_ABGR4444,
	DRM_FORMAT_RGBA4444,	DRM_FORMAT_BGRA4444,	DRM_FORMAT_XRGB1555,
	DRM_FORMAT_XBGR1555,	DRM_FORMAT_RGBX5551,	DRM_FORMAT_BGRX5551,
	DRM_FORMAT_ARGB1555,	DRM_FORMAT_ABGR1555,	DRM_FORMAT_RGBA5551,
	DRM_FORMAT_BGRA5551,	DRM_FORMAT_RGB565,	DRM_FORMAT_BGR565,
	DRM_FORMAT_XRGB8888,	DRM_FORMAT_XBGR8888,	DRM_FORMAT_RGBX8888,
	DRM_FORMAT_BGRX8888,	DRM_FORMAT_ARGB8888,	DRM_FORMAT_ABGR8888,
	DRM_FORMAT_RGBA8888,	DRM_FORMAT_BGRA8888,	DRM_FORMAT_ARGB2101010,
	DRM_FORMAT_ABGR2101010, DRM_FORMAT_RGBA1010102, DRM_FORMAT_BGRA1010102,
	DRM_FORMAT_YUYV,	DRM_FORMAT_YVYU,	DRM_FORMAT_UYVY,
	DRM_FORMAT_VYUY,	DRM_FORMAT_YVU420,	DRM_FORMAT_YUV420,
	DRM_FORMAT_NV12,	DRM_FORMAT_NV21,	DRM_FORMAT_NV16,
	DRM_FORMAT_NV61,	DRM_FORMAT_P010,
};

static const u32 primary_overlay_format1[] = {
	DRM_FORMAT_XRGB4444,	DRM_FORMAT_XBGR4444,	DRM_FORMAT_RGBX4444,
	DRM_FORMAT_BGRX4444,	DRM_FORMAT_ARGB4444,	DRM_FORMAT_ABGR4444,
	DRM_FORMAT_RGBA4444,	DRM_FORMAT_BGRA4444,	DRM_FORMAT_XRGB1555,
	DRM_FORMAT_XBGR1555,	DRM_FORMAT_RGBX5551,	DRM_FORMAT_BGRX5551,
	DRM_FORMAT_ARGB1555,	DRM_FORMAT_ABGR1555,	DRM_FORMAT_RGBA5551,
	DRM_FORMAT_BGRA5551,	DRM_FORMAT_RGB565,	DRM_FORMAT_BGR565,
	DRM_FORMAT_XRGB8888,	DRM_FORMAT_XBGR8888,	DRM_FORMAT_RGBX8888,
	DRM_FORMAT_BGRX8888,	DRM_FORMAT_ARGB8888,	DRM_FORMAT_ABGR8888,
	DRM_FORMAT_RGBA8888,	DRM_FORMAT_BGRA8888,	DRM_FORMAT_ARGB2101010,
	DRM_FORMAT_ABGR2101010, DRM_FORMAT_RGBA1010102, DRM_FORMAT_BGRA1010102,
	DRM_FORMAT_YUYV,	DRM_FORMAT_YVYU,	DRM_FORMAT_UYVY,
	DRM_FORMAT_VYUY,	DRM_FORMAT_YVU420,	DRM_FORMAT_YUV420,
	DRM_FORMAT_NV12,	DRM_FORMAT_NV21,	DRM_FORMAT_NV16,
	DRM_FORMAT_NV61,	DRM_FORMAT_P010,	DRM_FORMAT_YUV444,
};

static const u32 cursor_formats[] = { DRM_FORMAT_ARGB8888 };
static const u64 cursor_modifier[]={
	DRM_FORMAT_MOD_LINEAR,
	DRM_FORMAT_MOD_INVALID
};

static const u64 format_modifier0[] = {
	DRM_FORMAT_MOD_LINEAR,
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_LINEAR),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_XMAJOR),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_YMAJOR),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_TILE_8X8),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_TILE_8X4),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_XMAJOR_8X4),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_YMAJOR_4X8),
#ifdef CONFIG_VERISILICON_DEC
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_TILE_8X8_XMAJOR,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_TILE_8X4,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_TILE_4X8,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_256X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_128X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_64X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_TILE_16X8,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_32X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_TILE_32X8,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
#endif
	DRM_FORMAT_MOD_INVALID
};

static const u64 format_modifier1[] = {
	DRM_FORMAT_MOD_LINEAR,
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_LINEAR),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_XMAJOR),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_YMAJOR),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_TILE_8X8),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_TILE_8X4),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_XMAJOR_8X4),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_SUPER_TILED_YMAJOR_4X8),
	fourcc_mod_xdx_norm_code(DRM_FORMAT_MOD_XDX_TILE_MODE4X4),
	fourcc_mod_xdx_custom_code(DRM_FORMAT_MOD_XDX_TILE_MODE4X4),
	DRM_FORMAT_MOD_INVALID
};

static const u64 secondary_format_modifiers[] = {
	DRM_FORMAT_MOD_LINEAR,
#ifdef CONFIG_VERISILICON_DEC
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_256X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_128X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_64X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
	fourcc_mod_xdx_dec_code(DRM_FORMAT_MOD_XDX_DEC_RASTER_32X1,
				DRM_FORMAT_MOD_XDX_DEC_ALIGN_32),
#endif
	DRM_FORMAT_MOD_INVALID
};

#define FRAC_16_16(mult, div) (((mult) << 16) / (div))

static const struct dc8200_plane_info dc8200_planes[][PLANE_NUM] = {
	{
		/* DC_REV_0 */
		{
			.name = "Primary",
			.id = PRIMARY_PLANE_0,
			.type = DRM_PLANE_TYPE_PRIMARY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 0,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay",
			.id = OVERLAY_PLANE_0,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 2,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_1",
			.id = OVERLAY_PLANE_1,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(secondary_format_modifiers),
			.modifiers = secondary_format_modifiers,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 3,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Primary_1",
			.id = PRIMARY_PLANE_1,
			.type = DRM_PLANE_TYPE_PRIMARY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 1,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_2",
			.id = OVERLAY_PLANE_2,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 4,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_3",
			.id = OVERLAY_PLANE_3,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(secondary_format_modifiers),
			.modifiers = secondary_format_modifiers,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 5,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Cursor",
			.id = CURSOR_PLANE_0,
			.type = DRM_PLANE_TYPE_CURSOR,
			.num_formats = ARRAY_SIZE(cursor_formats),
			.formats = cursor_formats,
			.num_modifiers = ARRAY_SIZE(cursor_modifier),
			.modifiers = cursor_modifier,
			.min_width = 32,
			.min_height = 32,
			.max_width = 64,
			.max_height = 64,
			.rotation = 0,
			.degamma_size = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 255,
			.watermark = false,
			.color_mgmt = false,
			.roi = false,
		},
		{
			.name = "Cursor_1",
			.id = CURSOR_PLANE_1,
			.type = DRM_PLANE_TYPE_CURSOR,
			.num_formats = ARRAY_SIZE(cursor_formats),
			.formats = cursor_formats,
			.num_modifiers = ARRAY_SIZE(cursor_modifier),
			.modifiers = cursor_modifier,
			.min_width = 32,
			.min_height = 32,
			.max_width = 64,
			.max_height = 64,
			.rotation = 0,
			.degamma_size = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 255,
			.watermark = false,
			.color_mgmt = false,
			.roi = false,
		},
	},
	{
		/* DC_REV_1 */
		{
			.name = "Primary",
			.id = PRIMARY_PLANE_0,
			.type = DRM_PLANE_TYPE_PRIMARY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 0,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay",
			.id = OVERLAY_PLANE_0,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 2,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Primary_1",
			.id = PRIMARY_PLANE_1,
			.type = DRM_PLANE_TYPE_PRIMARY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 1,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_2",
			.id = OVERLAY_PLANE_2,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format0),
			.formats = primary_overlay_format0,
			.num_modifiers = ARRAY_SIZE(format_modifier0),
			.modifiers = format_modifier0,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 3,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Cursor",
			.id = CURSOR_PLANE_0,
			.type = DRM_PLANE_TYPE_CURSOR,
			.num_formats = ARRAY_SIZE(cursor_formats),
			.formats = cursor_formats,
			.num_modifiers = ARRAY_SIZE(cursor_modifier),
			.modifiers = cursor_modifier,
			.min_width = 32,
			.min_height = 32,
			.max_width = 64,
			.max_height = 64,
			.rotation = 0,
			.degamma_size = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 255,
			.watermark = false,
			.color_mgmt = false,
			.roi = false,
		},
		{
			.name = "Cursor_1",
			.id = CURSOR_PLANE_1,
			.type = DRM_PLANE_TYPE_CURSOR,
			.num_formats = ARRAY_SIZE(cursor_formats),
			.formats = cursor_formats,
			.num_modifiers = ARRAY_SIZE(cursor_modifier),
			.modifiers = cursor_modifier,
			.min_width = 32,
			.min_height = 32,
			.max_width = 64,
			.max_height = 64,
			.rotation = 0,
			.degamma_size = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 255,
			.watermark = false,
			.color_mgmt = false,
			.roi = false,
		},
	},
	{
		/* DC_REV_2 */
		{
			.name = "Primary",
			.id = PRIMARY_PLANE_0,
			.type = DRM_PLANE_TYPE_PRIMARY,
			.num_formats = ARRAY_SIZE(primary_overlay_format1),
			.formats = primary_overlay_format1,
			.num_modifiers = ARRAY_SIZE(format_modifier1),
			.modifiers = format_modifier1,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 0,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay",
			.id = OVERLAY_PLANE_0,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format1),
			.formats = primary_overlay_format1,
			.num_modifiers = ARRAY_SIZE(format_modifier1),
			.modifiers = format_modifier1,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 2,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_1",
			.id = OVERLAY_PLANE_1,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format1),
			.formats = primary_overlay_format1,
			.num_modifiers = ARRAY_SIZE(secondary_format_modifiers),
			.modifiers = secondary_format_modifiers,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 3,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Primary_1",
			.id = PRIMARY_PLANE_1,
			.type = DRM_PLANE_TYPE_PRIMARY,
			.num_formats = ARRAY_SIZE(primary_overlay_format1),
			.formats = primary_overlay_format1,
			.num_modifiers = ARRAY_SIZE(format_modifier1),
			.modifiers = format_modifier1,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 1,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_2",
			.id = OVERLAY_PLANE_2,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format1),
			.formats = primary_overlay_format1,
			.num_modifiers = ARRAY_SIZE(format_modifier1),
			.modifiers = format_modifier1,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = DRM_MODE_ROTATE_0 | DRM_MODE_ROTATE_90 |
				    DRM_MODE_ROTATE_180 | DRM_MODE_ROTATE_270 |
				    DRM_MODE_REFLECT_X | DRM_MODE_REFLECT_Y,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			.min_scale = FRAC_16_16(1, 3),
			.max_scale = FRAC_16_16(10, 1),
			.zpos = 4,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Overlay_3",
			.id = OVERLAY_PLANE_3,
			.type = DRM_PLANE_TYPE_OVERLAY,
			.num_formats = ARRAY_SIZE(primary_overlay_format1),
			.formats = primary_overlay_format1,
			.num_modifiers = ARRAY_SIZE(secondary_format_modifiers),
			.modifiers = secondary_format_modifiers,
			.min_width = 0,
			.min_height = 0,
			.max_width = 8192,
			.max_height = 8192,
			.rotation = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
			.blend_mode = BIT(DRM_MODE_BLEND_PIXEL_NONE) |
				      BIT(DRM_MODE_BLEND_PREMULTI) |
				      BIT(DRM_MODE_BLEND_COVERAGE),
			#endif
			.color_encoding = BIT(DRM_COLOR_YCBCR_BT709) |
					  BIT(DRM_COLOR_YCBCR_BT2020),
			.degamma_size = DEGAMMA_SIZE,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 5,
			.watermark = true,
			.color_mgmt = true,
			.roi = true,
		},
		{
			.name = "Cursor",
			.id = CURSOR_PLANE_0,
			.type = DRM_PLANE_TYPE_CURSOR,
			.num_formats = ARRAY_SIZE(cursor_formats),
			.formats = cursor_formats,
			.num_modifiers = ARRAY_SIZE(cursor_modifier),
			.modifiers = cursor_modifier,
			.min_width = 32,
			.min_height = 32,
			.max_width = 64,
			.max_height = 64,
			.rotation = 0,
			.degamma_size = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 255,
			.watermark = false,
			.color_mgmt = false,
			.roi = false,
		},
		{
			.name = "Cursor_1",
			.id = CURSOR_PLANE_1,
			.type = DRM_PLANE_TYPE_CURSOR,
			.num_formats = ARRAY_SIZE(cursor_formats),
			.formats = cursor_formats,
			.num_modifiers = ARRAY_SIZE(cursor_modifier),
			.modifiers = cursor_modifier,
			.min_width = 32,
			.min_height = 32,
			.max_width = 64,
			.max_height = 64,
			.rotation = 0,
			.degamma_size = 0,
			#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
			.min_scale = DRM_PLANE_NO_SCALING,
			.max_scale = DRM_PLANE_NO_SCALING,
			#else
			.min_scale = DRM_PLANE_HELPER_NO_SCALING,
			.max_scale = DRM_PLANE_HELPER_NO_SCALING,
			#endif
			.zpos = 255,
			.watermark = false,
			.color_mgmt = false,
			.roi = false,
		},
	},
};

static const struct dc8200_dc_info dc8200_dc_info[] = {
	{
		/* DC_REV_0 */
		.name = "DC8200",
		.panel_num = 2,
		.plane_num = 8,
		.planes = dc8200_planes[DC_REV_0],
		.layer_num = 6,
		.max_bpc = 10,
		#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
		.color_formats =
			DRM_COLOR_FORMAT_RGB444 | DRM_COLOR_FORMAT_YCRCB444 |
			DRM_COLOR_FORMAT_YCRCB422 | DRM_COLOR_FORMAT_YCRCB420,
		#else
		.color_formats =
			DRM_COLOR_FORMAT_RGB444 | DRM_COLOR_FORMAT_YCBCR444 |
			DRM_COLOR_FORMAT_YCBCR422 | DRM_COLOR_FORMAT_YCBCR420,
		#endif
		.gamma_size = GAMMA_SIZE,
		.gamma_bits = 12,
		.pitch_alignment = 128,
		.pipe_sync = false,
		.mmu_prefetch = false,
		.background = true,
		.panel_sync = true,
		.cap_dec = true,
	},
	{
		/* DC_REV_1 */
		.name = "DC8200",
		.panel_num = 2,
		.plane_num = 6,
		.planes = dc8200_planes[DC_REV_1],
		.layer_num = 4,
		.max_bpc = 10,
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
		.color_formats =
			DRM_COLOR_FORMAT_RGB444 | DRM_COLOR_FORMAT_YCRCB444 |
			DRM_COLOR_FORMAT_YCRCB422 | DRM_COLOR_FORMAT_YCRCB420,
#else
		.color_formats =
			DRM_COLOR_FORMAT_RGB444 | DRM_COLOR_FORMAT_YCBCR444 |
			DRM_COLOR_FORMAT_YCBCR422 | DRM_COLOR_FORMAT_YCBCR420,
#endif
		.gamma_size = GAMMA_SIZE,
		.gamma_bits = 12,
		.pitch_alignment = 128,
		.pipe_sync = false,
		.mmu_prefetch = false,
		.background = true,
		.panel_sync = true,
		.cap_dec = true,
	},
	{
		/* DC_REV_2 */
		.name = "DC8200",
		.panel_num = 2,
		.plane_num = 8,
		.planes = dc8200_planes[DC_REV_1],
		.layer_num = 6,
		.max_bpc = 10,
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
		.color_formats =
			DRM_COLOR_FORMAT_RGB444 | DRM_COLOR_FORMAT_YCRCB444 |
			DRM_COLOR_FORMAT_YCRCB422 | DRM_COLOR_FORMAT_YCRCB420,
#else
		.color_formats =
			DRM_COLOR_FORMAT_RGB444 | DRM_COLOR_FORMAT_YCBCR444 |
			DRM_COLOR_FORMAT_YCBCR422 | DRM_COLOR_FORMAT_YCBCR420,
#endif
		.gamma_size = GAMMA_SIZE,
		.gamma_bits = 12,
		.pitch_alignment = 128,
		.pipe_sync = false,
		.mmu_prefetch = false,
		.background = true,
		.panel_sync = true,
		.cap_dec = false,
	},
};

static const enum dc_out dc8200_out[DC_DISPLAY_NUM] = {
    DC_OUT_DP,
    DC_OUT_DP,
};
/*
static void dc8200_set_out(struct dc8200_dc *hw, enum dc_out out, u8 id)
{
	if (out <= DC_OUT_DP)
		hw->out[id] = out;
}
*/
static void gamma_ex_commit(struct dm_dc *dc)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	u8 panel_num = hw->info->panel_num;
	u16 i, j;
	u32 value;

	for (j = 0; j < panel_num; j++) {
		mutex_lock(&hw->gamma[j].gamma_mutex);
		if (hw->gamma[j].dirty) {
			if (hw->gamma[j].enable) {
				DC_WREG32(DC_DISPLAY_GAMMA_EX_INDEX + (j << 2),
					  0x00);
				for (i = 0; i < GAMMA_EX_SIZE; i++) {
					value = hw->gamma[j].gamma[i][2] |
						(hw->gamma[j].gamma[i][1]
						 << 16);
					DC_WREG32(DC_DISPLAY_GAMMA_EX_DATA +
							  (j << 2),
						  value);
					DC_WREG32(
						DC_DISPLAY_GAMMA_EX_ONE_DATA +
							(j << 2),
						hw->gamma[j].gamma[i][0]);
				}
				dc_set_clear(dc,
					     DC_DISPLAY_PANEL_CONFIG + (j << 2),
					     BIT(13), 0);
			} else {
				dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG + (j << 2),
						0, BIT(13));
			}
			hw->gamma[j].dirty = false;
		}
		mutex_unlock(&hw->gamma[j].gamma_mutex);
	}
}

static void plane_commit(struct dm_dc *dc)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct dc8200_plane *plane;
	const struct dc8200_plane_reg *reg;
	bool primary = false;
	u8 id, layer_num = hw->info->layer_num;
	u32 i, offset;

	for (i = 0; i < layer_num; i++) {
		plane = &hw->plane[i];
		id = hw->info->planes[i].id;
		offset = get_addr_offset(id);
		if (id == PRIMARY_PLANE_0 || id == PRIMARY_PLANE_1) {
			reg = &dc_plane_reg[0];
			primary = true;
		} else {
			reg = &dc_plane_reg[1];
			primary = false;
		}

		if (plane->fb.dirty) {
			if (plane->fb.data.enable) {
				DC_WREG32(reg->y_address + offset,
					  plane->fb.data.y_address);
				DC_WREG32(reg->u_address + offset,
					  plane->fb.data.u_address);
				DC_WREG32(reg->v_address + offset,
					  plane->fb.data.v_address);
				DC_WREG32(reg->y_stride + offset,
					  plane->fb.data.y_stride);
				DC_WREG32(reg->u_stride + offset,
					  plane->fb.data.u_stride);
				DC_WREG32(reg->v_stride + offset,
					  plane->fb.data.v_stride);
				DC_WREG32(reg->size + offset,
					  plane->fb.data.width |
						  (plane->fb.data.height
						   << 15));
				DC_WREG32(reg->water_mark + offset,
					  plane->fb.data.water_mark);

				if (plane->fb.data.clear_enable)
					DC_WREG32(reg->clear_value + offset,
						  plane->fb.data.clear_value);
			}

			if (primary) {
				dc_set_clear(
					dc, DC_FRAMEBUFFER_CONFIG + offset,
					(plane->fb.data.format << 26) |
						(plane->fb.data.uv_swizzle
						 << 25) |
						(plane->fb.data.swizzle << 23) |
						(plane->fb.data.tile_mode
						 << 17) |
						(plane->fb.data.yuv_color_space
						 << 14) |
						(plane->fb.data.rotation
						 << 11) |
						(plane->fb.data.clear_enable
						 << 8),
					(0x1F << 26) | BIT(25) | (0x03 << 23) |
						(0x1F << 17) | (0x07 << 14) |
						(0x07 << 11) | BIT(8));
				dc_set_clear(
					dc, DC_FRAMEBUFFER_CONFIG_EX + offset,
					(plane->fb.data.dec_enable << 1) |
						(plane->fb.data.enable << 13) |
						(plane->fb.data.zpos << 16) |
						(plane->fb.data.display_id
						 << 19),
					BIT(1) | BIT(13) | (0x07 << 16) |
						BIT(19));
			} else {
				dc_set_clear(
					dc, DC_OVERLAY_CONFIG + offset,
					(plane->fb.data.dec_enable << 27) |
						(plane->fb.data.clear_enable
						 << 25) |
						(plane->fb.data.enable << 24) |
						(plane->fb.data.format << 16) |
						(plane->fb.data.uv_swizzle
						 << 15) |
						(plane->fb.data.swizzle << 13) |
						(plane->fb.data.tile_mode
						 << 8) |
						(plane->fb.data.yuv_color_space
						 << 5) |
						(plane->fb.data.rotation << 2),
					BIT(27) | BIT(25) | BIT(24) |
						(0x1F << 16) | BIT(15) |
						(0x03 << 13) | (0x1F << 8) |
						(0x07 << 5) | (0x07 << 2));
				dc_set_clear(dc, DC_OVERLAY_CONFIG_EX + offset,
					     plane->fb.data.zpos |
						     (plane->fb.data.display_id
						      << 3),
					     0x07 | BIT(3));
			}
			plane->fb.dirty = false;
		}

		if (plane->scale.dirty) {
			if (plane->scale.data.enable) {
				DC_WREG32(reg->scale_factor_x + offset,
					  plane->scale.data.scale_factor_x);
				DC_WREG32(reg->scale_factor_y + offset,
					  plane->scale.data.scale_factor_y);
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG +
							     offset,
						     BIT(22), 0);
				else
					dc_set_clear(dc,
						     DC_OVERLAY_SCALE_CONFIG +
							     offset,
						     BIT(8), 0);
			} else {
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG +
							     offset,
						     0, BIT(22));
				else
					dc_set_clear(dc,
						     DC_OVERLAY_SCALE_CONFIG +
							     offset,
						     0, BIT(8));
			}
			plane->scale.dirty = false;
		}

		if (plane->pos.dirty) {
			DC_WREG32(reg->top_left + offset,
				  plane->pos.data.start_x |
					  (plane->pos.data.start_y << 15));
			DC_WREG32(reg->bottom_right + offset,
				  plane->pos.data.end_x | (plane->pos.data.end_y << 15));
			plane->pos.dirty = false;
		}
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
		if (plane->blend.dirty) {
			DC_WREG32(reg->src_global_color + offset,
				  plane->blend.data.alpha << 24);
			DC_WREG32(reg->dst_global_color + offset,
				  plane->blend.data.alpha << 24);
			switch (plane->blend.data.blend_mode) {
			case DC_BLEND_PREMULTI:
				DC_WREG32(reg->blend_config + offset, 0x3450);
				break;
			case DC_BLEND_COVERAGE:
				DC_WREG32(reg->blend_config + offset, 0x3950);
				break;
			case DC_BLEND_PIXEL_NONE:
				DC_WREG32(reg->blend_config + offset, 0x3548);
				break;
			default:
				break;
			}
			plane->blend.dirty = false;
		}
#else
		/* disable DC alpha blend */
		DC_WREG32(reg->blend_config + offset, 0x3452);
#endif

		if (plane->colorkey.dirty) {
			DC_WREG32(reg->color_key + offset,
				  plane->colorkey.colorkey);
			DC_WREG32(reg->color_key_high + offset,
				  plane->colorkey.colorkey_high);

			if (primary)
				dc_set_clear(dc, DC_FRAMEBUFFER_CONFIG + offset,
					     plane->colorkey.transparency << 9,
					     0x03 << 9);
			else
				dc_set_clear(dc, DC_OVERLAY_CONFIG + offset,
					     plane->colorkey.transparency,
					     0x03);

			plane->colorkey.dirty = false;
		}

		if (plane->roi.dirty) {
			if (plane->roi.data.enable) {
				DC_WREG32(reg->roi_origin + offset,
					  plane->roi.data.x | (plane->roi.data.y << 16));
				DC_WREG32(reg->roi_size + offset,
					  plane->roi.data.width |
						  (plane->roi.data.height << 16));
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     BIT(0), 0);
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     BIT(22), 0);
			} else {
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     0, BIT(0));
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     0, BIT(22));
			}
			plane->roi.dirty = false;
		}
	}
}

static void plane_ex_commit(struct dm_dc *dc)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct dc8200_plane *plane;
	const struct dc8200_plane_reg *reg;
	bool primary = false;
	u8 id, layer_num = hw->info->layer_num;
	u32 i, offset;

	for (i = 0; i < layer_num; i++) {
		plane = &hw->plane[i];
		id = hw->info->planes[i].id;
		offset = get_addr_offset(id);
		if (id == PRIMARY_PLANE_0 || id == PRIMARY_PLANE_1) {
			reg = &dc_plane_reg[0];
			primary = true;
		} else {
			reg = &dc_plane_reg[1];
			primary = false;
		}

		if (plane->fb.dirty) {
			if (is_rgb(plane->fb.data.format)) {
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     0 ,  BIT(6) | BIT(8));
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     0 , BIT(29) | BIT(30));
			} else {
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     BIT(8), BIT(6));
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     BIT(30), BIT(29));
				switch (plane->fb.data.yuv_color_space) {
				case DC_YUV_COLOR_SPACE_601:
					load_yuv_to_rgb(dc, reg, offset,
							YUV601_2RGB);
					break;
				case DC_YUV_COLOR_SPACE_709:
					load_yuv_to_rgb(dc, reg, offset,
							YUV709_2RGB);
					break;
				case DC_YUV_COLOR_SPACE_2020:
					load_yuv_to_rgb(dc, reg, offset,
							YUV2020_2RGB);
					break;
				default:
					break;
				}
			}
		}
		if (plane->degamma.dirty) {
			switch (plane->degamma.mode) {
			case XDX_DEGAMMA_DISABLE:
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     0, BIT(5));
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     0, BIT(28));
				break;
			case XDX_DEGAMMA_BT709:
				load_degamma_table(dc, reg, offset,
						   DEGAMMA_709);
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     BIT(5), 0);
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     BIT(28), 0);
				break;
			case XDX_DEGAMMA_BT2020:
				load_degamma_table(dc, reg, offset,
						   DEGAMMA_2020);
				if (primary)
					dc_set_clear(dc,
						     DC_FRAMEBUFFER_CONFIG_EX +
							     offset,
						     BIT(5), 0);
				else
					dc_set_clear(dc,
						     DC_OVERLAY_CONFIG + offset,
						     BIT(28), 0);
				break;
			default:
				break;
			}
			plane->degamma.dirty = false;
		}
	}
	plane_commit(dc);
}

static void setup_display(struct dm_dc *dc, uint32_t id)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct dc_display *display = &hw->display[id];
	u32 dpi_cfg, offset = id << 2;

	if (display->enable) {
		/* DPI just only output RGB format */
		if (hw->out[id] == DC_OUT_DPI) {
			switch (display->bpc) {
				case 6:
					display->dither_enable = true;
					dpi_cfg = 0x3;
					break;
				case 8:
					display->dither_enable = true;
					dpi_cfg = 0x5;
					break;
				case 10:
					dpi_cfg = 0x6;
					break;
				default:
					display->dither_enable = true;
					dpi_cfg = 0x5;
					break;
			}
			DC_WREG32(DC_DISPLAY_DPI_CONFIG + offset, dpi_cfg);
		}

		if (id == 0)
			dc_set_clear(dc, DC_DISPLAY_PANEL_START, 0,
				     BIT(0) | BIT(2));
		else
			dc_set_clear(dc, DC_DISPLAY_PANEL_START, 0,
				     BIT(1) | BIT(2));

		DC_WREG32(DC_DISPLAY_H + offset,
			  display->h_active | (display->h_total << 16));
		DC_WREG32(DC_DISPLAY_H_SYNC + offset,
			  display->h_sync_start | (display->h_sync_end << 15) |
				  (display->h_sync_polarity ? 0 : BIT(31)) |
				  BIT(30));
		DC_WREG32(DC_DISPLAY_V + offset,
			  display->v_active | (display->v_total << 16));
		DC_WREG32(DC_DISPLAY_V_SYNC + offset,
			  display->v_sync_start | (display->v_sync_end << 15) |
				  (display->v_sync_polarity ? 0 : BIT(31)) |
				  BIT(30));

		if (hw->info->pipe_sync) {
			switch (display->sync_mode) {
			case XDX_SINGLE_DC:
				dc_set_clear(dc, DC_FRAMEBUFFER_CONFIG_EX, 0,
					     BIT(3) | BIT(4));
				break;
			case XDX_MULTI_DC_PRIMARY:
				dc_set_clear(dc, DC_FRAMEBUFFER_CONFIG_EX,
					     BIT(3) | BIT(4), 0);
				break;
			case XDX_MULTI_DC_SECONDARY:
				dc_set_clear(dc, DC_FRAMEBUFFER_CONFIG_EX,
					     BIT(3), BIT(4));
				break;
			default:
				break;
			}
		}

		if (hw->info->background)
			DC_WREG32(DC_FRAMEBUFFER_BG_COLOR + offset,
				  display->bg_color);

		if (display->dither_enable) {
			DC_WREG32(DC_DISPLAY_DITHER_CONFIG + offset, BIT(31));
		} else {
			DC_WREG32(DC_DISPLAY_DITHER_CONFIG + offset, 0);
		}

		dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG + offset, BIT(12), 0);
		if (display->sync_enable)
			dc_set_clear(dc, DC_DISPLAY_PANEL_START,
				     BIT(2) | BIT(3), 0);
		else if (id == 0)
			dc_set_clear(dc, DC_DISPLAY_PANEL_START, BIT(0),
				     BIT(3));
		else
			dc_set_clear(dc, DC_DISPLAY_PANEL_START, BIT(1),
				     BIT(3));
	} else {
		dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG + offset, 0, BIT(12));
		if (id == 0)
			dc_set_clear(dc, DC_DISPLAY_PANEL_START, 0,
				     BIT(0) | BIT(2));
		else
			dc_set_clear(dc, DC_DISPLAY_PANEL_START, 0,
				     BIT(1) | BIT(2));
	}
}

static void setup_display_ex(struct dm_dc *dc, uint32_t id)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct dc_display *display = &hw->display[id];
	u32 dp_cfg, offset = id << 2;
	bool is_yuv = true;

	/* The preferred output color format is RGB444 */
	if (display->enable && hw->out[id] == DC_OUT_DP) {
		if (display->color_format & DRM_COLOR_FORMAT_RGB444)
		{
			is_yuv = false;
			switch (display->bpc) {
				case 6:
					display->dither_enable = true;
					dp_cfg = 1;
					break;
				case 8:
					display->dither_enable = true;
					dp_cfg = 2;
					break;
				case 10:
					dp_cfg = 3;
					break;
				default:
					display->dither_enable = true;
					dp_cfg = 2;
					break;
			}
		}
		#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
		else if(display->color_format & DRM_COLOR_FORMAT_YCRCB444)
		#else
		else if(display->color_format & DRM_COLOR_FORMAT_YCBCR444)
		#endif
		{
			switch (display->bpc) {
				case 8:
					dp_cfg = 0x4 << 4;
					break;
				case 10:
					dp_cfg = 0xa << 4;
					break;
				default:
					dp_cfg = 0x4 << 4;
					break;
			}
		}
		#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0)
		else if(display->color_format & DRM_COLOR_FORMAT_YCRCB422)
		#else
		else if(display->color_format & DRM_COLOR_FORMAT_YCBCR422)
		#endif
		{
			switch (display->bpc) {
				case 8:
					dp_cfg = 0x2 << 4;
					break;
				case 10:
					dp_cfg = 0x8 << 4;
					break;
				default:
					dp_cfg = 0x2 << 4;
					break;
			}
		} else {
			switch (display->bpc) {
				case 8:
					dp_cfg = 0x0 << 4;
					break;
				case 10:
					dp_cfg = 0x6 << 4;
					break;
				default:
					dp_cfg = 0x0 << 4;
					break;
			}
		}

		if (is_yuv)
			dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG + offset,
				     BIT(16), 0);
		else
			dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG + offset, 0,
				     BIT(16));
		DC_WREG32(DC_DISPLAY_DP_CONFIG + offset, dp_cfg | BIT(3));
	}

	if (hw->out[id] == DC_OUT_DPI)
		dc_set_clear(dc, DC_DISPLAY_DP_CONFIG + offset, 0, BIT(3));

	setup_display(dc, id);
}

static int dc8200_hw_init(struct dm_dc *dc)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct xdx_dm *dm = dc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);
	u8 i, id, panel_num, layer_num;
	u32 offset = 0;
	const struct dc8200_plane_reg *reg;

	layer_num = hw->info->layer_num;

	for (i = 0; i < layer_num; i++) {
		id = hw->info->planes[i].id;
		offset = get_addr_offset(id);
		if (id == PRIMARY_PLANE_0 || id == PRIMARY_PLANE_1)
			reg = &dc_plane_reg[0];
		else
			reg = &dc_plane_reg[1];
		load_default_filter(dc, reg, offset);
		load_rgb_to_rgb(dc, reg, offset, RGB2RGB);
	}

	panel_num = hw->info->panel_num;
	for (i = 0; i < panel_num; i++) {
		offset = i << 2;
		load_rgb_to_yuv(dc, offset, RGB2YUV);
		DC_WREG32(DC_DISPLAY_PANEL_CONFIG + offset, 0x5111);

		DC_WREG32(DC_DISPLAY_DITHER_TABLE_LOW + offset, DC_DISPLAY_DITHERTABLE_LOW);
		DC_WREG32(DC_DISPLAY_DITHER_TABLE_HIGH + offset, DC_DISPLAY_DITHERTABLE_HIGH);

		DC_WREG32(DC_DISPLAY_3DLUT_SCALE + offset , DC_DISPLAY_3DLUT_SCALE_DATA);
		DC_WREG32(DC_DISPLAY_3DLUT_OFFSET + offset , 0x0);
		DC_WREG32(DC_DISPLAY_3DLUT_OFFSET_EX + offset , 0x0);
		DC_WREG32(DC_DISPLAY_3DLUT_ENLARGE_DATA + offset , 0x0);
		DC_WREG32(DC_DISPLAY_3DLUT_INDEX + offset, 0x0);
		load_3dlut_table(dc, DC_DISPLAY_3DLUT_DATA + offset);

		offset = i ? DC_CURSOR_OFFSET : 0;
		DC_WREG32(DC_CURSOR_BACKGROUND + offset, 0x00FFFFFF);
		DC_WREG32(DC_CURSOR_FOREGROUND + offset, 0x00AAAAAA);
	}

	dc8200_mmu_init(hw, (struct dc8200_mmu*)xdev->vmm.dc.context);

	/* clear irq status */
	DC_RREG32(AQ_INTR_ACKNOWLEDGE);

	return 0;
}

static void dc8200_commit(struct dc8200_dc *hw)
{
	struct dm_dc *dc = &hw->base;

	gamma_ex_commit(dc);
	plane_ex_commit(dc);

	if (hw->qos.dirty) {
		dc_set_clear(dc, DC_QOS_CONFIG,
			     (hw->qos.data.high_value << 4) |
				     hw->qos.data.low_value,
			     0xFF);
		hw->qos.dirty = false;
	}
}

static void dc8200_plane_destory(struct dm_plane *dm_plane)
{
	struct dc8200_plane *hw_plane = to_dc8200_plane(dm_plane);

	memset(hw_plane, 0, sizeof(*hw_plane));
}
/*
static u8 get_stream_base(u8 id)
{
	u8 stream_base = 0;

	switch (id) {
	case OVERLAY_PLANE_0:
		stream_base = 3;
		break;
	case OVERLAY_PLANE_1:
		stream_base = 6;
		break;
	case PRIMARY_PLANE_1:
		stream_base = 16;
		break;
	case OVERLAY_PLANE_2:
		stream_base = 19;
		break;
	case OVERLAY_PLANE_3:
		stream_base = 22;
		break;
	default:
		break;
	}

	return stream_base;
}

static void dc8200_dec_config(struct dm_dc *dc, struct dm_plane *dm_plane,
			      struct dec_fb *dec_fb)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct dc8200_plane *plane = to_dc8200_plane(dm_plane);
	u8 stream_id;

	stream_id = get_stream_base(plane->hw_id);

	dec4001_config(&hw->dec400l, dec_fb, stream_id);
}
*/
static void dc8200_update_roi(struct dm_dc *dc, struct dm_plane *dm_plane,
			      struct dc_roi *roi)
{
	struct dc8200_plane *plane = to_dc8200_plane(dm_plane);

	if (plane) {
		memcpy(&plane->roi.data, roi, sizeof(*roi));
		plane->roi.dirty = true;
	}
}
/*
static u32 calc_factor(u32 src, u32 dest)
{
	u32 factor = 1 << 16;

	if ((src > 1) && (dest > 1))
		factor = ((src - 1) << 16) / (dest - 1);

	return factor;
}

static void update_scale(struct drm_plane_state *state, struct dc8200_roi *roi,
			 struct dc8200_scale *scale)
{
	int dst_w = drm_rect_width(&state->dst);
	int dst_h = drm_rect_height(&state->dst);
	int src_w, src_h, temp;

	scale->data.enable = false;

	if (roi->data.enable) {
		src_w = roi->data.width;
		src_h = roi->data.height;
	} else {
		src_w = drm_rect_width(&state->src) >> 16;
		src_h = drm_rect_height(&state->src) >> 16;
	}

	if (drm_rotation_90_or_270(state->rotation)) {
		temp = src_w;
		src_w = src_h;
		src_h = temp;
	}

	if (src_w != dst_w) {
		scale->data.scale_factor_x = calc_factor(src_w, dst_w);
		scale->data.enable = true;
	} else {
		scale->data.scale_factor_x = 1 << 16;
	}
	if (src_h != dst_h) {
		scale->data.scale_factor_y = calc_factor(src_h, dst_h);
		scale->data.enable = true;
	} else {
		scale->data.scale_factor_y = 1 << 16;
	}
}

static void dc8200_update_degamma(struct dm_dc *dc, struct dm_plane *dm_plane,
				  u32 mode)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	uint32_t id = dm_plane->id;
	struct dc8200_plane *plane = to_dc8200_plane(dm_plane);

	if (plane) {
		if (hw->info->planes[id].degamma_size) {
			plane->degamma.mode = mode;
			plane->degamma.dirty = true;
		} else {
			plane->degamma.dirty = false;
		}
	}
}
*/
static void dc8200_update_colorkey(struct dm_dc *dc, struct dm_plane *dm_plane,
				   struct dc_colorkey *colorkey)
{
	struct dc8200_plane *plane = to_dc8200_plane(dm_plane);

	if (plane) {
		plane->colorkey.colorkey = colorkey->colorkey;
		plane->colorkey.colorkey_high = colorkey->colorkey_high;
		plane->colorkey.transparency = (colorkey->transparency) ?
							     DC_TRANSPARENCY_KEY :
							     DC_TRANSPARENCY_OPAQUE;
		plane->colorkey.dirty = true;
	}
}
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
static void dc8200_update_plane(struct dm_dc *dc, struct dm_plane *dm_plane,
				struct dc_fb *fb, struct dc_scale *scale,
				struct dc_position *pos, struct dc_blend *blend)
{
	struct dc8200_plane *plane = to_dc8200_plane(dm_plane);

	if (plane) {
		if (fb) {
			if (fb->enable == false)
				plane->fb.data.enable = false;
			else
				memcpy(&plane->fb.data, fb, sizeof(*fb));
			plane->fb.dirty = true;
		}

		if (scale) {
			memcpy(&plane->scale.data, scale, sizeof(*scale));
			plane->scale.dirty = true;
		}

		if (pos) {
			memcpy(&plane->pos.data, pos, sizeof(*pos));
			plane->pos.dirty = true;
		}

		if (blend) {
			memcpy(&plane->blend.data, blend, sizeof(*blend));
			plane->blend.dirty = true;
		}
	}
}
#else
static void dc8200_update_plane(struct dm_dc *dc, struct dm_plane *dm_plane,
				struct dc_fb *fb, struct dc_scale *scale,
				struct dc_position *pos)
{
	struct dc8200_plane *plane = to_dc8200_plane(dm_plane);

	if (plane) {
		if (fb) {
			if (fb->enable == false)
				plane->fb.data.enable = false;
			else
				memcpy(&plane->fb.data, fb, sizeof(*fb));
			plane->fb.dirty = true;
		}

		if (scale) {
			memcpy(&plane->scale.data, scale, sizeof(*scale));
			plane->scale.dirty = true;
		}

		if (pos) {
			memcpy(&plane->pos.data, pos, sizeof(*pos));
			plane->pos.dirty = true;
		}

	}
}
#endif


static void dc8200_update_qos(struct dm_dc *dc, struct dc_qos *qos)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);

	memcpy(&hw->qos.data, qos, sizeof(*qos));
	hw->qos.dirty = true;
}

static void dc8200_update_cursor(struct dm_dc *dc, u8 id,
				 struct dc_cursor *cursor)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);

	memcpy(&hw->cursor[id].data, cursor, sizeof(*cursor));
	hw->cursor[id].dirty = true;
}

static bool dc8200_mod_supported(const struct dc8200_plane_info *plane_info,
				 u64 modifier)
{
	const u64 *mods;

	if (plane_info->modifiers == NULL)
		return false;

	for (mods = plane_info->modifiers; *mods != DRM_FORMAT_MOD_INVALID;
	     mods++) {
		if (*mods == modifier)
			return true;
	}

	return false;
}

static int dc8200_plane_check(struct dm_plane *dm_plane,
			      struct drm_plane_state *state)
{
	struct drm_framebuffer *fb = state->fb;
	struct drm_crtc *crtc = state->crtc;
	struct drm_crtc_state *crtc_state;
	struct dm_dc *dc = dm_plane->dc;
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct xdx_device *xdev = dm_to_xdev(dm_plane->dm);
	const struct dc8200_plane_info *plane_info;

	plane_info = &hw->info->planes[dm_plane->id];
	if (plane_info == NULL)
		return -EINVAL;

	if (fb->width < plane_info->min_width ||
	    fb->width > plane_info->max_width ||
	    fb->height < plane_info->min_height ||
	    fb->height > plane_info->max_height)
		dev_err_once(xdev->dev,
			     "buffer size may not support on plane%d.\n",
			     dm_plane->id);

	if ((dm_plane->base.type != DRM_PLANE_TYPE_CURSOR) &&
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
		(!dc8200_mod_supported(plane_info, fb->modifier)))
#else
		(!dc8200_mod_supported(plane_info, (u64)(fb->modifier[dm_plane->id]))))
#endif
	{
		dev_err(xdev->dev, "unsupported modifier on plane%d.\n",
			dm_plane->id);
		return -EINVAL;
	}

	crtc_state = drm_atomic_get_existing_crtc_state(state->state, crtc);
	if (IS_ERR(crtc_state))
		return -EINVAL;

	return drm_atomic_helper_check_plane_state(state, crtc_state,
						   plane_info->min_scale,
						   plane_info->max_scale, true,
						   true);
}

static const struct dm_plane_funcs dc8200_plane_funcs = {
	.destroy = dc8200_plane_destory,
	.update = dm_dc_plane_update,
	.disable = dm_dc_plane_disable,
	.check = dc8200_plane_check,
};

static const struct drm_prop_enum_list dc8200_degamma_mode_enum_list[] = {
	{ XDX_DEGAMMA_DISABLE, "disabled" },
	{ XDX_DEGAMMA_BT709, "preset degamma for BT709" },
	{ XDX_DEGAMMA_BT2020, "preset degamma for BT2020" },
};

static int dc8200_plane_init(struct xdx_dm *dm, struct dm_plane *dm_plane,
				struct dm_dc *dc, uint32_t possible_crtcs,
			     struct dc8200_plane_info *info, uint32_t layer_num,
			     uint32_t id)
{
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct drm_device *ddev = xdev_to_drm(xdev);
	int ret;

	if (!dm_plane || !info)
		return -EINVAL;

	dev_dbg(xdev->dev, "plane[%d]: %s\n", id, info->name);

	ret = dm_universal_plane_init(dm, dm_plane, dc, possible_crtcs, (struct dm_plane_funcs *)&dc8200_plane_funcs,
				      info->formats, info->num_formats,
				      info->modifiers, info->type, id, info->zpos, info->name);
	if (ret) {
		dev_err(xdev->dev, "dm failed to init plane %d\n", ret);
		return ret;
	}

	/* Set up the plane properties */
	if (info->degamma_size) {
		dm_plane->degamma_mode = drm_property_create_enum(
			ddev, 0, "DEGAMMA_MODE", dc8200_degamma_mode_enum_list,
			ARRAY_SIZE(dc8200_degamma_mode_enum_list));

		if (!dm_plane->degamma_mode) {
			ret = -ENOMEM;
			goto error_cleanup_plane;
		}

		drm_object_attach_property(&dm_plane->base.base,
					   dm_plane->degamma_mode,
					   XDX_DEGAMMA_DISABLE);
	}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	if (info->rotation) {
		ret = drm_plane_create_rotation_property(
			&dm_plane->base, DRM_MODE_ROTATE_0, info->rotation);
		if (ret)
			goto error_cleanup_plane;
	}
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
	if (info->blend_mode) {
		ret = drm_plane_create_blend_mode_property(&dm_plane->base,
							   info->blend_mode);
		if (ret)
			goto error_cleanup_plane;
		ret = drm_plane_create_alpha_property(&dm_plane->base);
		if (ret)
			goto error_cleanup_plane;
	}
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 17, 0))
	if (info->color_encoding) {
		ret = drm_plane_create_color_properties(
			&dm_plane->base, info->color_encoding,
			BIT(DRM_COLOR_YCBCR_LIMITED_RANGE),
			DRM_COLOR_YCBCR_BT709, DRM_COLOR_YCBCR_LIMITED_RANGE);
		if (ret)
			goto error_cleanup_plane;
	}
#endif
	if (info->zpos != 255) {
		ret = drm_plane_create_zpos_property(
			&dm_plane->base, info->zpos, 0, layer_num - 1);
		if (ret)
			goto error_cleanup_plane;
	}
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
	else {
		ret = drm_plane_create_zpos_immutable_property(&dm_plane->base,
							       info->zpos);
		if (ret)
			goto error_cleanup_plane;
	}
#endif

	if (info->watermark) {
		dm_plane->watermark_prop = drm_property_create(
			ddev, DRM_MODE_PROP_BLOB, "WATERMARK", 0);
		if (!dm_plane->watermark_prop) {
			ret = -ENOMEM;
			goto error_cleanup_plane;
		}

		drm_object_attach_property(&dm_plane->base.base,
					   dm_plane->watermark_prop, 0);
	}

	if (info->color_mgmt) {
		dm_plane->color_mgmt_prop = drm_property_create(
			ddev, DRM_MODE_PROP_BLOB, "COLOR_CONFIG", 0);
		if (!dm_plane->color_mgmt_prop) {
			ret = -ENOMEM;
			goto error_cleanup_plane;
		}

		drm_object_attach_property(&dm_plane->base.base,
					   dm_plane->color_mgmt_prop, 0);
	}

	if (info->roi) {
		dm_plane->roi_prop =
			drm_property_create(ddev, DRM_MODE_PROP_BLOB, "ROI", 0);
		if (!dm_plane->roi_prop) {
			ret = -ENOMEM;
			goto error_cleanup_plane;
		}

		drm_object_attach_property(&dm_plane->base.base,
					   dm_plane->roi_prop, 0);
	}

	if (info->type == DRM_PLANE_TYPE_PRIMARY) {
		ddev->mode_config.min_width = info->min_width;
		ddev->mode_config.min_height = info->min_height;
		ddev->mode_config.max_width = info->max_width;
		ddev->mode_config.max_height = info->max_height;
	}

	if (info->type == DRM_PLANE_TYPE_CURSOR) {
		ddev->mode_config.cursor_width = info->max_width;
		ddev->mode_config.cursor_height = info->max_height;
	}

	dm_plane->id = info->id;

	return 0;

error_cleanup_plane:
	drm_plane_cleanup(&dm_plane->base);
	return ret;
}

static void dc8200_crtc_destroy(struct dm_crtc *dm_crtc)
{
	struct dc8200_crtc *dc8200_crtc = to_dc8200_crtc(dm_crtc);

	kfree(dc8200_crtc);
}

static void dc8200_setup_display(struct dc8200_dc *hw,
				 struct dc_display *display)
{
	u8 id = display->id;

	memcpy(&hw->display[id], display, sizeof(*display));

	setup_display_ex(&hw->base, id);
}

static int dc8200_crtc_enable(struct dm_crtc *dm_crtc)
{
	struct drm_crtc *crtc = &dm_crtc->base;
	struct dm_crtc_state *dm_state = to_dm_crtc_state(crtc->state);
	struct dc8200_crtc_state *dc8200_state = to_dc8200_crtc_state(dm_state);
	struct drm_display_mode *mode = &crtc->state->adjusted_mode;
	struct dc_display display = {0};
	struct dm_dc *dc = dm_crtc->dc;
	struct dc8200_dc *hw = to_dc8200_dc(dc);

	display.color_format = dm_state->output_fmt;
	display.bpc = dm_state->bpc;
	display.h_active = mode->hdisplay;
	display.h_total = mode->htotal;
	display.h_sync_start = mode->hsync_start;
	display.h_sync_end = mode->hsync_end;
	if (mode->flags & DRM_MODE_FLAG_PHSYNC)
		display.h_sync_polarity = true;
	else
		display.h_sync_polarity = false;

	display.v_active = mode->vdisplay;
	display.v_total = mode->vtotal;
	display.v_sync_start = mode->vsync_start;
	display.v_sync_end = mode->vsync_end;
	if (mode->flags & DRM_MODE_FLAG_PVSYNC)
		display.v_sync_polarity = true;
	else
		display.v_sync_polarity = false;

	display.sync_mode = dc8200_state->sync_mode;
	display.bg_color = dc8200_state->bg_color;

	display.id = dm_crtc->id;
	display.sync_enable = dc8200_state->sync_enable;
	display.dither_enable = dc8200_state->dither_enable;

	display.enable = true;

#ifdef CONFIG_VERISILICON_MMU
	if (dc8200_state->mmu_prefetch == XDX_MMU_PREFETCH_ENABLE)
		dc8200_enable_mmu_prefetch(hw, true);
	else
		dc8200_enable_mmu_prefetch(hw, false);
#endif

	dc8200_setup_display(hw, &display);

	return 0;
}

static int dc8200_crtc_disable(struct dm_crtc *dm_crtc)
{
	struct dm_dc *dc = dm_crtc->dc;
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	struct dc_display display;

	display.id = dm_crtc->id;
	display.enable = false;
	dc8200_setup_display(hw, &display);

	return 0;
}

static bool dc8200_crtc_mode_fixup(struct dm_crtc *dm_crtc,
				   const struct drm_display_mode *mode,
				   struct drm_display_mode *adjusted_mode)
{
	/* TODO: */
	return true;
}

static void dc8200_update_gamma(struct dc8200_dc *hw, u8 id, u16 index, u16 r,
				u16 g, u16 b)
{

	if (index >= GAMMA_EX_SIZE)
		return;

	hw->gamma[id].gamma[index][0] = r;
	hw->gamma[id].gamma[index][1] = g;
	hw->gamma[id].gamma[index][2] = b;
	hw->gamma[id].dirty = true;
}

static void dc8200_enable_gamma(struct dc8200_dc *hw, u8 id, bool enable)
{
	hw->gamma[id].enable = enable;
	hw->gamma[id].dirty = true;
}

static int dc8200_crtc_set_gamma(struct dm_crtc *dm_crtc,
				 struct drm_color_lut *lut, unsigned int size)
{
	struct xdx_dm *dm = dm_crtc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct dm_dc *dc = dm_crtc->dc;
	struct dc8200_dc *hw = to_dc8200_dc(dc);
	int ret = 0;
	u16 *red, *green, *blue;
	u16 *new_red, *new_green, *new_blue, entry, i, j;
	u16 phase1_entry_num, phase2_entry_num, phase3_entry_num, total_entry_num;
	u8 bits, id;

	red = green = blue = NULL;
	new_red = new_green = new_blue = NULL;
	id = dm_crtc->id;
	bits = hw->info->gamma_bits;
	phase1_entry_num = 32, phase2_entry_num = 15, phase3_entry_num = 253, entry = 0;
	total_entry_num = phase1_entry_num + phase2_entry_num + phase3_entry_num;

	//extend 256entries gamma table to 300entries gamma table
	if (size != hw->info->gamma_size) {
		dev_err(xdev->dev, "gamma size does not match!\n");
		ret = -EFAULT;
		goto done;
	}

	red = (u16*)kmalloc(size * 3 * sizeof(u16), GFP_KERNEL);
	new_red = (u16*)kmalloc(GAMMA_EX_SIZE * 3 * sizeof(u16), GFP_KERNEL);
	if (!red || !new_red) {
		dev_err(xdev->dev, "alloc for new gamma table failed.\n");
		ret = -EFAULT;
		goto done;
	}
	green = red + size;
	blue = green + size;
	new_green = new_red + GAMMA_EX_SIZE;
	new_blue = new_green + GAMMA_EX_SIZE;
	for (i = 0; i < size; i++) {
		red[i] = drm_color_lut_extract(lut[i].red, bits);
		green[i] = drm_color_lut_extract(lut[i].green, bits);
		blue[i] = drm_color_lut_extract(lut[i].blue, bits);
	}

	for (; entry < phase1_entry_num; entry++) {
		new_red[entry] = ((red[1] >> 1) / phase1_entry_num) * entry  + red[0] +
				 entry * (((red[1] >> 1) % phase1_entry_num)) / phase1_entry_num;
		new_green[entry] = ((green[1] >> 1) / phase1_entry_num) * entry + green[0] +
				 entry * (((green[1] >> 1) % phase1_entry_num)) / phase1_entry_num;
		new_blue[entry] = ((blue[1] >> 1) / phase1_entry_num) * entry + blue[0] +
				 entry * (((blue[1] >> 1) % phase1_entry_num)) / phase1_entry_num;
	}
	new_red[entry] = red[1] >> 1;
	new_green[entry] = green[1] >> 1;
	new_blue[entry] = blue[1] >> 1;
	entry++;
	new_red[entry] = (red[1] >> 1) + (red[1] / 6);
	new_green[entry] = (green[1] >> 1) + (green[1] / 6);
	new_blue[entry] = (blue[1] >> 1) + (blue[1] / 6);
	entry++;
	new_red[entry] = (red[1] >> 1) + (red[1] / 3);
	new_green[entry] = (green[1] >> 1) + (green[1] / 3);
	new_blue[entry] = (blue[1] >> 1) + (blue[1] / 3);
	entry++;
	for (i = 1; i < 3; i++) {
		for (j = 0; j < 6; j++) {
			new_red[entry] = ((red[i+1] - red[i]) / 6) * j + red[i];
			new_green[entry] = ((green[i+1] - green[i]) / 6) * j + green[i];
			new_blue[entry] = ((blue[i+1] - blue[i]) / 6) * j + blue[i];
			entry++;
		}
	}
	for (; entry < total_entry_num; entry++) {
		new_red[entry] = red[entry - 44];
		new_green[entry] = green[entry - 44];
		new_blue[entry] = blue[entry - 44];
	}
	for (entry = 0; entry < GAMMA_EX_SIZE; entry++)
		dc8200_update_gamma(hw, id, entry, new_red[entry], new_green[entry], new_blue[entry]);
done:
	if (red)
		kfree(red);
	if (new_red)
		kfree(new_red);
	return ret;
}

static int dc8200_crtc_enable_gamma(struct dm_crtc *dm_crtc, bool enable)
{
	struct dm_dc *dc = dm_crtc->dc;
	struct dc8200_dc *dc8200_dc = to_dc8200_dc(dc);

	dc8200_enable_gamma(dc8200_dc, dm_crtc->id, enable);

	return 0;
}

static int dc8200_crtc_enable_vblank(struct dm_crtc *dm_crtc, bool enable)
{
	struct dm_dc *dc = dm_crtc->dc;
	union aq_intr_enbl intr_enbl;

	intr_enbl.value = DC_RREG32(AQ_INTR_ENBL);

	if (dm_crtc->id == 0)
		intr_enbl.bitfiles.dc_panel_0_intr = enable ? 1 : 0;
	else if (dm_crtc->id == 1)
		intr_enbl.bitfiles.dc_panel_1_intr = enable ? 1 : 0;
	else
		WARN_ON(1);

	DC_WREG32(AQ_INTR_ENBL, intr_enbl.value);

	return 0;
}

static void dc8200_enable_shadow_register(struct dc8200_dc *hw, bool enable)
{
	u32 i, offset;
	u8 id, layer_num = hw->info->layer_num;
	u8 panel_num = hw->info->panel_num;
	struct dm_dc *dc = &hw->base;

	for (i = 0; i < layer_num; i++) {
		id = hw->info->planes[i].id;
		offset = get_addr_offset(id);
		if (enable) {
			if (id == PRIMARY_PLANE_0 || id == PRIMARY_PLANE_1)
				dc_set_clear(dc,
					     DC_FRAMEBUFFER_CONFIG_EX + offset,
					     BIT(12), 0);
			else
				dc_set_clear(dc, DC_OVERLAY_CONFIG + offset,
					     BIT(31), 0);
		} else {
			if (id == PRIMARY_PLANE_0 || id == PRIMARY_PLANE_1)
				dc_set_clear(dc,
					     DC_FRAMEBUFFER_CONFIG_EX + offset,
					     0, BIT(12));
			else
				dc_set_clear(dc, DC_OVERLAY_CONFIG + offset, 0,
					     BIT(31));
		}
	}

	for (i = 0; i < panel_num; i++) {
		offset = i << 2;
		if (enable)
			dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG_EX + offset, 0,
				     BIT(0));
		else
			dc_set_clear(dc, DC_DISPLAY_PANEL_CONFIG_EX + offset,
				     BIT(0), 0);
	}
}

static int dc8200_crtc_commit(struct dm_crtc *dm_crtc)
{
	struct xdx_dm *dm = dm_crtc->dm;
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct device *dev = xdev_to_drm(xdev)->dev;
	struct dm_dc *dc = dm_crtc->dc;
	struct dc8200_dc *hw = to_dc8200_dc(dc);

	if (xdev->flags.dec_support)
		dec4001_commit(&hw->dec400l, dc);

	dc8200_enable_shadow_register(hw, false);
	dc8200_cursor_commit(dm_crtc);
	dc8200_commit(hw);
	dc8200_enable_shadow_register(hw, true);

	pm_runtime_mark_last_busy(dev);

	return 0;
}

static int dc8200_cursor_commit(struct dm_crtc *dm_crtc)
{
	struct dm_dc *dc = dm_crtc->dc;
	struct dc8200_dc *hw = to_dc8200_dc(dc);

	u8 plane_num = hw->info->plane_num;
	u8 layer_num = hw->info->layer_num;
	u8 cursor_num = plane_num - layer_num;
	u8 i = 0;
	u32 offset = 0;
	for (i = 0; i < cursor_num; i++) {
		if (hw->cursor[i].dirty) {
			offset = hw->cursor[i].data.display_id ?
						   DC_CURSOR_OFFSET :
						   0;
			if (hw->cursor[i].data.enable) {
				DC_WREG32(DC_CURSOR_ADDRESS + offset,
					  hw->cursor[i].data.address);
				DC_WREG32(DC_CURSOR_LOCATION + offset,
					  hw->cursor[i].data.x |
						  (hw->cursor[i].data.y << 16));
				dc_set_clear(dc, DC_CURSOR_CONFIG + offset,
						 (hw->cursor[i].data.hot_x << 16) |
							 (hw->cursor[i].data.hot_y
							  << 8) |
							 (hw->cursor[i].data.size
							  << 5) |
							 BIT(3) | BIT(2) | 0x02,
						 (0xFF << 16) | (0xFF << 8) |
							 (0x07 << 5) | 0x1F);
			} else {
				dc_set_clear(dc, DC_CURSOR_CONFIG + offset,
						 BIT(3), 0x03);
			}
			hw->cursor[i].dirty = false;
		}
	}

	return 0;
}

static struct dm_crtc_state *dc8200_crtc_state_create(struct dm_crtc *dm_crtc)
{
	struct dc8200_crtc_state *dc8200_crtc_state;

	dc8200_crtc_state = kzalloc(sizeof(*dc8200_crtc_state), GFP_KERNEL);
	if (!dc8200_crtc_state)
		return NULL;

	dc8200_crtc_state->sync_mode = XDX_SINGLE_DC;
#ifdef CONFIG_VERISILICON_MMU
	dc8200_crtc_state->mmu_prefetch = XDX_MMU_PREFETCH_DISABLE;
#endif

	return &dc8200_crtc_state->base;
}

static struct dm_crtc_state* dc8200_crtc_duplicate_state(struct dm_crtc_state *ori_state)
{
	struct dc8200_crtc_state *old = to_dc8200_crtc_state(ori_state);
	struct dc8200_crtc_state *new = kmemdup(old, sizeof(*old), GFP_KERNEL);

	if (!new)
		return NULL;

	return &new->base;
}

static void dc8200_crtc_state_destroy(struct dm_crtc_state *dm_crtc_state)
{
	struct dc8200_crtc_state *state = to_dc8200_crtc_state(dm_crtc_state);

	kfree(state);
}
/*
static int dc8200_crtc_set_property(struct dm_crtc *dm_crtc,
				    struct dm_crtc_state *dm_state,
				    struct drm_property *property, uint64_t val)
{
	struct dc8200_crtc_state *state = to_dc8200_crtc_state(dm_state);
	struct drm_crtc *crtc = &dm_crtc->base;
	struct drm_device *ddev = crtc->dev;
	struct xdx_device *xdev = drm_to_xdev(ddev);
	struct dc8200_crtc *dc8200_crtc = to_dc8200_crtc(dm_crtc);

	if (property == dc8200_crtc->sync_mode)
		state->sync_mode = val;
	else if (property == dc8200_crtc->mmu_prefetch)
		state->mmu_prefetch = val;
	else if (property == dc8200_crtc->bg_color)
		state->bg_color = val;
	else if (property == dc8200_crtc->panel_sync)
		state->sync_enable = val;
	else if (property == dc8200_crtc->dither)
		state->dither_enable = val;
	else {
		dev_err(xdev->dev, "wrong crtc property %p, name %s\n",
			property, property->name);
		return -EINVAL;
	}

	return 0;
}
*/
static int dc8200_crtc_get_property(struct dm_crtc *dm_crtc,
				    const struct dm_crtc_state *dm_state,
				    struct drm_property *property,
				    uint64_t *val)
{
	struct dc8200_crtc_state *state = to_dc8200_crtc_state((struct dm_crtc_state *)dm_state);
	struct drm_crtc *crtc = &dm_crtc->base;
	struct drm_device *ddev = crtc->dev;
	struct xdx_device *xdev = drm_to_xdev(ddev);
	struct dc8200_crtc *dc8200_crtc = to_dc8200_crtc(dm_crtc);

	if (property == dc8200_crtc->sync_mode)
		*val = state->sync_mode;
	else if (property == dc8200_crtc->mmu_prefetch)
		*val = state->mmu_prefetch;
	else if (property == dc8200_crtc->bg_color)
		*val = state->bg_color;
	else if (property == dc8200_crtc->panel_sync)
		*val = state->sync_enable;
	else if (property == dc8200_crtc->dither)
		*val = state->dither_enable;
	else {
		dev_err(xdev->dev, "wrong crtc property %p, name %s\n",
			property, property->name);
		return -EINVAL;
	}

	return 0;
}

static struct dm_crtc_funcs dc8200_crtc_funs = {
	.destroy = dc8200_crtc_destroy,
	.enable = dc8200_crtc_enable,
	.disable = dc8200_crtc_disable,
	.mode_fixup = dc8200_crtc_mode_fixup,
	.set_gamma = dc8200_crtc_set_gamma,
	.enable_gamma = dc8200_crtc_enable_gamma,
	.enable_vblank = dc8200_crtc_enable_vblank,
	.commit = dc8200_crtc_commit,
	.cursor_commit = dc8200_cursor_commit,
	.crtc_state_create = dc8200_crtc_state_create,
	.crtc_duplicate_state = dc8200_crtc_duplicate_state,
	.crtc_state_destroy = dc8200_crtc_state_destroy,
	.crtc_get_property = dc8200_crtc_get_property,
};

static const struct drm_prop_enum_list dc8200_sync_mode_enum_list[] = {
	{ XDX_SINGLE_DC, "single dc mode" },
	{ XDX_MULTI_DC_PRIMARY, "primary dc for multi dc mode" },
	{ XDX_MULTI_DC_SECONDARY, "secondary dc for multi dc mode" },
};

static const struct drm_prop_enum_list dc8200_mmu_prefetch_enum_list[] = {
	{ XDX_MMU_PREFETCH_DISABLE, "disable mmu prefetch" },
	{ XDX_MMU_PREFETCH_ENABLE, "enable mmu prefetch" },
};

static struct dc8200_crtc *dc8200_crtc_create(struct xdx_dm *dm,
						struct dm_dc *dc,
						struct dc8200_dc_info *info,
					      	struct dm_plane *primary,
					      	struct dm_plane *cursor,
					      	uint32_t id)
{
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct drm_device *ddev = xdev_to_drm(xdev);
	struct dc8200_crtc *dc8200_crtc;
	struct dm_crtc *dm_crtc;
	int ret;

	dc8200_crtc = kzalloc(sizeof(struct dc8200_crtc), GFP_KERNEL);
	if (!dc8200_crtc)
		return NULL;

	dm_crtc = &dc8200_crtc->base;

	ret = dm_universal_crtc_init(dm, dm_crtc, dc, &dc8200_crtc_funs,
				     primary, cursor, id);
	if (ret) {
		dev_err(xdev->dev, "dm: failed to init crtc %d\n", ret);
		goto err_free_crtc;
	}

	/* Set up the crtc properties */
	if (info->pipe_sync) {
		dc8200_crtc->sync_mode = drm_property_create_enum(
			ddev, 0, "SYNC_MODE", dc8200_sync_mode_enum_list,
			ARRAY_SIZE(dc8200_sync_mode_enum_list));

		if (!dc8200_crtc->sync_mode)
			goto err_cleanup_crts;

		drm_object_attach_property(&dm_crtc->base.base,
					   dc8200_crtc->sync_mode,
					   XDX_SINGLE_DC);
	}

	if (info->gamma_size) {
		ret = drm_mode_crtc_set_gamma_size(&dm_crtc->base,
						   info->gamma_size);
		if (ret)
			goto err_cleanup_crts;

		drm_crtc_enable_color_mgmt(&dm_crtc->base, 0, false,
					   info->gamma_size);
	}

	if (info->background) {
		dc8200_crtc->bg_color = drm_property_create_range(
			ddev, 0, "BG_COLOR", 0, 0xffffffff);

		if (!dc8200_crtc->bg_color)
			goto err_cleanup_crts;

		drm_object_attach_property(&dm_crtc->base.base,
					   dc8200_crtc->bg_color, 0);
	}

	if (info->panel_sync) {
		dc8200_crtc->panel_sync =
			drm_property_create_bool(ddev, 0, "SYNC_ENABLED");

		if (!dc8200_crtc->panel_sync)
			goto err_cleanup_crts;

		drm_object_attach_property(&dm_crtc->base.base,
					   dc8200_crtc->panel_sync, 0);
	}

	dc8200_crtc->dither =
		drm_property_create_bool(ddev, 0, "DITHER_ENABLED");
	if (!dc8200_crtc->dither)
		goto err_cleanup_crts;

	drm_object_attach_property(&dm_crtc->base.base, dc8200_crtc->dither, 0);

	if (info->mmu_prefetch) {
		dc8200_crtc->mmu_prefetch = drm_property_create_enum(
			ddev, 0, "MMU_PREFETCH",
			dc8200_mmu_prefetch_enum_list,
			ARRAY_SIZE(dc8200_mmu_prefetch_enum_list));
		if (!dc8200_crtc->mmu_prefetch)
			goto err_cleanup_crts;

		drm_object_attach_property(&dm_crtc->base.base,
					   dc8200_crtc->mmu_prefetch,
					   XDX_MMU_PREFETCH_DISABLE);
	}

	dc8200_crtc->max_bpc = info->max_bpc;
	dc8200_crtc->color_formats = info->color_formats;

	return dc8200_crtc;

err_cleanup_crts:
	drm_crtc_cleanup(&dm_crtc->base);

err_free_crtc:
	kfree(dc8200_crtc);

	return NULL;
}

static void dc8200_destroy(struct dm_dc *dc)
{
	struct dc8200_dc *dc8200_dc = to_dc8200_dc(dc);

	kfree(dc8200_dc);
}

static void dc8200_enable_interrupt(struct dm_dc *dc, bool enable)
{
	if (enable)
		DC_WREG32(AQ_INTR_ENBL, 0xFFFFFFFF);
	else
		DC_WREG32(AQ_INTR_ENBL, 0);
}

static void dc8200_crtc_handle_vblank(struct dm_crtc *dm_crtc, bool underflow)
{
	struct drm_crtc *crtc = &dm_crtc->base;
	struct dm_crtc_state *dm_state =  crtc->state ? to_dm_crtc_state(crtc->state) : NULL;
	struct dc8200_crtc_state *dc8200_state = dm_state ? to_dc8200_crtc_state(dm_state) : NULL;
	dc_irq_callback *vsync_cb = 0;
	struct list_head *temp = 0;

	drm_crtc_handle_vblank(crtc);

        if (dc8200_state)
	        dc8200_state->underflow = underflow;

        list_for_each(temp,&dm_crtc->dc->dc_irq_tbl)
        {
                vsync_cb = list_entry(temp,dc_irq_callback,list);
                if(vsync_cb->func)
                {
                        vsync_cb->func(vsync_cb->user_data);
                }
        }
}

static void dc8200_interrupt_handle(struct dm_dc *dc)
{
	union aq_intr_acknowledge ack;
	struct xdx_device *xdev = dm_to_xdev(dc->dm);

	if (unlikely(atomic_read(&xdev->in_runpm)))
		return;

	ack.value = DC_RREG32(AQ_INTR_ACKNOWLEDGE);

	if (ack.bitfiles.dc_panel_0_intr)
		dc8200_crtc_handle_vblank(
			dc->crtcs[0], ack.bitfiles.dc_panel_0_data_underflow);

	if (ack.bitfiles.dc_panel_1_intr)
		dc8200_crtc_handle_vblank(
			dc->crtcs[1], ack.bitfiles.dc_panel_0_data_underflow);
}

static void dc8200_query_hw_info(struct dm_dc *dc, struct dc_hw_info *info)
{
	struct dc8200_dc *hw = to_dc8200_dc(dc);

	info->pitch_alignment = hw->info->pitch_alignment;
}

static void dc8200_mmu_flush(struct dm_dc *dc)
{
	dc8200_mmu_reg_write(dc, SE_MMU_REG_CONFIG, 0x10);
	dc8200_mmu_reg_write(dc, SE_MMU_REG_CONFIG, 0x00);
}

static const struct dm_dc_funcs dc8200_funcs = {
	.destroy = dc8200_destroy,
	.reg_read = dc_reg_read,
	.reg_write = dc_reg_write,
	.enable_interrupt = dc8200_enable_interrupt,
	.hw_init = dc8200_hw_init,
	.update_roi = dc8200_update_roi,
	.update_colorkey = dc8200_update_colorkey,
	.update_plane = dc8200_update_plane,
	.update_qos = dc8200_update_qos,
	.update_cursor = dc8200_update_cursor,
	.interrupt_handle = dc8200_interrupt_handle,
	.mmu_flush = dc8200_mmu_flush,
	.query_hw_info = dc8200_query_hw_info,
};

int dc8200_early_init(struct xdx_dm *dm, u32 id)
{
	struct xdx_device *xdev = dm_to_xdev(dm);
	struct drm_device *ddev = xdev_to_drm(xdev);
	struct dc8200_dc_info *dc_info;
	struct dc8200_dc *dc8200_dc;
	struct dc8200_plane_info *plane_info;
	struct drm_plane *drm_plane, *tmp;
	int ret;
	uint32_t i;
	uint32_t plane_idx = 0, cusor_idx = 0;

	dc8200_dc = kzalloc(sizeof(*dc8200_dc), GFP_KERNEL);
	if (!dc8200_dc)
		return -ENOMEM;

	ret = dm_dc_universal_init(dm, &dc8200_dc->base, &dc8200_funcs, id);
	if (ret) {
		dev_err(xdev->dev, "failed to init dc %d\n", ret);
		goto err_free_dc;
	}

	/* PANGU using DC_REV_0, no need to read register */
	dc_info = (struct dc8200_dc_info *)&dc8200_dc_info[0];

	dc8200_dc->info = dc_info;
	dc8200_dc->base.gamma_size = dc_info->gamma_size;
	dc8200_dc->base.gamma_bits = dc_info->gamma_bits;

	/* Create planes */
	for (i = 0; i < dc_info->plane_num; i++) {
		uint32_t crtc_mask = 0;
		plane_info = (struct dc8200_plane_info *)&dc_info->planes[i];

#ifndef FALLTHROUGH_INTENDED
#define FALLTHROUGH_INTENDED __fallthrough
#endif
		switch (plane_info->type) {
		case DRM_PLANE_TYPE_OVERLAY:
			crtc_mask = ((1 << dc_info->panel_num) -1 ) << (dc_info->panel_num * id);
			FALLTHROUGH_INTENDED;
		case DRM_PLANE_TYPE_PRIMARY:
			ret = dc8200_plane_init(
				dm, &dc8200_dc->plane[plane_idx].base, &dc8200_dc->base,
				crtc_mask, plane_info, dc_info->layer_num, plane_info->id);
			dc8200_dc->plane[plane_idx].hw_id = plane_info->id;
			plane_idx++;
			break;
		case DRM_PLANE_TYPE_CURSOR:
			ret = dc8200_plane_init(
				dm, &dc8200_dc->cursor[cusor_idx].base, &dc8200_dc->base,
				crtc_mask, plane_info, dc_info->layer_num, plane_info->id);
			dc8200_dc->cursor[cusor_idx].hw_id = plane_info->id;
			cusor_idx++;
			break;
		default:
			dev_err(xdev->dev, "wrong plane type: %d\n",
				plane_info->type);
			break;
		}
		if (plane_idx > DC_LAYER_NUM || cusor_idx > DC_CURSOR_NUM)
			dev_err(xdev->dev,
				"wrong plane or cusor index: %d, %d\n",
				plane_idx, cusor_idx);

		if (ret)
			goto err_cleanup_planes;
	}

	/* Create crtc */
	for (i = 0; i < dc_info->panel_num; i++) {
		struct dc8200_crtc *dc8200_crtc;
		struct dm_plane *primary = NULL, *cursor = NULL;

		dc8200_dc->out[i] = dc8200_out[i];
		drm_for_each_plane(drm_plane, ddev) {
			if (!strcmp(drm_plane->name, "Primary") && (i == 0))
				primary = to_dm_plane(drm_plane);

			if (!strcmp(drm_plane->name, "Primary_1") && (i == 1))
				primary = to_dm_plane(drm_plane);

			if (!strcmp(drm_plane->name, "Cursor") && (i == 0))
				cursor = to_dm_plane(drm_plane);

			if (!strcmp(drm_plane->name, "Cursor_1") && (i == 1))
				cursor = to_dm_plane(drm_plane);
		}

		dc8200_crtc =
			dc8200_crtc_create(dm, &dc8200_dc->base, dc_info, primary, cursor, i);
		if (!dc8200_crtc) {
			ret = -ENOMEM;
			goto err_cleanup_planes;
		}
		mutex_init(&dc8200_dc->gamma[i].gamma_mutex);
	}
	mutex_init(&dc8200_dc->cursor_mtx);
	return 0;

err_cleanup_planes:
	list_for_each_entry_safe (drm_plane, tmp, &ddev->mode_config.plane_list,
				  head)
		if (drm_plane->possible_crtcs)
			drm_plane_cleanup(drm_plane);

err_free_dc:
	kfree(dc8200_dc);
	return ret;
}

void dc8200_hw_rpm_update(struct dc8200_dc *hw)
{
	dc8200_enable_shadow_register(hw, false);
	dc8200_commit(hw);
	dc8200_enable_shadow_register(hw, true);
}

void check_and_handle_underflow(struct dm_dc *dc)
{
	uint32_t plane_state0, plane_state1;

	plane_state0 = DC_RREG32(DC_PANEL_STATE0);
	plane_state1 = DC_RREG32(DC_PANEL_STATE1);
	if ((plane_state0 | plane_state1) & 0x20)
		/* IO read will clear the HW flag to make DC continue to run */
		dev_warn(dm_to_xdev(dc->dm)->dev, "dc underflow");
	dc8200_interrupt_handle(dc);
}
