/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __DM_CRTC_H__
#define __DM_CRTC_H__

#include <drm/drm_crtc.h>
#include <drm/drm_crtc_helper.h>

#include "dm_plane.h"

struct dm_crtc;
struct dm_dc;

struct dm_crtc_state {
	struct drm_crtc_state base;
	u32 bpc;
	u32 output_fmt;
};

static inline struct drm_crtc_state *
to_drm_crtc_state(struct dm_crtc_state *dm_crtc_state)
{
	return &dm_crtc_state->base;
}

static inline struct dm_crtc_state *
to_dm_crtc_state(struct drm_crtc_state *crtc_state)
{
	return container_of(crtc_state, struct dm_crtc_state, base);
}

struct dm_crtc_funcs {
	void (*destroy)(struct dm_crtc *dm_crtc);
	int (*enable)(struct dm_crtc *dm_crtc);
	int (*disable)(struct dm_crtc *dm_crtc);
	bool (*mode_fixup)(struct dm_crtc *dm_crtc,
			   const struct drm_display_mode *mode,
			   struct drm_display_mode *adjusted_mode);
	int (*set_gamma)(struct dm_crtc *dm_crtc, struct drm_color_lut *lut,
			 unsigned int size);
	int (*enable_gamma)(struct dm_crtc *dm_crtc, bool enable);
	int (*enable_vblank)(struct dm_crtc *dm_crtc, bool enable);
	int (*commit)(struct dm_crtc *dm_crtc);
	int (*cursor_commit)(struct dm_crtc *dm_crtc);
	struct dm_crtc_state *(*crtc_state_create)(struct dm_crtc *dm_crtc);
	void (*crtc_state_destroy)(struct dm_crtc_state *dm_crtc_state);
	struct dm_crtc_state *(*crtc_duplicate_state)(struct dm_crtc_state *ori_state);
	int (*crtc_set_property)(struct dm_crtc *dm_crtc, struct dm_crtc_state *state,
				  struct drm_property *property, uint64_t val);
	int (*crtc_get_property)(struct dm_crtc *dm_crtc,
				       const struct dm_crtc_state *state,
				       struct drm_property *property,
				       uint64_t *val);
};

struct dm_crtc {
	struct drm_crtc base;
	struct xdx_dm *dm;
	struct dm_dc *dc;
	int id;
	struct drm_pending_vblank_event *event;
	/* simulate vblank interrupt */
	struct hrtimer vblank_hrtimer;
	ktime_t period_ns;
	bool vblank_timer_enabled;
	const struct dm_crtc_funcs *funcs;
};

static inline struct drm_crtc *to_drm_crtc(struct dm_crtc *dm_crtc)
{
	return &dm_crtc->base;
}

static inline struct dm_crtc *to_dm_crtc(struct drm_crtc *crtc)
{
	return container_of(crtc, struct dm_crtc, base);
}

int dm_universal_crtc_init(struct xdx_dm *dm, struct dm_crtc *dm_crtc,
			   struct dm_dc *dc, struct dm_crtc_funcs *funcs,
			   struct dm_plane *primary, struct dm_plane *cursor,
			   uint32_t id);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0))
int xdx_drm_drv_enable_vblank(struct drm_device *ddev, unsigned int pipe);
void xdx_drm_drv_disable_vblank(struct drm_device *ddev, unsigned int pipe);
#endif
#endif /* __DM_CRTC_H__ */
