/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
//#include <string.h>
//#include "Debug.h"
//#include "HalReg.h"
//#include "TStamp.h"
//#include "MemIo.h"
#include "B0_HalVdma.h"
#include "utils.h"
#include <linux/delay.h>
#include "xdx_dma_dbglevel.h"
//#define DBG_ASSERT

#define SIZE_1K     1024
#define SIZE_64B    64
#define SIZE_32B    32
#define SIZE_16B    16
#define SIZE_8B     8

#define PRINT_DBG_VAR(var) XDX_DMA_INFO_VAR(var)
#define PRINT_DBG_VAR64(var) XDX_DMA_INFO_VAR64(var)

void VDMA_B0_Init(uint64_t vdmaBaseAddr, uint32_t vdmaChCnt)
{
    uint8_t vdmaCh;
    VDMA_B0_StreamId_T sid;

    VDMA_B0_SoftReset(vdmaBaseAddr);
#if 1
    // Common Register Init
    VDMA_B0_SetAxLen(vdmaBaseAddr, VDMA_B0_COM_AXLEN_1, VDMA_B0_COM_AXLEN_1);

    sid.RdSrcDescSid = VDMA_B0_GPA_SID;
    sid.RdSrcDataSid = VDMA_B0_GPA_SID;
    sid.PollingRdDataSid = VDMA_B0_GPA_SID;
    sid.RdDstDescSid = VDMA_B0_GPA_SID;
    sid.WrDstDataSid = VDMA_B0_GPA_SID;
    sid.PhyAddrSid = VDMA_B0_GPA_SID;

    for (vdmaCh = 0; vdmaCh < vdmaChCnt; ++vdmaCh) {
        VDMA_B0_SetChStreamId(vdmaBaseAddr, vdmaCh, &sid);
        VDMA_B0_EnDisCh(vdmaBaseAddr, vdmaCh, 1);
    }
	//pr_err("B0 VDMA Init done .\n");
#endif
}

void VDMA_B0_SoftReset(uint64_t vdmaBaseAddr)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
	//pr_err("dmaRegs = 0x%llx\n",dmaRegs);
    dmaRegs->comm.SOFT_RST = 1;
    udelay(10);
    dmaRegs->comm.SOFT_RST = 0;
}

void VDMA_B0_SetAxLen(uint64_t vdmaBaseAddr, uint32_t ndfAxlen, uint32_t hostAxlen)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(ndfAxlen <= VDMA_B0_COM_AXLEN_64);
    DBG_ASSERT(hostAxlen <= VDMA_B0_COM_AXLEN_64);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    dmaRegs->comm.HOST_AXI_RD_CTRL = hostAxlen;
    dmaRegs->comm.HOST_AXI_WR_CTRL = hostAxlen;
    dmaRegs->comm.NDF_AXI_RD_CTRL = ndfAxlen;
    dmaRegs->comm.NDF_AXI_WR_CTRL = ndfAxlen;
}

void VDMA_B0_SetBlkTrans(uint64_t vdmaBaseAddr, uint32_t ndfBlkTransEn, uint32_t hostBlkTransEn, uint32_t blkTransLen)
{
    VDMA_B0_Regs_T *dmaRegs;
    VDMA_B0_BlkTransCtrlReg_T blkTransCtrlReg;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(ndfBlkTransEn <= 1);
    DBG_ASSERT(hostBlkTransEn <= 1);
    DBG_ASSERT(blkTransLen <= VDMA_B0_BLKTRANSLEN_8K);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    blkTransCtrlReg.all = 0;
    blkTransCtrlReg.HostBlkTransEn = hostBlkTransEn;
    blkTransCtrlReg.NdfBlkTransEn = hostBlkTransEn;
    dmaRegs->comm.BLOCK_TRANS_CTRL = blkTransCtrlReg.all;
    dmaRegs->comm.BLOCK_TRANS_LEN = blkTransLen;
}

void VDMA_B0_SetChPriority(uint64_t vdmaBaseAddr, uint32_t ch0Prior, uint32_t ch1Prior,
        uint32_t ch2Prior, uint32_t ch3Prior, uint32_t chPriorTimeout)
{
    VDMA_B0_Regs_T *dmaRegs;
    VDMA_B0_ChPriorityReg_T chPriorityReg;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(ch0Prior <= VDMA_CHANNEL_PRIORITY_MAX);
    DBG_ASSERT(ch1Prior <= VDMA_CHANNEL_PRIORITY_MAX);
    DBG_ASSERT(ch2Prior <= VDMA_CHANNEL_PRIORITY_MAX);
    DBG_ASSERT(ch3Prior <= VDMA_CHANNEL_PRIORITY_MAX);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    chPriorityReg.all = 0;
    chPriorityReg.Ch0Priority = ch0Prior;
    chPriorityReg.Ch1Priority = ch1Prior;
    chPriorityReg.Ch2Priority = ch2Prior;
    chPriorityReg.Ch3Priority = ch3Prior;
    dmaRegs->comm.CHANNEL_PRIORITY = chPriorityReg.all;
    dmaRegs->comm.CHANEL_REQ_TIMEOUT_SIZE = chPriorTimeout;
}

uint32_t VDMA_B0_GetVDMcuId(uint64_t vdmaBaseAddr)
{
    DBG_ASSERT(0 != vdmaBaseAddr);

    return ((VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr)->comm.VD_MCU_ID;
}

void VDMA_B0_WrScrReg(uint64_t vdmaBaseAddr, uint32_t wrVal)
{
    DBG_ASSERT(0 != vdmaBaseAddr);

    ((VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr)->comm.SCRATCH = wrVal;
}

void VDMA_B0_SetChAxiCtrl(uint64_t vdmaBaseAddr, uint32_t channel,
        uint32_t ndfAxQos, uint32_t ndfAxCache, uint32_t hostAxQos, uint32_t hostAxCache)
{
    VDMA_B0_Regs_T *dmaRegs;
    VDMA_B0_ChAxCtrlReg_t chAxCtrlReg;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    chAxCtrlReg.all = 0;
    chAxCtrlReg.ArCache = ndfAxCache;
    chAxCtrlReg.AwCache = ndfAxCache;
    chAxCtrlReg.ArQos = ndfAxQos;
    chAxCtrlReg.AwQos = ndfAxQos;
    dmaRegs->ch[channel].NDF_AXI_CTRL = chAxCtrlReg.all;

    chAxCtrlReg.all = 0;
    chAxCtrlReg.ArCache = hostAxCache;
    chAxCtrlReg.AwCache = hostAxCache;
    chAxCtrlReg.ArQos = hostAxQos;
    chAxCtrlReg.AwQos = hostAxQos;
    dmaRegs->ch[channel].HOST_AXI_CTRL = chAxCtrlReg.all;
}

void VDMA_B0_SetChStreamId(uint64_t vdmaBaseAddr, uint32_t channel, VDMA_B0_StreamId_T* psSid)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(0 != psSid);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    dmaRegs->ch[channel].RD_SRC_DESC_STREAM_ID = psSid->RdSrcDescSid;
    dmaRegs->ch[channel].RD_SRC_DATA_STREAM_ID = psSid->RdSrcDataSid;
    dmaRegs->ch[channel].POLLING_RD_DATA_STREAM_ID = psSid->PollingRdDataSid;
    dmaRegs->ch[channel].RD_DST_DESC_STREAM_ID = psSid->RdDstDescSid;
    dmaRegs->ch[channel].WR_DST_DATA_STREAM_ID = psSid->WrDstDataSid;
    dmaRegs->ch[channel].PHYSICAL_ADDR_STREAM_ID = psSid->PhyAddrSid;
}

void VDMA_B0_EnDisCh(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t enDis)
{
    VDMA_B0_ChCtrlReg_T chCtrlReg;
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    chCtrlReg.all = dmaRegs->ch[channel].CTRL;
    chCtrlReg.DmaEnable = enDis;
    dmaRegs->ch[channel].CTRL = chCtrlReg.all;
}

void VDMA_B0_EnDisChIntr(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t enDis)
{
    VDMA_B0_ChCtrlReg_T chCtrlReg;
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    chCtrlReg.all = dmaRegs->ch[channel].CTRL;
    chCtrlReg.IntrEn = enDis;
    dmaRegs->ch[channel].CTRL = chCtrlReg.all;
}

void VDMA_B0_EnDisChSeqExec(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t enDis)
{
    VDMA_B0_ChCtrlReg_T chCtrlReg;
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    chCtrlReg.all = dmaRegs->ch[channel].CTRL;
    chCtrlReg.SeqExec = enDis;
    dmaRegs->ch[channel].CTRL = chCtrlReg.all;
}

uint32_t VDMA_B0_GetChCmdRemainSize(uint64_t ChannelBaseAddr)
{
    DBG_ASSERT(0 != ChannelBaseAddr);

    return ((VDMA_B0_ChanRegs_T*)ChannelBaseAddr)->REMAIN_CMD_BUF_SIZE;
}

void VDMA_B0_WrChRefBuf(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t startRefBufIdx, uint32_t *pu32WrBuf, uint32_t WrBytelen)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint32_t* pu32RefBuf;
    uint32_t i;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);
    DBG_ASSERT(WrBytelen != 0);
    DBG_ASSERT(pu32WrBuf != NULL);
    DBG_ASSERT((startRefBufIdx + WrBytelen)/sizeof(uint32_t) <= VDMA_B0_REFBUF_IDX_MAX + 1);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    pu32RefBuf = (uint32_t*)&(dmaRegs->ch[channel].REF_BUF[startRefBufIdx>>2]);
    for (i = 0; i < WrBytelen/sizeof(uint32_t); i++) {
        pu32RefBuf[i] = pu32WrBuf[i];
    }
}

// Random values need to be cleared
void VDMA_B0_ClrAndWrRefBufSingle(uint64_t vdmaCHBaseAddr, uint32_t RefBufIdx, uint32_t WrData)
{
	DBG_ASSERT(0 != vdmaCHBaseAddr);
	DBG_ASSERT(RefBufIdx <= VDMA_B0_REFBUF_IDX_MAX);
	((VDMA_B0_ChanRegs_T*)vdmaCHBaseAddr)->REF_BUF[RefBufIdx] = 0;
	((VDMA_B0_ChanRegs_T*)vdmaCHBaseAddr)->REF_BUF[RefBufIdx] = WrData;
}


void VDMA_B0_WrChCmdBuf(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t *pu64WrDataBuf, uint32_t WrDwlen)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t* CmdBuf;
    uint32_t i;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);
    DBG_ASSERT(WrDwlen != 0);
    DBG_ASSERT(pu64WrDataBuf != NULL);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    CmdBuf = (uint64_t *)&(dmaRegs->ch[channel].CMD[0]);
    for (i = 0; i < WrDwlen; i++) {
        CmdBuf[i] = pu64WrDataBuf[i];
    }
}

void VDMA_B0_WrCmdBuf(uint64_t vdmaCHBaseAddr, uint64_t *pu64WrDataBuf, uint32_t WrDwlen)
{
    uint64_t* CmdBuf;
	uint32_t i;

    DBG_ASSERT(0 != vdmaCHBaseAddr);
    DBG_ASSERT(WrDwlen != 0);
    DBG_ASSERT(pu64WrDataBuf != NULL);

	CmdBuf = (uint64_t*)&(((VDMA_B0_ChanRegs_T*)vdmaCHBaseAddr)->CMD[0]);

	for (i = 0; i < WrDwlen; i++)
	{
		CmdBuf[i] = pu64WrDataBuf[i];
	}
}

uint64_t VDMA_B0_RdCmdCmplIntSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t IntSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    IntSts = dmaRegs->ch[channel].CMD_CMPL_INTR_STS;

    return IntSts;
}

uint64_t VDMA_B0_CH_RdAndClrCmdCmplRawSts(uint64_t ChannelBaseAddr)
{
	uint64_t RawSts;

	//DBG_ASSERT(0 != VdmaBaseAddr);
	//DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	RawSts = ((VDMA_B0_ChanRegs_T*)ChannelBaseAddr)->CMD_CMPL_RAW_STS;
	((VDMA_B0_ChanRegs_T*)ChannelBaseAddr)->CMD_CMPL_CLR = RawSts;
	return RawSts;
}

uint64_t VDMA_B0_RdPollingTimeoutIntSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t IntSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    IntSts = dmaRegs->ch[channel].POLLING_TIMEOUT_INTR_STS;

    return IntSts;
}

uint64_t VDMA_B0_RdBusErrIntSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t IntSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    IntSts = dmaRegs->ch[channel].BUS_ERR_INTR_STS;

    return IntSts;
}

uint64_t VDMA_B0_RdCmdCmplRawSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].CMD_CMPL_RAW_STS;

    return rawSts;
}

uint64_t VDMA_B0_RdPollingTimeoutRawSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].POLLING_TIMEOUT_RAW_STS;

    return rawSts;
}

uint64_t VDMA_B0_RdAndClrPollingTimeoutRawSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T *)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].POLLING_TIMEOUT_RAW_STS;

	dmaRegs->ch[channel].POLLING_TIMEOUT_CLR = rawSts;

    return rawSts;
}


uint64_t VDMA_B0_RdBusErrRawSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].BUS_ERR_RAW_STS;

    return rawSts;
}

uint64_t VDMA_B0_RdAndClrBusErrRawSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;
    uint64_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T *)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].BUS_ERR_RAW_STS;

	dmaRegs->ch[channel].BUS_ERR_CLR = rawSts;

    return rawSts;
}


void VDMA_B0_WrCmdCmplClrReg(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t clrRegVal)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    dmaRegs->ch[channel].CMD_CMPL_CLR = clrRegVal;
}

void VDMA_B0_WrPollingTimeoutClrReg(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t clrRegVal)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    dmaRegs->ch[channel].POLLING_TIMEOUT_CLR = clrRegVal;
}

void VDMA_B0_WrBusErrClrReg(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t clrRegVal)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    dmaRegs->ch[channel].BUS_ERR_CLR = clrRegVal;
}

int32_t VDMA_B0_WaitXferCompleted(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t tag)
{
    uint64_t CmdCmplRawSts, PollingTimeoutRawSts, BusErrRawSts;
    uint64_t ExpCmdCmplRawSts;
    int32_t Ret = 0;

    ExpCmdCmplRawSts = (0x1ll<<tag);

    //pr_err("Wait for DMA completed\n");
    while (1) {
        CmdCmplRawSts = VDMA_B0_RdCmdCmplRawSts(vdmaBaseAddr, channel);
        if (CmdCmplRawSts) {
            //pr_err("CmdCmplRawSts=0x%llx\n", CmdCmplRawSts);
            if (CmdCmplRawSts != ExpCmdCmplRawSts) {
                //pr_err("Mismatch! ExpCmdCmplRawSts=0x%l\n", ExpCmdCmplRawSts);
                Ret = 1;
            }

            VDMA_B0_WrCmdCmplClrReg(vdmaBaseAddr, channel, CmdCmplRawSts);
            break;
        }

        PollingTimeoutRawSts = VDMA_B0_RdPollingTimeoutRawSts(vdmaBaseAddr, channel);
        if (PollingTimeoutRawSts) {
            //pr_err("PollingTimeoutRawSts=0x%l\n", PollingTimeoutRawSts);
            VDMA_B0_WrPollingTimeoutClrReg(vdmaBaseAddr, channel, PollingTimeoutRawSts);
            break;
        }

        BusErrRawSts = VDMA_B0_RdBusErrRawSts(vdmaBaseAddr, channel);
        if (BusErrRawSts) {
            //pr_err("BusErrRawSts=0x%l\n", BusErrRawSts);
            VDMA_B0_WrBusErrClrReg(vdmaBaseAddr, channel, BusErrRawSts);
            break;
        }

//        TStamp_DelayUS(1000);
    }

    return Ret;
}

void VDMA_B0_GenVdmaCmd(VDMA_B0_CmdCfg_T *vdmaCfg, VDMA_B0_CmdData_T *vdmaCmd)
{
    DBG_ASSERT(vdmaCfg != NULL);
    DBG_ASSERT(vdmaCmd != NULL);

    // clear VdmaCmd
    //memset(&vdmaCmd->cmd32[0], 0, sizeof(VDMA_B0_CmdData_T));

    // check param
//#ifdef DEBUG
    switch (vdmaCfg->OpCode) {
        case VDMA_B0_OPCODE_COPY_DSRC_DDST:
        case VDMA_B0_OPCODE_COPY_DSRC_LDST:
        case VDMA_B0_OPCODE_COPY_LSRC_DDST:
        case VDMA_B0_OPCODE_COPY_LSRC_LDST:
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->copy.DstAddr, VDMA_B0_COPY_ADDR_ALIGN));
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->copy.SrcAddr, VDMA_B0_COPY_ADDR_ALIGN));
            DBG_ASSERT(vdmaCfg->copy.LenUnit <= VDMA_B0_LENUNIT_1K);
            DBG_ASSERT((vdmaCfg->copy.SrcAddrMode & (~0xf)) == 0);
            DBG_ASSERT((vdmaCfg->copy.DstAddrMode & (~0xf)) == 0);
            DBG_ASSERT(vdmaCfg->copy.LogicOp <= VDMA_B0_LOGICOP_AND);

            if (VDMA_B0_OPCODE_COPY_LSRC_LDST != vdmaCfg->OpCode) {
                DBG_ASSERT((vdmaCfg->copy.TransferLen > 0) && (vdmaCfg->copy.TransferLen <= VDMA_B0_XFERLEN_LIMIT));
            }

            // use DstDesc
            if ((VDMA_B0_OPCODE_COPY_DSRC_LDST == vdmaCfg->OpCode) || (VDMA_B0_OPCODE_COPY_LSRC_LDST == vdmaCfg->OpCode)) {
                DBG_ASSERT(vdmaCfg->copy.DstDescNum >= 2);
            }

            // use SrcDesc
            if ((VDMA_B0_OPCODE_COPY_LSRC_DDST == vdmaCfg->OpCode) || (VDMA_B0_OPCODE_COPY_LSRC_LDST == vdmaCfg->OpCode)) {
                DBG_ASSERT(vdmaCfg->copy.SrcDescNum >= 2);
            }
            break;

        case VDMA_B0_OPCODE_2DCOPY:
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->copy2d.DstAddr, VDMA_B0_COPY_ADDR_ALIGN));
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->copy2d.SrcAddr, VDMA_B0_COPY_ADDR_ALIGN));
            DBG_ASSERT(vdmaCfg->copy2d.LenUnit <= VDMA_B0_LENUNIT_1K);
            DBG_ASSERT((vdmaCfg->copy2d.SrcAddrMode & (~0xf)) == 0);
            DBG_ASSERT((vdmaCfg->copy2d.DstAddrMode & (~0xf)) == 0);
            DBG_ASSERT((vdmaCfg->copy2d.Xstep > 0) && (vdmaCfg->copy2d.Xstep <= VDMA_B0_XFERLEN_LIMIT));
            DBG_ASSERT((vdmaCfg->copy2d.Ystep > 0) && (vdmaCfg->copy2d.Ystep <= VDMA_B0_XFERLEN_LIMIT));
            DBG_ASSERT((vdmaCfg->copy2d.Sstride >= vdmaCfg->copy2d.Xstep) && (vdmaCfg->copy2d.Sstride <= VDMA_B0_XFERLEN_LIMIT));
            DBG_ASSERT((vdmaCfg->copy2d.Dstride >= vdmaCfg->copy2d.Xstep) && (vdmaCfg->copy2d.Dstride <= VDMA_B0_XFERLEN_LIMIT));
            break;

        case VDMA_B0_OPCODE_WRITE_DIRECT:
            DBG_ASSERT((vdmaCfg->dwrite.TransferLen > 0) && (vdmaCfg->dwrite.TransferLen <= VDMA_B0_XFERLEN_LIMIT));
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->dwrite.DstAddr, VDMA_B0_DWRITE_ADDR_ALIGN));
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->dwrite.PatternLoc, VDMA_B0_DWRITE_PATTERN_LOC_ALIGN));
            DBG_ASSERT((vdmaCfg->dwrite.DstAddrMode & (~0xf)) == 0);
            DBG_ASSERT((vdmaCfg->dwrite.PatternLen <= VDMA_B0_DWRITE_PATTERN_LEN_LIMIT)
                    && (vdmaCfg->dwrite.PatternLen > 0));
            break;

        case VDMA_B0_OPCODE_POLLING_LESS:
        case VDMA_B0_OPCODE_POLLING_EQUAL:
        case VDMA_B0_OPCODE_POLLING_GREATER:
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->polling.SrcAddr, VDMA_B0_POLLING_ADDR_ALIGN));
            DBG_ASSERT(TEST_ALIGNED(vdmaCfg->polling.PatternLoc, VDMA_B0_POLLING_PATTERN_LOC_ALIGN));
            DBG_ASSERT((vdmaCfg->polling.SrcAddrMode & (~0xf)) == 0);
            break;

        default:
            XDX_DMA_ERR(NULL, "Unknown Opcode: 0x%x\n", vdmaCfg->OpCode);
            DBG_ASSERT(0);
            break;
    }
//#endif

    // generate Cmd
    switch (vdmaCfg->OpCode) {
        case VDMA_B0_OPCODE_COPY_DSRC_DDST:
        case VDMA_B0_OPCODE_COPY_DSRC_LDST:
        case VDMA_B0_OPCODE_COPY_LSRC_DDST:
        case VDMA_B0_OPCODE_COPY_LSRC_LDST:
            vdmaCmd->dw0.SrcAddr_47_3 = GET_BITS_U64(vdmaCfg->copy.SrcAddr, 47, 3);
            vdmaCmd->dw0.LogicOp = vdmaCfg->copy.LogicOp;
            vdmaCmd->dw1.LenUnit = vdmaCfg->copy.LenUnit;
            vdmaCmd->dw1.DstAddr_47_3 = GET_BITS_U64(vdmaCfg->copy.DstAddr, 47, 3);
            vdmaCmd->dw2.SrcAddrMode = vdmaCfg->copy.SrcAddrMode;
            vdmaCmd->dw2.DstAddrMode = vdmaCfg->copy.DstAddrMode;

            if (VDMA_B0_OPCODE_COPY_DSRC_DDST == vdmaCfg->OpCode) {
                // Use Xstep for transferLen
                vdmaCmd->dw1.Xstep_SrcDescNum = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.TransferLen);
            } else if (VDMA_B0_OPCODE_COPY_DSRC_LDST == vdmaCfg->OpCode) {
                // Use Xstep for transferLen, Ystep for dstDescNum
                vdmaCmd->dw1.Xstep_SrcDescNum = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.TransferLen);
                vdmaCmd->dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.DstDescNum);
            } else if (VDMA_B0_OPCODE_COPY_LSRC_DDST == vdmaCfg->OpCode) {
                // Use Xstep for srcDescNum, Ystep for trasnferLen
                vdmaCmd->dw1.Xstep_SrcDescNum = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.SrcDescNum);
                vdmaCmd->dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.TransferLen);
            } else if (VDMA_B0_OPCODE_COPY_LSRC_LDST == vdmaCfg->OpCode) {
                // Use Xstep for srcDescNum, Ystep for dstDescNum
                vdmaCmd->dw1.Xstep_SrcDescNum = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.SrcDescNum);
                vdmaCmd->dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh = VDMA_B0_LEN_ADJUST(vdmaCfg->copy.DstDescNum);
            }
            break;

        case VDMA_B0_OPCODE_2DCOPY:
            vdmaCmd->dw0.SrcAddr_47_3 = GET_BITS_U64(vdmaCfg->copy2d.SrcAddr, 47, 3);
            vdmaCmd->dw0.LogicOp = vdmaCfg->copy2d.LogicOp;
            vdmaCmd->dw1.LenUnit = vdmaCfg->copy2d.LenUnit;
            vdmaCmd->dw1.DstAddr_47_3 = GET_BITS_U64(vdmaCfg->copy2d.DstAddr, 47, 3);
            vdmaCmd->dw1.Xstep_SrcDescNum = VDMA_B0_LEN_ADJUST(vdmaCfg->copy2d.Xstep);
            vdmaCmd->dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh = VDMA_B0_LEN_ADJUST(vdmaCfg->copy2d.Ystep);
            vdmaCmd->dw2.SrcAddrMode = vdmaCfg->copy2d.SrcAddrMode;
            vdmaCmd->dw2.DstAddrMode = vdmaCfg->copy2d.DstAddrMode;
            vdmaCmd->dw2.Sstride_DWriteLenLow = VDMA_B0_LEN_ADJUST(vdmaCfg->copy2d.Sstride);
            vdmaCmd->dw2.DstrideHigh_PatternLoc = GET_BITS_U64(VDMA_B0_LEN_ADJUST(vdmaCfg->copy2d.Dstride), 15, 8);
            vdmaCmd->dw2.DstrideLow_PatternLen = GET_BITS_U64(VDMA_B0_LEN_ADJUST(vdmaCfg->copy2d.Dstride), 7, 0);
            break;

        case VDMA_B0_OPCODE_WRITE_DIRECT:
            vdmaCmd->dw1.DstAddr_47_3= GET_BITS_U64(vdmaCfg->dwrite.DstAddr, 47, 3);
            vdmaCmd->dw2.DstAddrMode = vdmaCfg->dwrite.DstAddrMode;
            vdmaCmd->dw2.DstrideLow_PatternLen = VDMA_B0_LEN_ADJUST(vdmaCfg->dwrite.PatternLen);
            vdmaCmd->dw2.DstrideHigh_PatternLoc = GET_BITS(vdmaCfg->dwrite.PatternLoc, 9, 2);
            vdmaCmd->dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh = GET_BITS(vdmaCfg->dwrite.TransferLen, 24, 16);
            vdmaCmd->dw2.Sstride_DWriteLenLow = VDMA_B0_LEN_ADJUST(GET_BITS(vdmaCfg->dwrite.TransferLen, 15, 0));
            break;

        case VDMA_B0_OPCODE_FENCE:
            break;

        case VDMA_B0_OPCODE_POLLING_LESS:
        case VDMA_B0_OPCODE_POLLING_EQUAL:
        case VDMA_B0_OPCODE_POLLING_GREATER:
            vdmaCmd->dw0.SrcAddr_47_3 = GET_BITS_U64(vdmaCfg->polling.SrcAddr, 47, 3);
            vdmaCmd->dw2.SrcAddrMode = vdmaCfg->polling.SrcAddrMode;
            vdmaCmd->dw2.DstrideHigh_PatternLoc = GET_BITS(vdmaCfg->polling.PatternLoc, 9, 2);
            break;
    }

    DBG_ASSERT(vdmaCfg->CommandTag <= VDMA_B0_COMMAND_TAG_LIMIT);
    DBG_ASSERT(vdmaCfg->IntrEn <= 1);
    DBG_ASSERT(vdmaCfg->PcieFuncNum <= 1);
    DBG_ASSERT(vdmaCfg->PcieDontSnoop <= 1);

    vdmaCmd->dw0.Opcode = vdmaCfg->OpCode;
    vdmaCmd->dw0.Tag = vdmaCfg->CommandTag;
    vdmaCmd->dw2.IntrEn = vdmaCfg->IntrEn;
    vdmaCmd->dw2.PcieFuncNum = vdmaCfg->PcieFuncNum;
    vdmaCmd->dw2.PcieDontSnoop = vdmaCfg->PcieDontSnoop;
}

void VDMA_B0_GenNonEndVdmaDesc(uint64_t dataBlkPtr, uint32_t transferLen, uint8_t lenUnit,  VDMA_B0_DescData_T *vdmaDesc)
{
    DBG_ASSERT(vdmaDesc != NULL);
    DBG_ASSERT(lenUnit <= VDMA_B0_LENUNIT_1K);
    DBG_ASSERT(transferLen <= VDMA_B0_XFERLEN_LIMIT);

    //vdmaDesc->all = 0;
    vdmaDesc->nonend.DataBlkPtr_47_3 = GET_BITS(dataBlkPtr, 47, 3);
    vdmaDesc->nonend.LenUnit = lenUnit;
    vdmaDesc->nonend.TransferLen = VDMA_B0_LEN_ADJUST(transferLen);
}

void VDMA_B0_GenEndVdmaDesc(uint64_t nextDescPtr, uint32_t nextDescNum, uint8_t linkEnd,  VDMA_B0_DescData_T *vdmaDesc)
{
    DBG_ASSERT(vdmaDesc != NULL);
    DBG_ASSERT(linkEnd <= 1);
    DBG_ASSERT(linkEnd == 1 || nextDescNum >= 2);

    // if linkEnd = 1, means scattergather list end here, and won't use nextDescPtr and nextDescNum
    //vdmaDesc->all = 0;
    vdmaDesc->end.LinkEnd = linkEnd;
    vdmaDesc->end.NextDescNum = VDMA_B0_LEN_ADJUST(nextDescNum);
    vdmaDesc->end.NextDescPtr_47_3 = GET_BITS(nextDescPtr, 47, 3);
}

void VDMA_B0_DumpComRegs(uint64_t vdmaBaseAddr)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    XDX_DMA_DBG(NULL, "== Dump VDMA ComReg Begin ==\n");

    PRINT_DBG_VAR(dmaRegs->comm.SOFT_RST                      );
    PRINT_DBG_VAR(dmaRegs->comm.DMA_IDLE                      );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_RD_CTRL              );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_WR_CTRL              );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_RD_CTRL               );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_WR_CTRL               );
    PRINT_DBG_VAR(dmaRegs->comm.CHANNEL_PRIORITY              );
    PRINT_DBG_VAR(dmaRegs->comm.CHANEL_REQ_TIMEOUT_SIZE       );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_PORT_OUTSTANDING_SIZE    );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_PORT_OUTSTANDING_SIZE     );
    PRINT_DBG_VAR(dmaRegs->comm.VD_MCU_VERSION                );
    PRINT_DBG_VAR(dmaRegs->comm.VD_MCU_ID                     );
    PRINT_DBG_VAR(dmaRegs->comm.SCRATCH                       );
    PRINT_DBG_VAR(dmaRegs->comm.PAD_MUX_CFG                   );
    PRINT_DBG_VAR(dmaRegs->comm.BLOCK_TRANS_CTRL              );
    PRINT_DBG_VAR(dmaRegs->comm.BLOCK_TRANS_LEN               );
    PRINT_DBG_VAR(dmaRegs->comm.DEBUG_CH_SEL                  );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_BEAT_CH_SDAT_CNT      );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_BEAT_CH_SDES_CNT      );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_BEAT_CH_DDES_CNT      );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_BEAT_CH_POLLING_CNT   );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_REQ_BEAT_CH_SDAT_CNT  );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_REQ_BEAT_CH_SDES_CNT  );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_REQ_BEAT_CH_DDES_CNT  );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_REQ_BEAT_CH_POLLING_CNT);
    PRINT_DBG_VAR(dmaRegs->comm.HOST_WR_BEAT_CH_CNT           );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_WR_STATE0            );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_WR_STATE1            );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_RD_STATE0            );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_RD_STATE1            );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_AXI_OUTSTANDING_STATE    );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_BEAT_CH_SDAT_CNT       );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_BEAT_CH_SDES_CNT       );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_BEAT_CH_DDES_CNT       );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_BEAT_CH_POLLING_CNT    );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_REQ_BEAT_CH_SDAT_CNT   );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_REQ_BEAT_CH_SDES_CNT   );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_REQ_BEAT_CH_DDES_CNT   );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_REQ_BEAT_CH_POLLING_CNT);
    PRINT_DBG_VAR(dmaRegs->comm.NDF_WR_BEAT_CH_CNT            );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_WR_STATE0             );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_WR_STATE1             );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_RD_STATE0             );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_RD_STATE1             );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_AXI_OUTSTANDING_STATE     );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_BEAT_CH_DDAT_H_CNT    );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_BEAT_CH_DDAT_N_CNT    );
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_REQ_BEAT_CH_DDAT_H_CNT);
    PRINT_DBG_VAR(dmaRegs->comm.HOST_RD_REQ_BEAT_CH_DDAT_N_CNT);
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_BEAT_CH_DDAT_H_CNT     );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_BEAT_CH_DDAT_N_CNT     );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_REQ_BEAT_CH_DDAT_H_CNT );
    PRINT_DBG_VAR(dmaRegs->comm.NDF_RD_REQ_BEAT_CH_DDAT_N_CNT );
}

void VDMA_B0_DumpChRegs(uint64_t vdmaBaseAddr, uint8_t channel)
{
    VDMA_B0_Regs_T *dmaRegs;

    DBG_ASSERT(0 != vdmaBaseAddr);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;

    XDX_DMA_DBG(NULL, "== Dump VDMA Ch[%d]Reg Begin ==\n", channel);

    PRINT_DBG_VAR(dmaRegs->ch[channel].CTRL);
    PRINT_DBG_VAR(dmaRegs->ch[channel].CHANNEL_IDLE);
    PRINT_DBG_VAR(dmaRegs->ch[channel].HOST_AXI_CTRL);
    PRINT_DBG_VAR(dmaRegs->ch[channel].NDF_AXI_CTRL);
    PRINT_DBG_VAR64(dmaRegs->ch[channel].CMD_CMPL_INTR_STS);
    PRINT_DBG_VAR64(dmaRegs->ch[channel].POLLING_TIMEOUT_INTR_STS);
    PRINT_DBG_VAR64(dmaRegs->ch[channel].BUS_ERR_INTR_STS);
    PRINT_DBG_VAR64(dmaRegs->ch[channel].CMD_CMPL_RAW_STS);
    PRINT_DBG_VAR64(dmaRegs->ch[channel].POLLING_TIMEOUT_RAW_STS);
    PRINT_DBG_VAR64(dmaRegs->ch[channel].BUS_ERR_RAW_STS);
    PRINT_DBG_VAR(dmaRegs->ch[channel].POLLING_INTERVAL);
    PRINT_DBG_VAR(dmaRegs->ch[channel].POLLING_MAX_SIZE);
    PRINT_DBG_VAR(dmaRegs->ch[channel].RD_SRC_DESC_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->ch[channel].RD_DST_DESC_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->ch[channel].RD_SRC_DATA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->ch[channel].POLLING_RD_DATA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->ch[channel].WR_DST_DATA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->ch[channel].PHYSICAL_ADDR_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->ch[channel].REMAIN_CMD_BUF_SIZE);
    PRINT_DBG_VAR(dmaRegs->ch[channel].CMD_FIFO_STS);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_FETCH_SRC_DESC_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_FETCH_DST_DESC_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_FETCH_SRC_DATA_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_POLLING_CMD_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_FETCH_SRC_DESC_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_FETCH_DST_DESC_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_FETCH_SRC_DATA_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_POLLING_CMD_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_PARSE0);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_PARSE1);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_PARSE2);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_RD_RESP_DEMUX);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_RD_RESP_DEMUX);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_WR_DST_DATA_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_WR_DST_DATA_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_WR_DIRECT_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].N_H_COPY_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].H_H_COPY_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].H_WR_DIRECT_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].H_POLLING_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].N_N_COPY_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].H_N_COPY_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].N_WR_DIRECT_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].N_POLLING_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].FENCE_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].RECEIVE_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].COMPLE_CMD_CNT);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DMA_ERROR_DETAIL);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_HSRC_FETCH_DST_DATA_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_NSRC_FETCH_DST_DATA_REQ);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_CMPL_RAW_STS[0]);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_CMPL_RAW_STS[1]);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_CMPL_CLR[0]);
    PRINT_DBG_VAR(dmaRegs->ch[channel].DBG_CMD_CMPL_CLR[1]);
}

void VDMA_B0_DumpVdmaCmd(VDMA_B0_CmdData_T *vdmaCmd)
{
    uint32_t i;

    DBG_ASSERT(vdmaCmd != NULL);

    XDX_DMA_DBG(NULL, "== Dump VDMA CMD Begin ==\n");

    PRINT_DBG_VAR(vdmaCmd->dw0.Opcode);
    PRINT_DBG_VAR(vdmaCmd->dw0.Tag);
    PRINT_DBG_VAR(vdmaCmd->dw0.LogicOp);
    PRINT_DBG_VAR(vdmaCmd->dw1.LenUnit);
    PRINT_DBG_VAR(vdmaCmd->dw2.PcieFuncNum);
    PRINT_DBG_VAR(vdmaCmd->dw2.PcieDontSnoop);
    PRINT_DBG_VAR(vdmaCmd->dw2.IntrEn);

    PRINT_DBG_VAR64((uint64_t)vdmaCmd->dw0.SrcAddr_47_3 << 3);
    PRINT_DBG_VAR64((uint64_t)vdmaCmd->dw1.DstAddr_47_3 << 3);
    PRINT_DBG_VAR(vdmaCmd->dw2.DstAddrMode);
    PRINT_DBG_VAR(vdmaCmd->dw2.SrcAddrMode);

    PRINT_DBG_VAR(vdmaCmd->dw1.Xstep_SrcDescNum);
    PRINT_DBG_VAR(vdmaCmd->dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh);
    PRINT_DBG_VAR(vdmaCmd->dw2.DstrideHigh_PatternLoc);
    PRINT_DBG_VAR(vdmaCmd->dw2.DstrideLow_PatternLen);
    PRINT_DBG_VAR(vdmaCmd->dw2.Sstride_DWriteLenLow);

    for (i = 0; i < 6; ++i) {
        XDX_DMA_DBG(NULL, "CmdData[%d] = 0x%x\n", i, vdmaCmd->cmd32[i]);
    }
}

void VMDA_B0_DumpNonEndVdmaDesc(VDMA_B0_DescData_T *vdmaDesc)
{
    DBG_ASSERT(vdmaDesc != NULL);

    XDX_DMA_DBG(NULL, "== Dump VDMA NON-END DESC Begin ==\n");

    PRINT_DBG_VAR64(vdmaDesc->all);
    PRINT_DBG_VAR64((uint64_t)vdmaDesc->nonend.DataBlkPtr_47_3);
    PRINT_DBG_VAR(vdmaDesc->nonend.LenUnit);
    PRINT_DBG_VAR(vdmaDesc->nonend.TransferLen);
}

void VMDA_B0_DumpEndVdmaDesc(VDMA_B0_DescData_T *vdmaDesc)
{
    DBG_ASSERT(vdmaDesc != NULL);

    XDX_DMA_DBG(NULL, "== Dump VDMA END DESC Begin ==\n");

    PRINT_DBG_VAR64((uint64_t)vdmaDesc->all);
    PRINT_DBG_VAR64((uint64_t)vdmaDesc->end.NextDescPtr_47_3);
    PRINT_DBG_VAR(vdmaDesc->end.LinkEnd);
    PRINT_DBG_VAR(vdmaDesc->end.NextDescNum);
}

static int32_t VDMA_B0_MemSetBase(uint64_t vdmaBaseAddr, uint32_t Channel, uint32_t MemSetPattern,
        uint8_t DstAddrMode, uint64_t DstAddr, uint32_t XferByteLen, uint8_t UseIntr)
{
    VDMA_B0_CmdCfg_T VdmaCfg;
    VDMA_B0_CmdData_T VdmaCmd;
    const uint32_t RefBufIdx = 0;
    const uint32_t PatternByteLen = 4;
    int32_t Ret = 0;

    DBG_ASSERT(0 == (XferByteLen % 4));

    // 2.1 Fill Reference Buffer
    VDMA_B0_WrChRefBuf(vdmaBaseAddr, Channel, RefBufIdx, (uint32_t*)&MemSetPattern, PatternByteLen);

    // 2.2 Generate Cmd
    memset(&VdmaCfg, 0, sizeof(VDMA_B0_CmdCfg_T));
    VdmaCfg.CommandTag = 0;
    VdmaCfg.IntrEn = 1;
    VdmaCfg.OpCode = VDMA_B0_OPCODE_WRITE_DIRECT;

    VdmaCfg.dwrite.DstAddr = DstAddr;
    VdmaCfg.dwrite.DstAddrMode = DstAddrMode;
    VdmaCfg.dwrite.PatternLen = PatternByteLen;
    VdmaCfg.dwrite.PatternLoc = RefBufIdx;
    VdmaCfg.dwrite.TransferLen = XferByteLen>>2;
    VDMA_B0_GenVdmaCmd(&VdmaCfg, &VdmaCmd);

    VDMA_B0_WrChCmdBuf(vdmaBaseAddr, Channel, VdmaCmd.cmd64, 3);

    // 4. Polling Status (DEBUG NOW)
    if (!UseIntr) {
        Ret = VDMA_B0_WaitXferCompleted(vdmaBaseAddr, Channel, VdmaCfg.CommandTag);
    }

    return Ret;
}

void calcLenUnitAndTransferLen(uint32_t XferByteLen, uint8_t* pu8LenUnit, uint32_t* pu32TransferLen)
{
    uint32_t transferLen;
    uint8_t lengthUnit;
    const uint32_t transferLenLimit = (0x1<<16);

    if (XferByteLen > SIZE_64B*transferLenLimit) {
        DBG_ASSERT(0 == (XferByteLen % SIZE_1K));
        lengthUnit = VDMA_B0_LENUNIT_1K;
        transferLen = XferByteLen / SIZE_1K;

    } else if (XferByteLen > SIZE_32B*transferLenLimit) {
        DBG_ASSERT(0 == (XferByteLen % SIZE_64B));
        lengthUnit = VDMA_B0_LENUNIT_64B;
        transferLen = XferByteLen / SIZE_64B;

    } else if (XferByteLen > SIZE_16B*transferLenLimit) {
        DBG_ASSERT(0 == (XferByteLen % SIZE_32B));
        lengthUnit = VDMA_B0_LENUNIT_32B;
        transferLen = XferByteLen / SIZE_32B;

    } else if (XferByteLen > SIZE_8B*transferLenLimit) {
        DBG_ASSERT(0 == (XferByteLen % SIZE_16B));
        lengthUnit = VDMA_B0_LENUNIT_16B;
        transferLen = XferByteLen / SIZE_16B;

    } else {
        DBG_ASSERT(0 == (XferByteLen % SIZE_8B));
        lengthUnit = VDMA_B0_LENUNIT_8B;
        transferLen = XferByteLen / SIZE_8B;
    }

    *pu8LenUnit = lengthUnit;
    *pu32TransferLen = transferLen;
}

static int32_t VDMA_B0_CopyBase(uint64_t vdmaBaseAddr, uint32_t Channel, VDMA_B0_CopyInfo_T *psCopyInfo, uint8_t UseIntr)
{
    VDMA_B0_CmdData_T VdmaCmd;
    VDMA_B0_CmdCfg_T VdmaCfg;
    uint32_t TransferLen;
    uint8_t LengthUnit;
    int32_t Ret = 0;

    calcLenUnitAndTransferLen(psCopyInfo->XferByteLen, &LengthUnit, &TransferLen);
    //pr_err("XferByteLen=0x%x, LengthUnit=0x%x, TransferLen=0x%x\n", psCopyInfo->XferByteLen, LengthUnit, TransferLen);

    memset(&VdmaCfg, 0, sizeof(VDMA_B0_CmdCfg_T));
    VdmaCfg.CommandTag = 0;
    VdmaCfg.IntrEn = 1;
    VdmaCfg.OpCode = psCopyInfo->OpCode;                        // VDMA_CMD_OPCODE_COPY_DSRC_DDST
	//pr_err("SrcAddr = 0x%llx DstAddr = 0x%llx \n",psCopyInfo->SrcAddr,psCopyInfo->DstAddr);
    VdmaCfg.copy.LogicOp = VDMA_B0_LOGICOP_NONE;
    VdmaCfg.copy.DstAddr = psCopyInfo->DstAddr;
    VdmaCfg.copy.DstAddrMode = psCopyInfo->DstAddrMode;         // VDMA_DRV_ADDRMODE_GPU_GPA
    VdmaCfg.copy.SrcAddr = psCopyInfo->SrcAddr;
    VdmaCfg.copy.SrcAddrMode = psCopyInfo->SrcAddrMode;
    VdmaCfg.copy.SrcDescNum = psCopyInfo->SrcDescNum;
    VdmaCfg.copy.DstDescNum = psCopyInfo->DstDescNum;
    VdmaCfg.copy.LenUnit = LengthUnit;
    VdmaCfg.copy.TransferLen = TransferLen;
    VDMA_B0_GenVdmaCmd(&VdmaCfg, &VdmaCmd);

//    VDMA_B0_DumpVdmaCmd(&VdmaCmd);

    VDMA_B0_WrChCmdBuf(vdmaBaseAddr, Channel, VdmaCmd.cmd64, 3);

    // 4. Polling Status (DEBUG NOW)
    if (!UseIntr) {
        Ret = VDMA_B0_WaitXferCompleted(vdmaBaseAddr, Channel, VdmaCfg.CommandTag);
    }

    return Ret;
}

int32_t VDMA_B0_CreateVdmaCmd(VDMA_B0_CopyInfo_T *psCopyInfo, VDMA_B0_CmdData_T *VdmaCmd,uint8_t CommandTag)
{
    VDMA_B0_CmdCfg_T VdmaCfg;
    uint32_t TransferLen;
    uint8_t LengthUnit;
    int32_t Ret = 0;

    calcLenUnitAndTransferLen(psCopyInfo->XferByteLen, &LengthUnit, &TransferLen);
   //pr_err("XferByteLen=0x%x, LengthUnit=0x%x, TransferLen=0x%x\n", psCopyInfo->XferByteLen, LengthUnit, TransferLen);

    memset(&VdmaCfg, 0, sizeof(VDMA_B0_CmdCfg_T));
    VdmaCfg.CommandTag = CommandTag;
    VdmaCfg.IntrEn = 1;
    VdmaCfg.OpCode = psCopyInfo->OpCode;                        // VDMA_CMD_OPCODE_COPY_DSRC_DDST
//   pr_err("SrcAddr = 0x%llx DstAddr = 0x%llx \n",psCopyInfo->SrcAddr,psCopyInfo->DstAddr);
    VdmaCfg.copy.LogicOp = VDMA_B0_LOGICOP_NONE;
    VdmaCfg.copy.DstAddr = psCopyInfo->DstAddr;
    VdmaCfg.copy.DstAddrMode = psCopyInfo->DstAddrMode;         // VDMA_DRV_ADDRMODE_GPU_GPA
    VdmaCfg.copy.SrcAddr = psCopyInfo->SrcAddr;
    VdmaCfg.copy.SrcAddrMode = psCopyInfo->SrcAddrMode;
    VdmaCfg.copy.SrcDescNum = psCopyInfo->SrcDescNum;
    VdmaCfg.copy.DstDescNum = psCopyInfo->DstDescNum;
    VdmaCfg.copy.LenUnit = LengthUnit;
    VdmaCfg.copy.TransferLen = TransferLen;
	VdmaCfg.CommandTag = CommandTag;
//	pr_err("TransferLen = 0x%llx\n",TransferLen);
    VDMA_B0_GenVdmaCmd(&VdmaCfg, VdmaCmd);

//    VDMA_B0_DumpVdmaCmd(&VdmaCmd);

//    VDMA_B0_WrChCmdBuf(vdmaBaseAddr, Channel, VdmaCmd.cmd64, 3);

    // 4. Polling Status (DEBUG NOW)
//    if (!UseIntr) {
//        Ret = VDMA_B0_WaitXferCompleted(vdmaBaseAddr, Channel, VdmaCfg.CommandTag);
//    }

    return Ret;
}


int32_t VDMA_B0_MemSetPolled(uint64_t vdmaBaseAddr, uint32_t Channel,
        uint32_t MemSetPattern, uint8_t DstAddrMode, uint64_t DstAddr,
        uint32_t XferByteLen)
{
    return VDMA_B0_MemSetBase(vdmaBaseAddr, Channel, MemSetPattern,
            DstAddrMode, DstAddr, XferByteLen, 0);
}

int32_t VDMA_B0_CopyPolled(uint64_t vdmaBaseAddr, uint32_t Channel, VDMA_B0_CopyInfo_T *psCopyInfo)
{
    return VDMA_B0_CopyBase(vdmaBaseAddr, Channel, psCopyInfo, 0);
}

int32_t VDMA_B0_MemSetIntr(uint64_t vdmaBaseAddr, uint32_t Channel,
        uint32_t MemSetPattern, uint8_t DstAddrMode, uint64_t DstAddr,
        uint32_t XferByteLen)
{
    return VDMA_B0_MemSetBase(vdmaBaseAddr, Channel, MemSetPattern,
            DstAddrMode, DstAddr, XferByteLen, 1);
}

int32_t VDMA_B0_CopyIntr(uint64_t vdmaBaseAddr, uint32_t Channel, VDMA_B0_CopyInfo_T *psCopyInfo)
{
    return VDMA_B0_CopyBase(vdmaBaseAddr, Channel, psCopyInfo, 1);
}

uint32_t VDMA_B0_RdCtrlReg(uint64_t vdmaBaseAddr, uint32_t channel)
{
	VDMA_B0_Regs_T *dmaRegs;
    uint32_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].CTRL;
	return rawSts;
}

uint32_t VDMA_B0_RdChannelIdleReg(uint64_t vdmaBaseAddr, uint32_t channel)
{
	VDMA_B0_Regs_T *dmaRegs;
    uint32_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].CHANNEL_IDLE;
	return rawSts;
}

uint32_t VDMA_B0_RdCmdBufOverflowSts(uint64_t vdmaBaseAddr, uint32_t channel)
{
	VDMA_B0_Regs_T *dmaRegs;
    uint32_t rawSts = 0;

    DBG_ASSERT(0 != vdmaBaseAddr);
    DBG_ASSERT(channel < VDMA_CHANNEL_TOTAL_NUM);

    dmaRegs =  (VDMA_B0_Regs_T*)(uint64_t)vdmaBaseAddr;
    rawSts = dmaRegs->ch[channel].CMD_FIFO_STS;
	return rawSts;
}

