/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#pragma once
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0))
#include <stddef.h>
#else
#include <linux/stddef.h>
#endif
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/init.h>

//#include "SysTypes.h"

/************************** Constant Definitions ****************************/
//#define VDMA_BASE_ADDR 0x20200000

#define VDMA_PORT_AXLEN_MAX         7
#define VDMA_CHANNEL_TOTAL_NUM      4
#define VDMA_STREAM_ID_MAX          1023
#define VDMA_COMMAND_TAG_TOTAL_NUM  64
#define VDMA_CLEAR_REG_WIDTH        32
#define VDMA_B0_REFBUF_IDX_MAX         127
#define VDMA_CHANNEL_PRIORITY_MAX   3
#define VDMA_CHANNEL_QOS_MAX        0xf

/* Default Config */
#define VDMA_DEF_POLLING_INTVAL     0x10000
#define VDMA_DEF_POLLING_MAX_SIZE   0x5

//#define VDMA_DEF_SVA_STREAM_ID            0x2
//#define VDMA_DEF_GPA_STREAM_ID            0x0

#define VDMA_B0_SVA_SRC_SID      0x2
#define VDMA_B0_SVA_DST_SID      0x1
#define VDMA_B0_GPA_SID          0x0

/* Align Requirement */
// Copy Cmd Align
#define VDMA_B0_COPY_ADDR_ALIGN             8

// Descriptor Align
#define VDMA_B0_DESC_DATAPTR_ALIGN          8   // A0 32
#define VDMA_B0_DESC_LINKPTR_ALIGN          8   // A0 16

// Direct Write Cmd Align
#define VDMA_B0_DWRITE_ADDR_ALIGN           32
#define VDMA_B0_DWRITE_PATTERN_LOC_ALIGN    4

#define VDMA_B0_REFBUF_OFFSET               2

// Pollinge Cmd Align
#define VDMA_B0_POLLING_ADDR_ALIGN          8
#define VDMA_B0_POLLING_PATTERN_LOC_ALIGN   8

/* Value Limit */
#define VDMA_B0_DWRITE_PATTERN_LEN_LIMIT    256
#define VDMA_B0_COMMAND_TAG_LIMIT           63

#define VDMA_B0_XFERLEN_WIDTH               22
#define VDMA_B0_XFERLEN_LIMIT               (0x1<<VDMA_B0_XFERLEN_WIDTH)



/**************************** Type Definitions (REG) ******************************/

/* Common Registers Map*/
typedef struct VDMA_B0_CommRegs_S {
    volatile uint32_t SOFT_RST                       ;   // 0x00, RW, write 1 assert, write 0 release
    volatile uint32_t DMA_IDLE                       ;   // 0x04, RO, if 1, DMA in IDLE state(all ch in idle state)
    volatile uint32_t rev0x08_0x0c[2]                ;   // 0x08 ~ 0x0C
    volatile uint32_t HOST_AXI_RD_CTRL               ;   // 0x10, SWWE, HOST ARLEN (3'h0:1, 3'h1:2, 3'h3:4, ... 3'h7:128)
    volatile uint32_t HOST_AXI_WR_CTRL               ;   // 0x14, SWWE, HOST AWLEN
    volatile uint32_t NDF_AXI_RD_CTRL                ;   // 0x18, SWWE, HOST ARLEN (3'h0:1, 3'h1:2, 3'h3:4, ... 3'h7:128)
    volatile uint32_t NDF_AXI_WR_CTRL                ;   // 0x1C, SWWE, HOST AWLEN
    volatile uint32_t CHANNEL_PRIORITY               ;   // 0x20, SWWE, Priority, 0 Lowest, 3 Highest
                                                    //      [1:0] CH0_PRIORITY, [3:2] CH1, [5:4] CH2, [7:6] CH3
    volatile uint32_t CHANEL_REQ_TIMEOUT_SIZE        ;   // 0x24, SWWE, if timeout, low priority ch will be served
    volatile uint32_t HOST_PORT_OUTSTANDING_SIZE     ;   // 0x28, SWWE
    volatile uint32_t NDF_PORT_OUTSTANDING_SIZE      ;   // 0x2C, SWWE
    volatile uint32_t rev0x30_0x3c[4]                ;   // 0x30 ~ 0x3c
    volatile uint32_t VD_MCU_VERSION                 ;   // 0x40 VD-MCU Version
    volatile uint32_t VD_MCU_ID                      ;   // 0x44, RO, 0 - VD-MCU0, 1- VD-MCU1
    volatile uint32_t rev0x48_0x4c[2]                ;   // 0x48 ~ 0x4c
    volatile uint32_t SCRATCH                        ;   // 0x50, RW, Debug Purpose
    volatile uint32_t PAD_MUX_CFG                    ;   // 0x54, RW, Debug Purpose
    volatile uint32_t rev0x58_0x5c[2]                ;   // 0x58 ~ 0x5c
    volatile uint32_t BLOCK_TRANS_CTRL               ;   // 0x60, SWWE, HOST/NDF block transfer enable
    volatile uint32_t BLOCK_TRANS_LEN                ;   // 0x64, SWWE, Port block transfer length (2'h0:2K, 2'h1:4k, 2'H2:8K)
    volatile uint32_t rev0x68_0xfc[38]               ;   // 0x68 ~ 0xFC
    volatile uint32_t DEBUG_CH_SEL                   ;   // 0x100
    volatile uint32_t HOST_RD_BEAT_CH_SDAT_CNT       ;   // 0x104
    volatile uint32_t HOST_RD_BEAT_CH_SDES_CNT       ;   // 0x108
    volatile uint32_t HOST_RD_BEAT_CH_DDES_CNT       ;   // 0x10C
    volatile uint32_t HOST_RD_BEAT_CH_POLLING_CNT    ;   // 0x110
    volatile uint32_t HOST_RD_REQ_BEAT_CH_SDAT_CNT   ;   // 0x114
    volatile uint32_t HOST_RD_REQ_BEAT_CH_SDES_CNT   ;   // 0x118
    volatile uint32_t HOST_RD_REQ_BEAT_CH_DDES_CNT   ;   // 0x11c
    volatile uint32_t HOST_RD_REQ_BEAT_CH_POLLING_CNT;   // 0x120
    volatile uint32_t HOST_WR_BEAT_CH_CNT            ;   // 0x124
    volatile uint32_t HOST_AXI_WR_STATE0             ;   // 0x128
    volatile uint32_t HOST_AXI_WR_STATE1             ;   // 0x12c
    volatile uint32_t HOST_AXI_RD_STATE0             ;   // 0x130
    volatile uint32_t HOST_AXI_RD_STATE1             ;   // 0x134
    volatile uint32_t HOST_AXI_OUTSTANDING_STATE     ;   // 0x138
    volatile uint32_t NDF_RD_BEAT_CH_SDAT_CNT        ;   // 0x13c
    volatile uint32_t NDF_RD_BEAT_CH_SDES_CNT        ;   // 0x140
    volatile uint32_t NDF_RD_BEAT_CH_DDES_CNT        ;   // 0x144
    volatile uint32_t NDF_RD_BEAT_CH_POLLING_CNT     ;   // 0x148
    volatile uint32_t NDF_RD_REQ_BEAT_CH_SDAT_CNT    ;   // 0x14c
    volatile uint32_t NDF_RD_REQ_BEAT_CH_SDES_CNT    ;   // 0x150
    volatile uint32_t NDF_RD_REQ_BEAT_CH_DDES_CNT    ;   // 0x154
    volatile uint32_t NDF_RD_REQ_BEAT_CH_POLLING_CNT ;   // 0x158
    volatile uint32_t NDF_WR_BEAT_CH_CNT             ;   // 0x15c
    volatile uint32_t NDF_AXI_WR_STATE0              ;   // 0x160
    volatile uint32_t NDF_AXI_WR_STATE1              ;   // 0x164
    volatile uint32_t NDF_AXI_RD_STATE0              ;   // 0x168
    volatile uint32_t NDF_AXI_RD_STATE1              ;   // 0x16c
    volatile uint32_t NDF_AXI_OUTSTANDING_STATE      ;   // 0x170
    volatile uint32_t HOST_RD_BEAT_CH_DDAT_H_CNT     ;   // 0x174
    volatile uint32_t HOST_RD_BEAT_CH_DDAT_N_CNT     ;   // 0x178
    volatile uint32_t HOST_RD_REQ_BEAT_CH_DDAT_H_CNT ;   // 0x17c
    volatile uint32_t HOST_RD_REQ_BEAT_CH_DDAT_N_CNT ;   // 0x180
    volatile uint32_t NDF_RD_BEAT_CH_DDAT_H_CNT      ;   // 0x184
    volatile uint32_t NDF_RD_BEAT_CH_DDAT_N_CNT      ;   // 0x188
    volatile uint32_t NDF_RD_REQ_BEAT_CH_DDAT_H_CNT  ;   // 0x18c
    volatile uint32_t NDF_RD_REQ_BEAT_CH_DDAT_N_CNT  ;   // 0x190
    volatile uint32_t rev0x194_0xffc[923]            ;   // 0x194 ~ 0xffc, pad to 4K
} VDMA_B0_CommRegs_T;
//COMPILE_ASSERT(sizeof(VDMA_B0_CommRegs_T) == 4096, "size fault");

typedef struct VDMA_B0_ChanRegs_S
{
    volatile uint32_t CTRL;                          // 0x00, DMA Channel Ctrl
                                                //      [0], SWWE, Channel Enable
                                                //      [1], SWWE, Interrupt Enable
                                                //      [2], SWWE, Cmd Sequence Execute Enable
    volatile uint32_t CHANNEL_IDLE;                  // 0x04, DMA Channel Idle
    volatile uint32_t HOST_AXI_CTRL;                 // 0x08, HOST ARQOS and AWQOS
                                                //      [3:0], SWWE, ARQOS
                                                //      [7:4], SWWE, ARCACHE
                                                //      [19:16], SWWE, AWQOS
                                                //      [23:20], SWWE, AWCACHE
    volatile uint32_t NDF_AXI_CTRL;                  // 0x0C, NDF ARQOS and AWQOS
                                                //      [3:0], SWWE, ARQOS
                                                //      [7:4], SWWE, ARCACHE
                                                //      [19:16], SWWE, AWQOS
                                                //      [23:20], SWWE, AWCACHE
    volatile uint64_t CMD_CMPL_INTR_STS;             // 0x10 ~ 0x14, RO, Command Completion Int Status (0x10: Tag31-0, 0x14: Tag63-32)
    volatile uint64_t POLLING_TIMEOUT_INTR_STS;      // 0x18 ~ 0x1C, RO, Polling Timeout Int Status
    volatile uint64_t BUS_ERR_INTR_STS;              // 0x20 ~ 0x24, RO, Bus Error Int Status
    volatile uint64_t CMD_CMPL_RAW_STS;              // 0x28 ~ 0x2C, RO, Command Completion Raw Status
    volatile uint64_t POLLING_TIMEOUT_RAW_STS;       // 0x30 ~ 0x34, RO, Polling Timeout Raw Status
    volatile uint64_t BUS_ERR_RAW_STS;               // 0x38 ~ 0x3C, RO, Bus Error Raw Status
    volatile uint64_t CMD_CMPL_CLR;                  // 0x40 ~ 0x44, W1P, Command Completion Clear
    volatile uint64_t POLLING_TIMEOUT_CLR;           // 0x48 ~ 0x4c, W1P, Polling Timeout Clear
    volatile uint64_t BUS_ERR_CLR;                   // 0x50 ~ 0x54, W1P, Bus Error Clear
    volatile uint32_t POLLING_INTERVAL;              // 0x58, SWWE, Polling Interval, Cycle
    volatile uint32_t POLLING_MAX_SIZE;              // 0x5C, SWWE, Polling Maximum Size
    volatile uint32_t RD_SRC_DESC_STREAM_ID;         // 0x60, SWWE, NDF read source descriptor axi request stream id
    volatile uint32_t RD_DST_DESC_STREAM_ID;         // 0x64, SWWE, NDF read destination descriptor axi request stream id
    volatile uint32_t RD_SRC_DATA_STREAM_ID;         // 0x68, SWWE, NDF read src data axi request stream id
    volatile uint32_t POLLING_RD_DATA_STREAM_ID;     // 0x6C, SWWE, NDF polling read data axi request stream id
    volatile uint32_t WR_DST_DATA_STREAM_ID;         // 0x70, SWWE, NDF write destination data axi request stream id
    volatile uint32_t PHYSICAL_ADDR_STREAM_ID;       // 0x74, SWWE, NDF port for GPA, use this stream id
    volatile uint32_t REMAIN_CMD_BUF_SIZE;           // 0x78, SWWE, Remain FIFO Cnt
    volatile uint32_t rev0x7c_0xfc[33];              // 0x7C ~ 0xFC
    volatile uint32_t CMD_FIFO_STS;                  // 0x100, W1C, Command FIFO overflow status
    volatile uint32_t DBG_HSRC_FETCH_SRC_DESC_REQ;   // 0X104
    volatile uint32_t DBG_HSRC_FETCH_DST_DESC_REQ;   // 0X108
    volatile uint32_t DBG_HSRC_FETCH_SRC_DATA_REQ;   // 0X10C
    volatile uint32_t DBG_HSRC_POLLING_CMD_REQ;      // 0X110
    volatile uint32_t DBG_NSRC_FETCH_SRC_DESC_REQ;   // 0X114
    volatile uint32_t DBG_NSRC_FETCH_DST_DESC_REQ;   // 0X118
    volatile uint32_t DBG_NSRC_FETCH_SRC_DATA_REQ;   // 0x11c
    volatile uint32_t DBG_NSRC_POLLING_CMD_REQ;      // 0x120
    volatile uint32_t DBG_CMD_PARSE0;                // 0x124
    volatile uint32_t DBG_CMD_PARSE1;                // 0x128
    volatile uint32_t DBG_CMD_PARSE2;                // 0x12C
    volatile uint32_t DBG_HSRC_RD_RESP_DEMUX;        // 0x130
    volatile uint32_t DBG_NSRC_RD_RESP_DEMUX;        // 0x134
    volatile uint32_t DBG_HSRC_WR_DST_DATA_REQ;      // 0x138
    volatile uint32_t DBG_NSRC_WR_DST_DATA_REQ;      // 0x13C
    volatile uint32_t DBG_WR_DIRECT_REQ;             // 0x140
    volatile uint32_t N_H_COPY_CMD_CNT;              // 0x144
    volatile uint32_t H_H_COPY_CMD_CNT;              // 0x148
    volatile uint32_t H_WR_DIRECT_CMD_CNT;           // 0x14C
    volatile uint32_t H_POLLING_CMD_CNT;             // 0x150
    volatile uint32_t N_N_COPY_CMD_CNT;              // 0x154
    volatile uint32_t H_N_COPY_CMD_CNT;              // 0x158
    volatile uint32_t N_WR_DIRECT_CMD_CNT;           // 0x15C
    volatile uint32_t N_POLLING_CMD_CNT;             // 0x160
    volatile uint32_t FENCE_CMD_CNT;                 // 0x164
    volatile uint32_t RECEIVE_CMD_CNT;               // 0x168
    volatile uint32_t COMPLE_CMD_CNT;                // 0x16C
    volatile uint32_t DMA_ERROR_DETAIL;              // 0x170
    volatile uint32_t DBG_HSRC_FETCH_DST_DATA_REQ;   // 0x174
    volatile uint32_t DBG_NSRC_FETCH_DST_DATA_REQ;   // 0x178
    volatile uint32_t DBG_CMD_CMPL_RAW_STS[2];       // 0x17C ~ 0x180
    volatile uint32_t DBG_CMD_CMPL_CLR[2];           // 0x184 ~ 0x188
    volatile uint32_t rev0x188_0x1fc[29];            // 0x18C ~ 0x1FC
    volatile uint32_t REF_BUF[128];                      // 0x200 ~ 0x3FC
    volatile uint32_t CMD[192];                  // 0x400 ~ 0x6FC
    volatile uint32_t rev0x700_0x7fc[64];            // 0x700 ~ 0x7FC, pad to 2K
} VDMA_B0_ChanRegs_T;
//COMPILE_ASSERT(sizeof(VDMA_B0_ChanRegs_T) == 2048, "size fault");
////COMPILE_ASSERT(offsetof(VDMA_B0_ChanRegs_T, DBG_CMD_CMPL_RAW_STS) == 0x180, "size fault");

typedef struct
{
    VDMA_B0_CommRegs_T comm;
    VDMA_B0_ChanRegs_T ch[4];
} VDMA_B0_Regs_T;

/* DMA REG */
typedef union {
    uint32_t all;
    struct {
        uint32_t Ch0Priority:2;
        uint32_t Ch1Priority:2;
        uint32_t Ch2Priority:2;
        uint32_t Ch3Priority:2;
        uint32_t Rsvd:24;
    };
} VDMA_B0_ChPriorityReg_T;
//COMPILE_ASSERT(sizeof(VDMA_B0_ChPriorityReg_T) == 4, "size fault");

typedef union {
    uint32_t all;
    struct {
        uint32_t HostBlkTransEn:1;
        uint32_t Rsvd0:15;
        uint32_t NdfBlkTransEn:1;
        uint32_t Rsvd:15;
    };
} VDMA_B0_BlkTransCtrlReg_T;
//COMPILE_ASSERT(sizeof(VDMA_B0_BlkTransCtrlReg_T) == 4, "size fault");

typedef union {
    uint32_t all;
    struct {
        uint32_t DmaEnable:1;
        uint32_t IntrEn:1;
        uint32_t SeqExec:1;
        uint32_t Rsvd:29;
    };
} VDMA_B0_ChCtrlReg_T;
//COMPILE_ASSERT(sizeof(VDMA_B0_ChCtrlReg_T) == 4, "size fault");

typedef union {
    uint32_t all;
    struct {
        uint32_t ArQos:4;
        uint32_t ArCache:4;
        uint32_t Rsvd1:8;
        uint32_t AwQos:4;
        uint32_t AwCache:4;
        uint32_t Rsvd2:8;
    };
} VDMA_B0_ChAxCtrlReg_t;
//COMPILE_ASSERT(sizeof(VDMA_B0_ChAxCtrlReg_t) == 4, "size fault");

/* DMA Command */
typedef union
{
    uint64_t all;
    struct
    {
        uint64_t SrcAddr_47_3:45;   // 44:0 Source Address[47:3]
        uint64_t LogicOp:3;         // 47:45 Logic Operation: 0:None 1:NOT 2:XOR 3:OR 4:AND
        uint64_t rsvd:2;            // 49:48 Rsvd
        uint64_t Tag:6;             // 55:50 Command Tag
        uint64_t Opcode:8;          // 63:56 Operation Code
    };
}DMAB0CmdDw0_T;

typedef union
{
    uint64_t all;
    struct
    {
        uint64_t Xstep_SrcDescNum:16;  // 15:0 XStep(OP80/81/84) or SrcDescriptorNum(OP82/83)
        uint64_t DstAddr_47_3:45;   // 60:16 Destination Address
        uint64_t LenUnit:3;         // 63:61 Length Unit: 8B/16B/32B/64B/1KB
    };
}DMAB0CmdDw1_T;

typedef union
{
    uint64_t all;
    struct
    {
        uint64_t PcieFuncNum:1;                 // 0
        uint64_t PcieDontSnoop:1;               // 1
        uint64_t IntrEn:1;                      // 2
        uint64_t rsvd1:1;                       // 3
        uint64_t DstrideLow_PatternLen:8;       // 11:4 DstStride[7:0](OP84) or PatternLocation
        uint64_t DstrideHigh_PatternLoc:8;      // 19:12 DstStride[15:8](OP84) or PatternLocation
        uint64_t Sstride_DWriteLenLow:16;       // 35:20 SrcStride(OP84) or DirectWriteLen[15:0]
        uint64_t Ystep_DstDescNum_DstXferLen_DWriteLenHigh:16;  // 51:36 Ystep(OP84) or DstDescNum(OP81/83)
                                                                //       or DstXferLen(OP82) or DirectWriteLen[24:16]
        uint64_t DstAddrMode:4;     // 55:52 Destination Address Mode
        uint64_t SrcAddrMode:4;     // 59:56 Source Address Mode
        uint64_t rsvd2:4;           // 63:60 Rsvd
    };
}DMAB0CmdDw2_T;

typedef union
{
    uint32_t cmd32[6];
    uint64_t cmd64[3];
    struct
    {
        DMAB0CmdDw0_T dw0;
        DMAB0CmdDw1_T dw1;
        DMAB0CmdDw2_T dw2;
    };
}VDMA_B0_CmdData_T;

/* DMA Descriptor */
typedef union
{
    uint64_t all;
    struct
    {
        uint64_t DataBlkPtr_47_3:45;    // 44:0
        uint64_t TransferLen:16;        // 60:45
        uint64_t LenUnit:3;             // 63:61
    }nonend;
    struct
    {
        uint64_t NextDescPtr_47_3:45;   // 44:0
        uint64_t NextDescNum:16;        // 60:45
        uint64_t LinkEnd:1;             // 61
        uint64_t rsvd:2;                // 63:62
    }end;
}VDMA_B0_DescData_T;

/* DMA Command Cfg */
typedef struct {
    uint64_t SrcAddr;
    uint64_t DstAddr;
    uint32_t TransferLen;       // if Linklist to Linklist, TransferLen is not used
    uint16_t SrcDescNum;        // used in L2D, L2L
    uint16_t DstDescNum;        // used in D2L, L2L
    uint8_t SrcAddrMode;        // SrcData and SrcDesc AddrMode
    uint8_t DstAddrMode;        // DstData and DstDesc AddrMode
    uint8_t LenUnit;            // used by Xstep, Ystep, Dstride, Sstride
    uint8_t LogicOp;            //
} VDMA_B0_CopyCfg_T;

typedef struct {
    uint64_t SrcAddr;
    uint64_t DstAddr;
    uint32_t Xstep;
    uint32_t Ystep;
    uint32_t Sstride;
    uint32_t Dstride;
    uint8_t SrcAddrMode;        // SrcData and SrcDesc AddrMode
    uint8_t DstAddrMode;        // DstData and DstDesc AddrMode
    uint8_t LenUnit;            // used by Xstep, Ystep, Dstride, Sstride
    uint8_t LogicOp;            //
} VDMA_B0_2DCopyCfg_T;

typedef struct {
    uint64_t DstAddr;
    uint32_t TransferLen;       // How many pattern Block will be transfer
    uint16_t PatternLen;        // Pattern Length (Unit Byte)
    uint16_t PatternLoc;        // Pattern Location (4Byte Align)
    uint8_t DstAddrMode;
    uint32_t patternData;
}VDMA_B0_DWriteCfg_T;

typedef struct {
    uint64_t SrcAddr;       // 8Byte Align
    uint8_t SrcAddrMode;
    uint8_t PatternLoc;     // Pattern Location (8Byte Align)
}VDMA_B0_PollingCfg_T;

typedef struct VDMA_CMD_Config1 {
    union {
        VDMA_B0_CopyCfg_T copy;
        VDMA_B0_2DCopyCfg_T copy2d;
        VDMA_B0_DWriteCfg_T dwrite;
        VDMA_B0_PollingCfg_T polling;
    };
    uint8_t OpCode;
    uint8_t CommandTag;
    uint8_t IntrEn;
    uint8_t PcieFuncNum;
    uint8_t PcieDontSnoop;
} VDMA_B0_CmdCfg_T;

/* CopyInfo */
typedef struct
{
    uint64_t SrcAddr;
    uint64_t DstAddr;
    uint32_t SrcDescNum;
    uint32_t DstDescNum;
    uint32_t XferByteLen;
    uint8_t SrcAddrMode;
    uint8_t DstAddrMode;
    uint8_t OpCode;
} VDMA_B0_CopyInfo_T;

/* StreamId */
typedef struct
{
    uint32_t RdSrcDescSid;
    uint32_t RdSrcDataSid;
    uint32_t PollingRdDataSid;
    uint32_t RdDstDescSid;
    uint32_t WrDstDataSid;
    uint32_t PhyAddrSid;
} VDMA_B0_StreamId_T;

typedef struct VDMA_B0_DESC_NODE{
	void *phy_node;
	void *virt_node;
	struct VDMA_B0_DESC_NODE *next;
	struct VDMA_B0_DESC_NODE *head;
	struct VDMA_B0_DESC_NODE *tail;
}VDMA_B0_DESC_Node_t;


/* VDMA DESC LinkHeadTail */
typedef struct VDMA_B0_DESC_LinkHeadTail {
	void* head;			// Pointer to DescLinkList Head (use Local Address)
	void* tail;			// Pointer to DescLinkList Tail (use Local Address)
	VDMA_B0_DESC_Node_t *node_info;
}VDMA_B0_DESC_LinkHeadTail_t;


typedef enum {
    VDMA_B0_COM_AXLEN_1 = 0x0,
    VDMA_B0_COM_AXLEN_2 = 0x1,
    VDMA_B0_COM_AXLEN_4 = 0x2,
    VDMA_B0_COM_AXLEN_8 = 0x3,
    VDMA_B0_COM_AXLEN_16 = 0x4,
    VDMA_B0_COM_AXLEN_32 = 0x5,
    VDMA_B0_COM_AXLEN_64 = 0x6
} VDMA_B0_AXLEN_T;

typedef enum {
    VDMA_B0_OPCODE_WRITE_DIRECT = 0x1,         // Write Pattern Data to Destination
    VDMA_B0_OPCODE_FENCE = 0x2,                // Fence Command
    VDMA_B0_OPCODE_POLLING_LESS = 0x50,        // Polling until Destination Value Less than Pattern Data
    VDMA_B0_OPCODE_POLLING_GREATER = 0x51,     // Polling until Destination Value Greater than Pattern Data
    VDMA_B0_OPCODE_POLLING_EQUAL = 0x52,       // Polling until Destination Value Equal to Pattern Data
    VDMA_B0_OPCODE_COPY_DSRC_DDST = 0x80,      // Direct Source to Direct Destination Copy
    VDMA_B0_OPCODE_COPY_DSRC_LDST = 0x81,      // Direct Source to Linklist Destination Copy
    VDMA_B0_OPCODE_COPY_LSRC_DDST = 0x82,      // Linklist Source to Direct Destination Copy
    VDMA_B0_OPCODE_COPY_LSRC_LDST = 0x83,      // Linklist Source to Linklist Destination Copy
    VDMA_B0_OPCODE_2DCOPY = 0x84               // 2D Copy
} VDMA_B0_OPCODE_T;

typedef enum {
    VDMA_B0_LOGICOP_NONE = 0x0,
    VDMA_B0_LOGICOP_NOT = 0x1,
    VDMA_B0_LOGICOP_XOR = 0x2,
    VDMA_B0_LOGICOP_OR = 0x3,
    VDMA_B0_LOGICOP_AND = 0x4,
} VDMA_B0_LOGICOP_T;

typedef enum {
    VDMA_B0_LENUNIT_8B = 0x0,
    VDMA_B0_LENUNIT_16B = 0x1,
    VDMA_B0_LENUNIT_32B = 0x2,
    VDMA_B0_LENUNIT_64B = 0x3,
    VDMA_B0_LENUNIT_1K = 0x4
} VDMA_B0_LENUNIT_T;

typedef enum {
    VDMA_B0_BLKTRANSLEN_2K = 0x0,
    VDMA_B0_BLKTRANSLEN_4K = 0x1,
    VDMA_B0_BLKTRANSLEN_8K = 0x3
} VDMA_B0_BLKTRANSLEN_T;

typedef enum {
    VDMA_B0_ADDRMODE_HOST_VA = 0x0,
    VDMA_B0_ADDRMODE_HOST_PA = 0x1,
    VDMA_B0_ADDRMODE_GPU_VA = 0x2,
    VDMA_B0_ADDRMODE_GPU_PA = 0x3
} VDMA_B0_AddrMode_t;


/***************** Macros (Inline Functions) Definitions ********************/
// Length Param in VDMA CMD/Register, should subtract 1
#define VDMA_B0_LEN_ADJUST_VAL     1
#define VDMA_B0_LEN_ADJUST(len)    (len-VDMA_B0_LEN_ADJUST_VAL)

#define PACK_ADDRMODE(descAddrMode, dataAddrMode)   ((descAddrMode<<2) | dataAddrMode)

void VDMA_B0_Init(uint64_t vdmaBaseAddr, uint32_t vdmaChCnt);
void VDMA_B0_SoftReset(uint64_t vdmaBaseAddr);
void VDMA_B0_SetAxLen(uint64_t vdmaBaseAddr, uint32_t ndfAxlen, uint32_t hostAxlen);
void VDMA_B0_SetBlkTrans(uint64_t vdmaBaseAddr, uint32_t ndfBlkTransEn, uint32_t hostBlkTransEn, uint32_t blkTransLen);
void VDMA_B0_SetChPriority(uint64_t vdmaBaseAddr, uint32_t ch0Prior, uint32_t ch1Prior,
        uint32_t ch2Prior, uint32_t ch3Prior, uint32_t chPriorTimeout);
uint32_t VDMA_B0_GetVDMcuId(uint64_t vdmaBaseAddr);
void VDMA_B0_WrScrReg(uint64_t vdmaBaseAddr, uint32_t wrVal);
void VDMA_B0_SetChAxiCtrl(uint64_t vdmaBaseAddr, uint32_t channel,
        uint32_t ndfAxQos, uint32_t ndfAxCache, uint32_t hostAxQos, uint32_t hostAxCache);
void VDMA_B0_SetChStreamId(uint64_t vdmaBaseAddr, uint32_t channel, VDMA_B0_StreamId_T* psSid);
void VDMA_B0_EnDisCh(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t enDis);
void VDMA_B0_EnDisChIntr(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t enDis);
void VDMA_B0_EnDisChSeqExec(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t enDis);
uint32_t VDMA_B0_GetChCmdRemainSize(uint64_t ChannelBaseAddr);
void VDMA_B0_WrChRefBuf(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t startRefBufIdx, uint32_t *pu32WrBuf, uint32_t WrBytelen);
void VDMA_B0_WrChCmdBuf(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t *pu64WrDataBuf, uint32_t WrDwlen);
void VDMA_B0_WrCmdBuf(uint64_t vdmaCHBaseAddr, uint64_t *pu64WrDataBuf, uint32_t WrDwlen);
uint64_t VDMA_B0_RdCmdCmplIntSts(uint64_t vdmaBaseAddr, uint32_t channel);
uint64_t VDMA_B0_CH_RdAndClrCmdCmplRawSts(uint64_t ChannelBaseAddr);
uint64_t VDMA_B0_RdPollingTimeoutIntSts(uint64_t vdmaBaseAddr, uint32_t channel);
uint64_t VDMA_B0_RdBusErrIntSts(uint64_t vdmaBaseAddr, uint32_t channel);
uint64_t VDMA_B0_RdCmdCmplRawSts(uint64_t vdmaBaseAddr, uint32_t channel);
uint64_t VDMA_B0_RdPollingTimeoutRawSts(uint64_t vdmaBaseAddr, uint32_t channel);
uint64_t VDMA_B0_RdBusErrRawSts(uint64_t vdmaBaseAddr, uint32_t channel);
void VDMA_B0_WrCmdCmplClrReg(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t clrRegVal);
void VDMA_B0_WrPollingTimeoutClrReg(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t clrRegVal);
void VDMA_B0_WrBusErrClrReg(uint64_t vdmaBaseAddr, uint32_t channel, uint64_t clrRegVal);
int32_t VDMA_B0_WaitXferCompleted(uint64_t vdmaBaseAddr, uint32_t channel, uint32_t tag);
void VDMA_B0_GenVdmaCmd(VDMA_B0_CmdCfg_T *vdmaCfg, VDMA_B0_CmdData_T *vdmaCmd);
void VDMA_B0_GenNonEndVdmaDesc(uint64_t dataBlkPtr, uint32_t transferLen, uint8_t lenUnit,  VDMA_B0_DescData_T *vdmaDesc);
void VDMA_B0_GenEndVdmaDesc(uint64_t nextDescPtr, uint32_t nextDescNum, uint8_t linkEnd,  VDMA_B0_DescData_T *vdmaDesc);

void VDMA_B0_DumpComRegs(uint64_t vdmaBaseAddr);
void VDMA_B0_DumpChRegs(uint64_t vdmaBaseAddr, uint8_t channel);
void VDMA_B0_DumpVdmaCmd(VDMA_B0_CmdData_T *vdmaCmd);
void VMDA_B0_DumpNonEndVdmaDesc(VDMA_B0_DescData_T *vdmaDesc);
void VMDA_B0_DumpEndVdmaDesc(VDMA_B0_DescData_T *vdmaDesc);

int32_t VDMA_B0_MemSetPolled(uint64_t vdmaBaseAddr, uint32_t Channel,
        uint32_t MemSetPattern, uint8_t DstAddrMode, uint64_t DstAddr,
        uint32_t XferByteLen);
int32_t VDMA_B0_CopyPolled(uint64_t vdmaBaseAddr, uint32_t Channel, VDMA_B0_CopyInfo_T *psCopyInfo);
int32_t VDMA_B0_MemSetIntr(uint64_t vdmaBaseAddr, uint32_t Channel,
        uint32_t MemSetPattern, uint8_t DstAddrMode, uint64_t DstAddr,
        uint32_t XferByteLen);
int32_t VDMA_B0_CopyIntr(uint64_t vdmaBaseAddr, uint32_t Channel, VDMA_B0_CopyInfo_T *psCopyInfo);
extern void calcLenUnitAndTransferLen(uint32_t XferByteLen, uint8_t* pu8LenUnit, uint32_t* pu32TransferLen);
int32_t VDMA_B0_CreateVdmaCmd(VDMA_B0_CopyInfo_T *psCopyInfo, VDMA_B0_CmdData_T *VdmaCmd,uint8_t CommandTag);

//void VDMA_DRV_B0_DescLink_AddToTail(VDMA_B0_DESC_LinkHeadTail_t* pList, VDMA_B0_DescData_T* VdmaDesc_virt,VDMA_B0_DescData_T* VdmaDesc_phy);

uint32_t VDMA_B0_RdCtrlReg(uint64_t vdmaBaseAddr, uint32_t Channel);
uint32_t VDMA_B0_RdChannelIdleReg(uint64_t VdmaBaseAddr, uint32_t Channel);
uint32_t VDMA_B0_RdCmdBufOverflowSts(uint64_t VdmaBaseAddr, uint32_t Channel);
uint64_t VDMA_B0_RdAndClrPollingTimeoutRawSts(uint64_t vdmaBaseAddr, uint32_t channel);
uint64_t VDMA_B0_RdAndClrBusErrRawSts(uint64_t vdmaBaseAddr, uint32_t channel);
void VDMA_B0_ClrAndWrRefBufSingle(uint64_t vdmaCHBaseAddr, uint32_t RefBufIdx, uint32_t WrData);


