/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef HAL_TDMA_H
#define HAL_TDMA_H
#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 0, 0))
#include <stddef.h>
#endif
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/init.h>

#define TDMA_COMMAND_TAG_LIMIT           63

typedef struct {
    volatile uint32_t SOFT_RST;                      // 0x00
    volatile uint32_t DMA_IDLE;                      // 0x04
    volatile uint32_t DMA_INTR_CTRL;                 // 0x08
    volatile uint32_t HOST_AXI_CTRL;                 // 0x0C
    volatile uint32_t NDF_AXI_CTRL;                  // 0x10
    volatile uint32_t NDF_WRITE_OUTSTANDING_NUM;     // 0x14
    volatile uint32_t RVA_STREAM_ID;                 // 0x18
    volatile uint32_t RPA_STREAM_ID;                 // 0x1C
    volatile uint32_t WVA_STREAM_ID;                 // 0x20
    volatile uint32_t WPA_STREAM_ID;                 // 0x24
    volatile uint32_t REMAIN_CMD_BUF_SIZE;           // 0x28
    volatile uint32_t CMD_FIFO_STS;                  // 0x2C
    volatile uint32_t SCRATCH;                       // 0x30
    volatile uint32_t GPU0_GPIO_CFG;                 // 0x34
    volatile uint32_t GPU1_GPIO_CFG;                 // 0x38
    volatile uint32_t rev0x3c_0x7c[17];              // 0x3C ~ 0x7c

    volatile uint64_t CMD_CMPL_INTR_STS;             // 0x80 ~ 0x84
    volatile uint64_t BUS_ERR_INTR_STS;              // 0x88 ~ 0x8C
    volatile uint64_t CMD_CMPL_RAW_STS;              // 0x90 ~ 0x94
    volatile uint64_t BUS_ERR_RAW_STS;               // 0x98 ~ 0x9C
    volatile uint64_t CMD_CMPL_RAW_CLR;              // 0xA0 ~ 0xA4
    volatile uint64_t BUS_ERR_RAW_CLR;               // 0xA8 ~ 0xAC
    volatile uint32_t rev0xb0_0xfc[20];              // 0xB0 ~ 0xFC

    volatile uint64_t DBG_CMD_CMPL_RAW_STS;          // 0x100 ~ 0x104
    volatile uint64_t DBG_CMD_CMPL_CLR;              // 0x108 ~ 0x10c
    volatile uint32_t DBG_PARSE_CNT0;                // 0x110
    volatile uint32_t DBG_PARSE_CNT1;                // 0x114
    volatile uint32_t DBG_PARSE_STATE;               // 0x118
    volatile uint32_t DBG_RD_CNT0;                   // 0x11C
    volatile uint32_t DBG_RD_CNT1;                   // 0x120
    volatile uint32_t DBG_RD_STATE;                  // 0x124
    volatile uint32_t DBG_WR_CNT0;                   // 0x128
    volatile uint32_t DBG_WR_CNT1;                   // 0x12C
    volatile uint32_t DBG_WR_CNT2;                   // 0x130
    volatile uint32_t DBG_WR_CNT3;                   // 0x134
    volatile uint32_t DBG_WR_CNT4;                   // 0x138
    volatile uint32_t DBG_WR_CNT5;                   // 0x13C
    volatile uint32_t DBG_WR_CNT6;                   // 0x140
    volatile uint32_t DBG_WR_CNT7;                   // 0x144
    volatile uint32_t DBG_WR_STATE;                  // 0x148
    volatile uint32_t DBG_H_AXI_RD_REAT_CNT;         // 0x14C
    volatile uint32_t DBG_H_AXI_RD_REQ_REAT_CNT;     // 0x150
    volatile uint32_t DBG_H_AXI_RD_STATE0;           // 0x154
    volatile uint32_t DBG_H_AXI_RD_STATE1;           // 0x158
    volatile uint32_t DBG_N_AXI_RD_REAT_CNT;         // 0x15C
    volatile uint32_t DBG_N_AXI_RD_REQ_REAT_CNT;     // 0x160
    volatile uint32_t DBG_N_AXI_RD_STATE0;           // 0x164
    volatile uint32_t DBG_N_AXI_RD_STATE1;           // 0x168
    volatile uint32_t DBG_N_AXI_WR_REAT_CNT;         // 0x16C
    volatile uint32_t DBG_N_AXI_WR_STATE0;           // 0x170
    volatile uint32_t DBG_N_AXI_WR_STATE1;           // 0x174
    volatile uint32_t DBG_AXI_OUTSTANDING_STATE0;    // 0x178
    volatile uint32_t DBG_AXI_OUTSTANDING_STATE1;    // 0x17C
    volatile uint32_t rev0x180_0x3fc[160];           // 0x180 ~ 0x3FC

    volatile uint32_t CMD[128];                      // 0x400
} TDMA_REGS_T;
//COMPILE_ASSERT(sizeof(TDMA_REGS_T) == 1536, "size fault");

typedef union {
    uint32_t all;
    struct {
        uint32_t IntrEn:1;
        uint32_t IntrDst:2;
        uint32_t Rsvd:29;
    };
} TDMA_IntrCtrlReg_T;
//COMPILE_ASSERT(sizeof(TDMA_IntrCtrlReg_T) == 4, "size fault");

typedef union {
    uint32_t all;
    struct {
        uint32_t ArQos:4;
        uint32_t ArCache:4;
        uint32_t ArLen:2;
        uint32_t Rsvd:22;
    };
} TDMA_HostAxiCtrl_T;
//COMPILE_ASSERT(sizeof(TDMA_HostAxiCtrl_T) == 4, "size fault");

typedef union {
    uint32_t all;
    struct {
        uint32_t ArQos:4;   // 3:0
        uint32_t ArProt:3;  // 6:4
        uint32_t Rsvd1:1;
        uint32_t ArCache:4; // 11:8
        uint32_t ArLen:2;   // 13:12
        uint32_t Rsvd2:2;
        uint32_t AwQos:4;   // 19:16
        uint32_t AwProt:3;  // 22:20
        uint32_t Rsvd3:1;
        uint32_t AwCache:4; // 27:24
        uint32_t AwLen:2;   // 29:28
        uint32_t Rsvd4:2;
    };
} TDMA_NdfAxiCtrl_T;
//COMPILE_ASSERT(sizeof(TDMA_NdfAxiCtrl_T) == 4, "size fault");

/* TDMA CMD */
typedef union
{
    uint64_t all;
    struct
    {
        uint64_t SrcAddr:48;        // 47:0 Source Address[47:0]
        uint64_t SrcAddrMode:2;     // 49:48 Source Address Mode
        uint64_t LenUnit:4;         // 53:50 Length Unit
        uint64_t Tag:6;             // 59:54 Command Tag
        uint64_t Opcode:4;          // 63:60 Operation Code
    };
} TDMACmdDw0_T;
//COMPILE_ASSERT(sizeof(TDMACmdDw0_T) == 8, "size fault");

typedef union
{
    uint64_t all;
    struct
    {
        uint64_t DontSnoop:1;       // 0
        uint64_t IntrEn:1;          // 1
        uint64_t DstAddr:40;        // 41:2 Destination Address[47:0]
        uint64_t DstAddrMode:1;     // 42
        uint64_t Rsvd:21;           // 63:43 Rsvd
    };
} TDMACmdDw1_T;
//COMPILE_ASSERT(sizeof(TDMACmdDw1_T) == 8, "size fault");

typedef union
{
    uint64_t all;
    struct
    {
        uint64_t Xstride:16;        // 15:0 stride in x direction
        uint64_t Ystep:15;          // 30:16 X Step: number of piexels in x direction
        uint64_t Xstep:15;          // 45:31 Y step: number of piexels in y direction
        uint64_t Rsvd:18;           // 63:46 Rsvd
    };
} TDMACmdDw2_T;
//COMPILE_ASSERT(sizeof(TDMACmdDw2_T) == 8, "size fault");

typedef union
{
    uint64_t all;
    struct
    {
        uint64_t Ystride:32;        // 31:0 stride in y direction
        uint64_t Zstep:12;          // 43:32 Z step: number of piexels in z direction
        uint64_t Rsvd:20;           // 63:44 Rsvd
    };
} TDMACmdDw3_T;
//COMPILE_ASSERT(sizeof(TDMACmdDw3_T) == 8, "size fault");

typedef union
{
    uint32_t cmd32[8];
    uint64_t cmd64[4];
    struct
    {
        TDMACmdDw0_T dw0;
        TDMACmdDw1_T dw1;
        TDMACmdDw2_T dw2;
        TDMACmdDw3_T dw3;
    };
} TDMA_CmdData_T;

/* TDMA_CmdCfg */
typedef struct
{
    uint64_t SrcAddr;
    uint64_t DstAddr;
    uint64_t Ystride;
    uint32_t Xstride;
    uint16_t Xstep;
    uint16_t Ystep;
    uint16_t Zstep;
    uint8_t SrcAddrMode;
    uint8_t DstAddrMode;
    uint8_t IntrEn;
    uint8_t DontSnoop;
    uint8_t LenUnit;
    uint8_t Tag;
    uint8_t Opcode;
} TDMA_CmdCfg_T;

/* Stream ID */
typedef struct
{
    uint16_t RvaSid;
    uint16_t RpaSid;
    uint16_t WvaSid;
    uint16_t WpaSid;
} TDMA_StreamId_T;

typedef enum {
    TDMA_LENUNIT_1B = 0x0,
    TDMA_LENUNIT_2B = 0x1,
    TDMA_LENUNIT_4B = 0x2,
    TDMA_LENUNIT_6B = 0x3,
    TDMA_LENUNIT_8B = 0x4,
    TDMA_LENUNIT_12B = 0x5,
    TDMA_LENUNIT_16B = 0x6
} TDMA_LENUNIT_T;

typedef enum {
    TDMA_COM_AXLEN_1 = 0x0,
    TDMA_COM_AXLEN_2 = 0x1,
    TDMA_COM_AXLEN_4 = 0x2,
    TDMA_COM_AXLEN_8 = 0x3,
} TDMA_AXLEN_T;

typedef enum {
    TDMA_SRCADDRMODE_HOST_VA = 0x0,
    TDMA_SRCADDRMODE_HOST_PA = 0x1,
    TDMA_SRCADDRMODE_GPU_VA = 0x2,
    TDMA_SRCADDRMODE_GPU_PA = 0x3
} TDMA_SRCADDRMODE_T;

typedef enum {
    TDMA_DSTADDRMODE_GPU_VA = 0x0,
    TDMA_DSTADDRMODE_GPU_PA = 0x1
} TDMA_DSTADDRMODE_T;

typedef enum {
    TDMA_INTRDST_ACPUHOST = 0x0,
    TDMA_INTRDST_RSVD = 0x1,
    TDMA_INTRDST_META0 = 0x2,
    TDMA_INTRDST_META1 = 0x3
} TDMA_INTRDST_T;

typedef enum {
    TDMA_OPCODE_2DCOPY = 0x1,         // 2D linear to twiddle copy
    TDMA_OPCODE_3DCOPY = 0x2,         // 3D linear to twiddle copy
} TDMA_OPCODE_T;

#define TDMA_LEN_ADJUST_VAL     1
#define TDMA_LEN_ADJUST(len)    (len-TDMA_LEN_ADJUST_VAL)

void InitTDMA(uint64_t gTdmaBaseAddr);
void TDMA_SoftReset(uint64_t tdmaBaseAddr);
void TDMA_EnDisIntr(uint64_t tdmaBaseAddr, uint32_t enDis);
void TDMA_SetIntrDst(uint64_t tdmaBaseAddr, uint32_t intrDst);
void TDMA_WrHostAxiCtrl(uint64_t tdmaBaseAddr, uint32_t regVal);
uint32_t TDMA_RdHostAxiCtrl(uint64_t tdmaBaseAddr);
void TDMA_WrNdfAxiCtrl(uint64_t tdmaBaseAddr, uint32_t regVal);
uint32_t TDMA_RdNdfAxiCtrl(uint64_t tdmaBaseAddr);
void TDMA_WrNdfOutStandingNum(uint64_t tdmaBaseAddr, uint32_t regVal);
void TDMA_SetStreamId(uint64_t tdmaBaseAddr, TDMA_StreamId_T *psSid);
uint32_t TDMA_GetCmdRemainSize(uint64_t tdmaBaseAddr);
void TDMA_WrCmdBuf(uint64_t tdmaBaseAddr, uint64_t *pu64WrDataBuf, uint32_t WrDwlen);
uint64_t TDMA_RdCmdCmplIntSts(uint64_t tdmaBaseAddr);
uint64_t TDMA_RdBusErrIntSts(uint64_t tdmaBaseAddr);
uint64_t TDMA_RdCmdCmplRawSts(uint64_t tdmaBaseAddr);
uint64_t TDMA_RdBusErrRawSts(uint64_t tdmaBaseAddr);
void TDMA_WrCmdCmplClrReg(uint64_t tdmaBaseAddr, uint64_t clrRegVal);
void TDMA_WrBusErrClrReg(uint64_t tdmaBaseAddr, uint64_t clrRegVal);
uint64_t TDMA_RdDbgCmdCmplRawSts(uint64_t tdmaBaseAddr);
void TDMA_WrDbgCmdCmplClrReg(uint64_t tdmaBaseAddr, uint64_t clrRegVal);
void TDMA_WrGpu0GpioCfg(uint64_t tdmaBaseAddr, uint32_t gpioCfg);
void TDMA_WrGpu1GpioCfg(uint64_t tdmaBaseAddr, uint32_t gpioCfg);
void TDMA_WrScrReg(uint64_t tdmaBaseAddr, uint32_t regVal);
void TDMA_DumpCmd(TDMA_CmdData_T *psTdmaCmd);
void TDMA_DumpRegs(uint64_t tdmaBaseAddr);
int32_t TDMA_Copy(uint64_t tdmaBaseAddr, TDMA_CmdCfg_T *psTdmaCfg, uint8_t useIntr);
void TDMA_GenCmd(TDMA_CmdCfg_T *psTdmaCfg, TDMA_CmdData_T *psTdmaCmd);
int32_t TDMA_WaitXferCompleted(uint64_t tdmaBaseAddr, uint32_t tag);
int32_t TDMA_CheckXferCompleted(uint64_t tdmaBaseAddr, uint32_t tag);

#endif
