/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef EXECTB_INTF_H_
#define EXECTB_INTF_H_

#include "platform.h"

/* Peripheral and SRAM base address */
#define SRAM_BASE                 0x20000000UL
#define PERIPH_BASE               0x40000000UL

/* Peripheral memory map */
#define GPIO_BASE                 PERIPH_BASE

#define GPIO0_BASE                (GPIO_BASE)
#define GPIO1_BASE                (GPIO_BASE       + 0x0800UL)
#define GPIO2_BASE                (GPIO_BASE       + 0x1000UL)
#define GPIO3_BASE                (GPIO_BASE       + 0x1800UL)

/*--------------------- General Purpose Input and Ouptut ---------------------*/
typedef union
{
   uint32_t WORD;
   uint16_t HALFWORD[2];
   uint8_t  BYTE[4];
} GPIO_Data_TypeDef;

typedef struct
{
  GPIO_Data_TypeDef DATA [256];
  GPIO_Data_TypeDef DIR;
  uint32_t RESERVED[3];
  GPIO_Data_TypeDef IE;
} GPIO_TypeDef;

#define GPIO0                     ((GPIO_TypeDef *) GPIO0_BASE)
#define GPIO1                     ((GPIO_TypeDef *) GPIO1_BASE)
#define GPIO2                     ((GPIO_TypeDef *) GPIO2_BASE)
#define GPIO3                     ((GPIO_TypeDef *) GPIO3_BASE)

////////////////////////////////////////////////////////////////////////////////
//
// execution_tb_mcu External GPIO Pin Assignments
//
////////////////////////////////////////////////////////////////////////////////

//
// Debug Driver Hardware Interface to the execution_tb
//
// Debug Driver --------------------------------------- execution_tb_mcu
//
// GPIO[31] 7 >----------------------------------------> Running
// GPIO[30] 6 >----------------------------------------> Error
// GPIO[29] 5 <----------------------------------------< Function Strobe
// GPIO[28] 4 <----------------------------------------< }
// GPIO[27] 3 <----------------------------------------< }
// GPIO[26] 2 <----------------------------------------< > Function Select
// GPIO[25] 1 <----------------------------------------< }
// GPIO[24] 0 <----------------------------------------< }
//
//

#define IO_BYTE 3

#define IO_RUNNING (1 << 7)
#define IO_ERROR (1 << 6)
#define IO_STROBE (1 << 5)
#define IO_FUNCSEL (0x1f)


//
// Character device for printf and miscellaneous signals
//
//
// GPIO[15] 7 >----------------------------------------> Character Strobe
// GPIO[14] 6 >----------------------------------------> }
// GPIO[13] 5 >----------------------------------------> }
// GPIO[12] 4 >----------------------------------------> } Character
// GPIO[11] 3 >----------------------------------------> > Value
// GPIO[10] 2 >----------------------------------------> }
// GPIO[ 9] 1 >----------------------------------------> }
// GPIO[ 8] 0 >----------------------------------------> }
//
//


#define PRINT_BYTE 1

#define PRINT_STROBE (1 << 7)
#define PRINT_CHAR 0x7F



//
// Test Completion and Status
//
//
// GPIO[ 7] 7 >---------------------------------------->
// GPIO[ 6] 6 >---------------------------------------->
// GPIO[ 5] 5 >---------------------------------------->
// GPIO[ 4] 4 >---------------------------------------->
// GPIO[ 3] 3 >---------------------------------------->
// GPIO[ 2] 2 >---------------------------------------->
// GPIO[ 1] 1 >----------------------------------------> Test PASS
// GPIO[ 0] 0 >----------------------------------------> Test Complete


#define MISC_BYTE 0

#define MISC_PASS (1 << 1)
#define MISC_COMPLETE (1 << 0)


////////////////////////////////////////////////////////////////////////////////
//
// execution_tb_mcu Internal GPIO Pin Assignments
//
////////////////////////////////////////////////////////////////////////////////

//
// GPIO1
//
// GPIO[31] 7 >----------> NMI
// GPIO[30] 6 >----------> gpio_loopout --> Delay -+
// GPIO[29] 5 <----------< gpio_loopin  <----------+
// GPIO[28] 4
// GPIO[27] 3
// GPIO[26] 2 >----------> DBGRESTART
// GPIO[25] 1 <----------< DBGRESTARTED
// GPIO[24] 0 >----------> EDBGRQ

#define INTDBG_BYTE 3

#define GPIO_NMI (1 << 7)
#define GPIO_LOOPOUT (1 << 6)
#define GPIO_LOOPIN  (1 << 5)
#define GPIO_DBGRESTART (1 << 2)
#define GPIO_DBGRESTARTED (1 << 1)
#define GPIO_EDBGRQ (1 << 0)


// GPIO[23] 7
// GPIO[22] 6
// GPIO[21] 5 <----------< HALTED
// GPIO[20] 4 <----------< LOCKUP
// GPIO[19] 3 <----------< SLEEPDEEP
// GPIO[18] 2 <----------< SLEEPING
// GPIO[17] 1 >----------> MTBOPERATE
// GPIO[16] 0

#define POWER_BYTE 2

#define GPIO_HALTED (1 << 5)
#define GPIO_LOCKUP (1 << 4)
#define GPIO_SLEEPDEEP (1 << 3)
#define GPIO_SLEEPING (1 << 2)
#define GPIO_MTBOPERATE (1 << 1)


// GPIO[15] 7 <----------< TXEV
// GPIO[14] 6 >----------> RXEV
// GPIO[13] 5 <----------< delayed_RXEV
// GPIO[12] 4
// GPIO[11] 3
// GPIO[10] 2
// GPIO[ 9] 1
// GPIO[ 8] 0

#define EVENT_BYTE 1

#define GPIO_TXEV (1 << 7)
#define GPIO_RXEV (1 << 6)
#define GPIO_delayed_RXEV (1 << 5)


// GPIO[7] 7 >----------> TSTART
// GPIO[6] 6 >----------> TSTOP
// GPIO[5] 5
// GPIO[4] 4
// GPIO[3] 3
// GPIO[2] 2 >----------> sys_iop_clr
// GPIO[1] 1 <----------< sys_iop_present
// GPIO[0] 0

#define MISC2_BYTE 0

#define GPIO_TSTART  (1 << 7)
#define GPIO_TSTOP   (1 << 6)
#define GPIO_iop_clr (1 << 2)
#define GPIO_IOP     (1 << 1)

//
// GPIO2
//
// GPIO[31:1] >----------> IRQ[31:1]


//
// GPIO3
//
// GPIO[31:0] >----------> IRQ[63:32]


#define TEST_PASS 0
#define TEST_FAIL 1



void EXECTB_Init(void);
void EXECTB_WriteChar(int ch);
void EXECTB_Exit(int return_code);


#endif /* EXECTB_INTF_H_ */
