/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
//#define DEBUG
#if defined(__linux__)
#include <linux/version.h>
#include <linux/uaccess.h>
#include <asm/current.h>
#include <asm/io.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/highmem.h>
#endif
#include "vdma_drv.h"
#include "xdx_dma_dbglevel.h"

//static VDMA_DRV_EnvConfig_t gVdmaEnvCfg;
#undef DBG_PRINTF
#define DBG_PRINTF(format, ...) XDX_DMA_DBG(NULL, format, ## __VA_ARGS__)

void VDMA_DRV_CH_EnableIntr(uint64_t ChannelBaseAdd)
{
	VDMA_CH_Ctrl_Reg_u VdmaChCtrl;

	VdmaChCtrl.val = VDMA_HAL_CH_RdCtrlReg(ChannelBaseAdd);
//	pr_info("VDMA_DRV_CH_EnableIntr val = 0x%x",VdmaChCtrl.val);
	VdmaChCtrl.f.IntrEn = 0x1;
	VDMA_HAL_CH_WrCtrlReg(ChannelBaseAdd, VdmaChCtrl.val);
}

void VDMA_DRV_CH_DisableIntr(uint64_t ChannelBaseAdd)
{
	VDMA_CH_Ctrl_Reg_u VdmaChCtrl;

	VdmaChCtrl.val = VDMA_HAL_CH_RdCtrlReg(ChannelBaseAdd);
//	pr_info("VDMA_DRV_CH_DisableIntr val = 0x%x",VdmaChCtrl.val);
	VdmaChCtrl.f.IntrEn = 0x0;
	VDMA_HAL_CH_WrCtrlReg(ChannelBaseAdd, VdmaChCtrl.val);
}

void VDMA_DRV_GenerateVdmaCmd(VDMA_CMD_Config_t *VdmaCfg, VDMA_CMD_Data_t *VdmaCmd)
{
	DBG_ASSERT(VdmaCfg != NULL);
	DBG_ASSERT(VdmaCmd != NULL);

	// clear VdmaCmd
	//memset(VdmaCmd, 0, sizeof(VDMA_CMD_Data_t));

	switch (VdmaCfg->OpCode) {
		case VDMA_DRV_OPCODE_COPY_DSRC_DDST:
		case VDMA_DRV_OPCODE_COPY_DSRC_LDST:
		case VDMA_DRV_OPCODE_COPY_LSRC_DDST:
		case VDMA_DRV_OPCODE_COPY_LSRC_LDST:
			if ((VDMA_DRV_OPCODE_COPY_DSRC_LDST == VdmaCfg->OpCode) || (VDMA_DRV_OPCODE_COPY_LSRC_LDST == VdmaCfg->OpCode)) {
				DBG_ASSERT(TEST_ALIGNED(VdmaCfg->CopyParam.DstAddr, VDMA_DRV_DESC_LINKPTR_ALIGN));	// Linklist use 16Byte Align
			} else {
				DBG_ASSERT(TEST_ALIGNED(VdmaCfg->CopyParam.DstAddr, VDMA_DRV_CMD_COPY_ADDR_ALIGN));
			}

			if ((VDMA_DRV_OPCODE_COPY_LSRC_DDST == VdmaCfg->OpCode) || (VDMA_DRV_OPCODE_COPY_LSRC_LDST == VdmaCfg->OpCode)) {
				DBG_ASSERT(TEST_ALIGNED(VdmaCfg->CopyParam.SrcAddr, VDMA_DRV_DESC_LINKPTR_ALIGN));	// Linklist use 16Byte Align
			} else {
				DBG_ASSERT(TEST_ALIGNED(VdmaCfg->CopyParam.SrcAddr, VDMA_DRV_CMD_COPY_ADDR_ALIGN));
			}

			DBG_ASSERT(VdmaCfg->CopyParam.SrcAddrMode <= VDMA_DRV_ADDRMODE_GPU_GPA);
			DBG_ASSERT(VdmaCfg->CopyParam.DstAddrMode <= VDMA_DRV_ADDRMODE_GPU_GPA);
			DBG_ASSERT(VdmaCfg->CopyParam.LengthUnit <= VDMA_DRV_LENUNIT_1M);

			if (VDMA_DRV_OPCODE_COPY_LSRC_LDST != VdmaCfg->OpCode) {
				DBG_ASSERT(VdmaCfg->CopyParam.TransferLen >= 1);
			}

			VdmaCmd->CmdDword0.LengthUnit = VdmaCfg->CopyParam.LengthUnit;
			VdmaCmd->CmdDword0.SrcAddr_47_3 = GET_BITS_U64(VdmaCfg->CopyParam.SrcAddr, 47, 3);
			VdmaCmd->CmdDword0.SrcAddrMode = VdmaCfg->CopyParam.SrcAddrMode;
			VdmaCmd->CmdDword1.DstAddr_47_3 = GET_BITS_U64(VdmaCfg->CopyParam.DstAddr, 47, 3);
			VdmaCmd->CmdDword1.DstAddrMode = VdmaCfg->CopyParam.DstAddrMode;
			VdmaCmd->CmdDword1.TransferLen = VDMA_DRV_LEN_ADJUST(VdmaCfg->CopyParam.TransferLen);
			break;

		case VDMA_DRV_OPCODE_WRITE_DIRECT:
			DBG_ASSERT(TEST_ALIGNED(VdmaCfg->WriteDirectParam.DstAddr, VDMA_DRV_CMD_DWRITE_ADDR_ALIGN));
			DBG_ASSERT(TEST_ALIGNED(VdmaCfg->WriteDirectParam.PatternLoc, VDMA_DRV_CMD_DWRITE_PATTERN_LOC_ALIGN));
			DBG_ASSERT(VdmaCfg->WriteDirectParam.DstAddrMode <= VDMA_DRV_ADDRMODE_GPU_GPA);
			DBG_ASSERT((VdmaCfg->WriteDirectParam.PatternLen <= VDMA_DRV_CMD_DWRITE_PATTERN_LEN_LIMIT)
					&& (VdmaCfg->WriteDirectParam.PatternLen > 0));
			DBG_ASSERT(VdmaCfg->WriteDirectParam.TransferLen > 0);

			VdmaCmd->CmdDword1.DstAddr_47_3= GET_BITS_U64(VdmaCfg->WriteDirectParam.DstAddr, 47, 3);
			VdmaCmd->CmdDword1.DstAddrMode = VdmaCfg->WriteDirectParam.DstAddrMode;
			VdmaCmd->CmdDword2.PatternLen = VDMA_DRV_LEN_ADJUST(VdmaCfg->WriteDirectParam.PatternLen);
			VdmaCmd->CmdDword2.PatternLoc_9_2 = GET_BITS(VdmaCfg->WriteDirectParam.PatternLoc, 9, 2);
			VdmaCmd->CmdDword1.TransferLen = VDMA_DRV_LEN_ADJUST(VdmaCfg->WriteDirectParam.TransferLen);
//			VdmaCmd->memset_info.pattern_data = VdmaCfg->WriteDirectParam.patternData;
//			VdmaCmd->memset_info.pattern_len = VdmaCfg->WriteDirectParam.PatternLen;
//			VdmaCmd->memset_info.transfer_len = VdmaCfg->WriteDirectParam.TransferLen;
//			VdmaCmd->memset_info.Dst_addr = VdmaCfg->WriteDirectParam.DstAddr;
			break;

		case VDMA_DRV_OPCODE_FENCE:
			break;

		case VDMA_DRV_OPCODE_POLLING_LESS:
		case VDMA_DRV_OPCODE_POLLING_EQUAL:
		case VDMA_DRV_OPCODE_POLLING_GREATER:
			DBG_ASSERT(TEST_ALIGNED(VdmaCfg->PollingParam.SrcAddr, VDMA_DRV_CMD_POLLING_ADDR_ALIGN));
			DBG_ASSERT(TEST_ALIGNED(VdmaCfg->PollingParam.PatternLoc, VDMA_DRV_CMD_POLLING_PATTERN_LOC_ALIGN));
			DBG_ASSERT(VdmaCfg->PollingParam.SrcAddrMode <= VDMA_DRV_ADDRMODE_GPU_GPA);

			VdmaCmd->CmdDword0.SrcAddr_47_3 = GET_BITS_U64(VdmaCfg->PollingParam.SrcAddr, 47, 3);
			VdmaCmd->CmdDword0.SrcAddrMode = VdmaCfg->PollingParam.SrcAddrMode;
			VdmaCmd->CmdDword2.PatternLoc_9_2 = GET_BITS(VdmaCfg->PollingParam.PatternLoc, 9, 2);
			break;

		default:
			DBG_PRINTF("Unknow Opcode: 0x%x\n", VdmaCfg->OpCode);
			DBG_ASSERT(0);
			break;
	}

	DBG_ASSERT(VdmaCfg->CommandTag <= VDMA_DRV_CMD_COMMAND_TAG_LIMIT);
	DBG_ASSERT(VdmaCfg->IntrEn <= 1);
	DBG_ASSERT(VdmaCfg->PcieFuncNum <= 1);
	DBG_ASSERT(VdmaCfg->PcieDontSnoop <= 1);

	VdmaCmd->CmdDword0.OpCode = VdmaCfg->OpCode;
	VdmaCmd->CmdDword0.CommandTag = VdmaCfg->CommandTag;
	VdmaCmd->CmdDword0.IntrEn = VdmaCfg->IntrEn;
	VdmaCmd->CmdDword2.PcieFuncNum = VdmaCfg->PcieFuncNum;
	VdmaCmd->CmdDword2.PcieDontSnoop = VdmaCfg->PcieDontSnoop;
}


void VDMA_DRV_DumpVdmaDescData(VDMA_DESC_Data_t *VdmaDesc)
{
	uint64_t DataBlkPtr;
	uint64_t NextDescPtr;
	uint32_t *VdmaDescData;
	uint32_t i;

	DBG_ASSERT(VdmaDesc != NULL);

	DBG_PRINTF("== Dump VDMA DESC DATA Begin ==\n");

	DataBlkPtr = (uint64_t)VdmaDesc->Dword1.DataBlkPtr_47_21<<21;
	DataBlkPtr |= (uint64_t)VdmaDesc->Dword0.DataBlkPtr_20_5<<5;

	NextDescPtr = VdmaDesc->Dword0.NextDescPtr_47_4<<4;

	DBG_PRINT_VAR64((uint64_t)VdmaDesc->Dword0.NextDescPtr_47_4);
	DBG_PRINTF("\tNextDescPtr=0x%llx\n", NextDescPtr);
	DBG_PRINT_VAR(VdmaDesc->Dword0.DataBlkPtr_20_5);
	DBG_PRINT_VAR(VdmaDesc->Dword1.DataBlkPtr_47_21);
	DBG_PRINTF("\tDataBlkPtr=0x%llx\n", DataBlkPtr);
	DBG_PRINT_VAR(VdmaDesc->Dword1.DataPtrAddrMode);
	DBG_PRINT_VAR(VdmaDesc->Dword1.LengthUnit);
	DBG_PRINT_VAR(VdmaDesc->Dword1.LinkEnd);
	DBG_PRINT_VAR(VdmaDesc->Dword1.TransferLen);


	VdmaDescData = (uint32_t*)(VdmaDesc);

	for (i = 0; i < 4; i++) {
		DBG_PRINTF("DescData[%d] = 0x%x\n", i, VdmaDescData[i]);
	}
}

void VDMA_DRV_DescLink_Init(VDMA_DESC_LinkHeadTail_t* pList)
{
	pList->tail = pList;
	pList->head = pList;
}

void* VDMA_DRV_DescLink_PopFromHead(VDMA_DESC_LinkHeadTail_t* pList)
{
	VDMA_DESC_Data_t* VdmaDesc;
//	uint64_t NextDescLocalAddr;
	uint64_t NextDescNdfAddr;

	if (SL_IS_NULL(pList, pList->head)) {
		VdmaDesc = NULL;
	} else {
		VdmaDesc = (VDMA_DESC_Data_t*)pList->head;

		NextDescNdfAddr = (VdmaDesc->Dword0.NextDescPtr_47_4<<4);
		if (0 != NextDescNdfAddr) {
	//		NextDescLocalAddr = VDMA_DRV_NdfToLocalAddr(NextDescNdfAddr);
		    pList->head = (void*)phys_to_virt(NextDescNdfAddr);
	//		pr_info("pop phy:0x%llx virt:0x%llx \n",NextDescNdfAddr,pList->head);
	//		pr_info("pList->tail = 0x%llx\n",pList->tail);
		} else {
			pList->head = pList;
		}
	}

	return VdmaDesc;
}

void VDMA_DRV_DumpVdmaDescList(VDMA_DESC_Data_t *VdmaDescHead)
{
	VDMA_DESC_LinkHeadTail_t VdmaDescList;
	VDMA_DESC_Data_t* VdmaDesc;

	DBG_ASSERT(VdmaDescHead != NULL);

	VDMA_DRV_DescLink_Init(&VdmaDescList);
	VdmaDescList.head = VdmaDescHead;

	while (1) {
		VdmaDesc = VDMA_DRV_DescLink_PopFromHead(&VdmaDescList);
		if (VdmaDesc == NULL) {
			break;
		} else {
			pr_info("dump virt = 0x%llx phys = 0x%llx\n",(uint64_t)VdmaDesc,(uint64_t)virt_to_phys(VdmaDesc));
			VDMA_DRV_DumpVdmaDescData(VdmaDesc);
		}
	}

}

void VDMA_DRV_DumpVdmaCmdCfg(VDMA_CMD_Config_t *VdmaCfg)
{
	DBG_ASSERT(VdmaCfg != NULL);

	DBG_PRINTF("== Dump VDMA CMD CONFIG Begin ==\n");

	DBG_PRINT_VAR(VdmaCfg->OpCode);
	DBG_PRINT_VAR(VdmaCfg->CommandTag);
	DBG_PRINT_VAR(VdmaCfg->IntrEn);
	DBG_PRINT_VAR(VdmaCfg->PcieFuncNum);
	DBG_PRINT_VAR(VdmaCfg->PcieDontSnoop);

	switch (VdmaCfg->OpCode) {
		case VDMA_DRV_OPCODE_WRITE_DIRECT:
			DBG_PRINTF("##CmdType: WriteDirect##\n");
			DBG_PRINT_VAR64(VdmaCfg->WriteDirectParam.DstAddr);
			DBG_PRINT_VAR(VdmaCfg->WriteDirectParam.DstAddrMode);
			DBG_PRINT_VAR(VdmaCfg->WriteDirectParam.PatternLen);
			DBG_PRINT_VAR(VdmaCfg->WriteDirectParam.PatternLoc);
			DBG_PRINT_VAR(VdmaCfg->WriteDirectParam.TransferLen);
			break;

		case VDMA_DRV_OPCODE_FENCE:
			DBG_PRINTF("##CmdType: Fence##\n");
			break;

		case VDMA_DRV_OPCODE_POLLING_LESS:
		case VDMA_DRV_OPCODE_POLLING_EQUAL:
		case VDMA_DRV_OPCODE_POLLING_GREATER:
			DBG_PRINTF("##CmdType: Polling##\n");
			DBG_PRINT_VAR64(VdmaCfg->PollingParam.SrcAddr);
			DBG_PRINT_VAR(VdmaCfg->PollingParam.SrcAddrMode);
			DBG_PRINT_VAR(VdmaCfg->PollingParam.PatternLoc);
			break;

		case VDMA_DRV_OPCODE_COPY_DSRC_DDST:
		case VDMA_DRV_OPCODE_COPY_DSRC_LDST:
		case VDMA_DRV_OPCODE_COPY_LSRC_DDST:
		case VDMA_DRV_OPCODE_COPY_LSRC_LDST:
			DBG_PRINTF("##CmdType: Copy##\n");
			DBG_PRINT_VAR(VdmaCfg->CopyParam.LengthUnit);
			DBG_PRINT_VAR64(VdmaCfg->CopyParam.SrcAddr);
			DBG_PRINT_VAR(VdmaCfg->CopyParam.SrcAddrMode);
			DBG_PRINT_VAR64(VdmaCfg->CopyParam.DstAddr);
			DBG_PRINT_VAR(VdmaCfg->CopyParam.DstAddrMode);
			DBG_PRINT_VAR(VdmaCfg->CopyParam.TransferLen);
			break;

		default:
			DBG_PRINTF("##CmdType: Unknown##\n");
			break;
	}
}

void VDMA_DRV_DumpVdmaCmdData(VDMA_CMD_Data_t *VdmaCmd)
{
	uint32_t *VdmaCmdData;
	uint32_t i;

	DBG_ASSERT(VdmaCmd != NULL);

	DBG_PRINTF("== Dump VDMA CMD DATA Begin ==\n");

	//DBG_PRINT_VAR64(VdmaCmd->NextCmd);
	//DBG_PRINT_VAR(VdmaCmd->CmdTimeoutVal);
	//DBG_PRINT_VAR(VdmaCmd->XferStatus);
	//DBG_PRINT_VAR(VdmaCmd->HCID);

	// CMD DWORD0
	DBG_PRINT_VAR64((uint64_t)VdmaCmd->CmdDword0.SrcAddr_47_3);
	DBG_PRINTF("\tSrcAddr=0x%llx\n", (uint64_t)(VdmaCmd->CmdDword0.SrcAddr_47_3<<3));
	DBG_PRINT_VAR(VdmaCmd->CmdDword0.SrcAddrMode);
	DBG_PRINT_VAR(VdmaCmd->CmdDword0.IntrEn);
	DBG_PRINT_VAR(VdmaCmd->CmdDword0.LengthUnit);
	DBG_PRINT_VAR(VdmaCmd->CmdDword0.CommandTag);
	DBG_PRINT_VAR(VdmaCmd->CmdDword0.OpCode);


	// CMD DWORD1
	DBG_PRINT_VAR64((uint64_t)VdmaCmd->CmdDword1.DstAddr_47_3);
	DBG_PRINTF("\tDstAddr=0x%llx\n", (uint64_t)(VdmaCmd->CmdDword1.DstAddr_47_3<<3));
	DBG_PRINT_VAR(VdmaCmd->CmdDword1.DstAddrMode);
	DBG_PRINT_VAR(VdmaCmd->CmdDword1.TransferLen);

	// CMD DWORD2
	DBG_PRINT_VAR(VdmaCmd->CmdDword2.PatternLen);
	DBG_PRINT_VAR(VdmaCmd->CmdDword2.PatternLoc_9_2);
	DBG_PRINTF("\tPatternLoc=0x%x\n", VdmaCmd->CmdDword2.PatternLoc_9_2<<2);
	DBG_PRINT_VAR(VdmaCmd->CmdDword2.PcieFuncNum);
	DBG_PRINT_VAR(VdmaCmd->CmdDword2.PcieDontSnoop);


	VdmaCmdData = (uint32_t*)&(VdmaCmd->CmdDword0);

	for (i = 0; i < 6; i++) {
		DBG_PRINTF("CmdData[%d] = 0x%x\n", i, VdmaCmdData[i]);
	}
}

void VDMA_DRV_DumpVdmaDescCfg(VDMA_DESC_Config_t *VdmaDescCfg)
{
	DBG_ASSERT(VdmaDescCfg != NULL);

	DBG_PRINTF("== Dump VDMA DESC CONFIG Begin ==\n");

	DBG_PRINT_VAR64(VdmaDescCfg->DataBlkPtr);
	DBG_PRINT_VAR(VdmaDescCfg->DataPtrAddrMode);
	DBG_PRINT_VAR(VdmaDescCfg->LengthUnit);
	DBG_PRINT_VAR(VdmaDescCfg->LinkEnd);
	DBG_PRINT_VAR(VdmaDescCfg->TransferLen);
}

void VDMA_GenVdmaDesc(uint64_t DataAddr, uint32_t transferLen, uint8_t lenUnit,
	VDMA_DESC_Data_t *vdmaDesc, bool is_host)
{
	DBG_ASSERT(vdmaDesc != NULL);
	DBG_ASSERT(lenUnit <= VDMA_DRV_LENUNIT_1M);
	DBG_ASSERT(transferLen <= VDMA_DRV_XFERLEN_LIMIT);

	vdmaDesc->Dword0.DataBlkPtr_20_5 = GET_BITS_U64(DataAddr, 20, 5);
	vdmaDesc->Dword1.DataBlkPtr_47_21 = GET_BITS_U64(DataAddr, 47, 21);
	vdmaDesc->Dword1.DataPtrAddrMode = is_host ? VDMA_DRV_ADDRMODE_HOST_SVA : VDMA_DRV_ADDRMODE_GPU_GPA;
	vdmaDesc->Dword1.LengthUnit = lenUnit; //use default lenght unit
	vdmaDesc->Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(transferLen);
	vdmaDesc->Dword1.LinkEnd = 1;
}

void VDMA_ADDVdmaNextDesc(uint64_t NextDescAddr, VDMA_DESC_Data_t *vdmaDesc)
{
	DBG_ASSERT(vdmaDesc != NULL);

	vdmaDesc->Dword0.NextDescPtr_47_4 = GET_BITS_U64(NextDescAddr, 47, 4);
	vdmaDesc->Dword1.LinkEnd = 0;
}

#if 0
/* VDMA Environment API */
void VDMA_DRV_InitEnv(uint64_t VdmaBaseAddr, VDMA_DRV_Master_t MasterType)
{
	DBG_ASSERT(VdmaBaseAddr != 0);
	DBG_ASSERT((MasterType == VDMA_DRV_MST_VDMCU0)
					|| (MasterType == VDMA_DRV_MST_VDMCU1)
					|| (MasterType == VDMA_DRV_MST_ACPU)
					|| (MasterType == VDMA_DRV_MST_HOST));

	gVdmaEnvCfg.Master = MasterType;
	gVdmaEnvCfg.VdmaBaseAddr = VdmaBaseAddr;
	gVdmaEnvCfg.VdMcuId = VDMA_HAL_COM_RdVdMcuId(VdmaBaseAddr);	// TODO: delete force
	gVdmaEnvCfg.MagicNum = VDMA_DRV_ENVINIT_FLAG;

	if (0 == gVdmaEnvCfg.VdMcuId) {
		gVdmaEnvCfg.VdmcuToNdfOffset = VDMA_DRV_VDMCU0_DRAM_NDF_STA - VDMA_DRV_VDMCU_DRAM_LOCAL_STA;
	} else if (1 == gVdmaEnvCfg.VdMcuId) {
		gVdmaEnvCfg.VdmcuToNdfOffset = VDMA_DRV_VDMCU1_DRAM_NDF_STA - VDMA_DRV_VDMCU_DRAM_LOCAL_STA;
	} else {
//		DBG_PRINTF("VdMcuId is illegal\n", gVdmaEnvCfg.VdMcuId);
		DBG_ASSERT(0);
	}

	DBG_PRINT_VAR(gVdmaEnvCfg.VdMcuId);
}

#ifdef DEBUG
static RET_CODE_t VDMA_DRV_IsEnvInit(void)
{
	if (VDMA_DRV_ENVINIT_FLAG == gVdmaEnvCfg.MagicNum) {
		return RET_SUCCESS;
	} else {
		return RET_FAILURE;
	}
}
#endif

/* VDMA Common Control API */
void VDMA_DRV_SoftReset(void)
{
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_COM_WrRstReg(gVdmaEnvCfg.VdmaBaseAddr, 1);
	// TODO: add delay here
	VDMA_HAL_COM_WrRstReg(gVdmaEnvCfg.VdmaBaseAddr, 0);
}

uint32_t VDMA_DRV_COM_GetMcuId(void)
{
	return VDMA_HAL_COM_RdVdMcuId(gVdmaEnvCfg.VdmaBaseAddr);
}

uint32_t VDMA_DRV_COM_GetDmaStatus(void)
{
	uint32_t DmaStatus;

	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	DmaStatus = VDMA_HAL_COM_RdDmaIdleReg(gVdmaEnvCfg.VdmaBaseAddr);

	DBG_ASSERT((VDMA_COM_STATUS_BUSY == DmaStatus) || (VDMA_COM_STATUS_IDLE == DmaStatus));

	return DmaStatus;
}

void VDMA_DRV_COM_WriteScrReg(uint32_t WrVal)
{
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_COM_WrScratchReg(gVdmaEnvCfg.VdmaBaseAddr, WrVal);
}

void VDMA_DRV_COM_ConfigPort(VDMA_PortCfg_t* VdmaPortCfg)
{
	DBG_ASSERT(VdmaPortCfg != NULL);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_COM_WrHostArLen(gVdmaEnvCfg.VdmaBaseAddr, VdmaPortCfg->HostArLen);
	VDMA_HAL_COM_WrHostAwLen(gVdmaEnvCfg.VdmaBaseAddr, VdmaPortCfg->HostAwLen);
	VDMA_HAL_COM_WrNdfArLen(gVdmaEnvCfg.VdmaBaseAddr, VdmaPortCfg->NdfArLen);
	VDMA_HAL_COM_WrNdfAwLen(gVdmaEnvCfg.VdmaBaseAddr, VdmaPortCfg->NdfAwLen);

	VDMA_HAL_COM_WrHostPortOutstandingSize(gVdmaEnvCfg.VdmaBaseAddr, VdmaPortCfg->HostOutstandingSize);
	VDMA_HAL_COM_WrNdfPortOutstandingSize(gVdmaEnvCfg.VdmaBaseAddr, VdmaPortCfg->NdfOutstandingSize);
}

void VDMA_DRV_COM_ConfigPriority(VDMA_PriorityCfg_t* VdmaPriorityCfg)
{
	VDMA_COM_ChPriority_Reg_u ChPriority;

	DBG_ASSERT(VdmaPriorityCfg != NULL);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());
	DBG_ASSERT(VdmaPriorityCfg->Ch0Priority <= VDMA_CHANNEL_PRIORITY_MAX);
	DBG_ASSERT(VdmaPriorityCfg->Ch1Priority <= VDMA_CHANNEL_PRIORITY_MAX);
	DBG_ASSERT(VdmaPriorityCfg->Ch2Priority <= VDMA_CHANNEL_PRIORITY_MAX);
	DBG_ASSERT(VdmaPriorityCfg->Ch3Priority <= VDMA_CHANNEL_PRIORITY_MAX);

	ChPriority.val = 0;
	ChPriority.f.Ch0Priority = VdmaPriorityCfg->Ch0Priority;
	ChPriority.f.Ch1Priority = VdmaPriorityCfg->Ch1Priority;
	ChPriority.f.Ch2Priority = VdmaPriorityCfg->Ch2Priority;
	ChPriority.f.Ch3Priority = VdmaPriorityCfg->Ch3Priority;

	VDMA_HAL_COM_WrChannelPriority(gVdmaEnvCfg.VdmaBaseAddr, ChPriority.val);
	VDMA_COM_WrChannelReqTimeout(gVdmaEnvCfg.VdmaBaseAddr, VdmaPriorityCfg->ChannelReqTimeout);
}

void VDMA_DRV_COM_InitToDefault(void)
{
	VDMA_PortCfg_t VdmaPortCfg;
	VDMA_PriorityCfg_t VdmaPriorityCfg;

#if defined(SUPPORT_PANGU_FPGA)
	VdmaPortCfg.HostArLen = VDMA_COM_AXLEN_4;
	VdmaPortCfg.HostAwLen = VDMA_COM_AXLEN_4;
	VdmaPortCfg.NdfArLen = VDMA_COM_AXLEN_8;
	VdmaPortCfg.NdfAwLen = VDMA_COM_AXLEN_8;
#else
	VdmaPortCfg.HostArLen = VDMA_COM_AXLEN_1;
	VdmaPortCfg.HostAwLen = VDMA_COM_AXLEN_1;
	VdmaPortCfg.NdfArLen = VDMA_COM_AXLEN_1;
	VdmaPortCfg.NdfAwLen = VDMA_COM_AXLEN_1;
#endif	
	VdmaPortCfg.HostOutstandingSize = 0x3ff;		// 1024
	VdmaPortCfg.NdfOutstandingSize = 0x3ff;

	VDMA_DRV_COM_ConfigPort(&VdmaPortCfg);

	VdmaPriorityCfg.Ch0Priority = VDMA_COM_PRIORITY_0;
	VdmaPriorityCfg.Ch1Priority = VDMA_COM_PRIORITY_0;
	VdmaPriorityCfg.Ch2Priority = VDMA_COM_PRIORITY_0;
	VdmaPriorityCfg.Ch3Priority = VDMA_COM_PRIORITY_0;
	VdmaPriorityCfg.ChannelReqTimeout = 0x10000;		// 1024*1024
	VDMA_DRV_COM_ConfigPriority(&VdmaPriorityCfg);
}


/* VDMA Channel Control API */
void VDMA_DRV_CH_Enable(uint32_t Channel)
{
	VDMA_CH_Ctrl_Reg_u VdmaChCtrl;

	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VdmaChCtrl.val = VDMA_HAL_CH_RdCtrlReg(gVdmaEnvCfg.VdmaBaseAddr, Channel);
//	pr_info("VDMA_DRV_CH_Enable val = 0x%x",VdmaChCtrl.val);
	VdmaChCtrl.f.DmaEnable = 0x1;
	VDMA_HAL_CH_WrCtrlReg(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChCtrl.val);
}

void VDMA_DRV_CH_Disable(uint32_t Channel)
{
	VDMA_CH_Ctrl_Reg_u VdmaChCtrl;

	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VdmaChCtrl.val = VDMA_HAL_CH_RdCtrlReg(gVdmaEnvCfg.VdmaBaseAddr, Channel);
	
//	pr_info("VDMA_DRV_CH_Disable val = 0x%x",VdmaChCtrl.val);
	VdmaChCtrl.f.DmaEnable = 0x0;
	VDMA_HAL_CH_WrCtrlReg(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChCtrl.val);
}


uint32_t VDMA_DRV_CH_GetStatus(uint32_t Channel)
{
	uint8_t ChannelStatus;

	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	ChannelStatus = VDMA_HAL_CH_RdChannelIdleReg(gVdmaEnvCfg.VdmaBaseAddr, Channel);

	DBG_ASSERT((VDMA_CH_STATUS_BUSY == ChannelStatus) || (VDMA_CH_STATUS_IDLE == ChannelStatus));

	return ChannelStatus;
}

void VDMA_DRV_CH_ConfigAxQos(uint32_t Channel, VDMA_ChAxQosCfg_t* VdmaChAxQosCfg)
{
	VDMA_CH_AxQos_Reg_u HostAxQos;
	VDMA_CH_AxQos_Reg_u NdfAxQos;

	DBG_ASSERT(VdmaChAxQosCfg != NULL);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());
	DBG_ASSERT(VdmaChAxQosCfg->HostArQos <= VDMA_CHANNEL_QOS_MAX);
	DBG_ASSERT(VdmaChAxQosCfg->HostAwQos <= VDMA_CHANNEL_QOS_MAX);
	DBG_ASSERT(VdmaChAxQosCfg->NdfArQos <= VDMA_CHANNEL_QOS_MAX);
	DBG_ASSERT(VdmaChAxQosCfg->NdfAwQos <= VDMA_CHANNEL_QOS_MAX);

	HostAxQos.val = 0;
	HostAxQos.f.ArQos = VdmaChAxQosCfg->HostArQos;
	HostAxQos.f.AwQos = VdmaChAxQosCfg->HostAwQos;
	VDMA_HAL_CH_WrHostAxQos(gVdmaEnvCfg.VdmaBaseAddr, Channel, HostAxQos.val);

	NdfAxQos.val = 0;
	NdfAxQos.f.ArQos = VdmaChAxQosCfg->NdfArQos;
	NdfAxQos.f.AwQos = VdmaChAxQosCfg->NdfAwQos;
	VDMA_HAL_CH_WrNdfAxQos(gVdmaEnvCfg.VdmaBaseAddr, Channel, NdfAxQos.val);
}

void VDMA_DRV_CH_ConfigPollingTimeout(uint32_t Channel, VDMA_ChPollingTimeoutCfg_t* VdmaChPollingTimeoutCfg)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());
	DBG_ASSERT(VdmaChPollingTimeoutCfg != NULL);

	VDMA_HAL_CH_WrPollingIntval(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChPollingTimeoutCfg->PollingIntval);
	VDMA_HAL_CH_WrPollingMaxSize(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChPollingTimeoutCfg->PollingMaxSize);
}

void VDMA_DRV_CH_ConfigStreamId(uint32_t Channel, VDMA_ChStreamIdCfg_t* VdmaChStreamIdCfg)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());
	DBG_ASSERT(VdmaChStreamIdCfg != NULL);

	VDMA_HAL_CH_WrSrcDescReadStreamId(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChStreamIdCfg->SrcDescReadStreamId);
	VDMA_HAL_CH_WrDstDescReadStreamId(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChStreamIdCfg->DstDescReadStreamId);
	VDMA_HAL_CH_WrSrcDataReadStreamId(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChStreamIdCfg->SrcDataReadStreamId);
	VDMA_HAL_CH_WrPollingRdDataStreamId(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChStreamIdCfg->PollingRdDataStreamId);
	VDMA_HAL_CH_WrDstDataWriteStreamId(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChStreamIdCfg->DstDataWriteStreamId);
	VDMA_HAL_CH_WrPhyAddrStreamId(gVdmaEnvCfg.VdmaBaseAddr, Channel, VdmaChStreamIdCfg->PhyAddrStreamId);
}

uint32_t VDMA_DRV_CH_GetRemainBufSize(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdRemainCmdBufSize(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint32_t VDMA_DRV_CH_GetCmdBufOverflowSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdCmdBufOverflowSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

void VDMA_DRV_CH_ClearCmdBufOverflowSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_CH_ClearCmdBufOverflowSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint64_t VDMA_DRV_CH_GetCmdCmplIntSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdCmdCmplIntSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint64_t VDMA_DRV_CH_GetPollingTimeoutIntSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdPollingTimeoutIntSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint64_t VDMA_DRV_CH_GetBusErrIntSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdBusErrIntSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint64_t VDMA_DRV_CH_GetCmdCmplRawSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdCmdCmplRawSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint64_t VDMA_DRV_CH_GetAndClrCmdCmplRawSts(uint64_t ChannelBaseAddr)
{
	uint64_t u64CmdCmpStatus, u64CmdCmpStatusRet = 0;
	//DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	//DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	// WA for host clear VDMA complete status reg delay issue
	do
	{
		u64CmdCmpStatus = VDMA_HAL_CH_RdAndClrCmdCmplRawSts(ChannelBaseAddr);
		
		if(0 == u64CmdCmpStatus)
		{
			break;
		}
		u64CmdCmpStatusRet |= u64CmdCmpStatus;
	}while(1);

	return u64CmdCmpStatusRet;
}

uint64_t VDMA_DRV_CH_GetPollingTimeoutRawSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdPollingTimeoutRawSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

uint64_t VDMA_DRV_CH_GetBusErrRawSts(uint32_t Channel)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	return VDMA_HAL_CH_RdBusErrRawSts(gVdmaEnvCfg.VdmaBaseAddr, Channel);
}

void VDMA_DRV_CH_ClearCmdCmplSts(uint32_t Channel, uint64_t MaskVal)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_CH_ClearCmdCmplSts(gVdmaEnvCfg.VdmaBaseAddr, Channel, MaskVal);
}

void VDMA_DRV_CH_ClearPollingTimeoutSts(uint32_t Channel, uint64_t MaskVal)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_CH_ClearPollingTimeoutSts(gVdmaEnvCfg.VdmaBaseAddr, Channel, MaskVal);
}

void VDMA_DRV_CH_ClearBusErrSts(uint32_t Channel, uint64_t MaskVal)
{
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_CH_ClearBusErrSts(gVdmaEnvCfg.VdmaBaseAddr, Channel, MaskVal);
}

void VDMA_DRV_CH_InitToDefault(uint32_t Channel)
{
	VDMA_ChAxQosCfg_t VdmaChAxQosCfg;
	VDMA_ChPollingTimeoutCfg_t VdmaChPollingTimeoutCfg;
	VDMA_ChStreamIdCfg_t VdmaChStreamIdCfg;

	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VdmaChAxQosCfg.HostArQos = 3;
	VdmaChAxQosCfg.HostAwQos = 3;
	VdmaChAxQosCfg.NdfArQos = 3;
	VdmaChAxQosCfg.NdfAwQos = 3;
	VDMA_DRV_CH_ConfigAxQos(Channel, &VdmaChAxQosCfg);

	VdmaChPollingTimeoutCfg.PollingIntval = VDMA_DRV_DEF_POLLING_INTVAL;
	VdmaChPollingTimeoutCfg.PollingMaxSize = VDMA_DRV_DEF_POLLING_MAX_SIZE;
	VDMA_DRV_CH_ConfigPollingTimeout(Channel, &VdmaChPollingTimeoutCfg);

	VdmaChStreamIdCfg.DstDataWriteStreamId = VDMA_DRV_DEF_STREAM_ID;
	VdmaChStreamIdCfg.DstDescReadStreamId = VDMA_DRV_DEF_STREAM_ID;
	VdmaChStreamIdCfg.PhyAddrStreamId = VDMA_DRV_DEF_STREAM_ID;
	VdmaChStreamIdCfg.PollingRdDataStreamId = VDMA_DRV_DEF_STREAM_ID;
	VdmaChStreamIdCfg.SrcDataReadStreamId = VDMA_DRV_DEF_STREAM_ID;
	VdmaChStreamIdCfg.SrcDescReadStreamId = VDMA_DRV_DEF_STREAM_ID;
	VDMA_DRV_CH_ConfigStreamId(Channel, &VdmaChStreamIdCfg);
}

/* VDMA Command API */
void VDMA_DRV_GenerateVdmaDesc(VDMA_DESC_Config_t *VdmaDescCfg, VDMA_DESC_Data_t *VdmaDesc)
{
	DBG_ASSERT(VdmaDesc != NULL);
	DBG_ASSERT(VdmaDescCfg != NULL);
	DBG_ASSERT(TEST_ALIGNED(VdmaDescCfg->DataBlkPtr, VDMA_DRV_DESC_DATAPTR_ALIGN));
	DBG_ASSERT(VdmaDescCfg->LengthUnit <= VDMA_DRV_LENUNIT_1M);
	DBG_ASSERT(VdmaDescCfg->LinkEnd <= 1);
	DBG_ASSERT(VdmaDescCfg->DataPtrAddrMode <= VDMA_DRV_ADDRMODE_GPU_GPA);
	DBG_ASSERT(VdmaDescCfg->TransferLen > 0);

	// clear VdmaCmd
	memset(VdmaDesc, 0, sizeof(VDMA_DESC_Data_t));

	// always set NextPtr to 0 when generating VDMA Descriptor
	// linklist operation will be handled by other function later
	VdmaDesc->Dword0.NextDescPtr_47_4 = 0;
	VdmaDesc->Dword0.DataBlkPtr_20_5 = GET_BITS_U64(VdmaDescCfg->DataBlkPtr, 20, 5);
	VdmaDesc->Dword1.DataBlkPtr_47_21 = GET_BITS_U64(VdmaDescCfg->DataBlkPtr, 47, 21);
	VdmaDesc->Dword1.DataPtrAddrMode = VdmaDescCfg->DataPtrAddrMode;
	VdmaDesc->Dword1.LengthUnit = VdmaDescCfg->LengthUnit;
	VdmaDesc->Dword1.TransferLen = VDMA_DRV_LEN_ADJUST(VdmaDescCfg->TransferLen);
	VdmaDesc->Dword1.LinkEnd = VdmaDescCfg->LinkEnd;
}


uint64_t VDMA_DRV_LocalToNdfAddr(uint32_t LocalAddr)
{
	uint64_t NdfAddr = 0;

	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	// TODO: add C2D address translation
	if ((LocalAddr >= VDMA_DRV_VDMCU_DRAM_LOCAL_STA) && (LocalAddr < VDMA_DRV_VDMCU_DRAM_LOCAL_END)) {
		NdfAddr = (uint64_t)LocalAddr + gVdmaEnvCfg.VdmcuToNdfOffset;
	} else {
		DBG_ASSERT(0);	// TODO: For test
	}

	return NdfAddr;
}

uint32_t VDMA_DRV_NdfToLocalAddr(uint64_t NdfAddr)
{
	uint32_t LocalAddr = 0;

	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	// TODO: add C2D address translation
	if ((NdfAddr >= VDMA_DRV_VDMCU0_DRAM_NDF_STA) && (NdfAddr < VDMA_DRV_VDMCU0_DRAM_NDF_END)) {
		DBG_ASSERT(gVdmaEnvCfg.VdMcuId == 0);
		LocalAddr = (uint32_t)NdfAddr - gVdmaEnvCfg.VdmcuToNdfOffset;
	} else if ((NdfAddr >= VDMA_DRV_VDMCU1_DRAM_NDF_STA) && (NdfAddr < VDMA_DRV_VDMCU1_DRAM_NDF_END)) {
		DBG_ASSERT(gVdmaEnvCfg.VdMcuId == 1);
		LocalAddr = (uint32_t)NdfAddr - gVdmaEnvCfg.VdmcuToNdfOffset;
	} else {
		DBG_ASSERT(0);	// TODO: For test
	}

	return LocalAddr;
}

void VDMA_DRV_DescLink_AddToTail(VDMA_DESC_LinkHeadTail_t* pList, VDMA_DESC_Data_t* VdmaDesc)
{
//	uint64_t VdmaDescNdfAddr;

//	VdmaDescNdfAddr = VDMA_DRV_LocalToNdfAddr((uint32_t)VdmaDesc);

//	pr_info("AddToTail = 0x%llx 0x%llx\n",(uint64_t)VdmaDesc,(uint64_t)virt_to_phys(VdmaDesc));
	if (SL_IS_NULL(pList, pList->head)) {
		pList->head = VdmaDesc;
		pList->tail = VdmaDesc;
	} else {
		((VDMA_DESC_Data_t*) pList->tail)->Dword0.NextDescPtr_47_4 = GET_BITS_U64((uint64_t)virt_to_phys(VdmaDesc), 47, 4);
		pr_info("cur ptr = 0x%llx  0x%llx next ptr = 0x%llx\n",(uint64_t)pList->tail,(uint64_t)virt_to_phys(pList->tail),(uint64_t)((VDMA_DESC_Data_t*) pList->tail)->Dword0.NextDescPtr_47_4<<4);
		pList->tail = VdmaDesc;
	}
	pr_info("pList->head = 0x%llx 0x%llx\n",(uint64_t)pList->head,(uint64_t)virt_to_phys(pList->head));
	pr_info("pList->tail = 0x%llx 0x%llx\n",(uint64_t)pList->tail,(uint64_t)virt_to_phys(pList->tail));
}

RET_CODE_t VDMA_DRV_CH_WriteCmd(uint32_t VdmaCh, VDMA_CMD_Data_t* VdmaCmd)
{
	DBG_ASSERT(VdmaCmd != NULL);
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

//	DBG_WritePostcode(0xffff0001);
//	VDMA_HAL_WritePostcode(gVdmaEnvCfg.VdmaBaseAddr,0xffff0002);
#if 0
	__maybe_unused uint32_t RemainBufSize;
	RemainBufSize = VDMA_HAL_CH_RdRemainCmdBufSize(gVdmaEnvCfg.VdmaBaseAddr, VdmaCh);
	if (0 != RemainBufSize) {
		VDMA_HAL_CH_WrCmdBufBurst(gVdmaEnvCfg.VdmaBaseAddr, VdmaCh, CmdBufPtr, 3);
		Ret = RET_SUCCESS;
	} else {
		DBG_PRINTF("Channel %d CmdBuf is full\n", VdmaCh);
		Ret = RET_ERR_DMACH_FIFOFULL;
	}
#else
	//VDMA_HAL_CH_WrCmdBufBurst(gVdmaEnvCfg.VdmaBaseAddr, VdmaCh, (uint64_t*)&(VdmaCmd->CmdDword0), 3);
#endif
	return RET_SUCCESS;
}

void VDMA_DRV_CH_Monitor(uint32_t val)
{
	pr_info("VDMA Monitor 0x%x ...\n",val);
	VDMA_HAL_WritePostcode(gVdmaEnvCfg.VdmaBaseAddr,val);
}

void VDMA_DRV_CH_WriteRefBuf(uint32_t VdmaCh, uint32_t StartRefBufIdx, uint32_t *WrDataBuf, uint32_t Wrlen)
{
	DBG_ASSERT(RET_SUCCESS == VDMA_DRV_IsEnvInit());

	VDMA_HAL_CH_WrRefBufBurst(gVdmaEnvCfg.VdmaBaseAddr, VdmaCh, StartRefBufIdx, WrDataBuf, Wrlen);
}

#endif
