/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef VDMA_DRV_H_
#define VDMA_DRV_H_

/***************************** Include Files ********************************/
#include "platform.h"
#include "vdma_hal.h"

/************************** Constant Definitions ****************************/
#define VDMA_DRV_ENVINIT_FLAG			0xBEEF1234


/* Align Requirement */
// Copy Cmd Align
#define VDMA_DRV_CMD_COPY_ADDR_ALIGN			8

// Direct Write Cmd Align
#define VDMA_DRV_CMD_DWRITE_ADDR_ALIGN			32
#define VDMA_DRV_CMD_DWRITE_PATTERN_LOC_ALIGN	4

#define VDMA_DRV_CMD_REFBUF_OFFSET  			2

// Pollinge Cmd Align
#define VDMA_DRV_CMD_POLLING_ADDR_ALIGN			8
#define VDMA_DRV_CMD_POLLING_PATTERN_LOC_ALIGN	8

// Descriptor Align
#define VDMA_DRV_DESC_DATAPTR_ALIGN				32
#define VDMA_DRV_DESC_LINKPTR_ALIGN				16


/* Value Limit */
#define VDMA_DRV_CMD_DWRITE_PATTERN_LEN_LIMIT	256
#define VDMA_DRV_CMD_COMMAND_TAG_LIMIT			63

// Length Param in VDMA CMD/Register, should subtract 1
#define VDMA_DRV_LEN_ADJUST_VAL		1l
#define VDMA_DRV_LEN_ADJUST(len) 	((len)-1)
#define VDMA_DRV_ADDR_SHIFT			(4)


/* Address Translation */
#define VDMA_DRV_VDMCU_DRAM_LOCAL_STA	0x20000000		// VD-MCU DRAM Start Address (Local Address)
#define VDMA_DRV_VDMCU_DRAM_LOCAL_END	0x20008000		// VD-MCU DRAM End Address (Local Address)

#define VDMA_DRV_VDMCU0_DRAM_NDF_STA	0x90400000
#define VDMA_DRV_VDMCU0_DRAM_NDF_END	0x90408000

#define VDMA_DRV_VDMCU1_DRAM_NDF_STA	0x90500000
#define VDMA_DRV_VDMCU1_DRAM_NDF_END	0x90508000


/* Default Config */
#define VDMA_DRV_DEF_POLLING_INTVAL		0x10000
#define VDMA_DRV_DEF_POLLING_MAX_SIZE	0x5

#define VDMA_DRV_DEF_STREAM_ID			0x0


/**************************** Type Definitions ******************************/
/* VDMA Port Config */
typedef struct VDMA_PortCfg {
	uint8_t HostArLen;
	uint8_t HostAwLen;
	uint8_t NdfArLen;
	uint8_t NdfAwLen;
	uint16_t HostOutstandingSize;
	uint16_t NdfOutstandingSize;
} VDMA_PortCfg_t;


/* VDMA Priority Config */
typedef struct VDMA_PriorityCfg {
	uint8_t Ch0Priority;
	uint8_t Ch1Priority;
	uint8_t Ch2Priority;
	uint8_t Ch3Priority;
	uint32_t ChannelReqTimeout;
} VDMA_PriorityCfg_t;


/* VDMA Channel AxQos Config */
typedef struct VDMA_ChAxQosCfg {
	uint8_t HostArQos;
	uint8_t HostAwQos;
	uint8_t NdfArQos;
	uint8_t NdfAwQos;
} VDMA_ChAxQosCfg_t;


/* VDMA Channel Polling Timeout Config */
typedef struct VDMA_ChPollingTimeoutCfg {
	uint32_t PollingIntval;
	uint32_t PollingMaxSize;
} VDMA_ChPollingTimeoutCfg_t;


/* VDMA Channel StreamId Config */
typedef struct VDMA_ChStreamIdCfg {
	uint32_t SrcDescReadStreamId;
	uint32_t DstDescReadStreamId;
	uint32_t SrcDataReadStreamId;
	uint32_t PollingRdDataStreamId;
	uint32_t DstDataWriteStreamId;
	uint32_t PhyAddrStreamId;
} VDMA_ChStreamIdCfg_t;

/* VDMA Command */
typedef struct {
	uint64_t SrcAddr_47_3:45;	// Source Address[47:3]
	uint64_t SrcAddrMode:2;		// Source Address Mode:0-HOST_SVA, 1-HOST_SPA, 2-GPU_SVA, 3-GPU_GPA
	uint64_t IntrEn:1;			// Interrupt Enable: 1-set intsts when cmd cmpl
	uint64_t LengthUnit:2;		// Tranfer Length Unit:0-8B, 1-64B, 2-1KB, 3-1MB
	uint64_t CommandTag:6;		// Command Tag
	uint64_t OpCode:8;			// Operation Code
}VDMA_CMD_DWORD0_t;

typedef struct {
	uint64_t DstAddr_47_3:45;		// Destination Address[47:3]
	uint64_t DstAddrMode:2;			// Destination Address Mode:0-HOST_SVA, 1-HOST_SPA, 2-GPU_SVA, 3-GPU_GPA
	uint64_t Rsvd:1;
	uint64_t TransferLen:16;		// Transfer Length
}VDMA_CMD_DWORD1_t;

typedef struct {
	uint64_t PatternLen:8;			// Pattern Length (Unit Byte) (RealLength - 1)
	uint64_t PatternLoc_9_2:8;		// Pattern Location (4Byte Align)
	uint64_t PcieFuncNum:1;			// PCIE Physical Function Number: PF0 or PF1
	uint64_t Rsvd1:1;
	uint64_t PcieDontSnoop:1;		// PCIE TLP Snoop Attribute:1-no snoop
	uint64_t Rsvd2:45;
}VDMA_CMD_DWORD2_t;

typedef struct {
	uint64_t Dst_addr;
	uint32_t pattern_data;
	uint32_t pattern_len;
	uint32_t transfer_len;
}DMA_MEMSET_T1;


typedef union VDMA_CMD_Data
{
	DMA_MEMSET_T1 memset_info;
	struct
	{
		VDMA_CMD_DWORD0_t CmdDword0;
		VDMA_CMD_DWORD1_t CmdDword1;
		VDMA_CMD_DWORD2_t CmdDword2;
	};
} VDMA_CMD_Data_t;


/* VDMA LinkList Descriptor */
typedef struct {
	uint64_t NextDescPtr_47_4:44;		// Next Descriptor Address [47:4], 16Byte Align
	uint64_t Rsvd:4;
	uint64_t DataBlkPtr_20_5:16;		// Data Address 20:5, 32Byte Align
}VDMA_DESC_DWORD0_t;

typedef struct {
	uint64_t DataBlkPtr_47_21:27;	// Data Address 47:21, 32Byte Align
	uint64_t Rsvd1:5;
	uint64_t DataPtrAddrMode:2;		// Address Mode
	uint64_t LengthUnit:2;			// Tranfer Length Unit
	uint64_t TransferLen:16;			// Transfer Length
	uint64_t LinkEnd:1;				// 1'b1 means the end of linklist
	uint64_t Rsvd2:11;
}VDMA_DESC_DWORD1_t;

typedef struct VDMA_DESC_Data {
	VDMA_DESC_DWORD0_t Dword0;
	VDMA_DESC_DWORD1_t Dword1;
}__attribute__((aligned(16))) VDMA_DESC_Data_t;


/* VDMA LinkList Descriptor Config */
typedef struct {
	uint64_t DataBlkPtr;
	uint16_t TransferLen;
	uint8_t LengthUnit;
	uint8_t DataPtrAddrMode;
	uint8_t LinkEnd;
} VDMA_DESC_Config_t;

/* VDMA Cmd Config */
typedef struct VDMA_Copy_Param {
	uint64_t SrcAddr;
	uint64_t DstAddr;
	uint8_t SrcAddrMode;
	uint8_t DstAddrMode;
	uint16_t LengthUnit;		// if Linklist to Linklist, LengthUnit is not used
	uint16_t TransferLen;		// if Linklist to Linklist, TransferLen is not used
}VDMA_Copy_Param_t;

typedef struct VDMA_WriteDirect_Param {
	uint64_t DstAddr;
	uint32_t DstAddrMode;
	uint32_t patternData;
	uint32_t PatternLen;		// Pattern Length (Unit Byte)
	uint16_t PatternLoc;		// Pattern Location (4Byte Align)
	uint16_t TransferLen;		// How many pattern Block will be transfer
}VDMA_WriteDirect_Param_t;

typedef struct VDMA_Polling_Param {
	uint64_t SrcAddr;		// 8Byte Align
	uint32_t SrcAddrMode;
	uint32_t PatternLoc;		// Pattern Location (8Byte Align)
}VDMA_Polling_Param_t;

typedef struct VDMA_CMD_Config {
	union
	{
		struct
		{
			uint32_t OpCode;
			uint32_t IntrEn;
			uint16_t PcieFuncNum;
			uint16_t PcieDontSnoop;
		};
		uint32_t RawData[3];
	};

	uint32_t CommandTag;
	union
	{
		VDMA_Copy_Param_t CopyParam;
		VDMA_WriteDirect_Param_t WriteDirectParam;
		VDMA_Polling_Param_t PollingParam;
	};
} VDMA_CMD_Config_t;


/* VDMA DESC LinkHeadTail */
typedef struct VDMA_DESC_LinkHeadTail {
	void* head;			// Pointer to DescLinkList Head (use Local Address)
	void* tail;			// Pointer to DescLinkList Tail (use Local Address)
}VDMA_DESC_LinkHeadTail_t;

/* VDMA Environment Config */
typedef enum {
	VDMA_DRV_MST_VDMCU0 = 0,
	VDMA_DRV_MST_VDMCU1 = 1,
	VDMA_DRV_MST_CMCU = 2,
	VDMA_DRV_MST_ACPU = 3,
	VDMA_DRV_MST_HOST = 4
} VDMA_DRV_Master_t;

typedef struct VDMA_DRV_EnvConfig {
	uint32_t MagicNum;
	uint32_t VdMcuId;
	uint64_t VdmaBaseAddr;
	VDMA_DRV_Master_t Master;
	uint64_t VdmcuToNdfOffset;
}VDMA_DRV_EnvConfig_t;


/* VDMA_DRV_PollingPattern_t */
typedef struct VDMA_DRV_PollingPattern {
	uint64_t RefData;
	uint64_t RefMask;		// if Bit[0] == 1, mask RefData Bit[0], and so on
} VDMA_DRV_PollingPattern_t;


/* VDMA Opcode */
typedef enum {
	VDMA_DRV_OPCODE_WRITE_DIRECT = 0x1,			// Write Pattern Data to Destination
	VDMA_DRV_OPCODE_FENCE = 0x2,				// Fence Command
	VDMA_DRV_OPCODE_POLLING_LESS = 0x50,		// Polling until Destination Value Less than Pattern Data
	VDMA_DRV_OPCODE_POLLING_GREATER = 0x51,		// Polling until Destination Value Greater than Pattern Data
	VDMA_DRV_OPCODE_POLLING_EQUAL = 0x52,		// Polling until Destination Value Equal to Pattern Data
	VDMA_DRV_OPCODE_COPY_DSRC_DDST = 0x80,		// Direct Source to Direct Destination Copy
	VDMA_DRV_OPCODE_COPY_DSRC_LDST = 0x81,		// Direct Source to Linklist Destination Copy
	VDMA_DRV_OPCODE_COPY_LSRC_DDST = 0x82,		// Linklist Source to Direct Destination Copy
	VDMA_DRV_OPCODE_COPY_LSRC_LDST = 0x83		// Linklist Source to Linklist Destination Copy
} VDMA_DRV_OpCode_t;

typedef enum {
	VDMA_COPY_DSRC_DDST_CFG = 0,
	VDMA_COPY_DSRC_LDST_CFG = 1,
	VDMA_COPY_LSRC_DDST_CFG = 2,
	VDMA_COPY_LSRC_LDST_CFG = 3,
}VDMA_DRV_OP_CFG_t;

/* VDMA AddrMode */
typedef enum {
	VDMA_DRV_ADDRMODE_HOST_SVA = 0x0,
	VDMA_DRV_ADDRMODE_HOST_SPA = 0x1,
	VDMA_DRV_ADDRMODE_GPU_SVA = 0x2,
	VDMA_DRV_ADDRMODE_GPU_GPA = 0x3
} VDMA_DRV_AddrMode_t;

/* VDMA LengthUnit */
typedef enum {
	VDMA_DRV_LENUNIT_8B = 0x0,
	VDMA_DRV_LENUNIT_64B = 0x1,
	VDMA_DRV_LENUNIT_1K = 0x2,
	VDMA_DRV_LENUNIT_1M = 0x3
} VDMA_DRV_LenUnit_t;

#define VDMA_DRV_LINKLIST_SHIFT		(3)
#define VDMA_DRV_LENUNIT_8B_SHIFT		(3)
#define VDMA_DRV_LENUNIT_64B_SHIFT		(6)
#define VDMA_DRV_LENUNIT_1K_SHIFT		(10)
#define VDMA_DRV_XFERLEN_LIMIT			(0x1 << 16)

//data size less than 64K, the 8bytes len-unit be recommended.
//although the 8bytes unit can max support 512KB.
#define  VDMA_MAX_SIZE_SL_8B				(1 << 19)
#define  VDMA_MAX_SIZE_SL_1K				(1 << 25)
#define  VDMA_ADDR_BEST_ALIGN			(1024UL)

/***************** Macros (Inline Functions) Definitions ********************/
#define SL_IS_NULL(pSl, pEntry)         ((void*)(pSl) == (void*)(pEntry))

/************************** Function Prototypes *****************************/
void VDMA_DRV_InitEnv(uint64_t VdmaBaseAddr, VDMA_DRV_Master_t MasterType);

void VDMA_DRV_SoftReset(void);
uint32_t VDMA_DRV_COM_GetDmaStatus(void);
uint32_t VDMA_DRV_COM_GetMcuId(void);
void VDMA_DRV_COM_WriteScrReg(uint32_t WrVal);
void VDMA_DRV_COM_ConfigPort(VDMA_PortCfg_t* VdmaPortCfg);
void VDMA_DRV_COM_ConfigPriority(VDMA_PriorityCfg_t* VdmaPriorityCfg);
void VDMA_DRV_COM_InitToDefault(void);

void VDMA_DRV_CH_Enable(uint32_t Channel);
void VDMA_DRV_CH_Disable(uint32_t Channel);
void VDMA_DRV_CH_EnableIntr(uint64_t ChannelBaseAdd);
void VDMA_DRV_CH_DisableIntr(uint64_t ChannelBaseAdd);
uint32_t VDMA_DRV_CH_GetStatus(uint32_t Channel);
void VDMA_DRV_CH_ConfigAxQos(uint32_t Channel, VDMA_ChAxQosCfg_t* VdmaChAxQosCfg);
void VDMA_DRV_CH_ConfigPollingTimeout(uint32_t Channel, VDMA_ChPollingTimeoutCfg_t* VdmaChPollingTimeoutCfg);
void VDMA_DRV_CH_ConfigStreamId(uint32_t Channel, VDMA_ChStreamIdCfg_t* VdmaChStreamIdCfg);
uint32_t VDMA_DRV_CH_GetRemainBufSize(uint32_t Channel);
uint32_t VDMA_DRV_CH_GetCmdBufOverflowSts(uint32_t Channel);
void VDMA_DRV_CH_ClearCmdBufOverflowSts(uint32_t Channel);
uint64_t VDMA_DRV_CH_GetCmdCmplIntSts(uint32_t Channel);
uint64_t VDMA_DRV_CH_GetPollingTimeoutIntSts(uint32_t Channel);
uint64_t VDMA_DRV_CH_GetBusErrIntSts(uint32_t Channel);
uint64_t VDMA_DRV_CH_GetCmdCmplRawSts(uint32_t Channel);
uint64_t VDMA_DRV_CH_GetAndClrCmdCmplRawSts(uint64_t ChannelBaseAddr);
uint64_t VDMA_DRV_CH_GetPollingTimeoutRawSts(uint32_t Channel);
uint64_t VDMA_DRV_CH_GetBusErrRawSts(uint32_t Channel);
void VDMA_DRV_CH_ClearCmdCmplSts(uint32_t Channel, uint64_t MaskVal);
void VDMA_DRV_CH_ClearPollingTimeoutSts(uint32_t Channel, uint64_t MaskVal);
void VDMA_DRV_CH_ClearBusErrSts(uint32_t Channel, uint64_t MaskVal);
void VDMA_DRV_CH_InitToDefault(uint32_t Channel);

void VDMA_GenVdmaDesc(uint64_t DataAddr, uint32_t transferLen,
		uint8_t lenUnit, VDMA_DESC_Data_t *vdmaDesc, bool is_host);
void VDMA_DRV_GenerateVdmaDesc(VDMA_DESC_Config_t *VdmaDescCfg, VDMA_DESC_Data_t *VdmaDesc);
void VDMA_DRV_GenerateVdmaCmd(VDMA_CMD_Config_t *VdmaCfg, VDMA_CMD_Data_t *VdmaCmd);
void VDMA_ADDVdmaNextDesc(uint64_t NextDescAddr, VDMA_DESC_Data_t *vdmaDesc);
uint64_t VDMA_DRV_LocalToNdfAddr(uint32_t LocalAddr);
uint32_t VDMA_DRV_NdfToLocalAddr(uint64_t NdfAddr);
void VDMA_DRV_DescLink_Init(VDMA_DESC_LinkHeadTail_t* pList);
void VDMA_DRV_DescLink_AddToTail(VDMA_DESC_LinkHeadTail_t* pList, VDMA_DESC_Data_t* VdmaDesc);
void* VDMA_DRV_DescLink_PopFromHead(VDMA_DESC_LinkHeadTail_t* pList);

RET_CODE_t VDMA_DRV_CH_WriteCmd(uint32_t VdmaCh, VDMA_CMD_Data_t* VdmaCmd);
void VDMA_DRV_CH_WriteRefBuf(uint32_t VdmaCh, uint32_t StartRefBufIdx, uint32_t *WrDataBuf, uint32_t Wrlen);

void VDMA_DRV_DumpVdmaCmdCfg(VDMA_CMD_Config_t *VdmaCfg);
void VDMA_DRV_DumpVdmaCmdData(VDMA_CMD_Data_t *VdmaCmd);
void VDMA_DRV_DumpVdmaDescCfg(VDMA_DESC_Config_t *VdmaDescCfg);
void VDMA_DRV_DumpVdmaDescData(VDMA_DESC_Data_t *VdmaDesc);
void VDMA_DRV_DumpVdmaDescList(VDMA_DESC_Data_t *VdmaDescHead);

void VDMA_DRV_CH_Monitor(uint32_t val);


#endif /* VDMA_DRV_H_ */
