/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include "xdx_device.h"
#include "xdx_irq.h"
#include "xdx_dma_drv.h"
#include "vdma_irq.h"

static int xdx_dma_irq_set(struct xdx_device *xdev, struct xdx_irq_src *src,
			unsigned type, enum xdx_interrupt_state state)
{
	struct xdx_dma_dev *sdc = container_of(src, struct xdx_dma_dev, irq_src);

	if (sdc->ops->irq_set_en) {
		sdc->ops->irq_set_en(sdc, !!state);
	} else {
//		XDX_DMA_ERR(sdc->dev, "no irq handler!!!");
		return -EINVAL;
	}
//	xdx_dma_chans_irq_set(sdc, state);
	return 0;
}

static int xdx_dma_irq_process(struct xdx_device *xdev, struct xdx_irq_src *src,
			struct xdx_irq_entry *entry)
{
	struct xdx_dma_dev *sdc = container_of(src, struct xdx_dma_dev, irq_src);
	int ret = 0;

	if (sdc->ops->irq_handler) {
//		xdx_dma_chans_irq_set(sdc, 0);
		ret = sdc->ops->irq_handler(sdc);
//		xdx_dma_chans_irq_set(sdc, 1);
	} else {
		XDX_DMA_ERR(sdc->dev, "no irq handler!!!");
		return -EINVAL;
	}
	return ret;
}

const struct xdx_irq_src_funcs xdx_dma_isr_funcs = {
	.set = xdx_dma_irq_set,
	.process = xdx_dma_irq_process,
};

int xdx_dma_irq_init(struct xdx_dma_dev *sdc)
{
	int ret = 0;
	struct xdx_dma_platdata_t *pdata;

	if (sdc->irq_cid < 0) {
		XDX_DMA_ERR(sdc->dev, "Invalid irq client ID:%d.", sdc->irq_cid);
		return -EINVAL;
	}
	sdc->irq_src.id = sdc->irq_cid;
	sdc->irq_src.funcs = &xdx_dma_isr_funcs;
	pdata = sdc->pdata;
	ret = xdx_irq_register_src(pdata->xdx_dev, &sdc->irq_src);
	if (ret < 0) {
		XDX_DMA_ERR(sdc->dev, "Register irq Failed.%d.", ret);
		return ret;
	}
	return 0;
}

int xdx_dma_irq_deinit(  struct xdx_dma_dev *sdc)
{
	int ret = 0;
	struct xdx_dma_platdata_t *pdata;

	if (sdc->irq_cid < 0) {
		XDX_DMA_ERR(sdc->dev, "Invalid irq client ID:%d.", sdc->irq_cid);
		return -EINVAL;
	}
	pdata = sdc->pdata;
	ret = xdx_irq_unregister_src(pdata->xdx_dev, &sdc->irq_src);
	if (ret < 0) {
		XDX_DMA_ERR(sdc->dev, "Register irq Failed.%d.", ret);
		return ret;
	}
	return 0;
}

