/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef XDX_DMA_H
#define XDX_DMA_H
#include <linux/clk.h>
#include <linux/delay.h>
#include <linux/dmaengine.h>
#include <linux/dmapool.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/of_dma.h>
#include <linux/of_device.h>
#include <linux/platform_device.h>
#include <linux/reset.h>
#include <linux/slab.h>
#include <linux/types.h>
#include <linux/dma-mapping.h>
#include <linux/version.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5,10,0))
#include <linux/dma-map-ops.h>
#endif
#include <linux/time.h>
#include <linux/timer.h>
#include <img_types.h>
#include <linux/mempool.h>
#include "virt-dma.h"
#include <asm/cacheflush.h>
#include <linux/pci.h>
#include "./hal/vdma_hal.h"
#include "./hal/vdma_drv.h"
#include <linux/random.h>
#include <linux/smp.h>
#include <linux/delay.h>
#include "B0_HalVdma.h"

//#include "osfunc.h"
//#include "dma_common.h"
#include "pangu.h"
#define CMD_MAX 			 (64)
#define XDX_VDMA_ADDR_ALIGN  (64)

/*
 * Hardware channels / ports representation
 *
 * The hardware is used in several SoCs, with differing numbers
 * of channels and endpoints. This structure ties those numbers
 * to a certain compatible string.
 */
struct xdx_dma_config {
	u32 nr_max_channels;
	u32 nr_max_requests;
	u32 nr_max_vchans;
	/*
	 * In the datasheets/user manuals of newer Allwinner SoCs, a special
	 * bit (bit 2 at register 0x20) is present.
	 * It's named "DMA MCLK interface circuit auto gating bit" in the
	 * documents, and the footnote of this register says that this bit
	 * should be set up when initializing the DMA controller.
	 * Allwinner A23/A33 user manuals do not have this bit documented,
	 * however these SoCs really have and need this bit, as seen in the
	 * BSP kernel source code.
	 */
	void (*set_burst_length)(u32 *p_cfg, s8 src_burst, s8 dst_burst);
	u32 src_addr_widths;
	u32 dst_addr_widths;
	bool has_mbus_clk;
};

/*
 * Hardware representation of the LLI
 *
 * The hardware will be fed the physical address of this structure,
 * and read its content in order to start the transfer.
 */
struct xdx_dma_lli
{
	VDMA_DESC_Data_t	xdma_desc;
	/*
	 * This field is not used by the DMA controller, but will be
	 * used by the CPU to go through the list (mostly for dumping
	 * or freeing it).
	 */
	u32					len;
	//dma_addr_t		dev_addr;
	//dma_addr_t			src;
	//dma_addr_t			dst;
	struct xdx_dma_lli	*v_lli_next;
};

typedef enum _trans_type
{

	Linklist = 1,
	Direct,
}Trans_type;

typedef enum _copy_type
{
	COPY_1D = 0,
	COPY_2D,
	COPY_TDMA,
}COPY_TYPE;


typedef struct {
	uint64_t Start_addr;
	uint64_t Dst_addr;
	uint64_t width;
	uint64_t height;
	uint64_t Srcstride;
	uint64_t Dststride;
	uint64_t depth;
	uint8_t LenUnit;
}DMA_INTERLEAV_T;


struct xdx_desc
{
	struct xdx_virt_dma_desc	vd;
	dma_addr_t					p_lli;			//linklist physical address
	phys_addr_t					host_addr;		//host physical address
	phys_addr_t 				dev_addr;
	struct xdx_dma_lli			*v_lli;
	struct xdx_dma_lli			*v_lli_next;
	atomic_t					Cmd_cnt;
	u32							transfer_len;	//desc total data size
	u32							direct;
	u32							linklist_len;
	VDMA_B0_DescData_T 			*VdmaDesc_vir;
	dma_addr_t 					VdmaDesc_phy;
	u8							*dmabuf_handle;
	dma_addr_t 					dmabuf_phy;
	u32							dmabuf_len;
	COPY_TYPE                   CopyType;
	DMA_INTERLEAV_T             Dma_interleav_info;
	//VDMA_CMD_Data_t 			*VdmaCmd[CMD_MAX];
	//VDMA_DESC_LinkHeadTail_t 	VdmaDevDescList[3];
	//VDMA_DESC_LinkHeadTail_t 	VdmaHostDescList[3];
};

struct xdx_cmd_reference
{
	struct xdx_desc		*desc;
	struct xdx_vchan	*vchan;
};

struct xdx_pchan
{
	uint64_t			DmaCmdPoolMask;
	struct xdx_cmd_reference cmdRef[CMD_MAX];
	spinlock_t			lock;
	u32					idx;
	u16					free_cmd;
	u16					submit_cmd;
	void __iomem		*base;
	//struct xdx_vchan	*vchan;
	//struct xdx_desc		*desc;
	//struct xdx_desc		*done;
};

struct xdx_vchan
{
	struct xdx_virt_dma_chan	vc;
	//struct list_head			node;
	//struct dma_slave_config	cfg;
	//struct xdx_pchan			*phy;
	u32							tag;
	u8							port;
	u8							irq_type;
	bool						cyclic;
};

struct xdx_dma_dev
{
	struct dma_device		slave;
	void __iomem			*base;
	//int						irq;
	spinlock_t				lock;
	//struct reset_control	*rstc;
	struct tasklet_struct	task;
	atomic_t				tasklet_shutdown;
	//struct list_head		pending;
	struct dma_pool			*pool;
	//struct dma_pool			*xdxpool;
	struct xdx_pchan		*pchans;
	struct xdx_vchan		*vchans;
	struct kmem_cache 		*txpool;	//txd cache pool
	u32						pendingMask;//pending vc mask
	atomic_t				cur_pID; 	//current p_channel ID
	const struct xdx_dma_config *cfg;
	VDMA_CMD_Config_t		*cmd_cfg;
	//volatile u32			*hostmisc_msi_addr;
	u32						num_pchans;
	u32						num_vchans;
	u32						max_request;
	bool 					IommuEnable;
};

struct vdma_funcs {
	void (*destroy)(struct xdx_dma_dev *dma);
	void (*reg_write)(struct xdx_dma_dev *dma, u32 reg, u32 value);
	u32 (*reg_read)(struct xdx_dma_dev *dma, u32 reg);
	void (*enable_interrupt)(struct xdx_dma_dev *dma, bool enable);
	int (*hw_init)(struct xdx_dma_dev *dma);
	void (*interrupt_handle)(struct xdx_dma_dev *dma);
};



extern struct vdma_funcs vdma_funcs_instance;
extern const struct vdma_funcs tdma_funcs_instance;

#define PVR_ALIGN(_x, _n)	(((_x)+((_n)-1U)) & ~((_n)-1U))
#define PVR_FLOOR_ALIGN(_x, _n)	((_x) & ~((_n)-1U))

static inline struct xdx_dma_dev *to_xdx_dma_dev(struct dma_device *d)
{
	return container_of(d, struct xdx_dma_dev, slave);
}

static inline struct xdx_vchan *to_xdx_vchan(struct dma_chan *chan)
{
	return container_of(chan, struct xdx_vchan, vc.chan);
}

int set_irq(int irq);
int xdx_dma_module_init(void __iomem	*base);
int xdx_tdma_module_init(void __iomem	*base);

void xdx_dma_module_exit(void);
struct platform_device *Get_dma_dev(void);
struct dma_async_tx_descriptor *xdx_dma_prep_sg(
		struct dma_chan *chan, struct scatterlist *sgl,
		unsigned int sg_len, enum dma_transfer_direction dir,VDMA_DRV_OpCode_t opt);
struct dma_async_tx_descriptor *xdx_dma_memset(
		struct dma_chan *chan, dma_addr_t DstAddr, uint32_t pattern_data,
		size_t pattern_len,size_t transfer_len, unsigned long flags);

//VDMA_CMD_Data_t *xdx_dma_createlinklist(struct xdx_dma_dev *sdev,struct xdx_vchan *vchan,struct xdx_dma_lli *v_lli,uint32_t dir,VDMA_DRV_OpCode_t opt);
VDMA_CMD_Data_t *xdx_dma_createlinklist(struct xdx_dma_dev *sdev,struct xdx_vchan *vchan,struct xdx_desc *txd,uint32_t dir,VDMA_DRV_OpCode_t opt);
VDMA_CMD_Data_t * xdx_dma_DirectToDirect(uint32_t VdmaCh,uint64_t src,uint64_t dst,uint64_t transfer_len,int dir);
VDMA_CMD_Data_t *xdx_dma_createlinklist_Header(struct xdx_dma_dev *sdev,struct xdx_vchan *vchan,struct xdx_desc *txd,uint32_t dir,VDMA_DRV_OpCode_t opt);
VDMA_CMD_Data_t *xdx_dma_createlinklist_Body(struct xdx_dma_dev *sdev,struct xdx_vchan *vchan,struct xdx_desc *txd,uint32_t dir,VDMA_DRV_OpCode_t opt);
VDMA_CMD_Data_t *xdx_dma_createlinklist_Tail(struct xdx_dma_dev *sdev,struct xdx_vchan *vchan,struct xdx_desc *txd,uint32_t dir,VDMA_DRV_OpCode_t opt);

static inline uint32_t GetLeadOnefromU64(uint64_t u64Mask)
{
	return (63U - __builtin_clzll(u64Mask));
}

static inline uint32_t GetLeadOnefromU32(uint32_t u32Mask)
{
	return (31U - __builtin_clz(u32Mask));
}

uint64_t vdma_get_ch_cmpl(void);

struct dma_async_tx_descriptor *xdx_dma_prep_dma_2Dcopy(
		struct dma_chan *chan,
		IMG_UINT64* src_addr,
		IMG_UINT64  dst_addr,
		int width,
		int height,
		int src_stride,
		int dst_stride,
		enum dma_transfer_direction dir,
		unsigned long flags);

struct dma_async_tx_descriptor *xdx_Tdma_prep_memcpy(
		struct dma_chan *chan, DMA_INTERLEAV_T *dma_interleav_info, unsigned long flags);


#ifdef GENERATE_LINKLIST
IMG_UINT64 xdx_dma_prep_sg2(
		struct scatterlist *sgl,
		unsigned int sg_len);
void free_linklist(IMG_DEV_PHYADDR Linklist_Head);
IMG_UINT64 Get_Direct_addr(IMG_DEV_PHYADDR Linklist_Head);

IMG_EXPORT PVRSRV_ERROR insert_chan(uintptr_t k, uintptr_t v);
IMG_EXPORT PVRSRV_ERROR remove_chan(uintptr_t k);
#endif


#endif
