/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/device.h>
#include "xdx_device.h"
#include "xdx_vdma.h"
#include "vdma_irq.h"
#include "rgxdevice.h"
#include "xdx_dma_drv.h"
#include "pangu.h"

#if defined(SUPPORT_DMA_TRANSFER) || defined(SUPPORT_DMA_TRANSFER_B0)
static void xdx_dma_info_init(struct platform_device_info *dma_device_info,
			struct xdx_device *xdev, const char *dev_name, struct xdx_dma_platdata_t *pdata)
{
	memset(dma_device_info, 0, sizeof(struct platform_device_info));
	dma_device_info->parent = &xdev->pdev->dev;
	dma_device_info->name = dev_name;
	dma_device_info->id = -2;
	dma_device_info->dma_mask = DMA_BIT_MASK(48);
	pdata->xdx_dev = xdev;
	dma_device_info->data = pdata;
	dma_device_info->size_data = sizeof(struct xdx_dma_platdata_t);
}
#endif

#ifdef SUPPORT_DMA_TRANSFER
static struct platform_device *xdx_vdmaa0_dev_init(struct xdx_device *xdev, resource_size_t base)
{
	struct platform_device_info dma_device_info;
	struct platform_device *dma_dev = NULL;
	struct xdx_vdma *vdma = &xdev->vdma;
	struct xdx_dma_platdata_t pdata;
	PVRSRV_DEVICE_NODE *psDeviceNode;
	const struct dma_map_ops *ops;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	struct resource res[2] = { 0 };
#else
	struct resource res[2];
	memset(res, 0, sizeof(res));
#endif

	res[0].start = base + PANGU_REG_VDMA0_BASE;
	res[0].end = res[0].start + (3 * PANGU_REG_VDMA0_SIZE) - 1;
	res[0].name = "base";
	res[0].flags = IORESOURCE_MEM;

	res[1].start = XDX_IRQ_CLIENTID_VDMA;
	res[1].end = XDX_IRQ_CLIENTID_VDMA;
	res[1].name = "irq-cid";
	res[1].flags =  IORESOURCE_REG;

	xdx_dma_info_init(&dma_device_info, xdev, XDX_VDMA_A0_NAME, &pdata);
	dma_device_info.res = res;
	dma_device_info.num_res = ARRAY_SIZE(res);
	dma_dev = platform_device_register_full(&dma_device_info);
	if (IS_ERR_OR_NULL(dma_dev)) {
		XDX_DMA_ERR(&xdev->pdev->dev, "Register VDMA_a0 device faild.%ld", PTR_ERR(dma_dev));
		return dma_dev;
	}

	psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
	ops = get_dma_ops(dma_dev->dev.parent);
	if (!ops)
		((PVRSRV_RGXDEV_INFO *)(psDeviceNode->pvDevice))->dma_dev = psDeviceNode->psDevConfig->pvOSDevice;
	else
		((PVRSRV_RGXDEV_INFO *)(psDeviceNode->pvDevice))->dma_dev = dma_dev->dev.parent;

	vdma->dma_dev = &dma_dev->dev;
	XDX_DMA_INFO(&dma_dev->dev, "device registered.");
	return dma_dev;
}

static struct platform_device *xdx_vdmab0_dev_init(struct xdx_device *xdev, resource_size_t base)
{
	struct platform_device_info dma_device_info;
	struct platform_device *dma_dev = NULL;
	struct xdx_vdma *vdma = &xdev->vdma;
	struct xdx_dma_platdata_t pdata;
	PVRSRV_DEVICE_NODE *psDeviceNode;
	const struct dma_map_ops *ops;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	struct resource res[2] = {0};
#else
	struct resource res[2];
	memset(res, 0, sizeof(res));
#endif

	res[0].start = base + PANGU_REG_VDMA0_BASE;
	res[0].end = res[0].start  + (3 * PANGU_REG_VDMA0_SIZE) - 1;
	res[0].name = "base";
	res[0].flags = IORESOURCE_MEM;

	res[1].start = XDX_IRQ_CLIENTID_VDMA;
	res[1].end = XDX_IRQ_CLIENTID_VDMA;
	res[1].name = "irq-cid";
	res[1].flags =  IORESOURCE_REG;

	xdx_dma_info_init(&dma_device_info, xdev, XDX_VDMA_B0_NAME, &pdata);
	dma_device_info.res = res;
	dma_device_info.num_res = ARRAY_SIZE(res);

	dma_dev = platform_device_register_full(&dma_device_info);
	if (IS_ERR_OR_NULL(dma_dev)) {
		XDX_DMA_ERR(&xdev->pdev->dev, "Register VDMA_b0 device faild.%ld", PTR_ERR(dma_dev));
		return dma_dev;
	}
	psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
	ops = get_dma_ops(dma_dev->dev.parent);
	if (!ops)
		((PVRSRV_RGXDEV_INFO *)(psDeviceNode->pvDevice))->dma_dev = psDeviceNode->psDevConfig->pvOSDevice;
	else
		((PVRSRV_RGXDEV_INFO *)(psDeviceNode->pvDevice))->dma_dev = dma_dev->dev.parent;
	vdma->dma_dev = &dma_dev->dev;
	XDX_DMA_INFO(&dma_dev->dev, "device:%s registered.", dev_name(&dma_dev->dev));
	return dma_dev;
}
#endif

#ifdef SUPPORT_DMA_TRANSFER_B0
static struct platform_device *xdx_tdmab0_dev_init(struct xdx_device *xdev, resource_size_t base)
{
	struct platform_device_info dma_device_info;
	struct platform_device *dma_dev = NULL;
	struct xdx_tdma *tdma = &xdev->tdma;
	struct xdx_dma_platdata_t pdata;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	struct resource res[3] = { 0 };
#else
	struct resource res[3];
	memset(res, 0, sizeof(res));
#endif

	res[0].start = base + SYS_PANGU_REG_TDMA_BASE;
	res[0].end = res[0].start + (3 * SYS_PANGU_REG_TDMA_SIZE) - 1;
	res[0].name = "base";
	res[0].flags = IORESOURCE_MEM;

	res[1].start = base + PANGU_REG_HOSTMISC_BASE;
	res[1].end = res[1].start + (PANGU_REG_HOSTMISC_SIZE) - 1;
	res[1].name = "hostmisc-base";
	res[1].flags = IORESOURCE_MEM;

	res[2].start = XDX_IRQ_CLIENTID_TDMA;
	res[2].end = XDX_IRQ_CLIENTID_TDMA;
	res[2].name = "irq-cid";
	res[2].flags =  IORESOURCE_REG;

	xdx_dma_info_init(&dma_device_info, xdev, XDX_TDMA_B0_NAME, &pdata);
	dma_device_info.res = res;
	dma_device_info.num_res = ARRAY_SIZE(res);

	dma_dev = platform_device_register_full(&dma_device_info);
	if (IS_ERR_OR_NULL(dma_dev)) {
		XDX_DMA_ERR(&xdev->pdev->dev, "Register TDMA_b0 device faild.%ld", PTR_ERR(dma_dev));
		return dma_dev;
	}
	tdma->dma_dev = &dma_dev->dev;
	XDX_DMA_INFO(&dma_dev->dev, "device:%s registered.", dev_name(&dma_dev->dev));
	return dma_dev;
}
#endif

#if defined(SUPPORT_DMA_TRANSFER) || defined(SUPPORT_DMA_TRANSFER_B0)
static int xdx_dmaa0_dev_init(struct xdx_device *xdev)
{
	struct xdx_vdma *vdma;
	PVRSRV_DEVICE_NODE *psDeviceNode;
	const struct dma_map_ops *ops;
	struct platform_device *dma_dev = NULL;

	XDX_DMA_DBG(xdev->dev, "dma_a0 dev init start");
	vdma = &xdev->vdma;
	psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;

	dma_dev = xdx_vdmaa0_dev_init(xdev, xdev->rmmio_base);
	if (IS_ERR_OR_NULL(dma_dev))
		return PTR_ERR(dma_dev);

	ops = get_dma_ops(dma_dev->dev.parent);
	((PVRSRV_RGXDEV_INFO *)(psDeviceNode->pvDevice))->dma_dev = dma_dev->dev.parent;
//	vdma->sdc = platform_get_drvdata(dma_dev);
	vdma->dma_dev = &dma_dev->dev;
	XDX_DMA_DBG(xdev->dev, "dma_a0 dev init success");
	return 0;
}

static int xdx_dmab0_dev_init(struct xdx_device *xdev)
{
	struct xdx_vdma *vdma;
	PVRSRV_DEVICE_NODE *psDeviceNode;
	struct platform_device *dma_dev = NULL;

	XDX_DMA_DBG(xdev->dev, "dma_b0 dev init start");
	vdma = &xdev->vdma;
	psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;

	dma_dev = xdx_vdmab0_dev_init(xdev, xdev->rmmio_base);
	if (IS_ERR_OR_NULL(dma_dev))
		return PTR_ERR(dma_dev);
#ifdef SUPPORT_DMA_TRANSFER_B0
	dma_dev = xdx_tdmab0_dev_init(xdev, xdev->rmmio_base);
	if (IS_ERR_OR_NULL(dma_dev))
		return PTR_ERR(dma_dev);
#endif
	XDX_DMA_DBG(xdev->dev, "dma_b0 dev init success");
	return 0;
}

int xdx_dma_dev_init(struct xdx_device *xdev)
{
	int ret = -1;

	switch(xdev->asic_type) {
	case CHIP_PANGU_A0:
		XDX_DMA_INFO(xdev->dev, "xdxdma A0 dev init start");
		ret = xdx_dmaa0_dev_init(xdev);
		break;
	case CHIP_PANGU_B0:
		ret = xdx_dmab0_dev_init(xdev);
		XDX_DMA_INFO(xdev->dev, "xdxdma B0 dev init start");
		break;
	default:
		ret = -EINVAL;
		break;
	}
	return 0;
}

int xdx_dma_dev_finit(struct xdx_device *xdev)
{
	PVRSRV_DEVICE_NODE *psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
	PVRSRVDeInitialiseDMA(psDeviceNode);
#ifdef SUPPORT_DMA_TRANSFER_B0
	platform_device_unregister(to_platform_device(xdev->tdma.dma_dev));
#endif
	platform_device_unregister(to_platform_device(xdev->vdma.dma_dev));
	return 0;
}
#else
int xdx_dma_dev_init(struct xdx_device *xdev)
{
	return 0;
}
int xdx_dma_dev_finit(struct xdx_device *xdev)
{
	return 0;
}
#endif

/* start define TDMA ip block */
#ifdef SUPPORT_DMA_TRANSFER_B0
static int xdx_tdma_resume(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct platform_device *pdev = NULL;

	/* TDMA need to reinit while resuming from suspend */
	if (NULL != xdev->tdma.dma_dev)
		pdev = to_platform_device(xdev->tdma.dma_dev);

	if (pdev) {
		struct xdx_dma_dev *sdc = platform_get_drvdata(pdev);
		if (sdc->ops->hw_init)
			sdc->ops->hw_init(sdc);
	}

	return 0;
}

static int xdx_tdma_suspend(void *handle)
{
	return 0;
}

static int xdx_tdma_early_init(void *handle)
{
	return 0;
}
static int xdx_tdma_early_fini(void *handle)
{
	return 0;
}

static int xdx_tdma_sw_init(void *handle)
{
	return 0;
}

static int xdx_tdma_sw_fini(void *handle)
{
	return 0;
}

static int xdx_tdma_hw_init(void *handle)
{
	return 0;
}

static int xdx_tdma_hw_fini(void *handle)
{
	return 0;
}

const struct xdx_ip_block_version tdma_ip_block_version = {
	.name = "tdma",
	.type = XDX_IP_BLOCK_TYPE_TDMA,
	.major = 1,
	.minor = 0,
	.rev = 0,
	.funcs = {
		.suspend = xdx_tdma_suspend,
		.resume = xdx_tdma_resume,
		.early_init = xdx_tdma_early_init,
		.early_fini = xdx_tdma_early_fini,
		.sw_init = xdx_tdma_sw_init,
		.sw_fini = xdx_tdma_sw_fini,
		.hw_init = xdx_tdma_hw_init,
		.hw_fini = xdx_tdma_hw_fini,
	},
};
#endif
/* end define TDMA ip block */
