/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __DC8200_MMU_H__
#define __DC8200_MMU_H__

#include "xdx_gem.h"
#include "xdx_vmm.h"

#define DC_INFINITE ((u32)(~0U))

#define DC_ENTRY_TYPE(x) (x & 0xF0)
#define DC_SINGLE_PAGE_NODE_INITIALIZE (~((1U << 8) - 1))

#define DC_INVALID_PHYSICAL_ADDRESS ~0ULL
#define DC_INVALID_ADDRESS ~0U

#if defined(SUPPORT_XDXGPU_RPM)
/* 4k mode */
#define MMU_MTLB_SHIFT 22
#else
/* 1k mode */
#define MMU_MTLB_SHIFT 24
#endif

#define MMU_STLB_4K_SHIFT 12
#define MMU_STLB_16M_SHIFT 24

#define MMU_MTLB_BITS (32 - MMU_MTLB_SHIFT)
#define MMU_PAGE_4K_BITS MMU_STLB_4K_SHIFT
#define MMU_STLB_4K_BITS (32 - MMU_MTLB_BITS - MMU_PAGE_4K_BITS)

#define MMU_MTLB_ENTRY_NUM (1 << MMU_MTLB_BITS)
#define MMU_MTLB_SIZE (MMU_MTLB_ENTRY_NUM << 2)
#define MMU_STLB_4K_ENTRY_NUM (1 << MMU_STLB_4K_BITS)
#define MMU_STLB_4K_SIZE (MMU_STLB_4K_ENTRY_NUM << 2)
#define MMU_PAGE_4K_SIZE (1 << MMU_STLB_4K_SHIFT)

#define MMU_MTLB_MASK (~((1U << MMU_MTLB_SHIFT) - 1))
#define MMU_STLB_4K_MASK ((~0U << MMU_STLB_4K_SHIFT) ^ MMU_MTLB_MASK)
#define MMU_PAGE_4K_MASK (MMU_PAGE_4K_SIZE - 1)

#define MMU_STLB_16M_ENTRY_NUM 16
#define MMU_STLB_16M_SIZE 64
#define MMU_PAGE_16M_SIZE (1 << MMU_STLB_16M_SHIFT)

/* page offset definitions. */
#define MMU_OFFSET_4K_BITS (32 - MMU_MTLB_BITS - MMU_STLB_4K_BITS)
#define MMU_OFFSET_4K_MASK ((1U << MMU_OFFSET_4K_BITS) - 1)

#define MMU_MTLB_PRESENT 0x00000001
#define MMU_MTLB_EXCEPTION 0x00000002
#define MMU_MTLB_SECURITY 0x00000010
#define MMU_MTLB_4K_PAGE 0x00000000
#define MMU_MTLB_16M_PAGE 0x0000000c

#if defined(CONFIG_SW64)
#define OSDeviceMemSetIO(a,b,c)  (void) memset_io((a), (b), (c))
#else
#define OSDeviceMemSetIO(a,b,c)  (void) memset((a), (b), (c))
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 8, 0))
struct drm_hash_item {
	struct hlist_node head;
	unsigned long key;
};

struct drm_open_hash {
	struct hlist_head *table;
	u8 order;
};
#endif

struct xdx_dc;
struct xdx_bo;

enum dc8200_mmu_type {
	DC_MMU_USED = (0 << 4),
	DC_MMU_SINGLE = (1 << 4),
	DC_MMU_FREE = (2 << 4),
};

enum dc8200_mmu_mode {
	MMU_MODE_4K,
	MMU_MODE_1K,
};

struct dc8200_mmu_stlb {
	u32 *logical;
	void *physical;
	u32 size;
	u64 physBase;
	u32 pageCount;
};

struct dc8200_va_node {
	union xdx_address addr;
	struct drm_hash_item hash;
	atomic_t refcnt;
};

struct dc8200_mmu {
	struct xdx_device *xdev;

	struct drm_open_hash res_ht;

	u32 mtlb_entry_num;
	u64 mtlb_physical;
	u32 *mtlb_logical;
	struct xdx_bo *mtlb_bo; /* master TLB */

	struct xdx_bo *stlbs_bo; /* all slave TLBs */

	void *safe_page_logical;
	u64 safe_page_physical;
	struct xdx_bo *safe_page_bo;

	u32 dynamic_mapping_start;

	void *stlbs; /* pointer array to dc_mmu_stlb instance */

	u64 stlb_physicals[MMU_MTLB_ENTRY_NUM];

	u32 page_table_entries;
	u32 page_table_size;
	u32 heap_list;

	u32 *map_logical;
	bool free_nodes;

	struct mutex page_table_mutex;

	struct mutex ht_mutex;

	enum dc8200_mmu_mode mode;

	void *static_stlb;
};

extern const struct xdx_mmu_funcs dc8200_mmu_funcs;

#endif /* __DC8200_MMU_H__ */
