/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#pragma once

#include "SysTypes.h"


#define GDDR_INFO_REG_ADDR 0x20C005FC
#define GDDR_PHY_COUNT     4

typedef enum
{
        HDMI_DP = 0,
        HDMI_VGA,
        HDMI_VGA_DVI,
        HDMI_DP_HDMI_VGA,
} DISPLAY_TYPE_T;

typedef enum
{
    cGPUClockFreq_800M = 0,
    cGPUClockFreq_900M,
    cGPUClockFreq_1000M,
    cGPUClockFreq_1050M,
    cGPUClockFreq_1100M,
    cGPUClockFreq_1150M,
    cGPUClockFreq_1200M,
    cGPUClockFreq_1250M,
    cGPUClockFreq_1300M,
    cGPUClockFreq_1350M,
    cGPUClockFreq_1400M,
    cGPUClockFreq_1450M,
    cGPUClockFreq_1500M,
    cGPUClockFreq_700M,
    cGPUClockFreq_600M,
    cGPUClockFreq_500M,
    cGPUClockFreq_400M,
    cGPUClockFreq_300M,
    cGPUClockFreq_200M,
    cGPUClockFreq_100M,
    cGPUClockFreq_50M,
    cGPUClockFreq_Max,
} GPUClockFreq_T;

const static U32 xdx_gpu_clock_freq_list[cGPUClockFreq_Max] = {
     800000000,
     900000000,
    1000000000,
    1050000000,
    1100000000,
    1150000000,
    1200000000,
    1250000000,
    1300000000,
    1350000000,
    1400000000,
    1450000000,
    1500000000,
     700000000,
     600000000,
     500000000,
     400000000,
     300000000,
     200000000,
     100000000,
      50000000,
};

typedef enum
{
    cGDDRClockFreq_500M = 0,
    cGDDRClockFreq_750M,
    cGDDRClockFreq_875M,
    cGDDRClockFreq_950M,
    cGDDRClockFreq_1000M,
    cGDDRClockFreq_Max,
} GDDRClockFreq_T;

const static U32 xdx_gddr_clock_freq_list[cGDDRClockFreq_Max] = {
     500000000,
     750000000,
     875000000,
     950000000,
    1000000000,
};

typedef enum
{
    cGDDRClockSpeed_8Gbps = 0,
    cGDDRClockSpeed_12Gbps,
    cGDDRClockSpeed_14Gbps,
    cGDDRClockSpeed_15_2Gbps,
    cGDDRClockSpeed_16Gbps,
    cGDDRClockSpeed_Max,
} GDDRClockSpeed_T;

const static U64 xdx_gddr_clock_speed_list[cGDDRClockSpeed_Max] = {
     8000000000,
    12000000000,
    14000000000,
    15200000000,
    16000000000,
};

typedef union
{
    U32 all;
    struct
    {
        U32 gddrMask            : 4;
        U32 interleave          : 4;
        U32 gddrOneDieSize      : 4;
        U32 gddrTotalSizeGB     : 8;
        U32 gddrSpeed           : 4;
        U32 vendorID            : 4;
        U32 resv                : 4;
    };
}GDDRInfo_T;

typedef union
{
    U64 all;
    struct
    {
        U64 gpuMask             : 4;
        U64 gpuFreq             : 8;
        U64 gCore0Volt          : 12;
        U64 gCore1Volt          : 12;
        U64 resv                : 28;
    };
}GPUInfo_T;

typedef struct
{
    U32 crcWriteCount[GDDR_PHY_COUNT];
    U32 crcReadCount[GDDR_PHY_COUNT];
    U32 crcWriteRetryCount[GDDR_PHY_COUNT];
    U32 crcReadRetryCount[GDDR_PHY_COUNT];
}GDDRStat_T;

typedef union
{
    struct
    {
        union
        {
            struct
            {

                U32 fwInfoVersion;
                U32 fwVersion;
                U32 fwVersionString[16];
                U32 vbiosVersion;
                U32 vbiosVersionString[16];
                GDDRInfo_T  gddrInfo;
                GPUInfo_T   gpuInfo;
                U32 boardInfo;
                U32 supportPowerInfo;
                U32 supportFanInfo;
                U32 supportTempInfo;
                U32 dcChannelInfo;
                U32 chipID[2];
                U32 cardType;
                U32 snNumber[8];
                U32 waterMarker;
                U32 vgaDetectFlag;
            };
            U8 pad0[2048];
        };

        union
        {
            struct
            {
                U32 curVolt;
                U32 curCurr;
                U32 curPower;
                U32 curRPM;
                U32 minRPM;
                U32 maxRPM;
                S32 curTemp;
                S32 maxHistoryTemp;
                S32 tempTH1;
                S32 tempTH2;
                S32 tempProtect;
                U32 curDuty;
                U32 minDuty;
                U32 maxDuty;
                U32 gddrTemp;
                U32 fanControlMode;
                U32 lpmSync;
                U32 curTotalPower;
                GDDRStat_T gddrstate;
            };
            U8 pad1[2048];
        };
    };
}PublicFirmwareInfo_T;

#define gFirmwareInfo ((PublicFirmwareInfo_T *)SHARED_SRAM_FIRMWARE_INFO_BASE)
#define FIRMWARE_INFO_SIZE SHARED_SRAM_FIRMWARE_INFO_SIZE
