/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_FW_H__
#define __XDX_FW_H__

#include <linux/types.h>
#include "fwshare/FirmwareInfo.h"

struct xdx_device;

#define CMCU_VARS_OFFSET       0x6FF000   //7MB-4KB
#define CMCU_VARS_SIZE         0x1000     //4KB
#define SHARED_VARS_OFFSET     0x700000   //7MB
#define ACPU_TRACE_OFFSET      0x800000   //8MB
#define ACPU_TRACE_SIZE        0x400000   //4MB
#define CMCU_TRACE_OFFSET      0xC00000   //12MB
#define CMCU_TRACE_SIZE        0x200000   //2MB
#define VDMCU0_TRACE_OFFSET    0xE00000   //14MB
#define VDMCU0_TRACE_SIZE      0x100000   //1MB
#define VDMCU1_TRACE_OFFSET    0xF00000   //15MB
#define VDMCU1_TRACE_SIZE      0x100000   //1MB

#define PARAMETER_PAGE_OFFSET  0x58000   //256KB+96KB
#define PARAMETER_PAGE_SIZE    0x1000    //4KB

#define ACPU_VARS_OFFSET       0x59000   //256KB+96KB+4KB
#define ACPU_VARS_SIZE         0x5000    //20KB

/* cache fw static info in KMD */
struct xdx_fw_static_info {
	/* static info acquired from KMD itself */
	u32 default_gpu_freq;
	u32 gpu_primary_id;
	u32 gddr_count;

	/* some but not all static info acquired from FW */

	/* GDDR Info */
	u32 gddr_size;
	u64 gddr_speed;
	u32 gddr_temp;
	u32 gddr_vendor;
	u32 gddr_type;

	/* GPU Info */
	u8  gpu_freq;
	u16 gpu_core0_volt;
	u16 gpu_core1_volt;

	/* FW Info */
	u32 fw_info_ver;
	u32 fw_ver;
	char fw_ver_str[64];

	/* Vbios Info */
	u32 vbios_ver;
	char vbios_ver_str[64];

	/* Product Name, Vendor, Chip Id and Serial Number */
	u16 device_id;
	u32 vendor;
	u32 chip_id;
	char serial_number[32];
	u32 card_type;

	/* RPM Info */
	u32 cur_rpm;
	u32 min_rpm;
	u32 max_rpm;

	/* Temp Info */
	s32 gpu_temp;
	s32 max_temp_history;
	s32 temp_min;
	s32 temp_max;
	s32 temp_prot;

	/* Duty Info */
	u32 curr_duty;
	u32 min_duty;
	u32 max_duty;

	/* Fan Ctrl Mode */
	u32 fan_ctrl_mode;

	/* is support fan */
	u32 support_fan_info;

	/* is support power */
	u32 support_power_info;

	/* is support temp info */
	u32 support_temp_info;

	/* is vga info */
	u32 vgaDetectFlag;
};

s32 get_gddr_temp(struct xdx_device *xdev);
s32 get_gpu_temp(struct xdx_device *xdev);
s32 get_gpu_maxHistoryTemp(struct xdx_device *xdev);
u32 get_gpu_freq(struct xdx_device *xdev);
u32 get_fan_rpm(struct xdx_device *xdev);
u32 get_fan_pwm(struct xdx_device *xdev);
u32 get_gpu_volt(struct xdx_device *xdev);
u32 get_gpu_current(struct xdx_device *xdev);
u32 get_gpu_power(struct xdx_device *xdev);
GDDRStat_T get_gddr_crc(struct xdx_device *xdev);
u32 get_gddr_type(struct xdx_device *xdev);

int cache_fw_static_info(struct xdx_device *xdev);

int xdx_fwinfo_init(struct xdx_device *xdev);
#endif /* __XDX_FW_H__ */
