/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include "linux/device.h"
#include "xdx_smmu_drv.h"
#include "xdx_device.h"

static uint32_t xdx_smmu_id;

#define XDX_SMMU_HW_W32(_addr, _off, _val) \
	writel((uint32_t)(_val), (void __iomem *)(_addr) + (_off))

#define XDX_SMMU_HW_R32(_addr, _off) \
	((uint32_t)readl((void __iomem *)(_addr) + (_off)))

__maybe_unused static uint64_t smmu_rreg64(void *reg_base, uint32_t off)
{
	uint64_t data, data1;

	wmb(); /* drain writebuffer */
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR, off);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR, 0);
	data = XDX_SMMU_HW_R32(reg_base, PANGU_IO_IND_ACCESS_DATA_OUTBAR);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR, (off + 4));
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR, 0);
	data1 = XDX_SMMU_HW_R32(reg_base, PANGU_IO_IND_ACCESS_DATA_OUTBAR);
	mb(); /* prevent compiler reordering around actual operate */
	data = data + (data1 << 32);
	return data;
}

__maybe_unused static void smmu_wreg64(void  *reg_base, uint32_t off, uint64_t value)
{
	wmb();/* drain writebuffer */
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR, off);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR, 0);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_DATA_OUTBAR, value&0xFFFFFFFF);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR, (off + 4));
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR, 0);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_DATA_OUTBAR, value>>32);
	mb(); /* prevent compiler reordering around actual operate */
}

__maybe_unused static void smmu_wreg32(void  *reg_base, uint32_t off, uint32_t value)
{
	wmb(); /* drain writebuffer */
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR, off);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR, 0);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_DATA_OUTBAR, value);
	mb(); /* prevent compiler reordering around actual operate */
}

__maybe_unused static uint32_t smmu_rreg32(void *reg_base, uint32_t off)
{
	uint32_t data;

	wmb(); /* drain writebuffer */
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR, off);
	XDX_SMMU_HW_W32(reg_base, PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR, 0);
	data = XDX_SMMU_HW_R32(reg_base, PANGU_IO_IND_ACCESS_DATA_OUTBAR);
	mb(); /* prevent compiler reordering around actual operate */
	return data;
}

static inline struct device *xdx_smmu_to_dev(struct xdx_smmu_s *xsmmu)
{
	return xsmmu->xdx_dev->dev;
}

static void SMMU500_ConfigSMMUPageTable(
	struct xdx_smmu_tcu_s *smmu_tcu,
	uint32_t sid_index, uint64_t value)
{
	void *base = smmu_tcu->dev_base;
	uint64_t tcu_offset = smmu_tcu->tcu_offset;
	uint32_t smmu_cb = 0;

	smmu_cb = XDX_SMMU_CB(smmu_tcu->cb_offset, sid_index);
	smmu_wreg64(base, tcu_offset + smmu_cb + ARM_SMMU_CB_TTBR0, value);
	smmu_wreg32(base, tcu_offset + ARM_SMMU_GR0_TLBIALLNSNH, 0x1);
	smmu_wreg32(base, tcu_offset + ARM_SMMU_GR0_sTLBGSYNC, 0x1);
	do {
	} while (smmu_rreg32(base, tcu_offset + ARM_SMMU_GR0_sTLBGSTATUS) != 0x0);
}

static void smm500_sync_tlb(struct xdx_smmu_tcu_s *smmu_tcu)
{
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_TLBIALLNSNH, 0x1);
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSYNC, 0x1);

	do {

	} while (smmu_rreg32(smmu_tcu->dev_base,
				smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSTATUS) != 0x0);
}

static void SMMU500ConfigGlobalRegister(struct xdx_smmu_tcu_s *smmu_tcu)
{
	union SMMU_CR0_u rCR0;
	union SMMU_ACR_u rACR;

	rCR0.all = 0;
	rCR0.CLIENTPD = 0;
	rCR0.SMCFCFG = 1;
	rCR0.USFCFG = 1;
	rCR0.GCFGFRE = 1;
	rCR0.GFRE = 1;
	rCR0.GFIE = 1;
	rCR0.GCFGFIE = 1;
	rCR0.MemAttr = 0x1;
	rCR0.MTCFG = 0x1;
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sCR0, rCR0.all);

	rACR.all = 0;
	rACR.S2CRB_TLBEN = 1;
	rACR.MMUDISB_TLBEN = 1;
	rACR.SMTNMB_TLBEN = 1;
	rACR.CACHE_LOCK = 0;
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sACR, rACR.all);
}

static void SMMU500ConfigIntegrationRegister(struct xdx_smmu_tcu_s *smmu_tcu)
{
	uint32_t SMMU_TBUQOS;

	SMMU_TBUQOS = 0x77;
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_TBUQOS(0), SMMU_TBUQOS);
}

static void SMMU500ConfigContextRegisterBypass(
	struct xdx_smmu_tcu_s *smmu_tcu,
	struct xdx_smmu_tbu_s *smmu_tbu)
{
	uint32_t sid_index = smmu_tbu->sid;
	union SMMU_S2CR_u rS2CR;
	union SMMU_SMR_u rSMR;
	union SMMU_CB_SCTLR_u rSCTLR;
	union SMMU_CBAR_u rCBAR;
	uint64_t smmu_cb = 0;

	smmu_cb = XDX_SMMU_CB(smmu_tcu->cb_offset, sid_index);
	rS2CR.all = 0;
	rS2CR.translation.TYPE = S2CR_TYPE_BYPASS;
	rS2CR.translation.CBNDX = sid_index;
	rS2CR.translation.MemAttr = 0x1;
	rS2CR.translation.MTCFG = 0x1;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_S2CR(sid_index), rS2CR.all);

	rSMR.all = 0;
	rSMR.VALID = 1;
	rSMR.MASK = 0;
	rSMR.ID = 0;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_SMR(sid_index), rSMR.all);

	rCBAR.all = 0;
	rCBAR.stage1.MemAttr = 0x1;
	rCBAR.stage1.TYPE = CBAR_TYPE_STAGE1_STAGE2_BYPASS;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR1_CBAR(sid_index), rSMR.all);

	rSCTLR.all = 0;
	rSCTLR.M = 1;
	rSCTLR.CFRE = 1;

	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_SCTLR, rSCTLR.all);
}

static void SMMU500ConfigContextRegister(
		struct xdx_smmu_tcu_s *smmu_tcu,
		struct xdx_smmu_tbu_s *smmu_tbu)
{
	uint32_t sid_index = smmu_tbu->sid;
	union SMMU_S2CR_u rS2CR;
	union SMMU_SMR_u rSMR;
	union SMMU_CB_SCTLR_u rSCTLR;
	union SMMU_CBAR_u rCBAR;
	union SMMU_CB_MAIR0_u rMAIR0;
	union SMMU_CB_MAIR1_u rMAIR1;
	union SMMU_CB_TCR_u rTCR;
	union SMMU_CB_TCR2_u rTCR2;
	uint64_t smmu_cb = 0;

	smmu_cb = XDX_SMMU_CB(smmu_tcu->cb_offset, sid_index);
	rMAIR0.all = 0;
	rMAIR0.Attr0 = DEVICE_MEMORY_nGnRnE;
	rMAIR0.Attr1 = DEVICE_MEMORY_nGnRE;
	rMAIR0.Attr2 = NORMAL_MEMORY_Outer_Inner_Non_Cacheable;
	rMAIR0.Attr3 = NORMAL_MEMORY_Outer_Non_Cacheable_Inner_WriteBack_ReadWriteAllocate;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_PRRR_MAIR0, rMAIR0.all);

	rMAIR1.all = 0;
	rMAIR1.Attr4 = NORMAL_MEMORY_Outer_Inner_WriteBack_ReadWriteAllocate;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_NMRR_MAIR1, rMAIR1.all);

	rTCR.all = 0;
	rTCR.IRGN0 = SMMU_TCR_IRGN0_NONCACHE;
	rTCR.ORGN0 = SMMU_TCR_ORGN0_NONCACHE;
	rTCR.SH0 = SMMU_TCR_SH0_INNER_SHARE;
	rTCR.TG0 = SMMU_TCR_TG0_4K;
	rTCR.T0SZ = 24;
	rTCR.T1SZ = 24;
	rTCR.EPD1 = 1;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_TCR, rTCR.all);

	rTCR2.all = 0;
	rTCR2.PASize = SMMU_TCR2_PHYSSIZE_40_1TB;
	rTCR2.TBI0 = 1;
	rTCR2.SEP = 3;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_TCR2, rTCR2.all);
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR1_CBA2R(sid_index), CBA2R_VA64);
	rS2CR.all = 0;
	rS2CR.translation.TYPE = S2CR_TYPE_TRANSLATION;
	rS2CR.translation.PRIVCFG = 0x2;
	rS2CR.translation.CBNDX = sid_index;
	rS2CR.translation.MemAttr = 0x1;
	rS2CR.translation.MTCFG = 0x1;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_S2CR(sid_index), rS2CR.all);
	rSMR.all = 0;
	rSMR.VALID = 1;

	rSMR.MASK = 0;
	rSMR.ID = sid_index;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_SMR(sid_index), rSMR.all);

	//SET PC
	rCBAR.all = 0;
	rCBAR.stage1.MemAttr = 0x1;
	rCBAR.stage1.TYPE = CBAR_TYPE_STAGE1_STAGE2_BYPASS;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR1_CBAR(sid_index), rCBAR.all);

	rSCTLR.all = 0;
	rSCTLR.M = 1;
	rSCTLR.CFRE = 1;
	rSCTLR.CFIE = 1;
	smmu_wreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_SCTLR, rSCTLR.all);
}

static void SMMU500InvalidateTLB(struct xdx_smmu_tcu_s *smmu_tcu)
{
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_TLBIALLNSNH, 0x1);
	smmu_wreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSYNC, 0x1);
}

#if defined(DEBUG)
static void SMMU500_DumpGlobalRegs(struct xdx_smmu_tcu_s *smmu_tcu)
{
	SMMU_DBG(smmu_tcu->p_dev, "***Dump %s Global Register***\n", smmu_tcu->name);
	SMMU_DBG(smmu_tcu->p_dev, "===Dump XDMA Global Register===\n");
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_sCR0 = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sCR0));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_ID0 = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_ID0));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_ID1 = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_ID1));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_sACR = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sACR));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_TBUQOS(0) = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_TBUQOS(0)));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_TLBIALLNSNH = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_TLBIALLNSNH));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_sTLBGSYNC = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSYNC));
	SMMU_DBG(smmu_tcu->p_dev, "ARM_SMMU_GR0_sTLBGSTATUS = 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSTATUS));
	SMMU_DBG(smmu_tcu->p_dev, "SMMU_GFSR= 0x%x\n",
		smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GFSR));
}

static void SMMU500_DumpCbCfgRegs(struct xdx_smmu_tcu_s *smmu_tcu, uint32_t sid_index)
{
	uint32_t smmu_cb = 0;

	smmu_cb = XDX_SMMU_CB(smmu_tcu->cb_offset, sid_index);
	SMMU_DBG(smmu_tcu->p_dev, "***Dump VENC Context Bank Config Register***\n");
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_GR1_CBA2R(%d)[0x%x] = 0x%x\n",
			sid_index, ARM_SMMU_GR1_CBA2R(sid_index),
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR1_CBA2R(sid_index)));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_GR0_S2CR(%d)[0x%x] = 0x%x\n",
			sid_index, ARM_SMMU_GR0_S2CR(sid_index),
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_S2CR(sid_index)));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_GR0_SMR(%d)[0x%x] = 0x%x\n",
		sid_index, ARM_SMMU_GR0_SMR(sid_index),
		smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_SMR(sid_index)));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_CB[%d]_TCR[0x%x] = 0x%x\n",
			sid_index, smmu_cb + ARM_SMMU_CB_TCR,
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_TCR));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_CB[%d]_TCR2[0x%x] = 0x%x\n",
			sid_index, smmu_cb + ARM_SMMU_CB_TCR2,
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_TCR2));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_CB[%d]_PRRR_MAIR0[0x%x] = 0x%x\n",
			sid_index, smmu_cb + ARM_SMMU_CB_PRRR_MAIR0,
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_PRRR_MAIR0));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_CB[%d]_NMRR_MAIR1[0x%x] = 0x%x\n",
			sid_index, smmu_cb + ARM_SMMU_CB_NMRR_MAIR1,
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_NMRR_MAIR1));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_CB(%d)_TTBR0[0x%x] = 0x%llx\n",
			sid_index, smmu_cb + ARM_SMMU_CB_TTBR0,
			smmu_rreg64(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_TTBR0));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_GR1_CBAR(%d)[0x%x] = 0x%x\n",
			sid_index, ARM_SMMU_GR1_CBAR(sid_index),
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR1_CBAR(sid_index)));
	SMMU_DBG(smmu_tcu->p_dev, "DUMP ARM_SMMU_CB[%d]_SCTLR[0x%x] = 0x%x\n",
			sid_index, smmu_cb + ARM_SMMU_CB_SCTLR,
			smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + smmu_cb + ARM_SMMU_CB_SCTLR));
	SMMU_DBG(smmu_tcu->p_dev, "SMMU_GFSR= 0x%x\n",
			smmu_rreg32(smmu_tcu->dev_base, smmu_tcu->tcu_offset + ARM_SMMU_GFSR));
}
#endif

static int _xdx_smmu_tbu_setup(
		struct xdx_smmu_tcu_s *smmu_tcu, struct xdx_smmu_tbu_s *smmu_tbu)
{
	if (smmu_tbu->flags & XDXTBU_FLAG_BYPASS)
		SMMU500ConfigContextRegisterBypass(smmu_tcu, smmu_tbu);
	else
		SMMU500ConfigContextRegister(smmu_tcu, smmu_tbu);
	return 0;
}
static int xdx_smmu_tbu_init(
		struct xdx_smmu_tcu_s *smmu_tcu,
		struct xdx_smmu_tbu_s *smmu_tbu,
		const struct xdx_smmu_tbu_cfg_s *tbu_cfg)
{
	smmu_tbu->p_tcu =  smmu_tcu;
	smmu_tbu->sid = tbu_cfg->sid;
	smmu_tbu->flags = tbu_cfg->flags;
	_xdx_smmu_tbu_setup(smmu_tcu, smmu_tbu);
	return 0;
}

int xdx_smmu_tcu_init(struct xdx_smmu_s *xsmmu,
			const struct xdx_smmu_tcu_cfg_s *tcu_cfg)
{
	struct xdx_smmu_tcu_s *smmu_tcu;
	int i;
	int ret;

	if (tcu_cfg->tcu_idx >= XDXSMMUTCU_MAX) {
		dev_err(xdx_smmu_to_dev(xsmmu),
				"SMMU:Invalid TCU index:%d!!", tcu_cfg->tcu_idx);
		return -EINVAL;
	}
	if (xsmmu->tcus[tcu_cfg->tcu_idx] != NULL) {
		dev_err(xdx_smmu_to_dev(xsmmu),
				"SMMU:TCU has been inited.(%d)", tcu_cfg->tcu_idx);
		return -EEXIST;
	}
	smmu_tcu = devm_kcalloc(xdx_smmu_to_dev(xsmmu),
					1, sizeof(struct xdx_smmu_tcu_s), GFP_KERNEL);
	if (!smmu_tcu)
		return -ENOMEM;
	memset(smmu_tcu, 0, sizeof(struct xdx_smmu_tcu_s));
	SMMU_INFO(xdx_smmu_to_dev(xsmmu), "SMMU tcu_idx:%d tbu_cnt:%d.",
			tcu_cfg->tcu_idx, tcu_cfg->tbu_cnt);

	xsmmu->tcus[tcu_cfg->tcu_idx] = smmu_tcu;
	smmu_tcu->p_dev = xdx_smmu_to_dev(xsmmu);
	smmu_tcu->dev_base = xsmmu->xdx_dev->rmmio;
	smmu_tcu->tcu_offset = tcu_cfg->base_offset;
	smmu_tcu->cb_offset = tcu_cfg->cb_offset;
	smmu_tcu->flags = tcu_cfg->flags;
	SMMU_DBG(xdx_smmu_to_dev(xsmmu),
			"xsmmu->xdx_dev->rmmio: %pK.", xsmmu->xdx_dev->rmmio);
	SMMU500ConfigGlobalRegister(smmu_tcu);
	SMMU500ConfigIntegrationRegister(smmu_tcu);
	for (i = 0; i < tcu_cfg->tbu_cnt; i++) {
		SMMU_DBG(xdx_smmu_to_dev(xsmmu),
					"tbu init.sid %d.", tcu_cfg->tbu_cfgs[i].sid);
		if (tcu_cfg->tbu_cfgs[i].sid >= XDXSMMU_SID_MAX) {
			SMMU_ERR(xdx_smmu_to_dev(xsmmu),
				"StreamID too big. %d.", tcu_cfg->tbu_cfgs[i].sid);
			return -EINVAL;
		}
		ret = xdx_smmu_tbu_init(smmu_tcu,
				&smmu_tcu->tbu_tab[tcu_cfg->tbu_cfgs[i].sid],
				&tcu_cfg->tbu_cfgs[i]);
		if (ret < 0) {
			SMMU_ERR(xdx_smmu_to_dev(xsmmu),
					"xdx_smmu_tbu_init failed. %d.", ret);
			return ret;
		}
	}
	SMMU500InvalidateTLB(smmu_tcu);
	do {
		udelay(10);
	} while (smmu_rreg32(smmu_tcu->dev_base,
				smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSTATUS) != 0);

#if defined(DEBUG)
	SMMU500_DumpGlobalRegs(smmu_tcu);
	for (i = 0; i < tcu_cfg->tbu_cnt; i++)
		SMMU500_DumpCbCfgRegs(smmu_tcu, tcu_cfg->tbu_cfgs[i].sid);
#endif

	return 0;
}

int xdx_smmu_tcus_deinit(struct xdx_smmu_s *xsmmu)
{
	int i;

	for (i = 0; i < XDXSMMUTCU_MAX; i++) {
		if (xsmmu->tcus[i] == NULL)
			continue;
		xdx_smmu_tcu_release(xsmmu->tcus[i]);
		devm_kfree(xsmmu->xdx_dev->dev, xsmmu->tcus[i]);
		xsmmu->tcus[i] = NULL;
	}
	return 0;
}

static inline int _xdx_smmu_tcu_resume(struct xdx_smmu_tcu_s *smmu_tcu)
{
	int i;
	int ret;

	SMMU500ConfigGlobalRegister(smmu_tcu);
	SMMU500ConfigIntegrationRegister(smmu_tcu);
	for (i = 0; i < XDXSMMU_SID_MAX; i++) {
		if (smmu_tcu->tbu_tab[i].p_tcu == NULL)
			continue;
		ret = _xdx_smmu_tbu_setup(smmu_tcu, &smmu_tcu->tbu_tab[i]);
		if (ret < 0) {
			SMMU_INFO(smmu_tcu->p_dev,
				"tbu resume sid:%d failed.(%d)", smmu_tcu->tbu_tab[i].sid, ret);
			return ret;
		}
		SMMU_DBG(smmu_tcu->p_dev,
				"tbu resume.sid %d.", smmu_tcu->tbu_tab[i].sid);
	}
	SMMU500InvalidateTLB(smmu_tcu);
	do {
		udelay(10);
	} while (smmu_rreg32(smmu_tcu->dev_base,
			smmu_tcu->tcu_offset + ARM_SMMU_GR0_sTLBGSTATUS) != 0);
	SMMU_INFO(smmu_tcu->p_dev, "tcu:%d resumed.", smmu_tcu->tcu_id);
	return 0;
}

int xdx_smmu_suspend(struct xdx_smmu_s *xsmmu)
{
	xsmmu->power_stat = XDX_SMMU_PWR_STAT_SUSPEND;
	return 0;
}

int xdx_smmu_resume(struct xdx_smmu_s *xsmmu)
{
	int i;
	int ret;

	if (xsmmu->power_stat != XDX_SMMU_PWR_STAT_SUSPEND) {
		SMMU_DBG(xdx_smmu_to_dev(xsmmu),
				"smmu power stat (%d) not allowed to resume.", xsmmu->power_stat);
		return 0;
	}

	for (i = 0; i < XDXSMMUTCU_MAX; i++) {
		if (!xsmmu->tcus[i])
			continue;

		ret = _xdx_smmu_tcu_resume(xsmmu->tcus[i]);
		if (ret < 0)
			return ret;
	}

	xsmmu->power_stat = XDX_SMMU_PWR_STAT_READY;
	return 0;
}

static int xdx_smmu_tbu_release(
			struct xdx_smmu_tcu_s *smmu_tcu,
			struct xdx_smmu_tbu_s *smmu_tbu)
{

	return 0;
}

int xdx_smmu_tcu_release(struct xdx_smmu_tcu_s *smmu_tcu)
{
	int i;

	for (i = 0; i < XDXSMMU_SID_MAX; i++) {
		if (smmu_tcu->tbu_tab[i].p_tcu == NULL)
			continue;
		xdx_smmu_tbu_release(smmu_tcu, &smmu_tcu->tbu_tab[i]);
	}
	return 0;
}

int xdx_smmu_group_sync_tlb(struct smmu_group_s *group)
{
	int i;
	struct xdx_smmu_tbu_s *tbu;
	struct xdx_smmu_tcu_s *last_tcu = NULL;

	if (!group)
		return -EINVAL;
	for (i = 0; i < group->gsid_cnt; i++) {
		tbu = group->tbus[i];
		if (last_tcu != tbu->p_tcu) {
			smm500_sync_tlb(tbu->p_tcu);
			last_tcu  = tbu->p_tcu;
		}
	}
	return 0;
}

int xdx_smmu_group_config_pagetab(struct smmu_group_s *group, uint64_t val)
{
	int i;
	struct xdx_smmu_tbu_s *tbu;

	for (i = 0; i < group->gsid_cnt; i++) {
		tbu = group->tbus[i];
		SMMU500_ConfigSMMUPageTable(tbu->p_tcu, tbu->sid, val);
		SMMU_DBG(group->xsmmu->xdx_dev->dev, "Set TBU:%x:%x base:%llx",
								tbu->p_tcu->tcu_id, tbu->sid, val);
	}
	return 0;
}

static struct xdx_smmu_tbu_s *xdx_smmu_get_tbu_gsid(
		struct xdx_smmu_s *xsmmu, uint32_t gsid)
{
	int i;
	struct xdx_smmu_tcu_s *tcu;

	if (GSID_TCU_ID(gsid) > XDXSMMUTCU_MAX || !xsmmu->tcus[GSID_TCU_ID(gsid)])
		return NULL;
	tcu = xsmmu->tcus[GSID_TCU_ID(gsid)];
	for (i = 0; i < XDXSMMU_SID_MAX; i++) {
		if (tcu->tbu_tab[i].p_tcu == NULL)
			return NULL;
		if (tcu->tbu_tab[i].sid == GSID_TBU_ID(gsid))
			return &tcu->tbu_tab[i];
	}
	return NULL;
}

struct smmu_group_s *xdx_smmu_find_group_gsid(
		struct xdx_smmu_s *xsmmu, uint32_t gsid)
{
	struct smmu_group_s *grp;
	int i;

	list_for_each_entry(grp, &xsmmu->grp_head, list) {
		for (i = 0; i < grp->gsid_cnt; i++) {
			if (grp->tbus[i]->sid == GSID_TBU_ID(gsid) &&
					grp->tbus[i]->p_tcu->tcu_id == GSID_TCU_ID(gsid)) {
				return grp;
			}
		}
	}
	return NULL;
}

struct smmu_group_s *xdx_smmu_creat_group(
			struct xdx_smmu_s *xsmmu,
			uint32_t gsid_cnt, uint32_t *gsids, uint32_t flags
)
{
	struct smmu_group_s *grp;
	int i;
	struct xdx_smmu_tbu_s *tbu;
	int tbu_cnt = 0;

	if (!gsid_cnt)
		return NULL;
	for (i = 0; i < gsid_cnt; i++) {
		if (xdx_smmu_find_group_gsid(xsmmu, gsids[i])) {
			SMMU_ERR(xdx_smmu_to_dev(xsmmu), "gsids:%x is in group:%d.",
				gsids[i], gsids[i]);
			return NULL;
		}
	}
	grp = devm_kcalloc(xdx_smmu_to_dev(xsmmu), 1,
					sizeof(struct smmu_group_s), GFP_KERNEL);
	if (!grp) {
		SMMU_ERR(xdx_smmu_to_dev(xsmmu), "Alloc group failed.");
		return NULL;
	}
	grp->grp_id = xdx_smmu_id++;
	grp->flags = flags;
	grp->gsid_cnt = gsid_cnt;
	grp->xsmmu = xsmmu;
	for (i = 0; i < gsid_cnt; i++) {
		tbu = xdx_smmu_get_tbu_gsid(xsmmu, gsids[i]);
		if (!tbu) {
			SMMU_ERR(xdx_smmu_to_dev(xsmmu), "Not find gsid(%x).", gsids[i]);
			goto err_out;
		}
		grp->tbus[tbu_cnt++] = tbu;
	}
	list_add(&grp->list, &xsmmu->grp_head);
	SMMU_INFO(xdx_smmu_to_dev(xsmmu), "Creat SMMU group:%d success.", grp->grp_id);
	return grp;

err_out:
	xdx_smmu_id--;
	devm_kfree(xdx_smmu_to_dev(xsmmu), grp);
	return NULL;
}

int xdx_smmu_destroy_group(struct smmu_group_s *grp)
{
	struct xdx_smmu_s *xsmmu;

	if (!grp)
		return -EINVAL;
	xsmmu = grp->xsmmu;
	list_del(&grp->list);
	xdx_smmu_group_config_pagetab(grp, 0);
	SMMU_INFO(xdx_smmu_to_dev(xsmmu),
			"destroy SMMU group(%d) success.", grp->grp_id);
	devm_kfree(xdx_smmu_to_dev(xsmmu), grp);
	return 0;
}
