/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef _XDX_SMMU_DRV_H_
#define _XDX_SMMU_DRV_H_

#include "xdx_shared.h"
#include "xdx_smmu.h"
#include "xdx_device.h"

#define SMMU_INFO(_dev, _fmt, ...) \
	do { \
		if (_dev) \
			dev_info(_dev, "SMMU %s() %d:"_fmt, __func__, __LINE__, ##__VA_ARGS__); \
		else \
			pr_info("SMMU %s() %d:"_fmt, __func__, __LINE__, ##__VA_ARGS__); \
	} while (0)

#define SMMU_DBG(_dev, _fmt, ...) \
	do { \
		if (_dev) \
			dev_dbg(_dev, "SMMU %s() %d:"_fmt, __func__, __LINE__, ##__VA_ARGS__); \
		else \
			pr_debug("SMMU %s() %d:"_fmt, __func__, __LINE__, ##__VA_ARGS__); \
	} while (0)

#define SMMU_ERR(_dev, _fmt, ...) \
	do { \
		if (_dev) \
			dev_err(_dev, "SMMU %s() %d:"_fmt, __func__, __LINE__, ##__VA_ARGS__); \
		else \
			pr_err("SMMU %s() %d:"_fmt, __func__, __LINE__, ##__VA_ARGS__); \
	} while (0)

//#define __V volatile
#ifdef __V
#undef __V
#endif

#define __V

#define COMPILE_ASSERT(expr, message) \
	extern int compile_assert_failed[(expr) ? 1 : -1] __maybe_unused

/* SMMU Register */
#define DEVICE_MEMORY_nGnRnE	0x00   //Armv7 Strong ordered
#define DEVICE_MEMORY_nGnRE	0x04   //Armv7 Device
#define DEVICE_MEMORY_nGRE	0x08
#define DEVICE_MEMORY_GRE	0x0C
#define NORMAL_MEMORY_Outer_Inner_Non_Cacheable					0x44
#define NORMAL_MEMORY_Outer_Non_Cacheable_Inner_WriteBack_ReadWriteAllocate   0x4F
#define NORMAL_MEMORY_Outer_Inner_WriteBack_ReadWriteAllocate	0xFF

#define SMMU_TCR_IRGN0_NONCACHE	(0)
#define SMMU_TCR_IRGN0_WB_WA	(1)
#define SMMU_TCR_IRGN0_WT		(2)
#define SMMU_TCR_IRGN0_WB		(3)

#define SMMU_TCR_ORGN0_NONCACHE	(0)
#define SMMU_TCR_ORGN0_WB_WA	(1)
#define SMMU_TCR_ORGN0_WT		(2)
#define SMMU_TCR_ORGN0_WB		(3)

#define SMMU_TCR_SH0_NON_SHARE		(0ULL)
#define SMMU_TCR_SH0_OUTER_SHARE	(1ULL)
#define SMMU_TCR_SH0_INNER_SHARE	(2ULL)

#define SMMU_TCR_TG0_4K		(0ULL)
#define SMMU_TCR_TG0_16K	(2ULL)
#define SMMU_TCR_TG0_64K	(1ULL)

#define SMMU_TCR2_PHYSSIZE_32_4GB		(0ULL)
#define SMMU_TCR2_PHYSSIZE_36_64GB		(1ULL)
#define SMMU_TCR2_PHYSSIZE_40_1TB		(2ULL)
#define SMMU_TCR2_PHYSSIZE_42_4TB		(3ULL)
#define SMMU_TCR2_PHYSSIZE_44_16TB		(4ULL)
#define SMMU_TCR2_PHYSSIZE_48_256TB		(5ULL)

#define CBA2R_VA64	(1)

#define S2CR_TYPE_TRANSLATION		(0x0)
#define S2CR_TYPE_BYPASS			(0x1)
#define S2CR_TYPE_FAULT				(0x2)
#define S2CR_TYPE_RSVD				(0x3)

#define CBAR_TYPE_STAGE2_ONLY			(0x0)
#define CBAR_TYPE_STAGE1_STAGE2_BYPASS	(0x1)
#define CBAR_TYPE_STAGE1_STAGE2_FUALT	(0x2)
#define CBAR_TYPE_STAGE1_STAGE2			(0x3)

/* Table Entry */
// #define VA_BITS			40
// #define PA_BITS			40
#define TTL0_ENTRY_COUNT   2
#define TTL1_TABLE_COUNT   TTL0_ENTRY_COUNT
#define TTL1_ENTRY_COUNT   512
#define TTL2_TABLE_COUNT   2
#define TTL2_ENTRY_COUNT   512
#define TTL3_TABLE_COUNT   2
#define TTL3_ENTRY_COUNT   512

#define TT_VALID 1
#define TT_INVALID 0
#define TT_TYPE_TABLE 1
#define TT_TYPE_BLOCK 0
#define TT_TYPE_PAGE  1
#define NEXT_TT_ADDRESS_SHIFT 12
#define TTL1_OA_SHIFT 30
#define TTL2_OA_SHIFT 21
#define TTL3_OA_SHIFT 12
#define SIZE_1GB		(UINT64_C(1)<<TTL1_OA_SHIFT)
#define SIZE_2MB		(UINT64_C(1)<<TTL2_OA_SHIFT)
#define SIZE_4KB		(UINT64_C(1)<<TTL3_OA_SHIFT)
#define SIZE_4GB		(UINT64_C(1)<<32)
#define SIZE_256MB	(UINT64_C(1)<<28)
#define SIZE_512MB	(UINT64_C(1)<<29)
#define SIZE_64KB		(UINT64_C(1)<<16)

#define STRONG_ORDERED_INDEX 0
#define DEVICE_MEMORY_INDEX  1
#define NORMAL_MEMORY_Outer_Inner_Non_Cacheable_INDEX 2
#define NORMAL_MEMORY_Outer_Non_Cacheable_Inner_WriteBack_ReadWriteAllocate_INDEX 3
#define NORMAL_MEMORY_Outer_Inner_WriteBack_ReadWriteAllocate_INDEX 4

#define SH_NON_SHAREABLE 0
#define SH_OUTER_SHAREABLE 2
#define SH_INNER_SHAREABLE 3

#define TT_EXECUTE_NEVER 1
#define TT_EXECUTE	0
#define TT_ENTRY_ACCESSED	1
#define TT_ENTRY_NON_ACCESSED 0

#define SSRAM_TTL1_INDEX  0
#define GDDR_TTL1_INDEX_START   4
#define GDDR_TTL1_INDEX_END	36

/* Configuration registers */
#define ARM_SMMU_GR0_BASE	0x0
#define ARM_SMMU_GR0_sCR0		0x0
/* Auxiliary Configuration register */
#define ARM_SMMU_GR0_sACR		0x10
/* Identification registers */
#define ARM_SMMU_GR0_ID0		0x20
#define ARM_SMMU_GR0_ID1		0x24
#define ARM_SMMU_GR0_ID2		0x28
#define ARM_SMMU_GR0_ID3		0x2c
#define ARM_SMMU_GR0_ID4		0x30
#define ARM_SMMU_GR0_ID5		0x34
#define ARM_SMMU_GR0_ID6		0x38

#define ARM_SMMU_GR0_ID7		0x3c
#define ARM_SMMU_GR0_sGFAR	0x40
#define ARM_SMMU_GR0_sGFSR		0x48

#define ARM_SMMU_GR0_sGFSYNR0		0x50
#define ARM_SMMU_GR0_sGFSYNR1		0x54
#define ARM_SMMU_GR0_sGFSYNR2		0x58

/* Global TLB invalidation */
#define ARM_SMMU_GR0_TLBIVMID		0x64
#define ARM_SMMU_GR0_TLBIALLNSNH	0x68
#define ARM_SMMU_GR0_TLBIALLH		0x6c
#define ARM_SMMU_GR0_sTLBGSYNC		0x70

#define ARM_SMMU_GR0_sTLBGSTATUS	0x74

/* Stream mapping registers */
#define ARM_SMMU_GR0_SMR(n)		(0x800 + ((n) << 2))
#define ARM_SMMU_SMR_VALID		BIT(31)
#define ARM_SMMU_SMR_MASK		GENMASK(31, 16)
#define ARM_SMMU_SMR_ID			GENMASK(15, 0)

#define ARM_SMMU_GR0_S2CR(n)		(0xc00 + ((n) << 2))
#define ARM_SMMU_S2CR_PRIVCFG		GENMASK(25, 24)

#define ARM_SMMU_GR1_BASE		0x1000
#define ARM_SMMU_GR1_CBAR(n)		(ARM_SMMU_GR1_BASE + ((n) << 2))

#define ARM_SMMU_GR1_CBFRSYNRA(n)	(ARM_SMMU_GR1_BASE + 0x400 + ((n) << 2))

#define ARM_SMMU_GR1_CBA2R(n)		(ARM_SMMU_GR1_BASE + 0x800 + ((n) << 2))

#define ARM_SMMU_INTEGRATION		0x2000
#define ARM_SMMU_TBUQOS(n)			(ARM_SMMU_INTEGRATION + 0x100 + ((n) << 2))

#define ARM_XDMA_SMMU_A0_CB_BASE	(0x20000)
#define ARM_VDMA_SMMU_A0_CB_BASE	(0x40000)

#define ARM_XDMA_SMMU_B0_CB_BASE	(0x8000)
#define ARM_VDMA_SMMU_B0_CB_BASE	(0x20000)
#define ARM_TDMA_SMMU_B0_CB_BASE	(0x8000)
#define ARM_VDEC_SMMU_B0_CB_BASE	(0x8000)
#define ARM_VENC_SMMU_B0_CB_BASE	(0x8000)

#define XDX_SMMU_CB(_cb_base, _n)	(_cb_base + (_n << PAGE_SHIFT))
#define ARM_SMMU_CB_SCTLR		0x0
#define ARM_SMMU_CB_ACTLR		0x4
#define ARM_SMMU_CB_RESUME		0x8
#define ARM_SMMU_CB_TCR2		0x10
#define ARM_SMMU_TCR2_SEP_UPSTREAM	0x7
#define ARM_SMMU_CB_TTBR0		0x20
#define ARM_SMMU_CB_TTBR1		0x28
#define ARM_SMMU_CB_TCR			0x30
#define ARM_SMMU_CB_CONTEXTIDR		0x34
#define ARM_SMMU_CB_PRRR_MAIR0		0x38
#define ARM_SMMU_CB_NMRR_MAIR1		0x3c
#define ARM_SMMU_GFSR			0x48
#define ARM_SMMU_CB_PAR			0x50
#define ARM_SMMU_CB_FSR			0x58
#define ARM_SMMU_CB_FAR			0x60
#define ARM_SMMU_CB_FSYNR0		0x68
#define ARM_SMMU_CB_FSYNR1		0x6C

#define ARM_SMMU_CB_S1_TLBIVA		0x600
#define ARM_SMMU_CB_S1_TLBIASID		0x610
#define ARM_SMMU_CB_S1_TLBIVAL		0x620
#define ARM_SMMU_CB_S2_TLBIIPAS2	0x630
#define ARM_SMMU_CB_S2_TLBIIPAS2L	0x638
#define ARM_SMMU_CB_TLBSYNC		0x7f0
#define ARM_SMMU_CB_TLBSTATUS		0x7f4
#define ARM_SMMU_CB_ATS1PR		0x800

#define ARM_SMMU_CB_ATSR		0x8f0
#define ARM_SMMU_sTLBGSTATUS_GSACTIVE	BIT(0)
struct TTTableDescriptor_s {
	uint64_t	valid : 1;
	uint64_t	type : 1;
	uint64_t	ignored2_11 : 10;
	uint64_t	nextTblAddr : 36;
	uint64_t	rev48_51 : 4;
	uint64_t	ignored52_58 : 7;
	uint64_t	PXNTable : 1;
	uint64_t	XNTable : 1;
	uint64_t	APTable : 2;
	uint64_t	NSTable : 1;
};
COMPILE_ASSERT(sizeof(struct TTTableDescriptor_s) == 8,
			"Size of structure TTTableDescriptor_T exceeds maximum expected size.");

struct TTL1BlockDescriptor_s {
	uint64_t	valid : 1;
	uint64_t	type : 1;
	uint64_t	AttrIndx : 3;
	uint64_t	NS : 1;
	uint64_t	AP : 2;
	uint64_t	SH : 2;
	uint64_t	AF : 1;
	uint64_t	nG : 1;
	uint64_t	rev12_15 : 4;
	uint64_t	nT : 1;
	uint64_t	rev17_29 : 13;
	uint64_t	OA30_47 : 18;
	uint64_t	rev48_49 : 2;
	uint64_t	GP : 1;
	uint64_t	DBM : 1;
	uint64_t	Contiguous : 1;
	uint64_t	PXN : 1;
	uint64_t	XN : 1;
	uint64_t	ignored55_58 : 4;
	uint64_t	PBHA : 4;
	uint64_t	ignored63 : 1;
};
COMPILE_ASSERT(sizeof(struct TTL1BlockDescriptor_s) == 8,
			"Size of structure struct TTL1BlockDescriptor_s exceeds maximum expected size.");

struct TTL2BlockDescriptor_s {
	uint64_t	valid : 1;
	uint64_t	type : 1;
	uint64_t	AttrIndx : 3;
	uint64_t	NS : 1;
	uint64_t	AP : 2;
	uint64_t	SH : 2;
	uint64_t	AF : 1;
	uint64_t	nG : 1;
	uint64_t	rev12_15 : 4;
	uint64_t	nT : 1;
	uint64_t	rev17_20 : 4;
	uint64_t	OA21_47 : 27;
	uint64_t	rev48_49 : 2;
	uint64_t	GP : 1;
	uint64_t	DBM : 1;
	uint64_t	Contiguous : 1;
	uint64_t	PXN : 1;
	uint64_t	XN : 1;
	uint64_t	ignored55_58 : 4;
	uint64_t	PBHA : 4;
	uint64_t	ignored63 : 1;
};
COMPILE_ASSERT(sizeof(struct TTL2BlockDescriptor_s) == 8,
			"Size of structure struct TTL2BlockDescriptor_s exceeds maximum expected size.");

struct TTL3PageDescriptor_s {
	uint64_t	valid : 1;
	uint64_t	type : 1;
	uint64_t	AttrIndx : 3;
	uint64_t	NS : 1;
	uint64_t	AP : 2;
	uint64_t	SH : 2;
	uint64_t	AF : 1;
	uint64_t	nG : 1;
	uint64_t	OA12_47 : 36;
	uint64_t	rev48_50 : 3;
	uint64_t	DBM : 1;
	uint64_t	Contiguous : 1;
	uint64_t	PXN : 1;
	uint64_t	XN : 1;
	uint64_t	ignored55_58 : 4;
	uint64_t	PBHA : 4;
	uint64_t	ignored63 : 1;
};
COMPILE_ASSERT(sizeof(struct TTL3PageDescriptor_s) == 8,
			"Size of structure TTL3BlockDescriptor_T exceeds maximum expected size.");


union TTEntry_u {
	uint64_t	all;
	struct TTTableDescriptor_s		tbl;
	struct TTL1BlockDescriptor_s	pd1Blk;
	struct TTL2BlockDescriptor_s	pd2Blk;
	struct TTL3PageDescriptor_s	ptPage;
};

union SMMU_CR0_u {
	__V uint32_t all;
	struct {
		__V uint32_t CLIENTPD :1;
		__V uint32_t GFRE :1;
		__V uint32_t GFIE :1;
		__V uint32_t EXIDENABLE :1;
		__V uint32_t GCFGFRE :1;
		__V uint32_t GCFGFIE :1;
		__V uint32_t TRANSIENTCFG :2;
		__V uint32_t STALLD :1;
		__V uint32_t GSE :1;
		__V uint32_t USFCFG :1;
		__V uint32_t VMIDPNE :1;
		__V uint32_t PTM :1;
		__V uint32_t FB :1;
		__V uint32_t BSU :2;
		__V uint32_t MemAttr :4;
		__V uint32_t MTCFG :1;
		__V uint32_t SMCFCFG :1;
		__V uint32_t SHCFG :2;
		__V uint32_t RACFG :2;
		__V uint32_t WACFG :2;
		__V uint32_t rev28_29 :2;
		__V uint32_t HYPMODE :1;
		__V uint32_t VMID16EN :1;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CR0_u) == 4, "size fault");

union SMMU_ACR_u {
	__V uint32_t all;
	struct {
		__V uint32_t rev0_1 :2;
		__V uint32_t S1WC2EN :1;
		__V uint32_t S2WC2EN :1;
		__V uint32_t IPA2PA_CEN :1;
		__V uint32_t rev5_7 :3;
		__V uint32_t SMTNMB_TLBEN :1;
		__V uint32_t MMUDISB_TLBEN :1;
		__V uint32_t S2CRB_TLBEN :1;
		__V uint32_t rev11_23 :13;
		__V uint32_t DP4K_TCUDISB :1;
		__V uint32_t DP4K_TBUDISB :1;
		__V uint32_t CACHE_LOCK :1;
		__V uint32_t rev27_31 :5;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_ACR_u) == 4, "size fault");

union SMMU_S2CR_u {
	__V uint32_t all;
	struct {
		__V uint32_t CBNDX :8;
		__V uint32_t SHCFG :2;
		__V uint32_t EXIDVALID :1;
		__V uint32_t MTCFG :1;
		__V uint32_t MemAttr :4;
		__V uint32_t TYPE :2;
		__V uint32_t NSCFG :2;
		__V uint32_t RACFG :2;
		__V uint32_t WACFG :2;
		__V uint32_t PRIVCFG :2;
		__V uint32_t INSTCFG :2;
		__V uint32_t TRANSIENTCFG :2;
		__V uint32_t rev30_31 :2;
	} translation;
	struct {
		__V uint32_t VMID :8;
		__V uint32_t SHCFG :2;
		__V uint32_t EXIDVALID :1;
		__V uint32_t MTCFG :1;
		__V uint32_t MemAttr :4;
		__V uint32_t TYPE :2;
		__V uint32_t NSCFG :2;
		__V uint32_t RACFG :2;
		__V uint32_t WACFG :2;
		__V uint32_t BSU :2;
		__V uint32_t FB :1;
		__V uint32_t rev27 :1;
		__V uint32_t TRANSIENTCFG :2;
		__V uint32_t rev30_31 :2;
	} bypass;
};
COMPILE_ASSERT(sizeof(union SMMU_S2CR_u) == 4, "size fault");

union SMMU_SMR_u {
	__V uint32_t all;
	struct {
		__V uint32_t ID :15;
		__V uint32_t rev15 :1;
		__V uint32_t MASK :15;
		__V uint32_t VALID :1;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_SMR_u) == 4, "size fault");

struct SMMUGlobal0_s {
	__V union SMMU_CR0_u SMMU_CR0;		//0x000
	__V uint32_t SMMU_SCR1;				//0x004
	__V uint32_t SMMU_CR2;				//0x008
	__V uint32_t rev0x00C;				//0x00C
	__V union SMMU_ACR_u SMMU_ACR;		//0x010
	__V uint32_t rev0x014_0x01C[3];		//0x014-0x01C
	__V uint32_t SMMU_IDR0;				//0x020
	__V uint32_t SMMU_IDR1;				//0x024
	__V uint32_t SMMU_IDR2;				//0x028
	__V uint32_t SMMU_IDR3;				//0x02C
	__V uint32_t SMMU_IDR4;				//0x030
	__V uint32_t SMMU_IDR5;				//0x034
	__V uint32_t SMMU_IDR6;				//0x038
	__V uint32_t SMMU_IDR7;				//0x03C
	__V uint64_t SMMU_GFAR;				//0x040
	__V uint32_t SMMU_GFSR;				//0x048
	__V uint32_t SMMU_GFSRRESTORE;		//0x04C
	__V uint32_t SMMU_GFSYNR0;			//0x050
	__V uint32_t SMMU_GFSYNR1;			//0x054
	__V uint32_t SMMU_GFSYNR2;			//0x058
	__V uint32_t rev0x05C;				//0x05C
	__V uint32_t SMMU_STLBIALL;			//0x060
	__V uint32_t SMMU_TLBIVMID;			//0x064
	__V uint32_t SMMU_TLBIALLNSNH;		//0x068
	__V uint32_t SMMU_TLBIALLH;			//0x06C
	__V uint32_t SMMU_TLBGSYNC;			//0x070
	__V uint32_t SMMU_TLBGSTATUS;		//0x074
	__V uint32_t SMMU_TLBIVAH;			//0x078
	__V uint32_t rev0x07C;				//0x07C
	__V uint32_t impl0x080[8];			//0x080-0x09C
	__V uint64_t SMMU_STLBIVALM;		//0x0A0
	__V uint64_t SMMU_STLBIVAM;			//0x0A8
	__V uint64_t SMMU_TLBIVALH64;		//0x0B0
	__V uint32_t SMMU_TLBIVMIDS1;		//0x0B8
	__V uint32_t SMMU_STLBIALLM;		//0x0BC
	__V uint64_t SMMU_TLBIVAH64;		//0x0C0
	__V uint32_t rev0x0C8_0x0FC[14];	//0x0C8-0x0FC
	__V uint64_t SMMU_GATS1UR;			//0x100
	__V uint64_t SMMU_GATS1UW;			//0x108
	__V uint64_t SMMU_GATS1PR;			//0x110
	__V uint64_t SMMU_GATS1PW;			//0x118
	__V uint64_t SMMU_GATS12UR;			//0x120
	__V uint64_t SMMU_GATS12UW;			//0x128
	__V uint64_t SMMU_GATS12PR;			//0x130
	__V uint64_t SMMU_GATS12PW;			//0x138
	__V uint32_t rev0x140_0x17C[16];	//0x140-0x17C
	__V uint64_t SMMU_GPAR;				//0x180
	__V uint32_t SMMU_GATSR;			//0x188
	__V uint32_t rev0x18C_0x3FC[157];	//0x18C-0x3FC
	__V uint32_t rev0x400_0x7FC[256];	//0x400-0x7FC
	__V union SMMU_SMR_u SMMU_SMR[128];	//0x800-0x9FC
	__V uint32_t rev0xA00_0xBFC[128];	//0xA00-0xBFC
	__V union SMMU_S2CR_u SMMU_S2CR[128];   //0xC00-0xDFC
	__V uint32_t rev0xE00_0xFCC[116];	//0xE00-0xFCC
	__V uint32_t impl0xFD0_0xFFC[12];	//0xFD0-0xFFC
};
COMPILE_ASSERT(sizeof(struct SMMUGlobal0_s) == 4096, "size fault");

union SMMU_CBAR_u {
	__V uint32_t all;
	struct {
		__V uint32_t VMID :8;
		__V uint32_t BPSHCFG :2;
		__V uint32_t HYPC :1;
		__V uint32_t FB :1;
		__V uint32_t MemAttr :4;
		__V uint32_t TYPE :2;
		__V uint32_t BSU :2;
		__V uint32_t RACFG :2;
		__V uint32_t WACFG :2;
		__V uint32_t IRPTNDX :8;
	} stage1;
};
COMPILE_ASSERT(sizeof(union SMMU_CBAR_u) == 4, "size fault");

struct SMMUGlobal1_s {
	__V union SMMU_CBAR_u SMMU_CBAR[128];	//0x000-0x1FC
	__V uint32_t rev0x200_0x3FC[128];	//0x200-0x3FC
	__V uint32_t SMMU_CBFRSYNRA[128];	//0x400-0x5FC
	__V uint32_t rev0x600_0x7FC[128];	//0x600-0x7FC
	__V uint32_t SMMU_CBA2R[128];		//0x800-0x9FC
	__V uint32_t rev0xA00_0xFFC[384];	//0xA00-0xFFC
};
COMPILE_ASSERT(sizeof(struct SMMUGlobal1_s) == 4096, "size fault");

struct SMMUIntegration_s {
	__V uint32_t SMMU_ITCTRL;				//0x000
	__V uint32_t SMMU_ITIP;				//0x004
	__V uint32_t SMMU_ITOP_GLBL;			//0x008
	__V uint32_t SMMU_ITOP_PERF_INDEX;	//0x00C
	__V uint32_t SMMU_ITOP_CXT0TO31_RAM0;	//0x010
	__V uint32_t SMMU_ITOP_CXT32TO63_RAM1;   //0x014
	__V uint32_t SMMU_ITOP_CXT64TO95_RAM2;   //0x018
	__V uint32_t SMMU_ITOP_CXT96TO127_RAM3;  //0x01c
	__V uint32_t rev0x020_0x0FC[56];		//0x020 ~ 0x0FC
	__V uint32_t SMMU_TBUQOS[4];			//0x100 ~ 0x10c
	__V uint32_t rev0x110_0x1FC[60];		//0x110 ~ 0x1FC
	__V uint32_t SMMU_PER;				//0x200
	__V uint32_t SMMU_TBU_PWR_STATUS;		//0x204
	__V uint32_t rev0x208_0xFFC[894];		//0x208 ~ 0xffc
};
COMPILE_ASSERT(sizeof(struct SMMUIntegration_s) == 4096, "size fault");

union SMMU_CB_SCTLR_u {
	__V uint32_t all;
	struct {
		__V uint32_t M :1;
		__V uint32_t TRE :1;
		__V uint32_t AFE :1;
		__V uint32_t AFED :1;
		__V uint32_t E :1;
		__V uint32_t CFRE :1;
		__V uint32_t CFIE :1;
		__V uint32_t CFCFG :1;
		__V uint32_t HUPCF :1;
		__V uint32_t WXN :1;
		__V uint32_t UWXN :1;
		__V uint32_t rev11 :1;
		__V uint32_t ASIDPNE :1;
		__V uint32_t rev13 :1;
		__V uint32_t TRANSIENTCFG :2;
		__V uint32_t MemAttr :4;
		__V uint32_t MTCFG :1;
		__V uint32_t rev21 :1;
		__V uint32_t SHCFG :2;
		__V uint32_t RACFG :2;
		__V uint32_t WACFG :2;
		__V uint32_t NSCFG :2;
		__V uint32_t UCI :1;
		__V uint32_t rev31 :1;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CB_SCTLR_u) == 4, "size fault");

union SMMU_CB_TCR_u {
	__V uint32_t all;
	struct {
		__V uint32_t T0SZ :6;
		__V uint32_t rev6 :1;
		__V uint32_t EPD0 :1;
		__V uint32_t IRGN0 :2;
		__V uint32_t ORGN0 :2;
		__V uint32_t SH0 :2;
		__V uint32_t TG0 :2;
		__V uint32_t T1SZ :6;
		__V uint32_t A1 :1;
		__V uint32_t EPD1 :1;
		__V uint32_t IRGN1 :2;
		__V uint32_t ORGN1 :2;
		__V uint32_t SH1 :2;
		__V uint32_t TG1 :2;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CB_TCR_u) == 4, "size fault");

union SMMU_CB_TCR2_u {
	__V uint32_t all;
	struct {
		__V uint32_t PASize :3;
		__V uint32_t rev3 :1;
		__V uint32_t AS :1;
		__V uint32_t TBI0 :1;
		__V uint32_t TBI1 :1;
		__V uint32_t rev7 :1;
		__V uint32_t HAD0 :1;
		__V uint32_t HAD1 :1;
		__V uint32_t HA :1;
		__V uint32_t HD :1;
		__V uint32_t rev12_13 :2;
		__V uint32_t NSCFG0 :1;
		__V uint32_t SEP :3;
		__V uint32_t rev18_29 :12;
		__V uint32_t NSCFG1 :1;
		__V uint32_t rev31 :1;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CB_TCR2_u) == 4, "size fault");

union SMMU_CB_TTBR_u {
	__V uint64_t all;
	struct {
		__V uint64_t rev0_11 :12;
		__V uint64_t BaseAddr :36;
		__V uint64_t ASID :16;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CB_TTBR_u) == 8, "size fault");

union SMMU_CB_MAIR0_u {
	__V uint32_t all;
	struct {
		__V uint32_t Attr0 :8;
		__V uint32_t Attr1 :8;
		__V uint32_t Attr2 :8;
		__V uint32_t Attr3 :8;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CB_MAIR0_u) == 4, "size fault");

union SMMU_CB_MAIR1_u {
	__V uint32_t all;
	struct {
		__V uint32_t Attr4 :8;
		__V uint32_t Attr5 :8;
		__V uint32_t Attr6 :8;
		__V uint32_t Attr7 :8;
	};
};
COMPILE_ASSERT(sizeof(union SMMU_CB_MAIR1_u) == 4, "size fault");

struct SMMUStage1CB_s {
	__V union SMMU_CB_SCTLR_u SMMU_CBn_SCTLR;   //0x000
	__V uint32_t SMMU_CBn_ACTLR;			//0x004
	__V uint32_t SMMU_CBn_RESUME;			//0x008
	__V uint32_t rev0x00C;					//0x00C
	__V union SMMU_CB_TCR2_u SMMU_CBn_TCR2;	//0x010
	__V uint32_t rev0x014_0x01C[3];			//0x014-0x01C
	__V union SMMU_CB_TTBR_u SMMU_CBn_TTBR0;	//0x020
	__V union SMMU_CB_TTBR_u SMMU_CBn_TTBR1;	//0x028
	__V union SMMU_CB_TCR_u SMMU_CBn_TCR;	//0x030
	__V uint32_t SMMU_CBn_CONTEXTIDR;		//0x034
	__V uint32_t SMMU_CBn_PRRR;				//0x038
	__V uint32_t SMMU_CBn_NMRR;				//0x03C
	__V uint32_t rev0x040_0x044[2];			//0x040-0x044
	__V uint32_t rev0x048_0x04C[2];			//0x048-0x04C
	__V uint64_t SMMU_CBn_PAR;				//0x050
	__V uint32_t SMMU_CBn_FSR;				//0x058
	__V uint32_t SMMU_CBn_FSRRESTORE;		//0x05C
	__V uint64_t SMMU_CBn_FAR;				//0x060
	__V uint32_t SMMU_CBn_FSYNR0;			//0x068
	__V uint32_t SMMU_CBn_FSYNR1;			//0x06C
	__V uint32_t rev0x070_0x5FC[356];		//0x070-0x5FC
	__V uint64_t SMMU_CBn_TLBIVA;			//0x600
	__V uint64_t SMMU_CBn_TLBIVAA;			//0x608
	__V uint32_t SMMU_CBn_TLBIASID;			//0x610
	__V uint32_t rev0x614;					//0x614
	__V uint32_t SMMU_CBn_TLBIALL;			//0x618
	__V uint32_t rev0x61C;					//0x61C
	__V uint64_t SMMU_CBn_TLBIVAL;			//0x620
	__V uint64_t SMMU_CBn_TLBIVAAL;			//0x628
	__V uint32_t rev0x630_0x7EC[112];		//0x630-0x7EC
	__V uint32_t SMMU_CBn_TLBSYNC;			//0x7F0
	__V uint32_t SMMU_CBn_TLBSSTATUS;		//0x7F4
	__V uint32_t rev0x7F8_0x7FC[2];			//0x7F8-0x7FC
	__V uint64_t SMMU_CBn_ATS1PR;			//0x800
	__V uint64_t SMMU_CBn_ATS1PW;			//0x808
	__V uint64_t SMMU_CBn_ATS1UR;			//0x810
	__V uint64_t SMMU_CBn_ATS1UW;			//0x818
	__V uint32_t rev0x820_0x8EC[52];		//0x820-0x8EC
	__V uint32_t SMMU_CBn_ATSR;				//0x8F0
	__V uint32_t rev0x8F4_0xCFC[259];		//0x8F4-0xCFC
	__V uint32_t impl0xD00_0xDFC[64];		//0xD00-0xDFC
	__V uint32_t SMMU_CBn_PMEVCNTR[15];		//0xE00-0xE38
	__V uint32_t rev0xE3C_0xE7C[17];		//0xE3C-0xE7C
	__V uint32_t SMMU_CBn_PMEVTYPER[15];	//0xE80-0xEB8
	__V uint32_t rev0xEBC_0xEFC[17];		//0xEBC-0xEFC
	__V uint32_t SMMU_CBn_PMCFGR;			//0xF00
	__V uint32_t SMMU_CBn_PMCR;				//0xF04
	__V uint32_t rev0xF08;					//0xF08
	__V uint32_t rev0xF0C_0xF1C[5];			//0xF0C-0xF1C
	__V uint32_t SMMU_CBn_PMCEID0;			//0xF20
	__V uint32_t SMMU_CBn_PMCEID1;			//0xF24
	__V uint32_t rev0xF28_0xF3C[6];			//0xF28-0xF3C
	__V uint32_t SMMU_CBn_PMCNTENSET;		//0xF40
	__V uint32_t SMMU_CBn_PMCNTENCLR;		//0xF44
	__V uint32_t SMMU_CBn_PMINTENSET;		//0xF48
	__V uint32_t SMMU_CBn_PMINTENCLR;		//0xF4C
	__V uint32_t SMMU_CBn_PMOVSCLR;			//0xF50
	__V uint32_t rev0xF54;					//0xF54
	__V uint32_t SMMU_CBn_PMOVSSET;			//0xF58
	__V uint32_t rev0xF5C_0xFB4[23];		//0xF5C-0xFB4
	__V uint32_t SMMU_CBn_PMAUTHSTATUS;		//0xFB8
	__V uint32_t rev0xFBC_0xFC8[4];			//0xFBC-0xFC8
	__V uint32_t rev0xFCC;					//0xFCC
	__V uint32_t rev0xFD0_0xFFC[12];		//0xFD0-0xFFC
};
COMPILE_ASSERT(sizeof(struct SMMUStage1CB_s) == 4096, "size fault");

struct SMMU500XDMARegs_s {
	__V struct SMMUGlobal0_s gr0; //OFFSET 0
	__V struct SMMUGlobal1_s gr1; //OFFSET 0x1000
	__V struct SMMUIntegration_s it; //OFFSET 0x2000
	__V uint8_t rev_pages[29][4096];
	__V struct SMMUStage1CB_s cb[32];
};
COMPILE_ASSERT(sizeof(struct SMMU500XDMARegs_s) == (64 * 4096), "size fault");

struct SMMU500VDMARegs_s {
	__V struct SMMUGlobal0_s gr0;
	__V struct SMMUGlobal1_s gr1;
	__V struct SMMUIntegration_s it;
	__V uint8_t rev_pages[61][4096];
	__V struct SMMUStage1CB_s cb[64];
};
COMPILE_ASSERT(sizeof(struct SMMU500VDMARegs_s) == (128 * 4096), "size fault");

/*
 *|_64:48_|_47:39_|_38:30_|_29:21_|_20:_12_|_11:0_|
 *|   0   |   PC  |  PD1  |  PD2  |   PT   |  PO  |
 */
#define XDX_SMMUC_VA_PC_INDEX_SHIFT              (39U)
#define XDX_SMMUC_VA_PC_INDEX_CLRMSK             (UINT64_C(0xFFFF007FFFFFFFFF))
#define XDX_SMMUC_VA_PD1_INDEX_SHIFT             (30U)
#define XDX_SMMUC_VA_PD1_INDEX_CLRMSK            (UINT64_C(0xFFFFFF803FFFFFFF))
#define XDX_SMMUC_VA_PD2_INDEX_SHIFT             (21U)
#define XDX_SMMUC_VA_PD2_INDEX_CLRMSK            (UINT64_C(0xFFFFFFFFC01FFFFF))
#define XDX_SMMUC_VA_PT_INDEX_SHIFT              (12U)
#define XDX_SMMUC_VA_PT_INDEX_CLRMSK             (UINT64_C(0xFFFFFFFFFFE00FFF))

#define XDX_sSMMUC_PT_DATA_PM_META_PROTECT_SHIFT    (62U)
#define XDX_SMMUC_PT_DATA_PM_META_PROTECT_CLRMSK   (UINT64_C(0xBFFFFFFFFFFFFFFF))
#define XDX_SMMUC_PT_DATA_PM_META_PROTECT_EN       (UINT64_C(0x4000000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_SHIFT            (58U)
#define XDX_SMMUC_PT_DATA_AXCACHE_CLRMSK           (UINT64_C(0xC3FFFFFFFFFFFFFF))
#define XDX_SMMUC_PT_DATA_AXCACHE_DEVNONBUFF       (UINT64_C(0x0000000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_DEVBUFF          (UINT64_C(0x0400000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_NORMNONBUFF      (UINT64_C(0x0800000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_NORMBUFF         (UINT64_C(0x0c00000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WTNOALLOC        (UINT64_C(0x1800000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WBNOALLOC        (UINT64_C(0x1c00000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WTRALLOC         (UINT64_C(0x2000000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WBRALLOC         (UINT64_C(0x2400000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WTWALLOC         (UINT64_C(0x2800000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WBWALLOC         (UINT64_C(0x2c00000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WTRWALLOC        (UINT64_C(0x3800000000000000))
#define XDX_SMMUC_PT_DATA_AXCACHE_WBRWALLOC        (UINT64_C(0x3c00000000000000))
#define XDX_SMMUC_PT_DATA_VP_PAGE_HI_SHIFT         (40U)
#define XDX_SMMUC_PT_DATA_VP_PAGE_HI_CLRMSK        (UINT64_C(0xFC0000FFFFFFFFFF))
#define XDX_SMMUC_PT_DATA_PAGE_SHIFT               (12U)
#define XDX_SMMUC_PT_DATA_PAGE_CLRMSK              (UINT64_C(0xFFFFFF0000000FFF))
#define XDX_SMMUC_PT_DATA_VP_PAGE_LO_SHIFT         (6U)
#define XDX_SMMUC_PT_DATA_VP_PAGE_LO_CLRMSK        (UINT64_C(0xFFFFFFFFFFFFF03F))
#define XDX_SMMUC_PT_DATA_ENTRY_PENDING_SHIFT      (5U)
#define XDX_SMMUC_PT_DATA_ENTRY_PENDING_CLRMSK     (UINT64_C(0xFFFFFFFFFFFFFFDF))
#define XDX_SMMUC_PT_DATA_ENTRY_PENDING_EN         (UINT64_C(0x0000000000000020))
#define XDX_SMMUC_PT_DATA_PM_SRC_SHIFT             (4U)
#define XDX_SMMUC_PT_DATA_PM_SRC_CLRMSK            (UINT64_C(0xFFFFFFFFFFFFFFEF))
#define XDX_SMMUC_PT_DATA_PM_SRC_EN                (UINT64_C(0x0000000000000010))
#define XDX_SMMUC_PT_DATA_CC_SHIFT                 (2U)
#define XDX_SMMUC_PT_DATA_CC_CLRMSK                (UINT64_C(0xFFFFFFFFFFFFFFFB))
#define XDX_SMMUC_PT_DATA_CC_EN                    (UINT64_C(0x0000000000000004))
#define XDX_SMMUC_PT_DATA_READ_ONLY_SHIFT          (1U)
#define XDX_SMMUC_PT_DATA_READ_ONLY_CLRMSK         (UINT64_C(0xFFFFFFFFFFFFFFFD))
#define XDX_SMMUC_PT_DATA_READ_ONLY_EN             (UINT64_C(0x0000000000000002))
#define XDX_SMMUC_PT_DATA_VALID_SHIFT              (0U)
#define XDX_SMMUC_PT_DATA_VALID_CLRMSK             (UINT64_C(0xFFFFFFFFFFFFFFFE))
#define XDX_SMMUC_PT_DATA_VALID_EN                 (UINT64_C(0x0000000000000001))

#define XDX_SMMUC_PD_DATA_ENTRY_PENDING_SHIFT      (40U)
#define XDX_SMMUC_PD_DATA_ENTRY_PENDING_CLRMSK     (UINT64_C(0xFFFFFEFFFFFFFFFF))
#define XDX_SMMUC_PD_DATA_ENTRY_PENDING_EN         (UINT64_C(0x0000010000000000))
#define XDX_SMMUC_PD_DATA_PT_BASE_SHIFT            (5U)
#define XDX_SMMUC_PD_DATA_PT_BASE_CLRMSK           (UINT64_C(0xFFFFFF000000001F))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_SHIFT          (1U)
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_CLRMSK         (UINT64_C(0xFFFFFFFFFFFFFFF1))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_4KB            (UINT64_C(0x0000000000000000))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_16KB           (UINT64_C(0x0000000000000002))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_64KB           (UINT64_C(0x0000000000000004))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_256KB          (UINT64_C(0x0000000000000006))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_1MB            (UINT64_C(0x0000000000000008))
#define XDX_SMMUC_PD_DATA_PAGE_SIZE_2MB            (UINT64_C(0x000000000000000a))
#define XDX_SMMUC_PD_DATA_VALID_SHIFT              (0U)
#define XDX_SMMUC_PD_DATA_VALID_CLRMSK             (UINT64_C(0xFFFFFFFFFFFFFFFE))
#define XDX_SMMUC_PD_DATA_VALID_EN                 (UINT64_C(0x0000000000000001))

#define XDX_SMMUC_PC_DATA_PD_BASE_SHIFT            (4U)
#define XDX_SMMUC_PC_DATA_PD_BASE_CLRMSK           (0x0000000FU)
#define XDX_SMMUC_PC_DATA_PD_BASE_ALIGNSHIFT       (12U)
#define XDX_SMMUC_PC_DATA_PD_BASE_ALIGNSIZE        (4096U)
#define XDX_SMMUC_PC_DATA_ENTRY_PENDING_SHIFT      (1U)
#define XDX_SMMUC_PC_DATA_ENTRY_PENDING_CLRMSK     (0xFFFFFFFDU)
#define XDX_SMMUC_PC_DATA_ENTRY_PENDING_EN         (0x00000002U)
#define XDX_SMMUC_PC_DATA_VALID_SHIFT              (0U)
#define XDX_SMMUC_PC_DATA_VALID_CLRMSK             (0xFFFFFFFEU)
#define XDX_SMMUC_PC_DATA_VALID_EN                 (0x00000001U)

#define XDX_SMMUC_PTE_PROTMASK	(XDX_SMMUC_PT_DATA_PM_META_PROTECT_EN | \
		~XDX_SMMUC_PT_DATA_AXCACHE_CLRMSK | \
		XDX_SMMUC_PT_DATA_ENTRY_PENDING_EN | \
		XDX_SMMUC_PT_DATA_PM_SRC_EN | \
		XDX_SMMUC_PT_DATA_CC_EN | \
		XDX_SMMUC_PT_DATA_READ_ONLY_EN | \
		XDX_SMMUC_PT_DATA_VALID_EN)

#define XDX_SMMUC_PDE_PROTMASK	(XDX_SMMUC_PD_DATA_ENTRY_PENDING_EN | \
		~XDX_SMMUC_PD_DATA_PAGE_SIZE_CLRMSK | \
		XDX_SMMUC_PD_DATA_VALID_EN)

#define XDX_SMMUC_PCE_PROTMASK	(XDX_SMMUC_PC_DATA_ENTRY_PENDING_EN | \
		XDX_SMMUC_PC_DATA_VALID_EN)

struct xdx_smmu_s;
struct xdx_smmu_tcu_s;
#define XDXSMMU_SID_MAX		8
#define XDXSMMU_GRP_TBUS_MAX	8

#define XDXTBU_FLAG_BYPASS (1 << 0)
struct xdx_smmu_tbu_s {
	uint32_t flags;
	uint32_t sid;
	struct xdx_smmu_tcu_s *p_tcu;
};

enum xdx_smmu_tcu_type {
	TDMA_TCU,
	VDMA_TCU,
	XDMA_TCU,
	VDEC_TCU,
	VENC_TCU,
	XDXSMMUTCU_MAX
};

#define XDXTCU_FLAG_CONFIGHOST (1 << 0)
struct xdx_smmu_tcu_s {
	struct device		*p_dev;
	uint32_t			tcu_id;
	char				*name;
	uint64_t			tcu_offset;
	void __iomem		*dev_base;
	struct xdx_smmu_tbu_s		tbu_tab[XDXSMMU_SID_MAX];
	uint64_t cb_offset;
	uint32_t flags;
};

struct smmu_group_s {
	struct list_head list;
	uint32_t grp_id;
	uint32_t flags;
	struct xdx_smmu_s *xsmmu;
	uint32_t gsid_cnt;
	struct xdx_smmu_tbu_s *tbus[XDXSMMU_GRP_TBUS_MAX];
};
enum xdx_smmu_pwr_stat_e {
	XDX_SMMU_PWR_STAT_READY = 0,
	XDX_SMMU_PWR_STAT_SUSPEND,
	XDX_SMMU_PWR_STAT_PWROFF,
};

struct xdx_smmu_s {
	struct xdx_smmu_tcu_s *tcus[XDXSMMUTCU_MAX];
	struct list_head grp_head;
	struct list_head ctx_head;
	uint32_t flags;
	void *env_data;
	uint32_t power_stat;
	struct xdx_device *xdx_dev;
};

struct xdx_smmu_tbu_cfg_s {
	uint32_t flags;
	uint32_t sid;
};

struct xdx_smmu_tcu_cfg_s {
	uint32_t flags;
	uint32_t tcu_idx;
	uint32_t tbu_cnt;
	uint64_t base_offset;
	uint64_t cb_offset;
	const struct xdx_smmu_tbu_cfg_s *tbu_cfgs;
};

struct xdx_smmu_cfg_s {
	uint32_t tcu_cnt;
	const struct xdx_smmu_tcu_cfg_s **tcu_tab;
};

#define GSID_ID(_tcu_id, __tbu_id) ((((_tcu_id) & 0xf) << 8) | ((__tbu_id) & 0xff))
#define GSID_TCU_ID(_gsid) ((_gsid) >> 8 & 0xf)
#define GSID_TBU_ID(_gsid) ((_gsid) & 0xff)

struct smmu_group_s *xdx_smmu_find_group_gsid(
	struct xdx_smmu_s *xsmmu, uint32_t gsid);
struct smmu_group_s *xdx_smmu_creat_group(
	struct xdx_smmu_s *xsmmu, uint32_t gsid_cnt, uint32_t *gsids, uint32_t flags);
int xdx_smmu_group_config_pagetab(struct smmu_group_s *group, uint64_t val);
int xdx_smmu_destroy_group(struct smmu_group_s *grp);
int xdx_smmu_tcu_init(struct xdx_smmu_s *xsmmu,
	const struct xdx_smmu_tcu_cfg_s *tcu_cfg);
int xdx_smmu_tcus_deinit(struct xdx_smmu_s *xsmmu);
int xdx_smmu_tcu_release(struct xdx_smmu_tcu_s *smmu_tcu);
int xdx_smmu_group_sync_tlb(struct smmu_group_s *group);
int xdx_smmu_plat_init(struct xdx_device *xdev);
int xdx_smmu_plat_deinit(struct xdx_device *xdev);
int xdx_smmu_suspend(struct xdx_smmu_s *xsmmu);
int xdx_smmu_resume(struct xdx_smmu_s *xsmmu);
int xdx_smmu_plat_resume(struct xdx_device *xdev);

static inline struct xdx_smmu_s *xdev_to_smmu_dev(struct xdx_device *xdev)
{
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);

	return (struct xdx_smmu_s *)(smmu->xsmmu);
}

#endif /* _XDX_SMMU_H */
