/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef HANTRO_H
#define HANTRO_H

#include <linux/ioctl.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0)) || (defined(REDHAT_83))
#include <linux/dma-resv.h>
#else
#include <linux/reservation.h>
#endif
#include "pvr_dma_resv.h"
#include <linux/dma-mapping.h>
#include <drm/drm_vma_manager.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 1, 0))
#include <drm/drm_gem_dma_helper.h>
#else
#include <drm/drm_gem_cma_helper.h>
#endif
#include <drm/drm_gem.h>
#include <linux/dma-buf.h>
#include <drm/drm.h>
#include <drm/drm_auth.h>
#include <linux/version.h>
#include <linux/list.h>
#if KERNEL_VERSION(4, 13, 0) <= LINUX_VERSION_CODE
#include <linux/dma-fence.h>
#endif
#if (KERNEL_VERSION(5, 5, 0) > LINUX_VERSION_CODE) && (!defined(REDHAT_83))
#include <drm/drmP.h>
#else
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <drm/drm_drv.h>
#include <drm/drm_fourcc.h>
#include <drm/drm_framebuffer.h>
#if KERNEL_VERSION(5, 8, 0) > LINUX_VERSION_CODE
#include <drm/drm_pci.h>
#endif
#endif

#include "hantro_metadata.h"

#define NO_RESIZEBAR_SIZE (256*0x100000)

/*these domain definitions are identical to hantro_bufmgr.h*/
#define HANTRO_DOMAIN_NONE 0x00000
#define HANTRO_CPU_DOMAIN 0x00001
#define HANTRO_HEVC264_DOMAIN 0x00002
#define HANTRO_JPEG_DOMAIN 0x00004
#define HANTRO_DECODER0_DOMAIN 0x00008
#define HANTRO_DECODER1_DOMAIN 0x00010
#define HANTRO_DECODER2_DOMAIN 0x00020

#define CONFIG_HWDEC BIT(0)
#define CONFIG_HWENC BIT(1)
#define CONFIG_L2CACHE BIT(2)
#define CONFIG_DEC400 BIT(3)
#define CONFIG_HANTROMMU BIT(4)
#define CONFIG_VCMD BIT(5)
#define CONFIG_AXIFE BIT(6)
#define CONFIG_INVAS BIT(7)
#define CONFIG_VPU_SMMU BIT(8)

#define KCORE(id) ((u32)(id) & 0xff)
#define NODETYPE(id) (((u32)(id) >> 8) & 0xff)
#define SLICE(id) ((u32)(id) >> 16)

/* slice index definition is unchanged.
 *for dec400/cache NODE(id) refers to its parent core number based on NODETYPE
 *for dec/enc, NODE(id) refers to its core num, and NODETYPE is useless.
 */
/*node type for NODETYPE(id), apply to be expanded */
#define NODE_TYPE_DEC BIT(0)
#define NODE_TYPE_ENC BIT(1)
#define NODE_TYPE_VCMD BIT(2)

#define CORE_MAX (4)

enum {
	CORE_VCE = 0,
	CORE_VCEJ = 1,
	CORE_CUTREE = 2,
	CORE_DEC400 = 3,
	CORE_MMU = 4,
	CORE_L2CACHE = 5,
	CORE_AXIFE = 6,
	CORE_APBFT = 7,
	CORE_MMU_1 = 8,
	CORE_AXIFE_1 = 9,
	CORE_TYPE_MAX
};

typedef enum {
	VCE,
	VCD_0,
	VCD_1,
	DECODER_G1_0,
	DECODER_G1_1,
	DECODER_G2_0,
	DECODER_G2_1,
} cache_client_type;

enum CoreType {
	/* Decoder */
	HW_VCD = 0,
	HW_VCDJ,
	HW_BIGOCEAN,
	HW_VCMD,
	HW_MMU, //if set HW_MMU_WR, then HW_MMU means HW_MMU_RD
	HW_MMU_WR,
	HW_DEC400,
	HW_L2CACHE,
	HW_SHAPER,
	/* Encoder*/
	/* Auxiliary IPs */
	HW_AXIFE,
	HW_AFBC,
	HW_CORE_MAX /* max number of cores supported */
};

typedef enum { DIR_RD = 0, DIR_WR, DIR_BI } driver_cache_dir;

struct hantro_drm_fb {
	struct drm_framebuffer fb;
	struct drm_gem_object *obj[4];
};

/**
 * The memta data location information exchange IOCTRL parameters.
 */
struct hantro_metadata_params {
	int handle; /* the handle of current bo */
	struct viv_vidmem_metadata meta_data; /* the meta data */
};

struct dmapriv {
	struct viv_vidmem_metadata meta_data;
	void *self; //ptr of parent cma
};

struct drm_gem_hantro_object {
	/* base of gem object */
	struct drm_gem_object base;

	struct dmapriv dmapriv;

	/* following is private data for hantro object */

	dma_addr_t paddr;
	dma_addr_t mem_base;
	struct sg_table *sgt;

	/* For objects with DMA memory allocated by GEM CMA */
	void *vaddr;
	struct page *pageaddr;
	unsigned long num_pages;
	/*fence ref*/
	struct dma_resv kresv;
	unsigned int ctxno;
	int handle;
	int sliceidx;
	int flag;
};

struct hantro_fencecheck {
	unsigned int handle;
	int ready;
};

struct hantro_domainset {
	unsigned int handle;
	unsigned int writedomain;
	unsigned int readdomain;
};

struct hantro_addrmap {
	unsigned int handle;
	unsigned long long vm_addr;
	unsigned long long phy_addr;
	unsigned long long mem_base;
};

struct hantro_regtransfer {
	unsigned long coreid;
	unsigned long offset;
	unsigned long size;
	const void *data;
	int benc; /*encoder core or decoder core*/
	int direction; /*0=read, 1=write*/
};

struct hantro_corenum {
	unsigned int deccore;
	unsigned int enccore;
};

#define HANTRO_FENCE_WRITE 1
struct hantro_acquirebuf {
	unsigned long handle;
	unsigned long flags;
	unsigned long timeout;
	unsigned long fence_handle;
};

struct hantro_releasebuf {
	unsigned long fence_handle;
};

struct core_desc {
	__u32 id; /* id of the core */
	__u32 type; /* type of core to be written */
	__u32 *regs; /* pointer to user registers */
	__u32 size; /* size of register space */
	__u32 reg_id;
};

struct nor64_parameter {
	unsigned long data;
	u32 id;
};

struct nor32_parameter {
	u32 data;
	u32 id;
};

typedef struct {
	unsigned int type_info;
	/*indicate which IP is contained
	 *in this subsystem and each uses one bit of this variable
	 */
	unsigned long offset[CORE_TYPE_MAX];
	unsigned long regSize[CORE_TYPE_MAX];
	int irq[CORE_TYPE_MAX];
	unsigned int id;
} SUBSYS_CORE_INFO;

typedef struct CoreWaitOut {
	u32 job_id[CORE_MAX];
	u32 irq_status[CORE_MAX];
	u32 irq_num;
	u32 id;
} CORE_WAIT_OUT;

struct axife_cfg {
	u8 axi_rd_chn_num;
	u8 axi_wr_chn_num;
	u8 axi_rd_burst_length;
	u8 axi_wr_burst_length;
	u8 fe_mode;
	u32 id;
};

/*********define the parameters to transfer to iocontrol**********/
typedef enum {
	CORE_FLAG = 0,
	VCMD_FLAG = 0x10,
	VCMD_BUF_FLAG = 0x11,
	DEC400_FLAG = 0x20,
	CACHE_FLAG = 0x30,
	MMU0_FLAG = 0x40,
	MMU1_FLAG = 0x41,
} SUB_NODE;

typedef enum {
	CODEC_DEC_FLGA = 0,
	CODEC_ENC_FLGA = 1,
} CODEC_GROP;

//use for mmap's offset parameter
//low pageshift bits are skipped since it can't be seen in driver.
//so standard DRM mmap's offset will starts from 1ul << (32 + pageshift)
//0 .. (1ul << (32 + pageshift) - 1) is used by our own mmap, such as core reg, vcmd buf.
//offset parameter is used as ioctl_id's ID_PAR.
#define MAPOFF_2COREID(a)		(((offset_t)(a)) >> PAGE_SHIFT)
// this is the least address value used for normal DRM map offset
#define VSI_MMAP_ADDRES_CEIL	(1ul << (32 + PAGE_SHIFT))
#define VSI_MMAP_ADDRES_CEIL_MMAP	(1ul << 32)

typedef union ioctl_id {
	unsigned int data;
	struct id_par {
		unsigned int sub_mod_idx : 8;
		unsigned int codec_idx : 8;
		unsigned int group_idx : 8;
		unsigned int node_idx : 8;
	} ID_PAR;
} hantro_ioctl_id;
/*********define the parameters to transfer to iocontrol**********/

/************* some cache related defines   ************/
//#define PCI_BUS
/*Define Cache&Shaper Offset from common base*/
#define SHAPER_OFFSET (0x8 << 2)
#define CACHE_ONLY_OFFSET (0x8 << 2)
#define CACHE_WITH_SHAPER_OFFSET (0x80 << 2)
/************* some cache related defines  end ************/

#endif /* HANTRO_H */
