/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include "xdx_gem.h"
#include "hantro_priv.h"
#include "hantrodec.h"
#include "hx280enc.h"
#include "hantrocache.h"
#include "hantrodec400.h"
#include "hantro_vcmd.h"
#include "hantro_mmu.h"
#include "hantro.h"
#ifdef __amd64__
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
#include <asm/set_memory.h>
#endif
#endif
#ifdef VSI_FPGA
#include "hantro_fpga.h"
#endif
#include "xdx_drv.h"
#include "physmem.h"

#ifndef USE_CMA
#define USE_CMA		0
#endif

void hantro_drm_fb_destroy(struct drm_framebuffer *fb);

int hantro_drm_fb_create_handle(struct drm_framebuffer *fb,
 			       struct drm_file *file_priv,
 			       unsigned int *handle);

int hantro_drm_fb_dirty(struct drm_framebuffer *fb,
 		       struct drm_file *file, unsigned int flags,
 		       unsigned int color, struct drm_clip_rect *clips,
 		       unsigned int num_clips);


int hantro_gem_dumb_create_internal(struct drm_file *file_priv,
 				   struct drm_device *dev,
 				   struct drm_mode_create_dumb *args);

int hantro_gem_dumb_create(struct drm_device *dev, void *data,
 			  struct drm_file *file_priv);

int hantro_gem_dumb_map_offset(struct drm_file *file_priv,
 			      struct drm_device *dev, uint32_t handle,
 			      uint64_t *offset);

int hantro_gem_dumb_destroy(struct drm_file *file,
 			struct drm_device *dev, u32 handle);

int hantro_destroy_dumb(struct drm_device *dev, void *data,
 		       struct drm_file *file_priv);

struct sg_table *hantro_gem_prime_get_sg_table(struct drm_gem_object *obj);

struct drm_gem_object *hantro_gem_prime_import_sg_table(struct drm_device *dev,
				 struct dma_buf_attachment *attach,
				 struct sg_table *sgt);

#if LINUX_VERSION_CODE<KERNEL_VERSION(5, 11, 0) 
void *hantro_gem_prime_vmap(struct drm_gem_object *obj);

void hantro_gem_prime_vunmap(struct drm_gem_object *obj, void *vaddr);

#elif ((LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))&&(LINUX_VERSION_CODE < KERNEL_VERSION(5,18,0)))
int hantro_gem_prime_vmap(struct drm_gem_object *obj,
				  struct dma_buf_map *map);
void hantro_gem_prime_vunmap(struct drm_gem_object *obj,
				    struct dma_buf_map *map);
#else
int hantro_gem_prime_vmap(struct drm_gem_object *obj,
				  struct iosys_map *map);
void hantro_gem_prime_vunmap(struct drm_gem_object *obj,
				    struct iosys_map *map);
#endif

int hantro_gem_prime_mmap(struct drm_gem_object *obj,
 			 struct vm_area_struct *vma);

void hantro_gem_free_object(struct drm_gem_object *gem_obj);

int hantro_gem_close(struct drm_device *dev, void *data,
 		    struct drm_file *file_priv);

int hantro_gem_open(struct drm_device *dev, void *data,
 		   struct drm_file *file_priv);

int hantro_map_vaddr(struct drm_device *dev, void *data,
 		    struct drm_file *file_priv);

int hantro_get_hwcfg(struct drm_device *dev, void *data,
 		    struct drm_file *file_priv);

int hantro_get_slicenum(struct drm_device *dev, void *data,
 		       struct drm_file *file_priv);

int hantro_gem_flink(struct drm_device *dev, void *data,
 		    struct drm_file *file_priv);

int hantro_map_dumb(struct drm_device *dev, void *data,
 		   struct drm_file *file_priv);

int hantro_drm_open(struct drm_device *dev, struct drm_file *file);

struct drm_gem_object *hantro_drm_gem_prime_import(struct drm_device *dev, struct dma_buf *dma_buf);

struct dma_buf *hantro_prime_export(struct drm_gem_object *obj,
 				   int flags);

int hantro_handle_to_fd(struct drm_device *dev, void *data,
 		       struct drm_file *file_priv);

int hantro_fd_to_handle(struct drm_device *dev, void *data,
 		       struct drm_file *file_priv);

int hantro_fb_create2(struct drm_device *dev, void *data,
 		     struct drm_file *file_priv);

int hantro_fb_create(struct drm_device *dev, void *data,
 		    struct drm_file *file_priv);

int hantro_get_version(struct drm_device *dev, void *data,
 		      struct drm_file *file_priv);

int hantro_get_cap(struct drm_device *dev, void *data,
			  struct drm_file *file_priv);

/*just a test API for any purpose*/
int hantro_test(struct drm_device *dev, void *data,
 	       struct drm_file *file_priv);

int hantro_getprimeaddr(struct drm_device *dev, void *data,
 		       struct drm_file *file_priv);

int hantro_ptr_to_phys(struct drm_device *dev, void *data,
 		      struct drm_file *file_priv);

int hantro_query_metadata(struct drm_device *dev, void *data,
 			 struct drm_file *file_priv);

int hantro_update_metadata(struct drm_device *dev, void *data,
 			  struct drm_file *file_priv);

int hantro_getmagic(struct drm_device *dev, void *data,
 		   struct drm_file *file_priv);

int hantro_authmagic(struct drm_device *dev, void *data,
 		    struct drm_file *file_priv);

int hantro_device_open(struct inode *inode, struct file *filp);

int hantro_device_release(struct inode *inode, struct file *filp);

int hantro_mmap(struct file *filp, struct vm_area_struct *vma);

void hantro_gem_vm_close(struct vm_area_struct *vma);

#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
int hantro_unload(struct drm_device *dev);
#else
void hantro_release(struct drm_device *dev);
#endif

int hantro_gem_prime_handle_to_fd(struct drm_device *dev,
					 struct drm_file *filp, uint32_t handle,
					 u32 flags, int *prime_fd);

#if KERNEL_VERSION(4, 13, 0) > LINUX_VERSION_CODE
/*we shall not support page fault. */
int hantro_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf);
#elif (KERNEL_VERSION(5, 0, 0) > LINUX_VERSION_CODE) && (!defined(REDHAT_83))
int hantro_vm_fault(struct vm_fault *vmf);
#else
vm_fault_t hantro_vm_fault(struct vm_fault *vmf);
#endif

u32 hantro_vblank_no_hw_counter(struct drm_device *dev,
				       unsigned int pipe);


struct drm_device *create_hantro_drm(struct device *dev);

long hantro_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);

