/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include "hantro_device.h"

#include "xdxvpu_irq.h"
#include "vcmdswhwregisters.h"
#include "hantro_vcmd.h"
#include "hantrocache.h"
#include "hantrodec.h"
#include "hx280enc.h"

#ifndef HAS_VCMD
#ifdef HAS_VC8000D
static int xdxvpu_dec_irq_status(int irq,struct xdx_device *xdev)
{
	int slicen = get_slicenumber();
	struct hantrodec_t *dev = NULL;
	int n;

    for (n = 0; n < slicen; n++) {
		XDX_DEV_DEBUG("dev->core_id:%d\n",dev->core_id);
		dev = get_decnodes(n, 0);
		hantrodec_isr(irq, dev);
	}
	return 0;
}
#endif

#ifdef HAS_VC8000E
static int xdxvpu_enc_irq_status(int irq,struct xdx_device *xdev)
{
	int i, slicen = get_slicenumber();
	struct hantroenc_t *pcore;

	for (i = 0; i < slicen; i++) {
		pcore = get_encnodes(i, 0);
	    while (pcore) {
			XDX_DEV_DEBUG("pcore->core_id:%d\n",pcore->core_id);
			hantroenc_isr(irq, pcore);
			pcore = pcore->next;
		}
	}
	return 0;
}
#endif
#endif

#ifdef HAS_VCMD
static u32 xdxvpu_vcmd_sub_irq_status(int irq, vcmd_dev_str *subsys_dev,struct xdx_device *xdev)
{
	u32 i;
	vcmd_core_str *subsys_core;

	subsys_core = subsys_dev->vcmd_core;

	for (i = 0; i < subsys_dev->subsys_num; i++) {
		XDX_DEV_DEBUG("subsys_core->core_id:%d\n",subsys_core->core_id);
		hantrovcmd_isr(irq, subsys_core);
		subsys_core = subsys_core->core_next;
	}
	return 0;
}

static int xdxvpu_vcmd_dec_handler(int irq, struct xdx_device *xdev)
{
	vcmd_slice_str *cur_slice;

	cur_slice = get_vcmd_slice_head();
	while (cur_slice) {
		if (cur_slice->dec_vcmd.subsys_num != 0)
			xdxvpu_vcmd_sub_irq_status(irq, &cur_slice->dec_vcmd, xdev);

		cur_slice = cur_slice->slice_next;
	}
	return 0;
}

static int xdxvpu_vcmd_enc_handler(int irq, struct xdx_device *xdev)
{
	vcmd_slice_str *cur_slice;

	cur_slice = get_vcmd_slice_head();
	while (cur_slice) {
		if (cur_slice->enc_vcmd.subsys_num != 0)
			xdxvpu_vcmd_sub_irq_status(irq, &cur_slice->enc_vcmd, xdev);

		cur_slice = cur_slice->slice_next;
	}
	return 0;
}

#endif

static int xdx_vpu_irq_set(struct xdx_device *xdev, struct xdx_irq_src *src,
		      unsigned type, enum xdx_interrupt_state state)
{
	uint32_t offset = PANGU_REG_HOST_MISC_BASE + mmMSI_MSIX_HW_ENABLE_PF0;
	uint32_t val = xdx_mmio_readl(xdev, offset);
	uint32_t vpu_mask = \
                        BIT(xdev->soc_funcs.get_msix_idx(MSIX_VDEC_INT)) |
                        BIT(xdev->soc_funcs.get_msix_idx(MSIX_VENC_INT));

	if(state == XDX_IRQ_STATE_ENABLE){
		val = val | vpu_mask;
	}else{
		val = val & (~vpu_mask);
	}
	xdx_mmio_writel(xdev, offset, val);
	return 0;
}

void vpuirq_dec_work_handle(struct work_struct *work)
{
	struct xdx_vpu *xdxvpu = container_of(work,struct xdx_vpu,vpuirq_dec_work);
	struct xdx_device *xdev = container_of(xdxvpu,struct xdx_device,vpu);

	xdxvpu_vcmd_dec_handler(xdxvpu->vpuirq_dec_status, xdev);
}
void vpuirq_enc_work_handle(struct work_struct *work)
{
	struct xdx_vpu *xdxvpu = container_of(work,struct xdx_vpu,vpuirq_enc_work);
	struct xdx_device *xdev = container_of(xdxvpu,struct xdx_device,vpu);

	xdxvpu_vcmd_enc_handler(xdxvpu->vpuirq_enc_status, xdev);
}

static int xdx_vpu_irq_process(struct xdx_device *xdev, struct xdx_irq_src *src,
			       struct xdx_irq_entry *entry)
{
	struct xdxvpu_dev_info *xdxvpu_pdev = NULL;
	u32 dev_type;
	u32 vcmd_ext_irq = 0;
	u32 i;

	if (entry->vec_idx == xdev->soc_funcs.get_msix_idx(MSIX_VDEC_INT))
		dev_type = VCMD_DEC_MODULE_TYPE_0;
	else if (entry->vec_idx ==  xdev->soc_funcs.get_msix_idx(MSIX_VENC_INT))
		dev_type = VCMD_ENC_MODULE_TYPE_0;
	else {
		dev_err(xdev->dev, "%s: wrong irq vector %d\n", __func__,
			entry->vec_idx);
		return -1;
	}

	for (i = 0; i < ARRAY_SIZE(xdxvpu_dev); i++) {
		if (xdxvpu_dev[i].sub_module_type == dev_type) {
			xdxvpu_pdev = &xdxvpu_dev[i];
			break;
		}
	}

	if (!xdxvpu_pdev) {
		dev_err(xdev->dev, "%s: wrong ip type %d\n", __func__,
			dev_type);
		return -1;
	}

#ifndef HAS_VCMD
	vcmd_ext_irq =
		vcmd_read_reg(xdxvpu_pdev->vcmd_reg_vaddr, VCMD_REG_IRQ_SOURCE);
	if (vcmd_ext_irq != 0) {
		if (vcmd_ext_irq & VCMD_EXT_IRQ_VCD) {
#ifdef HAS_VC8000D
			xdxvpu_dec_irq_status(entry->irq, xdev);
#endif
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_DEC400_VCD) {
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_L2Cache_VCD) {
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_MMU_VCD) {
		}
		/* enc */
		else if (vcmd_ext_irq & VCMD_EXT_IRQ_MMU_VCE) {
#ifdef HAS_VC8000E
			xdxvpu_enc_irq_status(entry->irq, xdev);
#endif
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_MMU_Cutree) {
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_DEC400_VCE) {
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_L2Cache_VCE) {
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_MMU_VCE) {
		} else if (vcmd_ext_irq & VCMD_EXT_IRQ_MMU_Cutree) {
		}
	}
#else
	vcmd_ext_irq =
		vcmd_read_reg(xdxvpu_pdev->vcmd_reg_vaddr, VCMD_REG_IRQ_STATUS);
	vcmd_write_reg(xdxvpu_pdev->vcmd_reg_vaddr, VCMD_REG_IRQ_STATUS, vcmd_ext_irq);

	/* directly return when vcmd_ext_irq is 0 and msi poll is enabled */
	if (entry->poll_enable && (vcmd_ext_irq == 0)) {
		return 0;
	}

	if (dev_type == VCMD_DEC_MODULE_TYPE_0) {
		xdev->vpu.vpuirq_dec_status = (int)vcmd_ext_irq;
		queue_work(xdev->vpu.vpu_workq, &xdev->vpu.vpuirq_dec_work);
	} else if (dev_type == VCMD_ENC_MODULE_TYPE_0) {
		xdev->vpu.vpuirq_enc_status = (int)vcmd_ext_irq;
		queue_work(xdev->vpu.vpu_workq, &xdev->vpu.vpuirq_enc_work);
	}
//	xdxvpu_vcmd_dec_handler(entry->irq, xdev);
//	xdxvpu_vcmd_enc_handler(entry->irq, xdev);
//	queue_work(xdev->vpu.vpu_workq, &xdev->vpu.vpuirq_dec_work);
//	queue_work(xdev->vpu.vpu_workq, &xdev->vpu.vpuirq_enc_work);
#endif
	return 0;
}

static struct xdx_irq_src_funcs xdx_vpu_src_funcs = {
	.set = xdx_vpu_irq_set,
	.process = xdx_vpu_irq_process,
};

int xdx_vpu_isr_install(struct xdx_device *xdev)
{
	struct xdx_irq_src *irq_src;

	irq_src = kmalloc(sizeof(*irq_src), GFP_KERNEL);
	if (!irq_src)
		return -ENOMEM;

	irq_src->id = XDX_IRQ_CLIENTID_VPU;
	irq_src->funcs = &xdx_vpu_src_funcs;

	xdx_irq_register_src(xdev, irq_src);

	xdev->vpu.irq_src = irq_src;

	return 0;
}

int vpu_isr_uninstall(struct xdx_device *xdev)
{
	int ret;

	ret = xdx_irq_unregister_src(xdev, xdev->vpu.irq_src);
	if (ret) {
		dev_err(xdev->dev, "failed to unregister irq source\n");
		return ret;
	}

	kfree(xdev->vpu.irq_src);
	xdev->vpu.irq_src = NULL;

	return 0;
}
