/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/delay.h>

#include <drm/xdxgpu_drm.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 14, 0))
#include <drm/drm_aperture.h>
#endif // (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 12, 0))
#include <drm/drm_fb_helper.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
#include <drm/drm_drv.h>
#endif
#include <drm/drm_framebuffer.h>
#include <drm/drm_plane.h>
#include <drm/drm_crtc.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0)) || (defined(REDHAT_83))
#include <drm/drm_device.h>
#include <drm/drm_file.h>
#endif
#include <drm/drm_modeset_lock.h>
#include <drm/drm_vma_manager.h>
#include <drm/drm_gem.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 13, 0))
#include <drm/drm_vblank.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0))
#include <drm/drm_irq.h>
#else
#include <drm/drmP.h>
#endif
#include "xdx_drv.h"
#include <drm/xdx_asic_type.h>

#include "xdx_device.h"
#include "xdx_drm.h"
#include "xdx_gfx.h"
#include "xdx_fdinfo.h"
#include "xdx_dmabuf.h"
#include "xdx_dma.h"
#include "xdx_vpu.h"
#include "xdx_debugfs.h"
#include "xdx_sysfs.h"
#include "xdx_gem.h"
#include "pm/xdx_pm.h"
#include "xdx_audio.h"

#include "pvr_device.h"
#include "pvr_bridge_k.h"
#include "pvr_sync_ioctl_drm.h"
#include "pvr_sync_ioctl_common.h"

#include "hantro_priv.h"
#include "hantro_drm.h"

#include "process_stats.h"
#include "rgxdevice.h"
#include "kernel_compatibility.h"

#include <drm/drm_pciids.h>
#include <linux/console.h>
#include <linux/module.h>
#include <linux/pm_runtime.h>
#include <linux/vga_switcheroo.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0)) || defined(REDHAT_83)
#include <drm/drm_probe_helper.h>
#else
#include <drm/drm_crtc_helper.h>
#endif
#include <linux/mmu_notifier.h>
#include <linux/suspend.h>
#include "vpu_xmem.h"
#include "pvr_gem.h"
#include "pm/xdx_fw.h"

#if defined(CONFIG_VGA_ARB)
#include <linux/vgaarb.h>
#endif

/* Kernel Module Parameters */
char *xdx_virtual_display = NULL;
int xdx_vram_limit;
int xdx_vis_vram_limit;
int xdx_msi_enable = 1;
int xdx_virtual_vblank;
int xdx_virtual_output;
int xdx_fake_mode;
uint32_t xdx_dc_heap_size = 0x14000000;

/**
 * DOC: virtual_display (charp)
 * Set to enable virtual display feature. This feature provides a virtual display hardware on headless boards
 * or in virtualized environments. It will be set like xxxx:xx:xx.x,x;xxxx:xx:xx.x,x. It's the pci address of
 * the device, plus the number of crtcs to expose. E.g., 0000:26:00.0,4 would enable 4 virtual crtcs on the pci
 * device at 26:00.0. The default is NULL.
 */
MODULE_PARM_DESC(virtual_display,
		 "Enable virtual display feature (the virtual_display will be set like xxxx:xx:xx.x,x;xxxx:xx:xx.x,x)");
module_param_named(virtual_display, xdx_virtual_display, charp, 0444);

/**
 * DOC: vramlimit (int)
 * Restrict the total amount of VRAM in MiB for testing.  The default is 0 (Use full VRAM).
 */
MODULE_PARM_DESC(vramlimit, "Restrict VRAM for testing, in megabytes");
module_param_named(vramlimit, xdx_vram_limit, int, 0600);

/**
 * DOC: vis_vramlimit (int)
 * Restrict the amount of CPU visible VRAM in MiB for testing.  The default is 0 (Use full CPU visible VRAM).
 */
MODULE_PARM_DESC(vis_vramlimit, "Restrict visible VRAM for testing, in megabytes");
module_param_named(vis_vramlimit, xdx_vis_vram_limit, int, 0444);

/**
 * DOC: virtual_vblank (int)
 * Using timer to simulate vblank interrupt.  The default is 0 (disabled virtual_vblank).
 */
MODULE_PARM_DESC(virtual_vblank, "using timer to simulate vblank interrupt");
module_param_named(virtual_vblank, xdx_virtual_vblank, int, 0600);

/**
 * DOC: virtual_output (int)
 * Create virtual encoder and connector  The default is 0 (disabled virtual_output).
 */
MODULE_PARM_DESC(virtual_output, "create a virtual encoder and connector");
module_param_named(virtual_output, xdx_virtual_output, int, 0600);

/**
 * DOC: fake_mode (int)
 * Using faked mode  The default is 0 (disabled fake_mode).
 */
MODULE_PARM_DESC(fake_mode, "create a virtual encoder and connector");
module_param_named(fake_mode, xdx_fake_mode, int, 0600);

/**
 * DOC: dc_heap_size (int)
 * reserved size for dc heap, the default size is 640M.
 */
MODULE_PARM_DESC(dc_heap_size, "reserved size for DC");
module_param_named(dc_heap_size, xdx_dc_heap_size, uint, 0600);

static DEFINE_MUTEX(xdx_connection_mutex);
static DEFINE_IDR(xdx_connection_idr);

/*
 * impl xdx_pm_runtime_resume_and_get
 */
static inline int xdx_pm_runtime_resume_and_get(struct device *dev)
{
	int ret;

	ret = __pm_runtime_resume(dev, RPM_GET_PUT);
	if (ret < 0) {
		pm_runtime_put_noidle(dev);
		return ret;
	}

	return 0;
}

static int xdx_drm_open(struct drm_device *ddev, struct drm_file *file_priv)
{
	struct xdx_device *xdev = drm_to_xdev(ddev);
	struct xdx_drm_fpriv *fpriv;
	int ret;

	file_priv->driver_priv = NULL;

#ifndef NO_HARDWARE
	ret = xdx_pm_runtime_resume_and_get (ddev->dev);
	if (ret < 0) {
		dev_err(xdev->dev, "failed to pm sync (%d)\n", ret);
		return ret;
	}
#endif

	fpriv = kzalloc(sizeof(*fpriv), GFP_KERNEL);
	if (unlikely(!fpriv)) {
		ret = -ENOMEM;
		goto out_suspend;
	}

	fpriv->dfile = file_priv;

	mutex_lock(&xdx_connection_mutex);
	ret = idr_alloc(&xdx_connection_idr, file_priv,
			1, 0, GFP_KERNEL);
	mutex_unlock(&xdx_connection_mutex);
	if (ret < 0) {
		dev_err(xdev->dev, "failed to alloc idr (%d)\n", ret);
		goto err_idr;
	}
	fpriv->connection_id = ret;

	ret = xdx_gfx_drm_open(xdev, fpriv);
	if (ret) {
		dev_err(xdev->dev, "gfx: failed to open device node (%d)\n", ret);
		goto err_gfx;
	}

#if defined(SUPPORT_VPU)
	ret = xdx_vpu_drm_open(xdev, fpriv);
	if (ret) {
		dev_err(xdev->dev, "vpu: failed to open device node (%d)\n", ret);
		goto err_vpu;
	}
#endif
	file_priv->driver_priv = fpriv;

	goto out_suspend;

#if defined(SUPPORT_VPU)
err_vpu:
#endif
err_gfx:
	mutex_lock(&xdx_connection_mutex);
	idr_remove(&xdx_connection_idr, fpriv->connection_id);
	mutex_unlock(&xdx_connection_mutex);
err_idr:
	kfree(fpriv);

out_suspend:
#ifndef NO_HARDWARE
	pm_runtime_mark_last_busy(ddev->dev);
	pm_runtime_put_autosuspend(ddev->dev);
#endif
	return ret;
}

static void xdx_drm_postclose(struct drm_device *ddev, struct drm_file *file_priv)
{
	struct xdx_device *xdev = drm_to_xdev(ddev);
	struct xdx_drm_fpriv *fpriv = (struct xdx_drm_fpriv *) file_priv->driver_priv;

	mutex_lock(&xdx_connection_mutex);
	idr_remove(&xdx_connection_idr, fpriv->connection_id);
	mutex_unlock(&xdx_connection_mutex);
	xdx_gfx_drm_release(xdev, file_priv->driver_priv);
#if defined(SUPPORT_VPU)
	xdx_vpu_drm_release(xdev, file_priv->driver_priv);
#endif
	kfree(file_priv->driver_priv);
	file_priv->driver_priv = NULL;
}

static void xdx_drm_lastclose(struct drm_device *ddev)
{
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 15, 0))
	struct xdx_device *xdev = drm_to_xdev(ddev);
	struct xdx_fbdev *fbdev = (struct xdx_fbdev *)xdev->fbdev;

	drm_fb_helper_restore_fbdev_mode_unlocked(&fbdev->helper);
#else
	drm_fb_helper_lastclose(ddev);
#endif
	vga_switcheroo_process_delayed_switch();
}

static int xdx_connection_info_query_ioctl(struct drm_device *ddev, void *data,
		       struct drm_file *filp)
{
	union drm_xdxgpu_connection_query *args = data;
	struct xdx_drm_fpriv *fpriv = (struct xdx_drm_fpriv *) filp->driver_priv;

	args->out.connection_id = fpriv->connection_id;

	return 0;
}

#if defined(SUPPORT_XDXGPU_OVERLAY_PLANE)

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 10, 0))
static inline bool drm_drv_uses_atomic_modeset(struct drm_device *dev)
{
        return drm_core_check_feature(dev, DRIVER_ATOMIC) ||
                (dev->mode_config.funcs && dev->mode_config.funcs->atomic_commit != NULL);
}
#endif

static int __xdx_setplane_atomic(struct drm_plane *plane,
			     struct drm_crtc *crtc,
			     struct drm_framebuffer *fb,
			     int32_t crtc_x, int32_t crtc_y,
			     uint32_t crtc_w, uint32_t crtc_h,
			     uint32_t src_x, uint32_t src_y,
			     uint32_t src_w, uint32_t src_h,
			     struct drm_modeset_acquire_ctx *ctx)
{
	WARN_ON(!drm_drv_uses_atomic_modeset(plane->dev));

	/* No fb means shut it down */
	if (!fb)
#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 12, 0))
		return plane->funcs->disable_plane(plane, ctx);
#else
		return plane->funcs->disable_plane(plane);
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 12, 0))
	return plane->funcs->update_plane(plane, crtc, fb,
					  crtc_x, crtc_y, crtc_w, crtc_h,
					  src_x, src_y, src_w, src_h, ctx);
#else
	return plane->funcs->update_plane(plane, crtc, fb,
					crtc_x, crtc_y, crtc_w, crtc_h,
					src_x, src_y, src_w, src_h);
#endif
}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0) && (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)) && (!defined(REDHAT_83)) && (!defined(CENTOS_81))
#define XDX_DRM_MODESET_LOCK_ALL_BEGIN(dev, ctx, flags, ret)		\
	drm_modeset_acquire_init(&ctx, flags);				\
modeset_lock_retry:							\
	ret = drm_modeset_lock_all_ctx(dev, &ctx);			\
	if (ret)							\
		goto modeset_lock_fail;

#define XDX_DRM_MODESET_LOCK_ALL_END(ctx, ret)				\
modeset_lock_fail:							\
	if (ret == -EDEADLK) {						\
		ret = drm_modeset_backoff(&ctx);			\
		if (!ret)						\
			goto modeset_lock_retry;			\
	}								\
	drm_modeset_drop_locks(&ctx);					\
	drm_modeset_acquire_fini(&ctx);
#endif
static int xdx_setplane_internal(struct drm_plane *plane,
			     struct drm_crtc *crtc,
			     struct drm_framebuffer *fb,
			     int32_t crtc_x, int32_t crtc_y,
			     uint32_t crtc_w, uint32_t crtc_h,
			     uint32_t src_x, uint32_t src_y,
			     uint32_t src_w, uint32_t src_h)
{
	int ret;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0) || defined(REDHAT_83) || defined(CENTOS_81)
	struct drm_modeset_acquire_ctx ctx;
	DRM_MODESET_LOCK_ALL_BEGIN(plane->dev, ctx,
				   DRM_MODESET_ACQUIRE_INTERRUPTIBLE, ret);
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	struct drm_modeset_acquire_ctx ctx;
	XDX_DRM_MODESET_LOCK_ALL_BEGIN(plane->dev, ctx,
				   DRM_MODESET_ACQUIRE_INTERRUPTIBLE, ret);
#else
	struct drm_modeset_acquire_ctx *ctx;
	drm_modeset_lock_all(plane->dev);
	ctx = plane->dev->mode_config.acquire_ctx;
#endif

	if (drm_drv_uses_atomic_modeset(plane->dev))
		ret = __xdx_setplane_atomic(plane, crtc, fb,
					crtc_x, crtc_y, crtc_w, crtc_h,
					src_x, src_y, src_w, src_h,
#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 10, 0))
					&ctx);
#else
					ctx);
#endif
	else
		ret = -EACCES;

#if ((LINUX_VERSION_CODE < KERNEL_VERSION(5, 8, 6)) && \
    (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0))) || defined(REDHAT_83) || defined(CENTOS_81)
	DRM_MODESET_LOCK_ALL_END(ctx, ret);
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 6))
	DRM_MODESET_LOCK_ALL_END(plane->dev, ctx, ret);
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	XDX_DRM_MODESET_LOCK_ALL_END(ctx, ret);
#else
	drm_modeset_unlock_all(plane->dev);
#endif
	return ret;
}

static int xdx_overlay_plane_set_ioctl(struct drm_device *dev, void *data,
						 struct drm_file *file_priv)
{
	struct drm_mode_set_plane *plane_req = data;
	struct drm_plane *plane;
	struct drm_crtc *crtc = NULL;
	struct drm_framebuffer *fb = NULL;
	int ret;
	if (!drm_core_check_feature(dev, DRIVER_MODESET))
		return -EOPNOTSUPP;
	/*
	 * First, find the plane, crtc, and fb objects.  If not available,
	 * we don't bother to call the driver.
	 */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
	plane = drm_plane_find(dev, file_priv, plane_req->plane_id);
#else
	plane = drm_plane_find(dev, plane_req->plane_id);
#endif
	if (!plane) {
		DRM_DEBUG_KMS("Unknown plane ID %d\n",
			      plane_req->plane_id);
		return -ENOENT;
	}

	if (plane->type != DRM_PLANE_TYPE_OVERLAY) {
		DRM_DEBUG_KMS("The Target plane:%d is not overlay plane\n",
			      plane_req->plane_id);
		return -ENOENT;
	}

	if (plane_req->fb_id) {
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
		fb = drm_framebuffer_lookup(dev, file_priv, plane_req->fb_id);
#else
		fb = drm_framebuffer_lookup(dev, plane_req->fb_id);
#endif
		if (!fb) {
			DRM_DEBUG_KMS("Unknown framebuffer ID %d\n",
				      plane_req->fb_id);
			return -ENOENT;
		}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
		crtc = drm_crtc_find(dev, file_priv, plane_req->crtc_id);
#else
		crtc = drm_crtc_find(dev, plane_req->crtc_id);
#endif
		if (!crtc) {
			drm_framebuffer_put(fb);
			DRM_DEBUG_KMS("Unknown crtc ID %d\n",
				      plane_req->crtc_id);
			return -ENOENT;
		}
	}

	ret = xdx_setplane_internal(plane, crtc, fb,
				plane_req->crtc_x, plane_req->crtc_y,
				plane_req->crtc_w, plane_req->crtc_h,
				plane_req->src_x, plane_req->src_y,
				plane_req->src_w, plane_req->src_h);

	if (fb)
		drm_framebuffer_put(fb);

	return ret;
}
#endif

static int xdx_info_ioctl(struct drm_device *dev, void *data, struct drm_file *filp)
{
	struct xdx_device *xdev = drm_to_xdev(dev);
	struct drm_xdxgpu_info *info = data;
	void __user *out = (void __user *)(uintptr_t)info->return_pointer;
	uint32_t size = info->return_size;

	if (!info->return_size || !info->return_pointer)
		return -EINVAL;

	switch (info->query) {
	case XDXGPU_INFO_PROCESS_INFO:
	{
		uint32_t real_size;
		IMG_UINT32 ui32LivePids;
		PVRSRV_PER_PROCESS_DATA *psPerProcessData;
		PVRSRV_ERROR err;

		err = PVRSRVQueryProcessDATA(&ui32LivePids,
						 &psPerProcessData);
		if (err) {
			dev_err(xdev->dev, "Fail to query process memory\n");
			return -err;
		}
		real_size =	ui32LivePids * sizeof(PVRSRV_PER_PROCESS_DATA);
		if ((size_t)size < real_size)
			dev_warn(xdev->dev, "return_size is too small\n");
		size = min(size, real_size);
		return copy_to_user(out, psPerProcessData, size) ? -EFAULT : 0;
	}
	case XDXGPU_INFO_UTIL_INFO:
	{
		PVRSRV_DEVICE_NODE *psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
		PVRSRV_DEVICE_DEBUG_INFO *psDebugInfo = &psDeviceNode->sDebugInfo;
		RGXFWIF_GPU_UTIL_STATS psGpuUtilStats;

		PVRSRV_RGXDEV_INFO *psDevInfo = psDeviceNode->pvDevice;
		PVRSRV_ERROR eError = PVRSRV_OK;

		eError = psDevInfo->pfnGetGpuUtilStats(psDeviceNode,
						       psDebugInfo->hGpuUtilUserDebugFS,
						       &psGpuUtilStats);

		return copy_to_user(out, &psGpuUtilStats,
				    min((size_t)size, sizeof(psGpuUtilStats))) ? -EFAULT : 0;
	}
	case XDXGPU_INFO_VRAM_INFO:
	{
		PVRSRV_DEVICE_NODE *psDeviceNode = (PVRSRV_DEVICE_NODE *)xdev->gfx.pvrdev;
		PVRSRV_PHYS_HEAP PhysHeapID = {PVRSRV_PHYS_HEAP_GPU_LOCAL};
		PHYS_HEAP_MEM_STATS sLocalMemInfo = {0};
		PVRSRV_ERROR eError;

		/*
		 * get information of PVRSRV_PHYS_HEAP_GPU_LOCAL only, if you
		 * want get other heap's info here, please change 1 to the
		 * number of heap wanted and add the heap idx into PhysHeapID
		 */
		eError = PVRSRVPhysHeapGetMemInfoKM(NULL, psDeviceNode, 1,
						    &PhysHeapID, &sLocalMemInfo);

		if (eError != PVRSRV_OK)
		{
			dev_err(xdev->dev, "%s, PVRSRVPhysHeapGetMemInfoKM error: %d!",
				__func__, eError);
			return -EFAULT;
		}

		size = min((size_t)size, sizeof(PHYS_HEAP_MEM_STATS));

		return copy_to_user(out, &sLocalMemInfo, size) ? -EFAULT : 0;
	}
	default:
		dev_err(xdev->dev, "Invalid request %d\n", info->query);
		return -EINVAL;
	}

	return 0;
}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 8, 0))
#define DRM_UNLOCKED		 BIT(4)
#endif

static const struct drm_ioctl_desc xdx_drm_ioctls[] = {
	DRM_IOCTL_DEF_DRV(PVR_SRVKM_CMD, PVRSRV_BridgeDispatchKM,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(PVR_SRVKM_INIT, drm_pvr_srvkm_init,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
#if defined(SUPPORT_NATIVE_FENCE_SYNC) && !defined(USE_PVRSYNC_DEVNODE)
	DRM_IOCTL_DEF_DRV(PVR_SYNC_RENAME_CMD, pvr_sync_rename_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(PVR_SYNC_FORCE_SW_ONLY_CMD, pvr_sync_force_sw_only_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(PVR_SW_SYNC_CREATE_FENCE_CMD, pvr_sw_sync_create_fence_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(PVR_SW_SYNC_INC_CMD, pvr_sw_sync_inc_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
#endif
	DRM_IOCTL_DEF_DRV(XDXGPU_GEM_CREATE, xdx_gem_create_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_GEM_QUERY, xdx_gem_info_query_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_EXPORT_TO_PVR_HANDLE, xdx_gem_export_to_pvr_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_IMPORT_FROME_PVR_HANDLE , xdx_gem_import_from_pvr_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_CONNECTION_QUERY , xdx_connection_info_query_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_INFO_QUERY , xdx_device_info_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
#if defined(SUPPORT_XDXGPU_OVERLAY_PLANE)
	DRM_IOCTL_DEF_DRV(XDXGPU_OVERLAY_PLANE_SET, xdx_overlay_plane_set_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),
#endif
	DRM_IOCTL_DEF_DRV(XDXGPU_INFO, xdx_info_ioctl,
			  DRM_RENDER_ALLOW | DRM_UNLOCKED),

#if defined(SUPPORT_VPU)
///////////////////////////codec ioctl start///////////////////////////////
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_PRIME_HANDLE_TO_FD, hantro_handle_to_fd,
		      DRM_UNLOCKED | DRM_RENDER_ALLOW),

	/*hantro specific ioctls*/
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_TESTCMD, hantro_test,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_GETPADDR, hantro_map_vaddr,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_HWCFG, hantro_get_hwcfg,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_TESTREADY, hantro_testbufvalid,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_SETDOMAIN, hantro_setdomain,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_ACQUIREBUF, hantro_acquirebuf,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_RELEASEBUF, hantro_releasebuf,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_GETPRIMEADDR, hantro_getprimeaddr,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_PTR_PHYADDR, hantro_ptr_to_phys,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_QUERY_METADATA, hantro_query_metadata,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_UPDATE_METADATA, hantro_update_metadata,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_GET_SLICENUM, hantro_get_slicenum,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),
	DRM_IOCTL_DEF_DRV(XDXGPU_VPU_HANTRO_GET_VCMDSUP, hantro_get_vcmdsup,
		      DRM_RENDER_ALLOW | DRM_UNLOCKED),

///////////////////////////codec ioctl end///////////////////////////////
#endif
};

static int xdx_fops_flush(struct file *f, fl_owner_t id)
{
/*	struct drm_file *file_priv = f->private_data;
	struct drm_device *ddev = file_priv->minor->dev;
	struct xdx_device *xdev = drm_to_xdev(ddev);
*/
	return 0;
}


//////////////////use drm_gem_mmap maybe more suitable /////////////////
#if 0
static int xdx_fops_mmap(struct file *filp, struct vm_area_struct *vma)
{
	int ret = 0;
	struct drm_gem_object *obj = NULL;
	struct drm_gem_hantro_object *cma_obj;
	struct drm_vma_offset_node *node;
	unsigned long page_num = (vma->vm_end - vma->vm_start) >> PAGE_SHIFT;
	int sgtidx = 0;
	struct scatterlist *pscatter = NULL;
	struct slice_info *pslice;
	struct device *dev;

	if (mutex_lock_interruptible(&hantro_dev.drm_dev->struct_mutex))
		return -EBUSY;
	drm_vma_offset_lock_lookup(hantro_dev.drm_dev->vma_offset_manager);
	node = drm_vma_offset_exact_lookup_locked(
		hantro_dev.drm_dev->vma_offset_manager, vma->vm_pgoff,
		vma_pages(vma));

	if (likely(node)) {
		obj = container_of(node, struct drm_gem_object, vma_node);
		if (!kref_get_unless_zero(&obj->refcount))
			obj = NULL;
	}
	drm_vma_offset_unlock_lookup(hantro_dev.drm_dev->vma_offset_manager);

	if (!obj) {
		mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
		return -EINVAL;
	}
	hantro_unref_drmobj(obj);
	cma_obj = to_drm_gem_hantro_obj(obj);

	if (page_num > cma_obj->num_pages) {
		mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
		return -EINVAL;
	}
	if (!(cma_obj->flag & HANTRO_GEM_FLAG_IMPORT)) {
		pslice = getslicenode(cma_obj->sliceidx);
		if (!pslice) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return -EINVAL;
		}
		dev = pslice->dev;
	} else {
		dev = obj->dev->dev;
	}
	if ((cma_obj->flag & HANTRO_GEM_FLAG_IMPORT) == 0) {
		if (cma_obj->vaddr == 0) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return -EINVAL;
		}
		ret = drm_gem_mmap_obj(
			obj, drm_vma_node_size(node) << PAGE_SHIFT, vma);

		if (ret) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return ret;
		}
	} else {
		pscatter = &cma_obj->sgt->sgl[sgtidx];
#ifdef __amd64__
		set_memory_uc((unsigned long)cma_obj->vaddr, (int)page_num);
#endif
		vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
		/*else mmap report uncached error for some importer, e.g. i915*/
	}

	if (cma_obj->flag & HANTRO_GEM_FLAG_USEVMALLOC) {
		int i, j;
		unsigned long uaddr = vma->vm_start;
		void *address = cma_obj->vaddr;
		struct page *pages = NULL;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0))
		vma->vm_flags |= VM_LOCKED;
		vma->vm_flags &= ~VM_PFNMAP;
#else
		vm_flags_set(vma, VM_LOCKED);
		vm_flags_clear(vma, VM_PFNMAP);
#endif
		for (i = 0; i < page_num; i++) {
			pages = (vmalloc_to_page(address));
			if (IS_ERR(pages) || !page_count(pages) ||
			    vm_insert_page(vma, uaddr, pages)) {
				pr_err("alloc page %d fail = %lx:%lx\n", i,
				       (unsigned long)address,
				       (unsigned long)pages);
				address = cma_obj->vaddr;
				for (j = 0; j < i; j++) {
					pages = (vmalloc_to_page(address));
					unref_page(pages);
					address += PAGE_SIZE;
				}
				mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
				return -ENOMEM;
			}
			ref_page(pages);
			uaddr += PAGE_SIZE;
			address += PAGE_SIZE;
		}
	} else {
#ifndef VSI_FPGA
		vma->vm_pgoff = 0;
		if (dma_mmap_coherent(dev, vma, cma_obj->vaddr, cma_obj->paddr,
				      page_num << PAGE_SHIFT)) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return -EAGAIN;
		}
#else
		remap_pfn_range(vma, vma->vm_start,
				cma_obj->paddr >> PAGE_SHIFT,
				(vma->vm_end - vma->vm_start), PAGE_SHARED);
#endif
	}
	vma->vm_private_data = cma_obj;
	mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
	return ret;
}
#endif

static int xdx_fops_mmap_common(struct file *filp, struct vm_area_struct *vma)
{
#if defined(SUPPORT_VPU)
	struct drm_file *priv = filp->private_data;
	struct drm_device *dev = priv->minor->dev;
	struct drm_gem_object *obj = NULL;
	struct drm_vma_offset_node *node;
	struct drm_gem_hantro_object *cma_obj;

#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 13, 0))
	if (drm_dev_is_unplugged(dev))
#else
	if (drm_device_is_unplugged(dev))
#endif
		return -ENODEV;

	hantro_mmaplog("%s :%lx", __func__, vma->vm_pgoff);
	if (vma->vm_pgoff < VSI_MMAP_ADDRES_CEIL_MMAP)
		return hantro_map_internal_address(filp, vma);

	drm_vma_offset_lock_lookup(dev->vma_offset_manager);
	vma->vm_pgoff -= VSI_MMAP_ADDRES_CEIL_MMAP;
	node = drm_vma_offset_exact_lookup_locked(dev->vma_offset_manager,
						  vma->vm_pgoff, vma_pages(vma));
	if (likely(node)) {
			obj = container_of(node, struct drm_gem_object, vma_node);
			if (!kref_get_unless_zero(&obj->refcount))
					obj = NULL;
	}
	drm_vma_offset_unlock_lookup(dev->vma_offset_manager);

	if (!obj)
			return -EINVAL;

	xdx_gem_object_put(obj);

	cma_obj = to_drm_gem_hantro_obj(obj);
	if (!cma_obj)
		return xdx_gem_mmap(filp,vma);
	else
		return hantro_mmap(filp,vma);

#else
	return xdx_gem_mmap(filp,vma);
#endif
}

static long xdx_fops_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	unsigned int nr = _IOC_NR(cmd);
	struct drm_file *file_priv = filp->private_data;
	struct drm_device *ddev = file_priv->minor->dev;
	long ret;
#ifndef NO_HARDWARE
	struct xdx_device *xdev = drm_to_xdev(ddev);

	if (cmd == VCMD_IOCH_POLLING_CMDBUF &&
	    atomic_read(&xdev->in_runpm))
		return 0;

	if (cmd != VCMD_IOCH_POLLING_CMDBUF) {
		ret = xdx_pm_runtime_resume_and_get(ddev->dev);
		if (ret < 0)
			return ret;
	}
#endif

	if (_IOC_TYPE(cmd) == DRM_IOCTL_BASE)
		ret = drm_ioctl(filp, cmd, arg);

#if defined(SUPPORT_VPU)
	else if (_IOC_TYPE(cmd) == HANTRO_IOC_MAGIC)
		ret = hantro_ioctl(filp, cmd, arg);
#endif
	else {
		printk("invalid xdx ioctl cmd type:%d nr:%d", _IOC_TYPE(cmd), nr);
		ret = -EINVAL;
	}
#ifndef NO_HARDWARE
	if (cmd != VCMD_IOCH_POLLING_CMDBUF) {
		pm_runtime_mark_last_busy(ddev->dev);
		pm_runtime_put_autosuspend(ddev->dev);
	}
#endif
	return ret;
}

static int xdx_fops_open(struct inode *inode, struct file *filp)
{
	int ret = 0;
#if defined(SUPPORT_VPU)
	int vcmd_en;
	hantro_ioctl_id ioctl_id_par;
#endif
	ret = drm_open(inode, filp);

#if defined(SUPPORT_VPU)
	ioctl_id_par.ID_PAR.node_idx = 0;
	vcmd_en = hantro_get_vcmdsup(NULL, &ioctl_id_par.data, NULL);
	//vcmd_en = 0;
	if (vcmd_en == 0) {
#if USE_HW == 1 /*hw init*/
#ifdef HAS_VC8000D
	hantrodec_open(inode, filp);
#endif
#ifdef HAS_CACHECORE
	cache_open(inode, filp);
#endif
#endif
	} else {
//		hantrovcmd_open(inode, filp);
	}
#endif
	return ret;
}

static int xdx_fops_release(struct inode *inode, struct file *filp)
{
	int ret = 0;
#if 1//temporary code

#if defined(SUPPORT_VPU)
	struct xdx_device *xdev = NULL;
	int vcmd_en;
#ifdef HAS_MMU
	int tmp = 0;
#endif
	hantro_ioctl_id ioctl_id_par;

	ioctl_id_par.ID_PAR.node_idx = 0;
	xdev = get_file_xdev(filp);
	vcmd_en = hantro_get_vcmdsup(NULL, &ioctl_id_par.data, NULL);
    //vcmd_en = 0;
    if (vcmd_en == 0) {
#if USE_HW == 1
#ifdef HAS_CACHECORE
	cache_release(filp);
#endif
#ifdef HAS_VC8000D
	hantrodec_release(filp);
#endif
#ifdef HAS_VC8000E
	hantroenc_release();
#endif
#ifdef HAS_MMU
	tmp = 0;
	if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0)
		hantroMMURelease(filp);
#endif
	} else {
		hantrovcmd_release(inode, filp);
#ifdef HAS_MMU
		tmp = 0;
		if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
			tmp = 1;
#endif
		}
		if(tmp == 0)
			vpu_mmu_release(filp);
#endif
	}
#endif
#endif
#endif

	ret = drm_release(inode, filp);

	return ret;
}

extern int hantro_mmap(struct file *filp, struct vm_area_struct *vma);
const struct file_operations xdx_drm_fops = {
	.owner = THIS_MODULE,
	.open = xdx_fops_open,
	.flush = xdx_fops_flush,
	.release = xdx_fops_release,
	.unlocked_ioctl = xdx_fops_ioctl,
	/**
	 * @brief abandon using PVRSRV_MMap which doesn't match libdrm framewrok,
	 * therefore we need refactoring the user mode function OSMMapPMR as
	 * the following steps:
	 * 1. change hPMR handle to GEM handle (new ioctl)
	 * 2. query the mmap offset with GEM handle (new ioctl)
	 * 3. call mmap with the new offset;
	 */
//	.mmap = hantro_mmap,
	.mmap = xdx_fops_mmap_common,
	.poll = drm_poll,
	.read = drm_read,
	.compat_ioctl = drm_compat_ioctl,
#ifdef CONFIG_PROC_FS
	.show_fdinfo = xdx_drm_show_fdinfo
#endif
};

int xdx_file_to_fpriv(struct file *filp, struct xdx_drm_fpriv **fpriv)
{
	struct drm_file *file;

	if (!filp)
		return -EINVAL;

	if (filp->f_op != &xdx_drm_fops) {
		return -EINVAL;
	}

	file = filp->private_data;
	*fpriv = file->driver_priv;
	return 0;
}

static void xdx_drm_release(struct drm_device *dev)
{
	struct xdx_device *xdev = drm_to_xdev(dev);

#ifndef NO_HARDWARE
	/* TODO: release software resouces */
	pci_set_drvdata(xdev->pdev, NULL);
#endif
#if LINUX_VERSION_CODE<KERNEL_VERSION(4, 10, 0)
	drm_dev_unregister(dev);
#elif LINUX_VERSION_CODE<KERNEL_VERSION(5, 8, 0)
	drm_dev_fini(dev);
#else
	drm_dev_put(dev);
#endif
	kfree(dev);
}
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 1, 0)) && (!defined(REDHAT_83))
static void put_pages(struct xdx_bo *bo)
{
	if (bo->pages) {
		pvr_pmr_put_pages(bo->psPMR, bo->pages);
		bo->pages = NULL;
	}
}

void xdx_bo_put_pages(struct xdx_bo *bo)
{
	put_pages(bo);
}
static int xdx_gem_object_open(struct drm_gem_object *gobj,
			       struct drm_file *file_priv)
{
/*	struct xdx_bo *bo = to_xdx_gem(gobj);
	struct xdx_device *xdev = drm_to_xdev(bo->base.dev);
	struct xdx_drm_fpriv *fpriv = file_priv->driver_priv;
*/
	// TODO: do nothing now, for future use
	return 0;
}

static void xdx_gem_object_close(struct drm_gem_object *gobj,
				 struct drm_file *file_priv)
{
/*	struct xdx_bo *bo = to_xdx_gem(gobj);
	struct xdx_device *xdev = drm_to_xdev(bo->base.dev);
	struct xdx_drm_fpriv *fpriv = file_priv->driver_priv;
*/
	// TODO: do nothing now, for future use
}
static void xdx_gem_object_free(struct drm_gem_object *gobj)
{
	struct xdx_bo *bo = to_xdx_gem(gobj);
	struct xdx_device *xdev = drm_to_xdev(bo->base.dev);

	xdx_bo_vunmap(bo);
	//("xdx_gem_object_free complete!\n");
#ifndef NO_HARDWARE
	if (!xdev->flags.virtual_display) {
		struct xdx_bo_va *va, *tmp;

		list_for_each_entry_safe (va, tmp, &bo->va_list, node)
			xdx_vm_bo_remove(&xdev->vmm, va);
	}
#endif

#ifdef SUPPORT_VPU
	vpumem_remove(NULL, bo);
#endif

	xdx_bo_reserve(bo, true);
	xdx_bo_put_pages(bo);
	pvr_pmr_put(bo->psPMR);
	xdx_bo_unreserve(bo);

	drm_gem_object_release(gobj);

	if (bo->cma_obj)
		kfree(bo->cma_obj);

	kvfree(bo);
}
static void *xdx_gem_object_vmap(struct drm_gem_object *gobj)
{
	struct xdx_bo *bo = to_xdx_gem(gobj);
	int ret;
    // printk("xdx_gem_object_vmap complete!\n");
	ret = xdx_bo_vmap(bo);
	if (ret)
		return NULL;

	return bo->vaddr;
}

static void xdx_gem_object_vunmap(struct drm_gem_object *gobj, void *vaddr)
{
	struct xdx_bo *bo = to_xdx_gem(gobj);
	// printk("xdx_gem_object_vunmap complete!\n");
	xdx_bo_vunmap(bo);
}

static struct dma_buf *xdx_gem_prime_export(struct drm_device *dev, struct drm_gem_object *gobj, int flags)
{
	struct xdx_bo *bo = to_xdx_gem(gobj);
	struct xdx_device *xdev = drm_to_xdev(bo->base.dev);
	struct dma_buf *buf;

	buf = pvr_export_dmabuf(xdev->gfx.pvrdev, bo->psPMR, gobj, flags);
	if (IS_ERR(buf))
		dev_err(xdev->dev, "failed to export obj %p to dmabuf\n", bo);
	return buf;
}
#endif
static const struct drm_driver xdx_drm_driver = {
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 1, 0)) && (!defined(REDHAT_83))
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0))
	.driver_features = DRIVER_ATOMIC | DRIVER_GEM | DRIVER_RENDER |
			   DRIVER_PRIME| DRIVER_MODESET,
#else
	.driver_features = DRIVER_ATOMIC | DRIVER_GEM | DRIVER_RENDER |
			   DRIVER_PRIME| DRIVER_MODESET | DRIVER_SYNCOBJ,
#endif
#else
	.driver_features = DRIVER_ATOMIC | DRIVER_GEM | DRIVER_RENDER |
			   DRIVER_MODESET | DRIVER_SYNCOBJ |
			   DRIVER_SYNCOBJ_TIMELINE,
#endif
	.open = xdx_drm_open,
	.postclose = xdx_drm_postclose,
	.lastclose = xdx_drm_lastclose,
	.ioctls = xdx_drm_ioctls,
	.num_ioctls = ARRAY_SIZE(xdx_drm_ioctls),
	.dumb_create = xdx_drm_dumb_create,
	.dumb_map_offset = xdx_drm_dumb_mmap,
	.fops = &xdx_drm_fops,
#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 10, 0))
	.release = xdx_drm_release,
#endif
	.prime_handle_to_fd = drm_gem_prime_handle_to_fd,
	.prime_fd_to_handle = drm_gem_prime_fd_to_handle,
	.gem_prime_import = xdx_gem_prime_import,
#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0))
	.gem_prime_mmap = xdx_gem_object_mmap,
#endif
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0))
	.get_vblank_counter = drm_vblank_no_hw_counter,
	.enable_vblank = xdx_drm_drv_enable_vblank,
	.disable_vblank = xdx_drm_drv_disable_vblank,
#endif
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 1, 0)) && (!defined(REDHAT_83))
    .gem_open_object = xdx_gem_object_open,
	.gem_free_object = xdx_gem_object_free,
	.gem_close_object = xdx_gem_object_close,
	.gem_prime_vmap = xdx_gem_object_vmap,
	.gem_prime_vunmap = xdx_gem_object_vunmap,
	.gem_vm_ops = &xdx_gem_vm_ops,
	.gem_prime_export = xdx_gem_prime_export,
#endif
	.name = DRIVER_NAME,
	.desc = DRIVER_DESC,
	.date = DRIVER_DATE,
	.major = KMS_DRIVER_MAJOR,
	.minor = KMS_DRIVER_MINOR,
	.patchlevel = KMS_DRIVER_PATCHLEVEL,
};

#ifndef NO_HARDWARE
#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(4, 9, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(4, 20, 0)))
static int xdx_remove_conflict_fb(struct pci_dev *pdev)
{
        struct apertures_struct *ap;
        bool primary = false;

        ap = alloc_apertures(1);
        if(!ap)
                return -ENOMEM;

        ap->ranges[0].base = pci_resource_start(pdev, 2);
        ap->ranges[0].size = pci_resource_len(pdev, 2);

        drm_fb_helper_remove_conflicting_framebuffers(ap, "xdxgpudrmfb", primary);
        kfree(ap);

        return 0;
}
#endif

static int xdx_pci_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
{
	struct drm_device *ddev;
	struct xdx_device *xdev;
	int ret, retry = 0;
	enum xdx_asic_type asic_type;

	asic_type = (enum xdx_asic_type)ent->driver_data;


	switch(asic_type) {
	case CHIP_PANGU_A0:
		dev_info(&pdev->dev, "Pangu A0 Graphics detected\n");
		break;
	case CHIP_PANGU_B0:
		dev_info(&pdev->dev, "Pangu B0 Graphics detected\n");
		break;
	default:
		dev_err(&pdev->dev, "wrong asic type\n");
		return -ENODEV;
	}
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0))
	xdev = devm_drm_dev_alloc(&pdev->dev,(struct drm_driver*)&xdx_drm_driver, typeof(*xdev), ddev);
	if (IS_ERR(xdev)) {
		dev_err(&pdev->dev, "failed to alloc drm dev\n");
		return PTR_ERR(xdev);
	}
#else
	ddev = drm_dev_alloc((struct drm_driver *)&xdx_drm_driver, &pdev->dev);
	if (IS_ERR(ddev))
		return PTR_ERR(ddev);

	xdev = kzalloc(sizeof(*xdev), GFP_KERNEL);
	if (xdev == NULL) {
		drm_dev_put(ddev);
		return -ENOMEM;
	}

	ddev->dev_private = (void *)xdev;
	xdev->ddev = ddev;
	ddev->pdev = pdev;
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0))
		ret = drm_aperture_remove_conflicting_pci_framebuffers(pdev,&xdx_drm_driver);
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 14, 0))
		ret = drm_aperture_remove_conflicting_pci_framebuffers(pdev,"xdxgpudrmfb");
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0))
        ret = drm_fb_helper_remove_conflicting_pci_framebuffers(pdev,"xdxgpudrmfb");
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0))
        ret = drm_fb_helper_remove_conflicting_pci_framebuffers(pdev,0,"xdxgpudrmfb");
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 9, 0))
        xdx_remove_conflict_fb(pdev);
#endif

	ret = xdx_device_resize_fb_bar(pdev,2,
			xdx_pci_rebar_get_possible_sizes(pdev,2));
	if(ret)
		dev_err(&pdev->dev,
			"error - pci_resize_fb_bar returned %d\n", ret);

	xdev->dev  = &pdev->dev;
	xdev->pdev = pdev;
	xdev->asic_type = asic_type;
	ddev = xdev_to_drm(xdev);

	ret = pci_enable_device(pdev);
	if (ret) {
		dev_err(xdev->dev, "failed to enable pci device, ret(%d)\n",
			ret);
		goto err_drm_dev_put;
	}

   	 #if((LINUX_VERSION_CODE>=KERNEL_VERSION(5,10,0))&&(LINUX_VERSION_CODE<KERNEL_VERSION(5,14,0)))
        ddev->pdev=pdev;
	#endif
	pci_set_master(pdev);

	pci_set_drvdata(pdev, ddev);

#if defined(CONFIG_VGA_ARB)
	vga_set_legacy_decoding(pdev, VGA_RSRC_NONE);
#endif
	dma_set_mask_and_coherent(xdev->dev, DMA_BIT_MASK(48));
	dma_set_max_seg_size(xdev->dev, UINT_MAX);

	ret = xdx_device_init(xdev);
	if (ret)
		goto err_pci;

retry_init:
	ret = drm_dev_register(ddev, ent->driver_data);
	if (ret == -EAGAIN && ++retry <= 3) {
		dev_info(xdev->dev, "retry init %d\n", retry);
		/* Don't request EX mode too frequently which is attacking */
		msleep(5000);
		goto retry_init;
	} else if (ret) {
		dev_err(xdev->dev, "failed to register drm device, ret(%d)\n", ret);
		goto err_drm_dev_unload;
	}

	xdx_sysfs_init(xdev);
	ret = xdx_debugfs_init(xdev);
	if (ret)
		dev_err(xdev->dev, "Creating debugfs files failed (%d).\n", ret);

	if (xdev->enable_runpm) {
		/* setup runtime pm */
		pm_runtime_use_autosuspend(ddev->dev);
		pm_runtime_set_autosuspend_delay(ddev->dev, 5000);
		pm_runtime_allow(ddev->dev);
		pm_runtime_mark_last_busy(ddev->dev);
		pm_runtime_put_autosuspend(ddev->dev);
	}

	return ret;

err_drm_dev_unload:
	xdx_device_fini(xdev);
err_pci:
	pci_disable_device(pdev);
err_drm_dev_put:
	drm_dev_put(ddev);
	kfree(xdev);
	return ret;
}

static void xdx_pci_remove(struct pci_dev *pdev)
{
	struct drm_device *ddev = pci_get_drvdata(pdev);
	struct xdx_device *xdev = drm_to_xdev(ddev);

	if (xdev->enable_runpm) {
		pm_runtime_get_sync(ddev->dev);
		pm_runtime_forbid(ddev->dev);
		pm_runtime_dont_use_autosuspend(ddev->dev);
	}

	xdx_sysfs_fini(xdev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 13, 0))
	drm_dev_unplug(ddev);
#else
	drm_unplug_dev(ddev);
#endif
	xdx_device_fini(xdev);
	pci_disable_device(pdev);
	pci_set_drvdata(pdev, NULL);
#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0)
	drm_dev_unregister(ddev);
#elif LINUX_VERSION_CODE < KERNEL_VERSION(5, 8, 0)
	drm_dev_fini(ddev);
#else
	drm_dev_put(ddev);
#endif
}

static void xdx_pci_shutdown(struct pci_dev *pdev)
{
#if 0
	struct drm_device *ddev = pci_get_drvdata(pdev);
	struct xdx_device *xdev = drm_to_xdev(ddev);

	drm_dev_unplug(ddev);
	drm_kms_helper_poll_disable(ddev);
	drm_atomic_helper_shutdown(ddev);

#ifdef SUPPORT_AUDIO
	xdx_audio_fini(xdev);
#endif

#ifdef SUPPORT_PANGU_ACPU
	xdx_switch_power_state(xdev, XDX_PS2);
#endif
#else
	/* TBD:
	 *  Move disable code above to pci_driver.remove handler.
	 *  We have not much to do in shutdown handler (Except
	 *  we should suspend all IP blocks when run in VM)
	 */
	return;
#endif
}

#endif /* NO_HARDWARE */

static int xdx_pmops_prepare(struct device *dev)
{
	dev_warn(dev, "%s: no implemented\n", __func__);
	return 0;
}

static void xdx_pmops_complete(struct device *dev)
{
	/* nothing to do */
	dev_warn(dev, "%s: no implemented\n", __func__);
}

static int xdx_pmops_suspend(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
	int r;

	atomic_set(&xdev->in_s3, 1);
	r = xdx_device_suspend(xdev, true);
	atomic_set(&xdev->in_s3, 0);

	return r;
}

static int xdx_pmops_resume(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
	int r;

	r = xdx_device_resume(xdev, true);

	return r;
}

static int xdx_pmops_freeze(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
	int r;

	atomic_set(&xdev->in_s4, 1);
	r = xdx_device_suspend(xdev, true);
	atomic_set(&xdev->in_s4, 0);
	if (r)
		return r;

#if 0
	return xdx_device_reset(xdev);
#else
	/* TBD:
	 *  Enable reset after we implemented it.
	 *  For now return OK or it will fail hibernation
	 */
	return 0;
#endif
}

static int xdx_pmops_thaw(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
	int r;

	atomic_set(&xdev->in_s4, 1);
	r = xdx_device_resume(xdev, true);
	atomic_set(&xdev->in_s4, 0);
	return r;
}

static int xdx_pmops_poweroff(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);

	return xdx_device_suspend(xdev, true);
}

static int xdx_pmops_restore(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);

	return xdx_device_resume(xdev, true);
}

static int xdxgpu_runtime_idle_check_display(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
	int ret = 0;

	if (xdx_device_is_dc_supported(xdev->asic_type)) {
		struct drm_crtc *crtc;

		drm_for_each_crtc(crtc, ddev) {
			drm_modeset_lock(&crtc->mutex, NULL);
			if (crtc->state->active)
				ret = -EBUSY;
			drm_modeset_unlock(&crtc->mutex);
			if (ret < 0)
				break;
		}
	} else {
		struct drm_connector *list_connector;
		struct drm_connector_list_iter iter;

		mutex_lock(&ddev->mode_config.mutex);
		drm_modeset_lock(&ddev->mode_config.connection_mutex, NULL);

		drm_connector_list_iter_begin(ddev, &iter);
		drm_for_each_connector_iter(list_connector, &iter) {
			if (list_connector->dpms ==  DRM_MODE_DPMS_ON) {
				ret = -EBUSY;
				break;
			}
		}

		drm_connector_list_iter_end(&iter);

		drm_modeset_unlock(&ddev->mode_config.connection_mutex);
		mutex_unlock(&ddev->mode_config.mutex);
	}

	if (ret == -EBUSY)
		dev_dbg(dev, "failing to power off - crtc active\n");
	return ret;
}

static int xdx_pmops_runtime_suspend(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
	int ret;

#if defined(SUPPORT_XDXGPU_RPM)
	int i;

	drm_kms_helper_poll_disable(xdev_to_drm(xdev));
	atomic_set(&xdev->in_runpm, 1);

	for (i = 0; i < xdev->num_ip_blocks; i++) {
		/* low power mode with display on is requested */
		if (xdev->ip_blocks[i]->version.type == XDX_IP_BLOCK_TYPE_DC) {
			if (xdev->fw_st_info.gddr_count > 1)
				ret = xdx_dm_rpm_suspend(&xdev->dm);
		}
		else
			ret = xdev->ip_blocks[i]->version.funcs.suspend(xdev);
		if (ret) {
			DRM_ERROR("runtime suspend of IP block <%s> failed %d",
				  xdev->ip_blocks[i]->version.name, ret);
			goto rpm_sus_err;
		}

		xdev->ip_blocks[i]->status.hw_init = false;
	}
	xdx_runtime_suspend(xdev);
#else
	atomic_set(&xdev->in_runpm, 1);
	ret = xdxgpu_runtime_idle_check_display(dev);
	if (ret)
		goto rpm_sus_err;
	ret = xdx_device_suspend(xdev, false);
#endif /* SUPPORT_XDXGPU_RPM */
	if (ret)
rpm_sus_err:
		atomic_set(&xdev->in_runpm, 0);
	return ret;
}

static int xdx_pmops_runtime_resume(struct device *dev)
{
	struct drm_device *ddev = dev_get_drvdata(dev);
	struct xdx_device *xdev = drm_to_xdev(ddev);
#if defined(SUPPORT_XDXGPU_RPM)
	int i;
	int ret;

	xdx_runtime_resume(xdev);
	/*
	 * pcie clock resume need few milliseconds after set PCI-D0
	 * pcie access during this resuming period would
	 * make host axi bus hang
	 * so wait 10ms here
	*/
	msleep(10);

	for (i = 0; i < xdev->num_ip_blocks; i++) {
		/* display is not turned off during rpm suspend*/
		if (xdev->ip_blocks[i]->version.type == XDX_IP_BLOCK_TYPE_DC) {
			if (xdev->fw_st_info.gddr_count > 1)
				ret = xdx_dm_rpm_resume(&xdev->dm);
		}
		else
			ret = xdev->ip_blocks[i]->version.funcs.resume(xdev);
		if (ret) {
			DRM_ERROR("runtime resume of IP block <%s> failed %d\n",
				  xdev->ip_blocks[i]->version.name, ret);
			return ret;
		}

		xdev->ip_blocks[i]->status.hw_init = true;
	}
#else
	xdx_device_resume(xdev, false);
#endif /* SUPPORT_XDXGPU_RPM */
	atomic_set(&xdev->in_runpm, 0);

	drm_kms_helper_poll_enable(xdev_to_drm(xdev));
	return 0;
}



static int xdx_pmops_runtime_idle(struct device *dev)
{
	int ret = 0;

	ret = xdxgpu_runtime_idle_check_display(dev);
	pm_runtime_mark_last_busy(dev);
	pm_runtime_autosuspend(dev);
	return ret;
}

static const struct dev_pm_ops xdx_pm_ops = {
	.prepare = xdx_pmops_prepare,
	.complete = xdx_pmops_complete,
	.suspend = xdx_pmops_suspend,
	.resume = xdx_pmops_resume,
	.freeze = xdx_pmops_freeze,
	.thaw = xdx_pmops_thaw,
	.poweroff = xdx_pmops_poweroff,
	.restore = xdx_pmops_restore,
	.runtime_suspend = xdx_pmops_runtime_suspend,
	.runtime_resume = xdx_pmops_runtime_resume,
	.runtime_idle = xdx_pmops_runtime_idle,
};

#ifndef NO_HARDWARE
/**
 * xdx_pci_error_detected - Called when a PCI error is detected.
 * @pdev: PCI device struct
 * @state: PCI channel state
 *
 * Description: Called when a PCI error is detected.
 *
 * Return: PCI_ERS_RESULT_NEED_RESET or PCI_ERS_RESULT_DISCONNECT.
 */
static pci_ers_result_t xdx_pci_error_detected(struct pci_dev *pdev,
					pci_channel_state_t state)
{
	dev_warn(&pdev->dev, "%s: no implemented\n", __func__);
	return PCI_ERS_RESULT_DISCONNECT;
}

/**
 * xdx_pci_mmio_enabled - Enable MMIO and dump debug registers
 * @pdev: pointer to PCI device
 */
static pci_ers_result_t xdx_pci_mmio_enabled(struct pci_dev *pdev)
{
	dev_info(&pdev->dev, "PCI error: mmio enabled callback!!\n");

	/* TODO - dump whatever for debugging purposes */

	/* This called only if xdx_pci_error_detected returns
	 * PCI_ERS_RESULT_CAN_RECOVER. Read/write to the device still
	 * works, no need to reset slot.
	 */
	return PCI_ERS_RESULT_RECOVERED;
}

/**
 * xdx_pci_slot_reset - Called when PCI slot has been reset.
 * @pdev: PCI device struct
 *
 * Description: This routine is called by the pci error recovery
 * code after the PCI slot has been reset, just before we
 * should resume normal operations.
 */
static pci_ers_result_t xdx_pci_slot_reset(struct pci_dev *pdev)
{
	dev_warn(&pdev->dev, "%s: no implemented\n", __func__);
	return PCI_ERS_RESULT_RECOVERED;
}

/**
 * xdx_pci_resume() - resume normal ops after PCI reset
 * @pdev: pointer to PCI device
 *
 * Called when the error recovery driver tells us that its
 * OK to resume normal operation.
 */
static void xdx_pci_resume(struct pci_dev *pdev)
{
	dev_warn(&pdev->dev, "%s: no implemented\n", __func__);
}

static struct pci_error_handlers xdx_pci_err_handler = {
	.error_detected = xdx_pci_error_detected,
	.mmio_enabled = xdx_pci_mmio_enabled,
	.slot_reset = xdx_pci_slot_reset,
	.resume = xdx_pci_resume,
};

static const struct pci_device_id pciidlist[] = {
	{ 0x1EED, 0x0100, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },
	{ 0x1EED, 0x1100, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },
        { 0x1EED, 0x10A0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX110
        { 0x1EED, 0x10F0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX115
        { 0x1EED, 0x1140, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX120
        { 0x1EED, 0x1150, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX120S
        { 0x1EED, 0x1160, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX121
        { 0x1EED, 0x1170, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX121S
        { 0x1EED, 0x1190, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX125
        { 0x1EED, 0x11E0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX130
        { 0x1EED, 0x1230, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX135
        { 0x1EED, 0x1280, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX140
        { 0x1EED, 0x1290, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX141
        { 0x1EED, 0x12D0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX145
        { 0x1EED, 0x1320, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX150
        { 0x1EED, 0x1330, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX150S
        { 0x1EED, 0x1340, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX151
        { 0x1EED, 0x1350, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX151S
        { 0x1EED, 0x1360, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX151T
        { 0x1EED, 0x1370, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX155
        { 0x1EED, 0x13C0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX160T
        { 0x1EED, 0x13D0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX160S
        { 0x1EED, 0x1410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX165
        { 0x1EED, 0x1460, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX170
        { 0x1EED, 0x14B0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX175
        { 0x1EED, 0x1500, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX180
        { 0x1EED, 0x1510, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX180S
        { 0x1EED, 0x1520, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX180T
        { 0x1EED, 0x1550, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX185
        { 0x1EED, 0x15A0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX190
        { 0x1EED, 0x15B0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX190S
        { 0x1EED, 0x15F0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX195
        { 0x1EED, 0x10A2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX110M
        { 0x1EED, 0x1142, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX120M
        { 0x1EED, 0x11E2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX130M
        { 0x1EED, 0x1323, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX R1500
        { 0x1EED, 0x1373, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX R1550
        { 0x1EED, 0x13C3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX R1600
        { 0x1EED, 0x1503, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX R1800
        { 0x1EED, 0x15A3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX R1900
        { 0x1EED, 0x1084, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX E1080
        { 0x1EED, 0x1094, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX E1090
        { 0x1EED, 0x10A4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_B0 },   //XDX E1100
        { 0x1EED, 0x1144, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX E1200
        { 0x1EED, 0x11E4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX E1300
        { 0x1EED, 0x1324, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1500
        { 0x1EED, 0x13D4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1600D
        { 0x1EED, 0x13E4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1600Q
        { 0x1EED, 0x13F4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1690Q
        { 0x1EED, 0x1404, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX V1600D
        { 0x1EED, 0x1414, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX V1600Q
        { 0x1EED, 0x1504, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1800
        { 0x1EED, 0x15A4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1900
        { 0x1EED, 0x15B4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1910
        { 0x1EED, 0x15C4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1900D
        { 0x1EED, 0x15D4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, CHIP_PANGU_A0 },   //XDX X1900Q
	{ 0, 0, 0 }
};
MODULE_DEVICE_TABLE(pci, pciidlist);

static struct pci_driver xdx_pci_driver = {
	.name = DRIVER_NAME,
	.id_table = pciidlist,
	.probe = xdx_pci_probe,
	.remove = xdx_pci_remove,
	.shutdown = xdx_pci_shutdown,
	.driver.pm = &xdx_pm_ops,
	.err_handler = &xdx_pci_err_handler,
};

#else
static int xdx_platform_probe(struct platform_device *pdev)
{
	struct drm_device *ddev;
	struct xdx_device *xdev;
	int ret;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0))
	xdev = devm_drm_dev_alloc(&pdev->dev, &xdx_drm_driver, typeof(*xdev), ddev);
	if (IS_ERR(xdev)) {
		dev_err(&pdev->dev, "failed to alloc drm dev\n");
		return PTR_ERR(xdev);
	}
#else
	ddev = drm_dev_alloc(&xdx_drm_driver, &pdev->dev);
	if (IS_ERR(ddev))
		return PTR_ERR(ddev);

	xdev = kzalloc(sizeof(*xdev), GFP_KERNEL);
	if (xdev == NULL) {
		drm_dev_put(ddev);
		return -ENOMEM;
	}

	ddev->dev_private = (void *)xdev;
	xdev->ddev = ddev;
	//ddev->pdev = pdev;
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))
			ret = drm_fb_helper_remove_conflicting_pci_framebuffers(pdev,"xdxgpudrmfb");
#else
			ret = drm_fb_helper_remove_conflicting_pci_framebuffers(pdev,0,"xdxgpudrmfb");
#endif

	xdev->dev  = &pdev->dev;
	xdev->pdev = pdev;
	xdev->asic_type = CHIP_PANGU_A0;
	ddev = xdev_to_drm(xdev);

	dev_set_drvdata(ddev->dev, ddev);

	ret = xdx_device_init(xdev);
	if (ret)
		goto err_drm_dev_put;

	ret = drm_dev_register(ddev, 0);
	if (ret)
		goto err_drm_dev_unload;

	drm_fbdev_generic_setup(ddev, 0);

	ret = xdx_debugfs_init(xdev);
	if (ret)
		dev_err(xdev->dev, "Creating debugfs files failed (%d).\n", ret);

	return ret;

err_drm_dev_unload:
	xdx_device_fini(xdev);
err_drm_dev_put:
	drm_dev_put(ddev);
	kfree(xdev);
	return ret;
}

static int xdx_platform_remove(struct platform_device *pdev)
{
	struct drm_device *ddev = platform_get_drvdata(pdev);
	struct xdx_device *xdev = drm_to_xdev(ddev);

	drm_dev_unregister(ddev);
	xdx_device_fini(xdev);
	drm_dev_put(ddev);
	kfree(xdev);

	return 0;
}

static void xdx_platform_shutdown(struct platform_device *pdev)
{
	struct drm_device *ddev = platform_get_drvdata(pdev);
	struct xdx_device *xdev = drm_to_xdev(ddev);

	xdx_device_ip_suspend(xdev);
}

static const struct of_device_id xdx_of_ids[] = {
	{},
};

#if !defined(CHROMIUMOS_KERNEL) || !defined(MODULE)
MODULE_DEVICE_TABLE(of, xdx_of_ids);
#endif

static struct platform_device_id xdx_platform_ids[] = {
	{ "xdxgpu", 0 },
	{ }
};

#if !defined(CHROMIUMOS_KERNEL) || !defined(MODULE)
MODULE_DEVICE_TABLE(platform, xdx_platform_ids);
#endif

static struct platform_driver xdx_platform_driver = {
	.driver = {
		.name = DRIVER_NAME,
		.of_match_table	= of_match_ptr(xdx_of_ids),
		.pm		= NULL,
	},
	.id_table		= xdx_platform_ids,
	.probe			= xdx_platform_probe,
	.remove			= xdx_platform_remove,
	.shutdown		= xdx_platform_shutdown,
};

static struct platform_device *pdevice;

static int xdx_platform_device_register(void)
{
#if defined(MODULE)
	struct platform_device_info dev_info = {
		.name = "xdxgpu",
		.id = -2,
#if defined(NO_HARDWARE)
		/* Not all cores have 40 bit physical support, but this
		 * will work unless > 32 bit address is returned on those cores.
		 * In the future this will be fixed more correctly.
		 */
		.dma_mask = DMA_BIT_MASK(40),
#else
		.dma_mask = DMA_BIT_MASK(32),
#endif
	};

	pdevice = platform_device_register_full(&dev_info);
	if (IS_ERR(pdevice)) {
		DRM_ERROR("unable to register device (err=%ld)\n", PTR_ERR(pdevice));
		return -ENODEV;
	}
#endif /* defined(MODULE) */

	return 0;
}

static void xdx_platform_device_unregister(void)
{
#if defined(MODULE)
	platform_device_unregister(pdevice);
#endif /* defined(MODULE) */
}

#endif /*NO_HARDWARE*/

static int __init xdx_drv_init(void)
{
	int ret;
#if LINUX_VERSION_CODE>=KERNEL_VERSION(5, 17, 0)
	if (drm_firmware_drivers_only()) {
		DRM_ERROR("video nomodeset has set.\n");
		return -EINVAL;
	}
#else
	if (vgacon_text_force()) {
		DRM_ERROR("VGACON disables xdxgpu kernel modesetting.\n");
		return -EINVAL;
	}
#endif

	ret = PVRSRVDriverInit();
	if (ret) {
		DRM_ERROR("failed to init XDXGPU gfx driver");
		return ret;
	}

#if defined(SUPPORT_DMA_TRANSFER)
	ret = xdx_dma_drv_init();
	if (ret) {
		DRM_ERROR("failed to init XDXGPU DMA driver");
		return ret;
	}
#endif

#ifndef NO_HARDWARE
	/* let modprobe override vga console setting */
	ret = pci_register_driver(&xdx_pci_driver);

#else
	ret = platform_driver_register(&xdx_platform_driver);
	if (ret)
		return ret;
	xdx_platform_device_register();
#endif
	return ret;
}

static void __exit xdx_drv_exit(void)
{
	PVRSRVDriverDeinit();
#if defined(SUPPORT_DMA_TRANSFER)
	xdx_dma_drv_exit();
#endif
#ifndef NO_HARDWARE
	pci_unregister_driver(&xdx_pci_driver);
#else
	xdx_platform_device_unregister();
	platform_driver_unregister(&xdx_platform_driver);
#endif
	mmu_notifier_synchronize();
}

module_init(xdx_drv_init);
module_exit(xdx_drv_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL and additional rights");
MODULE_FIRMWARE("xdxgpu/rgx.fw.35.4.1632.23");
MODULE_FIRMWARE("xdxgpu/rgx.sh.35.4.1632.23");
