/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/debugfs.h>
#include <linux/list.h>
#include <linux/module.h>
#include <linux/uaccess.h>
#include <linux/reboot.h>
#include <linux/syscalls.h>
#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 11, 0))
#include <drm/drm_debugfs.h>
#endif

#include "xdx_fdinfo.h"
#include "xdx_drm.h"
#include "xdx_device.h"

/**
 * @brief show the process information to DRM framework
 *        such as: memory usage, fence status and so on,
 *        call function seq_printf to print the above info
 * 
 * @param m 
 * @param f 
 */
void xdx_drm_show_fdinfo(struct seq_file *m, struct file *f)
{
	struct xdx_drm_fpriv *fpriv;
	uint32_t bus, dev, fn, domain;
	struct drm_file *file = f->private_data;
	struct xdx_device *xdev = drm_to_xdev(file->minor->dev);
	int ret;

	ret = xdx_file_to_fpriv(f, &fpriv);
	if (ret)
		return;

	bus = xdev->pdev->bus->number;
	domain = pci_domain_nr(xdev->pdev->bus);
	dev = PCI_SLOT(xdev->pdev->devfn);
	fn = PCI_FUNC(xdev->pdev->devfn);

	seq_printf(m, "pdev:\t%04x:%02x:%02x.%d\n",
        domain, bus, dev, fn);
    /* TDDO: add more */
}
