/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_VMM_H__
#define __XDX_VMM_H__

#include "xdx_shared.h"

struct xdx_bo;
struct xdx_device;

union xdx_address {
	struct {
		uint32_t low;
		uint32_t high;
	};
	uint64_t value;
};

struct xdx_mmu_funcs {
	void *(*create)(struct xdx_device *xdev);
	int (*destrory)(void *context);
	int (*bo_map)(void *context, struct xdx_bo *bo, union xdx_address *va);
	int (*bo_unmap)(void *context, struct xdx_bo *bo, union xdx_address va);
};

struct xdx_vm {
	char name[64];
	void *context;
	struct xdx_mmu_funcs *funcs;
	void *hw_mmu_mutex;
};

struct xdx_bo_va {
	struct list_head node;
	struct xdx_vm *vm;
	struct xdx_bo *bo;
	union xdx_address addr;
};

struct xdx_vmm {
	/* dc, vdma using global page table */
	struct xdx_vm dc;
	struct xdx_vm vdma;
	struct xdx_vm vpu;
};

int xdx_vmm_init(struct xdx_vmm *vmm);
int xdx_vmm_fini(struct xdx_vmm *vmm);

struct xdx_bo_va *xdx_vm_bo_find(struct xdx_vmm *vmm, struct xdx_vm *vm,
				 struct xdx_bo *bo);
struct xdx_bo_va *xdx_vm_bo_add(struct xdx_vmm *vmm, struct xdx_vm *vm,
				struct xdx_bo *bo);
void xdx_vm_bo_remove(struct xdx_vmm *vmm, struct xdx_bo_va *va);
#endif /* __XDX_VMM_H__ */
