/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/uaccess.h>
#include <linux/module.h>
#include <linux/vmalloc.h>
#include <linux/interrupt.h>
/* Our header */
#include "hantro.h"
#include "hantro_priv.h"
#include "hantro_device.h"
#include "hantro_vcmd.h"
#include "bidirect_list.h"
#include "vcmdswhwregisters.h"
#include <linux/delay.h>
#ifdef HAS_MMU
#include "hantro_mmu.h"
#endif
#ifdef SUPPORT_VPU_SMMU
#include "vpu_smmu.h"
#endif
#ifdef VSI_FPGA
#include "hantro_fpga.h"
#endif

#include "xdx_gem.h"
#include "pangu.h"
#include "xdxvpu_irq.h"
#include "vpu_xmem.h"
#include "osfunc_common.h"

#ifndef USE_DTB_PROBE
#define VCMD_INVALIDE_MODULE_TYPE				(-1)

#ifndef ASIC_SWREG_AMOUNT
#define ASIC_SWREG_AMOUNT                  		543 //from encswhwregister.h
#endif

#define WAIT_CMDBUF_TIMEOUT			3000

#define VCMD_ENCODER_REGISTER_SIZE 				(ASIC_SWREG_AMOUNT * 4)
#define VCMD_DECODER_REGISTER_SIZE              (1023 * 4)
#define VCMD_IM_REGISTER_SIZE 					(ASIC_SWREG_AMOUNT * 4)
#define VCMD_JPEG_ENCODER_REGISTER_SIZE 		(ASIC_SWREG_AMOUNT * 4)
#define VCMD_JPEG_DECODER_REGISTER_SIZE         (1023 * 4)

//video encoder vcmd configuration

#define VCMD_ENC_IO_ADDR_0      			(0x1b00000) /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_0 					(ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_ENC_INT_PIN_0 					-1
#define VCMD_ENC_MODULE_TYPE_0 				0
#define VCMD_ENC_MODULE_MAIN_ADDR_0 		(0x1000) /*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_0		(0xffff) //0x6000 /*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_0 		(0xffff)
#define VCMD_ENC_MODULE_MMU_ADDR_0 			(0x2000)
#define VCMD_ENC_MODULE_MMU1_ADDR_0 		(0xffff)
#define VCMD_ENC_MODULE_AXIFE0_ADDR_0 		(0x3000)
#define VCMD_ENC_MODULE_AXIFE1_ADDR_0 		(0Xffff)

#define VCMD_ENC_IO_ADDR_1                                                     \
	0x90218000 /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_1 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_ENC_INT_PIN_1 -1
#define VCMD_ENC_MODULE_TYPE_1 0
#define VCMD_ENC_MODULE_MAIN_ADDR_1                                            \
	0x1000 /*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_1                                          \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_1 0XFFFF
#define VCMD_ENC_MODULE_MMU_ADDR_1 0XFFFF
#define VCMD_ENC_MODULE_AXIFE0_ADDR_1 0XFFFF
#define VCMD_ENC_MODULE_AXIFE1_ADDR_1 0XFFFF

#define VCMD_ENC_IO_ADDR_2                                                     \
	0x92000 /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_2 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_ENC_INT_PIN_2 -1
#define VCMD_ENC_MODULE_TYPE_2 0
#define VCMD_ENC_MODULE_MAIN_ADDR_2                                            \
	0x0000 /*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_2                                          \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_2 0XFFFF
#define VCMD_ENC_MODULE_MMU_ADDR_2 0XFFFF
#define VCMD_ENC_MODULE_AXIFE0_ADDR_2 0XFFFF
#define VCMD_ENC_MODULE_AXIFE1_ADDR_2 0XFFFF

#define VCMD_ENC_IO_ADDR_3                                                     \
	0x93000 /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_3 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_ENC_INT_PIN_3 -1
#define VCMD_ENC_MODULE_TYPE_3 0
#define VCMD_ENC_MODULE_MAIN_ADDR_3                                            \
	0x0000 /*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_3                                          \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_3 0XFFFF
#define VCMD_ENC_MODULE_MMU_ADDR_3 0XFFFF
#define VCMD_ENC_MODULE_AXIFE0_ADDR_3 0XFFFF
#define VCMD_ENC_MODULE_AXIFE1_ADDR_3 0XFFFF

//video encoder cutree/IM  vcmd configuration

#define VCMD_IM_IO_ADDR_0 0x94000 /*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_0 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_IM_INT_PIN_0 -1
#define VCMD_IM_MODULE_TYPE_0 1
#define VCMD_IM_MODULE_MAIN_ADDR_0                                             \
	0x1000 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_0                                           \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_0 0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_0 0XFFFF
#define VCMD_IM_MODULE_AXIFE0_ADDR_0 0XFFFF
#define VCMD_IM_MODULE_AXIFE1_ADDR_0 0XFFFF

#define VCMD_IM_IO_ADDR_1 0xa1000 /*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_1 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_IM_INT_PIN_1 -1
#define VCMD_IM_MODULE_TYPE_1 1
#define VCMD_IM_MODULE_MAIN_ADDR_1                                             \
	0x0000 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_1                                           \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_1 0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_1 0XFFFF
#define VCMD_IM_MODULE_AXIFE0_ADDR_1 0XFFFF
#define VCMD_IM_MODULE_AXIFE1_ADDR_1 0XFFFF

#define VCMD_IM_IO_ADDR_2 0xa2000 /*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_2 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_IM_INT_PIN_2 -1
#define VCMD_IM_MODULE_TYPE_2 1
#define VCMD_IM_MODULE_MAIN_ADDR_2                                             \
	0x0000 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_2                                           \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_2 0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_2 0XFFFF
#define VCMD_IM_MODULE_AXIFE0_ADDR_2 0XFFFF
#define VCMD_IM_MODULE_AXIFE1_ADDR_2 0XFFFF

#define VCMD_IM_IO_ADDR_3 0xa3000 /*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_3 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_IM_INT_PIN_3 -1
#define VCMD_IM_MODULE_TYPE_3 1
#define VCMD_IM_MODULE_MAIN_ADDR_3                                             \
	0x0000 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_3                                           \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_3 0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_3 0XFFFF
#define VCMD_IM_MODULE_AXIFE0_ADDR_3 0XFFFF
#define VCMD_IM_MODULE_AXIFE1_ADDR_3 0XFFFF

//video decoder vcmd configuration

#define VCMD_DEC_IO_ADDR_0  			0x1a00000  /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_0 				(ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_DEC_INT_PIN_0 				-1
#define VCMD_DEC_MODULE_TYPE_0 			2
#define VCMD_DEC_MODULE_MAIN_ADDR_0     0x0800 //0x0800 /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_0   0xffff //0x2000 //0x6000 /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_0_B0 	0xFFFF  //0xFFFF
#define VCMD_DEC_MODULE_L2CACHE_ADDR_0 	0x0400  //0x2000
#define VCMD_DEC_MODULE_MMU_ADDR_0 		0x1400 //0XFFFF
#define VCMD_DEC_MODULE_MMU1_ADDR_0 	0xffff // 0x1400 //0XFFFF
#define VCMD_DEC_MODULE_AXIFE0_ADDR_0_B0 	0x0400 //0XFFFF
#define VCMD_DEC_MODULE_AXIFE0_ADDR_0 	0x0200 //0XFFFF
#define VCMD_DEC_MODULE_AXIFE1_ADDR_0 	0xffff

#define VCMD_DEC_IO_ADDR_1				0xb1000 /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_1 				(ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_DEC_INT_PIN_1 				-1
#define VCMD_DEC_MODULE_TYPE_1 			2
#define VCMD_DEC_MODULE_MAIN_ADDR_1   	0x0000 /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_1   0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_1 	0XFFFF
#define VCMD_DEC_MODULE_MMU_ADDR_1 		0XFFFF
#define VCMD_DEC_MODULE_AXIFE0_ADDR_1 	0XFFFF
#define VCMD_DEC_MODULE_AXIFE1_ADDR_1 	0xFFFF

#define VCMD_DEC_IO_ADDR_2              0xb2000 /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_2 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_DEC_INT_PIN_2 -1
#define VCMD_DEC_MODULE_TYPE_2 2
#define VCMD_DEC_MODULE_MAIN_ADDR_2                                            \
	0x0000 /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_2                                          \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_2 0XFFFF
#define VCMD_DEC_MODULE_MMU_ADDR_2 0XFFFF
#define VCMD_DEC_MODULE_AXIFE0_ADDR_2 0XFFFF
#define VCMD_DEC_MODULE_AXIFE1_ADDR_2 0xFFFF

#define VCMD_DEC_IO_ADDR_3                                                     \
	0xb3000 /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_3 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_DEC_INT_PIN_3 -1
#define VCMD_DEC_MODULE_TYPE_3 2
#define VCMD_DEC_MODULE_MAIN_ADDR_3                                            \
	0x0000 /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_3                                          \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_3 0XFFFF
#define VCMD_DEC_MODULE_MMU_ADDR_3 0XFFFF
#define VCMD_DEC_MODULE_AXIFE0_ADDR_3 0XFFFF
#define VCMD_DEC_MODULE_AXIFE1_ADDR_3 0xFFFF

//JPEG encoder vcmd configuration

#define VCMD_JPEGE_IO_ADDR_0                                                   \
	0x90218000 /*customer specify according to own platform*/
#define VCMD_JPEGE_IO_SIZE_0 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGE_INT_PIN_0 -1
#define VCMD_JPEGE_MODULE_TYPE_0 3
#define VCMD_JPEGE_MODULE_MAIN_ADDR_0                                          \
	0x1000 /*customer specify according to own platform*/
#define VCMD_JPEGE_MODULE_DEC400_ADDR_0                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGE_MODULE_L2CACHE_ADDR_0 0XFFFF
#define VCMD_JPEGE_MODULE_MMU_ADDR_0 0XFFFF
#define VCMD_JPEGE_MODULE_MMU1_ADDR_0 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE0_ADDR_0 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE1_ADDR_0 0XFFFF

#define VCMD_JPEGE_IO_ADDR_1                                                   \
	0xC1000 /*customer specify according to own platform*/
#define VCMD_JPEGE_IO_SIZE_1 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGE_INT_PIN_1 -1
#define VCMD_JPEGE_MODULE_TYPE_1 3
#define VCMD_JPEGE_MODULE_MAIN_ADDR_1                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGE_MODULE_DEC400_ADDR_1                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGE_MODULE_L2CACHE_ADDR_1 0XFFFF
#define VCMD_JPEGE_MODULE_MMU_ADDR_1 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE0_ADDR_1 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE1_ADDR_1 0XFFFF

#define VCMD_JPEGE_IO_ADDR_2                                                   \
	0xC2000 /*customer specify according to own platform*/
#define VCMD_JPEGE_IO_SIZE_2 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGE_INT_PIN_2 -1
#define VCMD_JPEGE_MODULE_TYPE_2 3
#define VCMD_JPEGE_MODULE_MAIN_ADDR_2                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGE_MODULE_DEC400_ADDR_2                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGE_MODULE_L2CACHE_ADDR_2 0XFFFF
#define VCMD_JPEGE_MODULE_MMU_ADDR_2 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE0_ADDR_2 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE1_ADDR_2 0XFFFF

#define VCMD_JPEGE_IO_ADDR_3                                                   \
	0xC3000 /*customer specify according to own platform*/
#define VCMD_JPEGE_IO_SIZE_3 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGE_INT_PIN_3 -1
#define VCMD_JPEGE_MODULE_TYPE_3 3
#define VCMD_JPEGE_MODULE_MAIN_ADDR_3                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGE_MODULE_DEC400_ADDR_3                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGE_MODULE_L2CACHE_ADDR_3 0XFFFF
#define VCMD_JPEGE_MODULE_MMU_ADDR_3 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE0_ADDR_3 0XFFFF
#define VCMD_JPEGE_MODULE_AXIFE1_ADDR_3 0XFFFF

//JPEG decoder vcmd configuration

#define VCMD_JPEGD_IO_ADDR_0                                                   \
	0xD0000 /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_0 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGD_INT_PIN_0 -1
#define VCMD_JPEGD_MODULE_TYPE_0 4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_0                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_0                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_0 0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_0 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE0_ADDR_0 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE1_ADDR_0 0XFFFF

#define VCMD_JPEGD_IO_ADDR_1                                                   \
	0xD1000 /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_1 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGD_INT_PIN_1 -1
#define VCMD_JPEGD_MODULE_TYPE_1 4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_1                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_1                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_1 0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_1 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE0_ADDR_1 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE1_ADDR_1 0XFFFF

#define VCMD_JPEGD_IO_ADDR_2                                                   \
	0xD2000 /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_2 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGD_INT_PIN_2 -1
#define VCMD_JPEGD_MODULE_TYPE_2 4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_2                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_2                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_2 0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_2 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE0_ADDR_2 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE1_ADDR_2 0XFFFF

#define VCMD_JPEGD_IO_ADDR_3                                                   \
	0xD3000 /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_3 (ASIC_VCMD_SWREG_AMOUNT * 4) /* bytes */
#define VCMD_JPEGD_INT_PIN_3 -1
#define VCMD_JPEGD_MODULE_TYPE_3 4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_3                                          \
	0x0000 /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_3                                        \
	0XFFFF /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_3 0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_3 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE0_ADDR_3 0XFFFF
#define VCMD_JPEGD_MODULE_AXIFE1_ADDR_3 0XFFFF

static struct vcmd_config vcmd_core_array[1][3] = {
	{
#ifdef HAS_VC8000E
		//encoder configuration
		{ VCMD_ENC_IO_ADDR_0,
		  VCMD_ENC_IO_SIZE_0,
		  VCMD_ENC_INT_PIN_0,
		  VCMD_ENC_MODULE_TYPE_0,
		  VCMD_ENC_MODULE_MAIN_ADDR_0,
		  VCMD_ENC_MODULE_DEC400_ADDR_0,
		  VCMD_ENC_MODULE_L2CACHE_ADDR_0,
		  VCMD_ENC_MODULE_MMU_ADDR_0,
		  VCMD_ENC_MODULE_MMU1_ADDR_0,
		  { VCMD_ENC_MODULE_AXIFE0_ADDR_0,
		    VCMD_ENC_MODULE_AXIFE1_ADDR_0 } },
#if 0
		{ VCMD_JPEGE_IO_ADDR_0,
		  VCMD_JPEGE_IO_SIZE_0,
		  VCMD_JPEGE_INT_PIN_0,
		  VCMD_JPEGE_MODULE_TYPE_0,
		  VCMD_JPEGE_MODULE_MAIN_ADDR_0,
		  VCMD_JPEGE_MODULE_DEC400_ADDR_0,
		  VCMD_JPEGE_MODULE_L2CACHE_ADDR_0,
		  VCMD_JPEGE_MODULE_MMU_ADDR_0,
		  VCMD_JPEGE_MODULE_MMU1_ADDR_0,
		  { VCMD_JPEGE_MODULE_AXIFE0_ADDR_0,
		    VCMD_JPEGE_MODULE_AXIFE1_ADDR_0 } },
#endif
#endif
#if HAS_VC8000D
		{ VCMD_DEC_IO_ADDR_0,
		  VCMD_DEC_IO_SIZE_0,
		  VCMD_DEC_INT_PIN_0,
		  VCMD_DEC_MODULE_TYPE_0,
		  VCMD_DEC_MODULE_MAIN_ADDR_0,
		  VCMD_DEC_MODULE_DEC400_ADDR_0,
		  VCMD_DEC_MODULE_L2CACHE_ADDR_0,
		  VCMD_DEC_MODULE_MMU_ADDR_0,
		  VCMD_DEC_MODULE_MMU1_ADDR_0,
		  { VCMD_DEC_MODULE_AXIFE0_ADDR_0,
		    VCMD_DEC_MODULE_AXIFE1_ADDR_0 }
		},

#endif
	},
};

#endif

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 10, 0))
#define from_timer(var, callback_timer, timer_fieldname) \
	container_of(callback_timer, typeof(*var), timer_fieldname)
#endif

#ifdef VPU_USE_HRTIMER
static enum hrtimer_restart vpu_time_handle(struct hrtimer *t)
#else
void vpu_time_handle(struct timer_list *t)
#endif
{
	struct vpu_timer *timer = from_timer(timer, t, timer);

	timer->delta++;
#ifdef VPU_USE_HRTIMER
	if (timer->delta > 9) {
#else
	if (timer->delta > 25) {
#endif
		timer->delta = 0;
		timer->count = timer->sum;
		timer->sum = 0;
	}

	timer->sum += timer->c;
	timer->c = 0;
#ifndef VPU_USE_HRTIMER
	mod_timer(&timer->timer, jiffies + msecs_to_jiffies(40));
#else
	timer->record = 0;
	hrtimer_forward_now(t, timer->kt);
	return HRTIMER_RESTART;
#endif
}

void vcmd_timer_init(vcmd_dev_str *subsys_dev)
{
	struct vpu_timer *timer = &subsys_dev->vcmd_timer;

	timer->c = 0;
	timer->count = 0;
	timer->sum = 0;
	timer->delta = 0;
#ifndef VPU_USE_HRTIMER
	timer_setup(&timer->timer, vpu_time_handle, 0);
	timer->timer.expires = jiffies + msecs_to_jiffies(40);
	add_timer(&timer->timer);
#else
	timer->record = 0;
	timer->kt = ktime_set(0, 100*1000*1000); //100ms
	hrtimer_init(&timer->timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
	timer->timer.function = vpu_time_handle;
	hrtimer_start(&timer->timer, timer->kt, HRTIMER_MODE_REL);
#endif
}

void vcmd_timer_deinit(vcmd_dev_str *subsys_dev)
{
	struct vpu_timer *timer = &subsys_dev->vcmd_timer;

#ifndef VPU_USE_HRTIMER
	del_timer(&timer->timer);
#else
	hrtimer_cancel(&timer->timer);
#endif
}

//#define VCMD_DEBUG_INTERNAL
static void printk_vcmd_register_debug(const void *hwregs, char *info);

static void allocate_cmdbuf_Index(vcmd_dev_str *subsys_dev, u16 i,
			   struct noncache_mem *new_cmdbuf_addr,
			   struct noncache_mem *new_status_cmdbuf_addr);

static void destroy_cmdbuf_node(bi_list_node *cmdbuf_node);			   

/*process manager object management*/
static struct process_manager_obj *create_process_manager_obj(void)
{
	struct process_manager_obj *process_manager_obj = NULL;

	process_manager_obj = vmalloc(sizeof(*process_manager_obj));
	if (!process_manager_obj) {
		XDX_DEV_ERROR("%s\n", "vmalloc for process_manager_obj fail!");
		return process_manager_obj;
	}
	memset(process_manager_obj, 0, sizeof(*process_manager_obj));
	return process_manager_obj;
}

static void
free_process_manager_obj(struct process_manager_obj *process_manager_obj)
{
	if (!process_manager_obj) {
		XDX_DEV_ERROR("%s\n", "free_process_manager_obj NULL");
		return;
	}
	//free current cmdbuf_obj
	vfree(process_manager_obj);
}

static bi_list_node *create_process_manager_node(void)
{
	bi_list_node *current_node = NULL;
	struct process_manager_obj *process_manager_obj = NULL;

	process_manager_obj = create_process_manager_obj();
	if (!process_manager_obj) {
		XDX_DEV_ERROR("%s\n", "create_process_manager_obj fail!");
		return NULL;
	}
	process_manager_obj->total_exe_time = 0;
	spin_lock_init(&process_manager_obj->spinlock);
	init_waitqueue_head(&process_manager_obj->wait_queue);
	current_node = bi_list_create_node();
	if (!current_node) {
		XDX_DEV_ERROR("%s\n", "bi_list_create_node fail!");
		free_process_manager_obj(process_manager_obj);
		return NULL;
	}
	current_node->data = (void *)process_manager_obj;
	return current_node;
}

static void free_process_manager_node(bi_list_node *process_node)
{
	struct process_manager_obj *process_manager_obj = NULL;

	if (!process_node) {
		return;
	}
	process_manager_obj = (struct process_manager_obj *)process_node->data;
	//free struct process_manager_obj
	free_process_manager_obj(process_manager_obj);
	//free current process_manager_obj entity.
	bi_list_free_node(process_node);
}

static long release_process_node_cleanup(bi_list *list)
{
	bi_list_node *new_process_node = NULL;

	while (1) {
		new_process_node = list->head;
		if (!new_process_node)
			break;
		//remove node from list
		bi_list_remove_node(list, new_process_node);
		//remove node from list
		free_process_manager_node(new_process_node);
	}
	return 0;
}

static long destroy_cmdbuf_node_cleanup(bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;

	while (1) {
		new_cmdbuf_node = list->head;
		if (!new_cmdbuf_node)
			break;
		//remove node from list
		bi_list_remove_node(list, new_cmdbuf_node);
		//remove node from list
		destroy_cmdbuf_node(new_cmdbuf_node);
	}
	return 0;
}

static int sku_check(struct vcmd_dev *vcmd_dev)
{
	u32 sku_size = vcmd_dev->sku_size;
	struct process_manager_obj *p = NULL;
	int ret = 0;
	vcmd_slice_str *cur_slice = NULL;
	int n_dec = 0;
	int n_enc = 0;

	cur_slice = get_vcmd_slice_head();
	list_for_each_entry(p, &vcmd_dev->pro_mngr, list) {
		if (p->subsys_dev != NULL) {
			if (p->subsys_dev == &cur_slice->dec_vcmd) {
				n_dec++;
			} else if (p->subsys_dev == &cur_slice->enc_vcmd) {
				n_enc++;
			} else {
				XDX_DEV_ERROR("subsys illegality \n");
			}
		} else {
			XDX_DEV_DEBUG("obj subsys_dev is NULL\n");
		}
	}

	XDX_DEV_ERROR("n_dec:%d, n_enc:%d \n", n_dec, n_enc);

	if (vcmd_dev->type == HANTRO_DECODER) {
		XDX_DEV_ERROR();
		switch (sku_size) {
		case 4096*2160*70:
			if (n_dec > 3)
				return -EBUSY;
			break;
		case 4096*2160*130:
			if (n_dec > 4)
				return -EBUSY;
			break;
		default:
			if (n_dec > 3)
				return -EBUSY;
			break;
		}
	} else if (vcmd_dev->type == HANTRO_ENCODER) {
		XDX_DEV_ERROR();
		switch (sku_size) {
		case 1920*1088*130:
			if (n_enc > 3)
				return -EBUSY;
			break;
		case 4096*2160*70:
			if (n_enc > 4)
				return -EBUSY;
			break;
		default:
			if (n_enc > 2)
				return -EBUSY;
			break;
		}
	} else {
		XDX_DEV_ERROR("bad type \n");
		return -EINVAL;
	}

	return ret;
}

static void create_kernel_process_manager(vcmd_dev_str *subsys_dev)
{
	bi_list_node *process_manager_node;
	struct process_manager_obj *process_manager_obj = NULL;

	process_manager_node = create_process_manager_node();
	process_manager_obj =
		(struct process_manager_obj *)process_manager_node->data;
	process_manager_obj->filp = NULL;
	bi_list_insert_node_tail(&subsys_dev->global_process_manager,
				 process_manager_node);
}
#if 0
static u32
wait_process_resource_rdy(struct process_manager_obj *process_manager_obj)
{
	return process_manager_obj->total_exe_time <=
	       PROCESS_MAX_SUM_OF_IMAGE_SIZE;
}
#else

static bool process_res_rdy(vcmd_dev_str *subsys_dev, u32 etime)
{

	while (1) {

#ifndef VPU_USE_HRTIMER
		if (etime * subsys_dev->vcmd_timer.c < subsys_dev->sku_size/25)
			return true;
#else
		//if (etime + subsys_dev->vcmd_timer.record <= subsys_dev->sku_size/10) {
		if (etime + subsys_dev->vcmd_timer.record <= subsys_dev->sku_size/5) {
			subsys_dev->vcmd_timer.record += etime;
			return true;
		}
#endif
		XDX_DEV_ERROR("process_res_rdy. etime:%d, subsys_dev->vcmd_timer.record %d\n", etime, subsys_dev->vcmd_timer.record);
		msleep(5);
	}

	return etime * subsys_dev->vcmd_timer.c < subsys_dev->sku_size;
} 
#endif

#define HANTRO_AXIFE_OFFSET 0
#define HANTRO_AXIFE_IOSIZE (64*4)
#define AXI_REG10_SW_FRONTEND_EN           10*4 //0x28
#define AXI_REG11_SW_WORK_MODE             11*4 //0x2c

static void AXIFEEnable(volatile unsigned char *hwregs)
{
	if (!hwregs)
		return;

	//AXI FE pass through
	iowrite32(0x0, (void *)(hwregs + HANTRO_AXIFE_OFFSET + AXI_REG11_SW_WORK_MODE));
	XDX_DEV_DEBUG("AXI FE: 0x2C = 0x%x\n", ioread32((void *)(hwregs + AXI_REG11_SW_WORK_MODE)));
	iowrite32(0x2, (void *)(hwregs + HANTRO_AXIFE_OFFSET + AXI_REG10_SW_FRONTEND_EN));
	XDX_DEV_DEBUG("AXI FE: 0x28 = 0x%x\n", ioread32((void *)(hwregs + AXI_REG10_SW_FRONTEND_EN)));
}

static int vcmd_reserve_IO(vcmd_dev_str *subsys_dev)
{
	u32 hwid;
	unsigned int i;
	int j;
	u32 found_hw = 0;
	vcmd_core_str *subsys_core;
	u64 base_addr, axife_base_addr;
	u64 size;
	struct vcmd_config *vcmd_cfg;
	volatile u8 *axife_hwregs;
	struct xdxvpu_dev_info *xdxvpu_pdev;

	subsys_core = subsys_dev->vcmd_core;

	for (i = 0; i < subsys_dev->subsys_num; i++) {
		vcmd_cfg = &subsys_core->vcmd_core_cfg;

		base_addr = vcmd_cfg->vcmd_base_addr;
		size = vcmd_cfg->vcmd_iosize;
		if (size != 0) {
#if 0
			if (!request_mem_region(base_addr, size,
						"hantrovcmd")) {
				XDX_DEV_ERROR("hantrovcmd: failed to reserve vcmd HW regs\n");
				return -EBUSY;
			}
			subsys_core->hwregs = (u8 *)ioremap_wc(base_addr, size);
#else
			xdxvpu_pdev = get_xdxvpu_dev_info(vcmd_cfg->sub_module_type);
			if (!xdxvpu_pdev)
				return -ENODEV;

			subsys_core->hwregs = (u8 *)xdxvpu_pdev->vcmd_reg_vaddr;
			XDX_DEV_DEBUG("subsys_core->hwregs:0%p\n",subsys_core->hwregs);
#endif
		}

		/*read hwid and check validness and store it*/
		hwid = (u32)ioread32((void *)subsys_core->hwregs);
		subsys_core->hw_version_id = hwid;

		/* check for vcmd HW ID */
		if (((hwid >> 16) & 0xFFFF) != VCMD_HW_ID) {
			XDX_DEV_DEBUG("hantrovcmd: HW not found at 0x%llx\n",
				(unsigned long long)subsys_core->vcmd_core_cfg
					.vcmd_base_addr);
			iounmap((void *)subsys_core->hwregs);
			release_mem_region(
				subsys_core->vcmd_core_cfg.vcmd_base_addr,
				subsys_core->vcmd_core_cfg.vcmd_iosize);
			subsys_core->hwregs = NULL;
			subsys_core = subsys_core->core_next;
			continue;
		}
		for (j = 0; j < 2; j++) {
			if (vcmd_cfg->submodule_axife_addr[j] != 0xffff) {
				axife_base_addr =
					base_addr +
					vcmd_cfg->submodule_axife_addr[j];
				XDX_DEV_DEBUG("%s: axife addr is=%llx\n", __func__,
					axife_base_addr);
				size = HANTRO_AXIFE_IOSIZE;
				if (!request_mem_region(axife_base_addr, size,
							"hantroaxife")) {
					XDX_DEV_ERROR("hantrovcmd: failed to reserve axife HW regs\n");
					return -EBUSY;
				}
				axife_hwregs =
					(u8 *)ioremap_wc(axife_base_addr, size);
				AXIFEEnable(axife_hwregs);
				iounmap((void *)axife_hwregs);
				release_mem_region(axife_base_addr, size);
			}
		}

		found_hw = 1;

		XDX_DEV_DEBUG("hantrovcmd: HW at base <0x%llx> with ID <0x%08x>\n",
			(unsigned long long)
				subsys_core->vcmd_core_cfg.vcmd_base_addr,
			hwid);
		subsys_core = subsys_core->core_next;
	}

	if (found_hw == 0) {
		XDX_DEV_ERROR("hantrovcmd: NO ANY HW found!!\n");
		return -1;
	}

	return 0;
}

static void vcmd_release_IO(vcmd_dev_str *subsys_dev)
{
	u32 i;
	vcmd_core_str *subsys_core;

	subsys_core = subsys_dev->vcmd_core;

	for (i = 0; i < subsys_dev->subsys_num; i++) {
		if (subsys_core->hwregs) {
			iounmap((void *)subsys_core->hwregs);
			release_mem_region(
				subsys_core->vcmd_core_cfg.vcmd_base_addr,
				subsys_core->vcmd_core_cfg.vcmd_iosize);
			subsys_core->hwregs = NULL;
		}
		subsys_core = subsys_core->core_next;
	}

}

static void vcmd_reset_asic(vcmd_dev_str *subsys_dev)
{
	unsigned int i, n;
	u32 result;
	vcmd_core_str *subsys_core;

	subsys_core = subsys_dev->vcmd_core;

	for (n = 0; n < subsys_dev->subsys_num; n++) {
		if (subsys_core->hwregs) {
			//disable interrupt at first
			vcmd_write_reg((const void *)subsys_core->hwregs,
				       VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
			//reset all
			vcmd_write_reg((const void *)subsys_core->hwregs,
				       VCMD_REGISTER_CONTROL_OFFSET, 0x0002);
			//read status register
			result =
				vcmd_read_reg((const void *)subsys_core->hwregs,
					      VCMD_REGISTER_INT_STATUS_OFFSET);
			//clean status register
			vcmd_write_reg((const void *)subsys_core->hwregs,
				       VCMD_REGISTER_INT_STATUS_OFFSET, result);
			for (i = VCMD_REGISTER_CONTROL_OFFSET;
			     i < subsys_core->vcmd_core_cfg.vcmd_iosize;
			     i += 4) {
				//set all register 0
				vcmd_write_reg(
					(const void *)subsys_core->hwregs, i,
					0x0000);
			}
			//enable all interrupt
			vcmd_write_reg((const void *)subsys_core->hwregs,
				       VCMD_REGISTER_INT_CTL_OFFSET,
				       0xffffffff);
		}
		subsys_core = subsys_core->core_next;
	}
}

static void vcmd_reset_current_asic(vcmd_core_str *subsys_core)
{
	u32 result;

	if (subsys_core->hwregs) {
		//disable interrupt at first
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
		//reset all
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_CONTROL_OFFSET, 0x0002);
		//read status register
		result = vcmd_read_reg((const void *)subsys_core->hwregs,
				       VCMD_REGISTER_INT_STATUS_OFFSET);
		//clean status register
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_INT_STATUS_OFFSET, result);
	}
}

static void vcmd_start(vcmd_core_str *subsys_core,
		       bi_list_node *first_linked_cmdbuf_node)
{
	u64 cmdbuf_bus_addr;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	struct vcmd_dev *subsys_dev = subsys_core->parent_dev;

	if (subsys_core->working_state == WORKING_STATE_IDLE) {
		if ((first_linked_cmdbuf_node) &&
		    subsys_core->sw_cmdbuf_rdy_num) {
			cmdbuf_obj = (struct cmdbuf_obj *)
					     first_linked_cmdbuf_node->data;
#ifdef VCMD_DEBUG_INTERNAL
			printk_vcmd_register_debug(
				(const void *)subsys_core->hwregs,
				"vcmd_start enters");
#endif

			//0x40
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_AXI_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_MASTER_OUT_CLK_GATE_DISABLE,
				/* this bit should be set 1 only
				 * when need to reset dec400.
				 */
				1);
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_CORE_CLK_GATE_DISABLE, 0);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_ABORT_MODE, 0);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_RESET_CORE, 0);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_RESET_ALL, 0);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_START_TRIGGER,
						       0);
			//0x48
			if (subsys_core->hw_version_id <= HW_ID_1_0_C) {
				vcmd_set_register_mirror_value(
					subsys_core->reg_mirror,
					HWIF_VCMD_IRQ_INTCMD_EN, 0xffff);
			} else {
				vcmd_set_register_mirror_value(
					subsys_core->reg_mirror,
					HWIF_VCMD_IRQ_JMPP_EN, 1);
				vcmd_set_register_mirror_value(
					subsys_core->reg_mirror,
					HWIF_VCMD_IRQ_JMPD_EN, 1);
			}

			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_IRQ_RESET_EN,
						       1);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_IRQ_ABORT_EN,
						       1);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_IRQ_CMDERR_EN,
						       1);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_IRQ_TIMEOUT_EN,
						       1);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_IRQ_BUSERR_EN,
						       1);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_IRQ_ENDCMD_EN,
						       1);
			//0x4c
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_TIMEOUT_EN, 1);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_TIMEOUT_CYCLES,
						       0x7fffffff);

			cmdbuf_bus_addr=
					  (u64)(cmdbuf_obj->cmdbuf_busAddress -
						    subsys_dev->base_ddr_addr);
#ifdef HAS_MMU
			cmdbuf_bus_addr= (u64)(cmdbuf_obj->mmu_cmdbuf_bus_address);
#endif
#ifdef SUPPORT_VPU_SMMU
			if(hantro_dev.asic_type == CHIP_PANGU_B0)
				cmdbuf_bus_addr= (u64)(cmdbuf_obj->smmu_cmdbuf_bus_address);
#endif
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR,
				(u32)(cmdbuf_bus_addr));
			if (sizeof(size_t) == 8) {
				vcmd_set_register_mirror_value(
					subsys_core->reg_mirror,
					HWIF_VCMD_EXECUTING_CMD_ADDR_MSB,
					(u32)((u64)(cmdbuf_bus_addr) >>
					      32));
			} else {
				vcmd_set_register_mirror_value(
					subsys_core->reg_mirror,
					HWIF_VCMD_EXECUTING_CMD_ADDR_MSB, 0);
			}
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_EXE_CMDBUF_LENGTH,
				(u32)((cmdbuf_obj->cmdbuf_size + 7) / 8));
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_RDY_CMDBUF_COUNT,
				subsys_core->sw_cmdbuf_rdy_num);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_MAX_BURST_LEN,
						       0x10);
			if (subsys_core->hw_version_id > HW_ID_1_0_C) {
				vcmd_write_register_value(
					(const void *)subsys_core->hwregs,
					subsys_core->reg_mirror,
					HWIF_VCMD_CMDBUF_EXECUTING_ID,
					(u32)cmdbuf_obj->cmdbuf_id);
			}
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x40,
				       subsys_core->reg_mirror[0x40 / 4]);
			vcmd_write_reg(
				(const void *)subsys_core->hwregs, 0x44,
				vcmd_read_reg((const void *)subsys_core->hwregs,
					      0x44));
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x48,
				       subsys_core->reg_mirror[0x48 / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x4c,
				       subsys_core->reg_mirror[0x4c / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x50,
				       subsys_core->reg_mirror[0x50 / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x54,
				       subsys_core->reg_mirror[0x54 / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x58,
				       subsys_core->reg_mirror[0x58 / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x5c,
				       subsys_core->reg_mirror[0x5c / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x60,
				       subsys_core->reg_mirror[0x60 / 4]);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x64,
				       0xffffffff); //not interrupt cpu

			subsys_core->working_state = WORKING_STATE_WORKING;
			//start
			vcmd_set_register_mirror_value(
				subsys_core->reg_mirror,
				HWIF_VCMD_MASTER_OUT_CLK_GATE_DISABLE,
				/*this bit should be set 1 only
				 *when need to reset dec400.
				 */
				0);
			vcmd_set_register_mirror_value(subsys_core->reg_mirror,
						       HWIF_VCMD_START_TRIGGER,
						       1);
			vcmd_write_reg((const void *)subsys_core->hwregs, 0x40,
				       subsys_core->reg_mirror[0x40 / 4]);
#ifdef VCMD_DEBUG_INTERNAL
			printk_vcmd_register_debug(subsys_core->hwregs,
						   "vcmd_start exits ");
#endif
		}
	}
}

/*cmdbuf object management*/
static struct cmdbuf_obj *create_cmdbuf_obj(void)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	cmdbuf_obj = vmalloc(sizeof(*cmdbuf_obj));
	if (!cmdbuf_obj) {
		XDX_DEV_ERROR("%s\n", "vmalloc for cmdbuf_obj fail!");
		return cmdbuf_obj;
	}
	memset(cmdbuf_obj, 0, sizeof(*cmdbuf_obj));
	return cmdbuf_obj;
}

static void free_cmdbuf_obj(struct cmdbuf_obj *cmdbuf_obj)
{
	if (!cmdbuf_obj) {
		XDX_DEV_ERROR("%s\n", "remove_cmdbuf_obj NULL");
		return;
	}
	//free current cmdbuf_obj
	vfree(cmdbuf_obj);
}

static void free_cmdbuf_mem(vcmd_dev_str *subsys_dev)
{
	wake_up_interruptible_all(&subsys_dev->vcmd_cmdbuf_memory_wait);
}

static bi_list_node *create_cmdbuf_node(vcmd_dev_str *subsys_dev, u16 uIndex)
{
	bi_list_node *current_node;
	struct cmdbuf_obj *cmdbuf_obj;
	struct noncache_mem new_cmdbuf_addr;
	struct noncache_mem new_status_cmdbuf_addr;

	allocate_cmdbuf_Index(subsys_dev, uIndex, &new_cmdbuf_addr, &new_status_cmdbuf_addr);

	cmdbuf_obj = create_cmdbuf_obj();
	if (!cmdbuf_obj) {
		XDX_DEV_ERROR("%s\n", "create_cmdbuf_obj fail!");
		return NULL;
	}
	cmdbuf_obj->cmdbuf_busAddress = new_cmdbuf_addr.busAddress;
	cmdbuf_obj->mmu_cmdbuf_bus_address = new_cmdbuf_addr.mmu_bus_address;
	cmdbuf_obj->cmdbuf_virtualAddress = new_cmdbuf_addr.virtualAddress;
#ifdef SUPPORT_VPU_SMMU
	if(hantro_dev.asic_type == CHIP_PANGU_B0)
		cmdbuf_obj->smmu_cmdbuf_bus_address = new_cmdbuf_addr.smmu_bus_address;
#endif
	cmdbuf_obj->cmdbuf_size = new_cmdbuf_addr.size;
	cmdbuf_obj->cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;
	//XDX_DEV_ERROR("CREATE CMDBUF index %x, id %x\n", uIndex, cmdbuf_obj->cmdbuf_id);
	cmdbuf_obj->status_busAddress = new_status_cmdbuf_addr.busAddress;
	cmdbuf_obj->status_virtualAddress =
		new_status_cmdbuf_addr.virtualAddress;
	cmdbuf_obj->status_size = new_status_cmdbuf_addr.size;
	cmdbuf_obj->uFreed = 1;
	current_node = bi_list_create_node();
	if (!current_node) {
		XDX_DEV_ERROR("%s\n", "bi_list_create_node fail!");
		free_cmdbuf_obj(cmdbuf_obj);
		return NULL;
	}
	current_node->data = (void *)cmdbuf_obj;
	current_node->next = NULL;
	current_node->previous = NULL;
	return current_node;
}

static int pop_cmdbuf(vcmd_dev_str *subsys_dev, bi_list_node **pcurrent_node)
{
	bi_list_node *new_cmdbuf_node;
	unsigned long flags;

	spin_lock_irqsave(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);
	
	new_cmdbuf_node = subsys_dev->global_vcmd_res.head;
	if (!new_cmdbuf_node)
	{
		spin_unlock_irqrestore(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);	
		return 0;
	}
	//remove node from list
	bi_list_remove_node(&subsys_dev->global_vcmd_res, new_cmdbuf_node);
	spin_unlock_irqrestore(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);
	*pcurrent_node = new_cmdbuf_node;

	return 1;
}

static bi_list_node *pop_cmdbuf_node(vcmd_dev_str *subsys_dev)
{
	bi_list_node *current_node = NULL;

	if (wait_event_interruptible(subsys_dev->vcmd_cmdbuf_memory_wait,
				     pop_cmdbuf(subsys_dev,
						     &current_node)))
	{
		XDX_DEV_ERROR("vcmd res NULL! *****************\n");
		return NULL;
	}
	current_node->next = NULL;
	current_node->previous = NULL;
	return current_node;
}

static void free_cmdbuf_node(vcmd_dev_str *subsys_dev,
			     bi_list_node *cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	unsigned long flags;

	if (!cmdbuf_node) {
		XDX_DEV_ERROR("%s\n", "remove_cmdbuf_node NULL");
		return;
	}

#ifdef VPU_XMEM
	if (xvcmd_debug_start)
		memset(cmdbuf_obj->cmdbuf_virtualAddress,0xdeaf,cmdbuf_obj->cmdbuf_size);
#endif

	spin_lock_irqsave(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);
	cmdbuf_obj = (struct cmdbuf_obj *)cmdbuf_node->data;
	if(0 == cmdbuf_obj->uFreed)
	{
		cmdbuf_obj->uFreed = 1;
		bi_list_insert_node_tail(&subsys_dev->global_vcmd_res, cmdbuf_node);
	}
	else
	{
		XDX_DEV_ERROR("free_cmdbuf_node id %x, already free**************\n", cmdbuf_obj->cmdbuf_id);
	}
	
	spin_unlock_irqrestore(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);
	

	//free cmdbuf mem in pool
	free_cmdbuf_mem(subsys_dev);
}

static void destroy_cmdbuf_node(bi_list_node *cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj;

	if (!cmdbuf_node) {
		XDX_DEV_ERROR("%s\n", "remove_cmdbuf_node NULL");
		return;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)cmdbuf_node->data;
#ifdef VPU_XMEM
	if (xvcmd_debug_start)
	 	memset(cmdbuf_obj->cmdbuf_virtualAddress,0xdeaf,cmdbuf_obj->cmdbuf_size);
#endif
	XDX_DEV_ERROR("destroy_cmdbuf_node id %x\n", cmdbuf_obj->cmdbuf_id);
	free_cmdbuf_obj(cmdbuf_obj);
	bi_list_free_node(cmdbuf_node);
}

//just remove, not free the node.
static bi_list_node *remove_cmdbuf_node_from_list(bi_list *list,
						  bi_list_node *cmdbuf_node)
{
	if (!cmdbuf_node) {
		XDX_DEV_ERROR("%s\n", "remove_cmdbuf_node_from_list  NULL");
		return NULL;
	}
	if (cmdbuf_node->next) {
		bi_list_remove_node(list, cmdbuf_node);
		return cmdbuf_node;
	} else {
		//the last one, should not be removed.
		return NULL;
	}
}

static long release_cmdbuf_node(vcmd_dev_str *subsys_dev, bi_list *list,
				bi_list_node *cmdbuf_node)
{
  bi_list_node *new_cmdbuf_node = NULL;
  struct cmdbuf_obj *cmdbuf_obj = NULL;
  /*get cmdbuf object according to cmdbuf_id*/
  new_cmdbuf_node = cmdbuf_node;
  if (!new_cmdbuf_node)
    return -1;
  //remove node from list
  new_cmdbuf_node = remove_cmdbuf_node_from_list(list, new_cmdbuf_node);
  if (new_cmdbuf_node) {
    //free node
    cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
    subsys_dev->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
    free_cmdbuf_node(subsys_dev, new_cmdbuf_node);
    return 0;
  }
  return 1;
}

//calculate executing_time of each vcmd
static u32 calculate_executing_time_after_node(bi_list_node *exe_cmdbuf_node)
{
	u32 time_run_all = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;

	while (1) {
		if (!exe_cmdbuf_node)
			break;
		cmdbuf_obj_temp = (struct cmdbuf_obj *)exe_cmdbuf_node->data;
		time_run_all += cmdbuf_obj_temp->executing_time;
		exe_cmdbuf_node = exe_cmdbuf_node->next;
	}
	return time_run_all;
}

static u32
calculate_executing_time_after_node_high_priority(bi_list_node *exe_cmdbuf_node)
{
	u32 time_run_all = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;

	if (!exe_cmdbuf_node)
		return time_run_all;
	cmdbuf_obj_temp = (struct cmdbuf_obj *)exe_cmdbuf_node->data;
	time_run_all += cmdbuf_obj_temp->executing_time;
	exe_cmdbuf_node = exe_cmdbuf_node->next;
	while (1) {
		if (!exe_cmdbuf_node)
			break;
		cmdbuf_obj_temp = (struct cmdbuf_obj *)exe_cmdbuf_node->data;
		if (cmdbuf_obj_temp->priority == CMDBUF_PRIORITY_NORMAL)
			break;
		time_run_all += cmdbuf_obj_temp->executing_time;
		exe_cmdbuf_node = exe_cmdbuf_node->next;
	}
	return time_run_all;
}

static void allocate_cmdbuf_Index(vcmd_dev_str *subsys_dev, u16 i,
			   struct noncache_mem *new_cmdbuf_addr,
			   struct noncache_mem *new_status_cmdbuf_addr)
{
	//there is one cmdbuf at least
	new_cmdbuf_addr->virtualAddress =
		subsys_dev->vcmd_buf_mem_pool->virtualAddress +
		i * CMDBUF_MAX_SIZE /
			4;
	new_cmdbuf_addr->busAddress =
		subsys_dev->vcmd_buf_mem_pool->busAddress +
		i * CMDBUF_MAX_SIZE;

	new_cmdbuf_addr->mmu_bus_address =
		subsys_dev->vcmd_buf_mem_pool->mmu_bus_address +
		i * CMDBUF_MAX_SIZE;
#ifdef SUPPORT_VPU_SMMU
	if(hantro_dev.asic_type == CHIP_PANGU_B0){
		new_cmdbuf_addr->smmu_bus_address =
			subsys_dev->vcmd_buf_mem_pool->smmu_bus_address +
			i * CMDBUF_MAX_SIZE;
	}
#endif
	new_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
	new_cmdbuf_addr->cmdbuf_id = i;
	new_status_cmdbuf_addr->virtualAddress =
		subsys_dev->vcmd_status_buf_mem_pool
			->virtualAddress +
		i * CMDBUF_MAX_SIZE / 4;
	new_status_cmdbuf_addr->busAddress =
		subsys_dev->vcmd_status_buf_mem_pool
			->busAddress +
		i * CMDBUF_MAX_SIZE;

	new_status_cmdbuf_addr->mmu_bus_address =
		subsys_dev->vcmd_status_buf_mem_pool
			->mmu_bus_address +
		i * CMDBUF_MAX_SIZE;
#ifdef SUPPORT_VPU_SMMU
	if(hantro_dev.asic_type == CHIP_PANGU_B0){
		new_status_cmdbuf_addr->smmu_bus_address =
			subsys_dev->vcmd_status_buf_mem_pool
				->smmu_bus_address +
			i * CMDBUF_MAX_SIZE;
	}
#endif
	new_status_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
	new_status_cmdbuf_addr->cmdbuf_id = i;
}

static bi_list_node *get_cmdbuf_node_in_list_by_addr(vcmd_dev_str *subsys_dev,
						     size_t cmdbuf_addr,
						     bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	new_cmdbuf_node = list->head;
	while (1) {
		if (!new_cmdbuf_node)
			return NULL;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (((cmdbuf_obj->cmdbuf_busAddress -
		      subsys_dev->base_ddr_addr) <= cmdbuf_addr) &&
		    (((cmdbuf_obj->cmdbuf_busAddress -
		       subsys_dev->base_ddr_addr + cmdbuf_obj->cmdbuf_size) >
		      cmdbuf_addr))) {
			return new_cmdbuf_node;
		}
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
	return NULL;
}

static int wait_abort_rdy(vcmd_core_str *subsys_core)
{
	return subsys_core->working_state == WORKING_STATE_IDLE;
}

static int select_vcmd(vcmd_dev_str *subsys_dev, bi_list_node *new_cmdbuf_node)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *curr_cmdbuf_node = NULL;
	bi_list *list = NULL;
	u32 executing_time = 0xffff;
	int counter = 0;
	unsigned long flags = 0;
	u32 hw_rdy_cmdbuf_num = 0;
	size_t exe_cmdbuf_addr = 0;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	u32 cmdbuf_id = 0;
	vcmd_core_str *subsys_core;
	vcmd_core_str *smallest_core;
	u16 *position;

	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;

	position = &subsys_dev->vcmd_position[cmdbuf_obj->module_type];

	//there is an empty vcmd to be used
	while (1) {
		subsys_core = subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
						      [*position];
		list = &subsys_core->list_manager;
		spin_lock_irqsave(&subsys_core->spinlock, flags);
		if (!list->tail) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			cmdbuf_obj->core_id = subsys_core->core_id;
			return 0;
		} else {
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
		}
		if (counter >=
		    subsys_dev->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}

	/*there is a vcmd which tail node -> cmdbuf_run_done == 1.
	 * It means this vcmd has nothing to do, so we select it.
	 */
	counter = 0;
	while (1) {
		subsys_core = subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
						      [*position];
		list = &subsys_core->list_manager;
		spin_lock_irqsave(&subsys_core->spinlock, flags);
		curr_cmdbuf_node = list->tail;
		if (!curr_cmdbuf_node) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			cmdbuf_obj->core_id = subsys_core->core_id;
			return 0;
		}
		cmdbuf_obj_temp = (struct cmdbuf_obj *)curr_cmdbuf_node->data;
		if (cmdbuf_obj_temp->cmdbuf_run_done == 1) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			cmdbuf_obj->core_id = subsys_core->core_id;
			return 0;
		} else {
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
		}
		if (counter >=
		    subsys_dev->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	//another case, tail = executing node, and
	//vcmd=pend state (finish but not generate interrupt)
	counter = 0;
	while (1) {
		subsys_core = subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
						      [*position];
		list = &subsys_core->list_manager;
		//read executing cmdbuf address
		if (subsys_core->hw_version_id <= HW_ID_1_0_C) {
			hw_rdy_cmdbuf_num = vcmd_get_register_value(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_EXE_CMDBUF_COUNT);
		} else {
			hw_rdy_cmdbuf_num =
				*(subsys_core->vcmd_reg_mem_virtualAddress +
				  VCMD_EXE_CMDBUF_COUNT);
			if (hw_rdy_cmdbuf_num != subsys_core->sw_cmdbuf_rdy_num)
				hw_rdy_cmdbuf_num += 1;
		}
		spin_lock_irqsave(&subsys_core->spinlock, flags);
		curr_cmdbuf_node = list->tail;
		if (!curr_cmdbuf_node) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			cmdbuf_obj->core_id = subsys_core->core_id;
			return 0;
		}

		if (subsys_core->sw_cmdbuf_rdy_num == hw_rdy_cmdbuf_num) {
			bi_list_insert_node_tail(list, new_cmdbuf_node);
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			cmdbuf_obj->core_id = subsys_core->core_id;
			return 0;
		} else {
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
		}
		if (counter >=
		    subsys_dev->vcmd_type_core_num[cmdbuf_obj->module_type])
			break;
	}
	//there is no idle vcmd,if low priority,calculate
	// exe time, select the least one.
	// or if high priority, calculate the exe time,
	// select the least one and abort it.
	if (cmdbuf_obj->priority == CMDBUF_PRIORITY_NORMAL) {
		counter = 0;
		//calculate total execute time of all devices
		while (1) {
			subsys_core =
				subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
							[*position];
			//read executing cmdbuf address
			if (subsys_core->hw_version_id <= HW_ID_1_0_C) {
				exe_cmdbuf_addr = VCMDGetAddrRegisterValue(
					(const void *)subsys_core->hwregs,
					subsys_core->reg_mirror,
					HWIF_VCMD_EXECUTING_CMD_ADDR);
				list = &subsys_core->list_manager;
				spin_lock_irqsave(&subsys_core->spinlock,
						  flags);
				//get the executing cmdbuf node.
				curr_cmdbuf_node =
					get_cmdbuf_node_in_list_by_addr(
						subsys_dev, exe_cmdbuf_addr,
						list);

				//calculate total execute time of this device
				subsys_core->total_exe_time =
					calculate_executing_time_after_node(
						curr_cmdbuf_node);
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
			} else {
				/*cmdbuf_id = vcmd_get_register_value(
				 *(const void *)dev->hwregs,dev->reg_mirror,
				 *HWIF_VCMD_CMDBUF_EXECUTING_ID);
				 */
				cmdbuf_id = *(
					subsys_core->vcmd_reg_mem_virtualAddress +
					EXECUTING_CMDBUF_ID_ADDR + 1);
				spin_lock_irqsave(&subsys_core->spinlock,
						  flags);
				if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM ||
				    cmdbuf_id == 0) {
					XDX_DEV_ERROR("cmdbuf_id greater than the ceiling !!\n");
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return -1;
				}
				//get the executing cmdbuf node.
				curr_cmdbuf_node =
					subsys_dev
						->global_cmdbuf_node[cmdbuf_id];
				if (!curr_cmdbuf_node) {
					list = &subsys_core->list_manager;
					curr_cmdbuf_node = list->head;
					while (1) {
						if (!curr_cmdbuf_node)
							break;
						cmdbuf_obj_temp =
							(struct cmdbuf_obj *)
								curr_cmdbuf_node
									->data;
						if (cmdbuf_obj_temp
							    ->cmdbuf_data_linked &&
						    cmdbuf_obj_temp->cmdbuf_run_done ==
							    0)
							break;
						curr_cmdbuf_node =
							curr_cmdbuf_node->next;
					}
				}

				//calculate total execute time of this device
				subsys_core->total_exe_time =
					calculate_executing_time_after_node(
						curr_cmdbuf_node);
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
			}
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
			if (counter >= subsys_dev->vcmd_type_core_num
					       [cmdbuf_obj->module_type])
				break;
		}

		//find the device with the least total_exe_time.
		counter = 0;
		executing_time = 0xffffffff;
		while (1) {
			subsys_core =
				subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
							[*position];
			if (subsys_core->total_exe_time <= executing_time) {
				executing_time = subsys_core->total_exe_time;
				smallest_core = subsys_core;
			}
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
			if (counter >= subsys_dev->vcmd_type_core_num
					       [cmdbuf_obj->module_type])
				break;
		}
		//insert list
		list = &smallest_core->list_manager;
		spin_lock_irqsave(&smallest_core->spinlock, flags);
		bi_list_insert_node_tail(list, new_cmdbuf_node);
		spin_unlock_irqrestore(&smallest_core->spinlock, flags);
		cmdbuf_obj->core_id = smallest_core->core_id;
		return 0;
	} else {
		//CMDBUF_PRIORITY_HIGH
		counter = 0;
		//calculate total execute time of all devices
		while (1) {
			subsys_core =
				subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
							[*position];
			if (subsys_core->hw_version_id <= HW_ID_1_0_C) {
				//read executing cmdbuf address
				exe_cmdbuf_addr = VCMDGetAddrRegisterValue(
					(const void *)subsys_core->hwregs,
					subsys_core->reg_mirror,
					HWIF_VCMD_EXECUTING_CMD_ADDR);
				list = &subsys_core->list_manager;
				spin_lock_irqsave(&subsys_core->spinlock,
						  flags);
				//get the executing cmdbuf node.
				curr_cmdbuf_node =
					get_cmdbuf_node_in_list_by_addr(
						subsys_dev, exe_cmdbuf_addr,
						list);

				//calculate total execute time of this device
				subsys_core->total_exe_time =
					calculate_executing_time_after_node_high_priority(
						curr_cmdbuf_node);
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
			} else {
				/*cmdbuf_id = vcmd_get_register_value(
				 *(const void *)dev->hwregs,
				 *dev->reg_mirror,
				 *HWIF_VCMD_CMDBUF_EXECUTING_ID);
				 */
				cmdbuf_id = *(
					subsys_core->vcmd_reg_mem_virtualAddress +
					EXECUTING_CMDBUF_ID_ADDR);
				spin_lock_irqsave(&subsys_core->spinlock,
						  flags);
				if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM ||
				    cmdbuf_id == 0) {
					XDX_DEV_ERROR("cmdbuf_id greater than the ceiling !!\n");
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return -1;
				}
				//get the executing cmdbuf node.
				curr_cmdbuf_node =
					subsys_dev
						->global_cmdbuf_node[cmdbuf_id];
				if (!curr_cmdbuf_node) {
					list = &subsys_core->list_manager;
					curr_cmdbuf_node = list->head;
					while (1) {
						if (!curr_cmdbuf_node)
							break;
						cmdbuf_obj_temp =
							(struct cmdbuf_obj *)
								curr_cmdbuf_node
									->data;
						if (cmdbuf_obj_temp
							    ->cmdbuf_data_linked &&
						    cmdbuf_obj_temp->cmdbuf_run_done ==
							    0)
							break;
						curr_cmdbuf_node =
							curr_cmdbuf_node->next;
					}
				}

				//calculate total execute time of this device
				subsys_core->total_exe_time =
					calculate_executing_time_after_node(
						curr_cmdbuf_node);
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
			}
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
			if (counter >= subsys_dev->vcmd_type_core_num
					       [cmdbuf_obj->module_type])
				break;
		}

		//find the smallest device.
		counter = 0;
		executing_time = 0xffffffff;
		while (1) {
			subsys_core =
				subsys_dev->vcmd_manager[cmdbuf_obj->module_type]
							[*position];
			if (subsys_core->total_exe_time <= executing_time) {
				executing_time = subsys_core->total_exe_time;
				smallest_core = subsys_core;
			}
			*position = *position + 1;
			if (*position >= subsys_dev->vcmd_type_core_num
						 [cmdbuf_obj->module_type])
				*position = 0;
			counter++;
			if (counter >= subsys_dev->vcmd_type_core_num
					       [cmdbuf_obj->module_type])
				break;
		}
		//abort the vcmd and wait
		vcmd_write_register_value((const void *)smallest_core->hwregs,
					  smallest_core->reg_mirror,
					  HWIF_VCMD_START_TRIGGER, 0);
		if (subsys_dev->type == HANTRO_ENCODER) {
			subsys_dev->software_triger_abort = 1;
			if (wait_event_interruptible(
				    smallest_core->wait_abort_queue,
				    wait_abort_rdy(subsys_core))) {
				subsys_dev->software_triger_abort = 0;
				return -ERESTARTSYS;
			}
			subsys_dev->software_triger_abort = 0;
		} else {
			if (wait_event_interruptible(
				    smallest_core->wait_abort_queue,
				    wait_abort_rdy(subsys_core))) {
				return -ERESTARTSYS;
			}
		}
		/* need to select inserting position again
		 *because hw maybe have run to the next node.
		 *CMDBUF_PRIORITY_HIGH
		 */
		spin_lock_irqsave(&smallest_core->spinlock, flags);
		curr_cmdbuf_node = smallest_core->list_manager.head;
		while (1) {
			//if list is empty or tail,insert to tail
			if (!curr_cmdbuf_node)
				break;
			cmdbuf_obj_temp =
				(struct cmdbuf_obj *)curr_cmdbuf_node->data;
			/*if find the first node which priority is normal,
			 *insert node prior to  the node
			 */
			if ((cmdbuf_obj_temp->priority ==
			     CMDBUF_PRIORITY_NORMAL) &&
			    (cmdbuf_obj_temp->cmdbuf_run_done == 0))
				break;
			curr_cmdbuf_node = curr_cmdbuf_node->next;
		}
		bi_list_insert_node_before(list, curr_cmdbuf_node,
					   new_cmdbuf_node);
		cmdbuf_obj->core_id = smallest_core->core_id;
		spin_unlock_irqrestore(&smallest_core->spinlock, flags);

		return 0;
	}
	return 0;
}

static inline void OSWriteMemoryBarrier(volatile void *hReadback)
{
	wmb();

	if (hReadback)
	{
		/* Force a read-back to memory to avoid posted writes on certain buses
		 * e.g. PCI(E)
		 */
		(void) (*(volatile u32 *)((void *) (hReadback)));
	}
}

static void vcmd_link_cmdbuf(vcmd_core_str *subsys_core,
			     bi_list_node *last_linked_cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *next_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	struct cmdbuf_obj *next_cmdbuf_obj = NULL;
	u32 *jmp_addr;
	u32 vcmd_array[3];
	u32 operation_code;
	u64 next_cmd_buf_addr;

	struct vcmd_dev *subsys_dev = subsys_core->parent_dev;

	new_cmdbuf_node = last_linked_cmdbuf_node;
	//for the first cmdbuf.
	if (new_cmdbuf_node) {
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_data_linked == 0) {
			subsys_core->sw_cmdbuf_rdy_num++;
			cmdbuf_obj->cmdbuf_data_linked = 1;
			subsys_core->duration_without_int = 0;
			if (cmdbuf_obj->has_end_cmdbuf == 0) {
				if (cmdbuf_obj->no_normal_int_cmdbuf == 1) {
					subsys_core->duration_without_int =
						cmdbuf_obj->executing_time;
					//maybe nop is modified, so write back.
					if (subsys_core->duration_without_int >= 
					    INT_MIN_SUM_OF_IMAGE_SIZE) {
						jmp_addr = cmdbuf_obj->cmdbuf_virtualAddress + (cmdbuf_obj->cmdbuf_size >> 2);
						operation_code = *(jmp_addr - 4);
						operation_code = JMP_IE_1 | operation_code;
						*(jmp_addr - 4) = operation_code;
						subsys_core->duration_without_int = 0;
					}
				}
			}
		}
	}
	while (1) {
		if (!new_cmdbuf_node)
			break;
		if (!new_cmdbuf_node->next)
			break;
		next_cmdbuf_node = new_cmdbuf_node->next;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		next_cmdbuf_obj = (struct cmdbuf_obj *)next_cmdbuf_node->data;
		if (cmdbuf_obj->has_end_cmdbuf == 0 && !next_cmdbuf_obj->cmdbuf_run_done) {
			//need to link, current cmdbuf link to next cmdbuf
			jmp_addr = cmdbuf_obj->cmdbuf_virtualAddress +
				   (cmdbuf_obj->cmdbuf_size >> 2);
			if (subsys_core->hw_version_id > HW_ID_1_0_C) {
				//set next cmdbuf id
				vcmd_array[2] = next_cmdbuf_obj->cmdbuf_id;
				//*(jmp_addr - 1) = next_cmdbuf_obj->cmdbuf_id;
			}
			else
			{
				vcmd_array[2] = *(jmp_addr - 1);
			}
			next_cmd_buf_addr = (u64)(next_cmdbuf_obj->cmdbuf_busAddress - subsys_dev->base_ddr_addr);

#ifdef HAS_MMU
			next_cmd_buf_addr = (u64)(next_cmdbuf_obj->mmu_cmdbuf_bus_address);
#endif
#ifdef SUPPORT_VPU_SMMU
			if(hantro_dev.asic_type == CHIP_PANGU_B0){
				next_cmd_buf_addr = (u64)(next_cmdbuf_obj->smmu_cmdbuf_bus_address);
			}
#endif
			if (sizeof(size_t) == 8) {
				vcmd_array[1] = (u32)((u64)(next_cmd_buf_addr) >> 32);
				// *(jmp_addr - 2) = (u32)(
				// 	(u64)(next_cmd_buf_addr) >>	32);
			} else {
				vcmd_array[1] = 0;
				//*(jmp_addr - 2) = 0;
			}
			vcmd_array[0] = (u32)(next_cmd_buf_addr);
			memcpy((jmp_addr - 3), vcmd_array, sizeof(vcmd_array)); 

			//*(jmp_addr - 3) = (u32)(next_cmd_buf_addr);
			operation_code = *(jmp_addr - 4);
			operation_code &= 0xFFFF0000;
			*(jmp_addr - 4) = (u32)(operation_code | JMP_RDY_1 | ((next_cmdbuf_obj->cmdbuf_size + 7) >> 3));
			// *(jmp_addr - 4) =
			// 	(u32)(operation_code | JMP_RDY_1 |
			// 	      ((next_cmdbuf_obj->cmdbuf_size + 7) / 8));
			
			// guarantee gddr write is complete to avoid cmd_err added by liyao
			OSWriteMemoryBarrier(jmp_addr - 4);
			//wmb(); 
			//(void) (*(volatile u32 *)((void *) (jmp_addr - 4)));

			next_cmdbuf_obj->cmdbuf_data_linked = 1;
			subsys_core->sw_cmdbuf_rdy_num++;
			//modify nop code of next cmdbuf
			if (next_cmdbuf_obj->has_end_cmdbuf == 0) {
				if (next_cmdbuf_obj->no_normal_int_cmdbuf ==
				    1) {
					subsys_core->duration_without_int +=
						next_cmdbuf_obj->executing_time;

					//maybe we see the modified nop before abort,
					//so need to write back.
					if (subsys_core->duration_without_int >=
					    INT_MIN_SUM_OF_IMAGE_SIZE) {
						jmp_addr = next_cmdbuf_obj->cmdbuf_virtualAddress + (next_cmdbuf_obj->cmdbuf_size >> 2);
						operation_code = *(jmp_addr - 4);
						operation_code = JMP_IE_1 | operation_code;
						*(jmp_addr - 4) = operation_code;
						subsys_core->duration_without_int = 0;
					}
				}
			} else {
				subsys_core->duration_without_int = 0;
			}
#ifdef VCMD_DEBUG_INTERNAL
		{
		u32 i;

		XDX_DEV_DEBUG("vcmd link, last cmdbuf content\n");
		for (i = cmdbuf_obj->cmdbuf_size / 4 - 8; i < cmdbuf_obj->cmdbuf_size / 4; i++)
			XDX_DEV_DEBUG("current linked cmdbuf data %d =0x%x\n",
				i, *(cmdbuf_obj->cmdbuf_virtualAddress + i));
		}
#endif
		}
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
}

/* delink all the cmd buffers from the cmdbuf in front of last_linked_cmdbuf_node
 * to head of the list. All the cmd bufs marked as X will be delinked.
 * E.g.,
 * X->X->...->X->L->L->        ...        ->L
 * ^             ^                          ^
 * head          last_linked_cmdbuf_node    tail
 */
static void vcmd_delink_cmdbuf(vcmd_core_str *subsys_core,
			       bi_list_node *last_linked_cmdbuf_node)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	new_cmdbuf_node = last_linked_cmdbuf_node;
	while (1) {
		if (!new_cmdbuf_node)
			break;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_data_linked)
			cmdbuf_obj->cmdbuf_data_linked = 0;
		else
			break;
		new_cmdbuf_node = new_cmdbuf_node->next;
	}
	subsys_core->sw_cmdbuf_rdy_num = 0;
}

static long reserve_cmdbuf(struct file *filp, vcmd_dev_str *subsys_dev,
			   struct exchange_parameter *input_para)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *process_manager_node = NULL;
	struct process_manager_obj *process_manager_obj = NULL;
	unsigned long flags;
	size_t offset;

	input_para->cmdbuf_id = 0;
	if (input_para->cmdbuf_size > CMDBUF_MAX_SIZE)
		return -1;
	spin_lock_irqsave(&subsys_dev->vcmd_process_manager_lock, flags);
	process_manager_node = subsys_dev->global_process_manager.head;
	while (1) {
		if (!process_manager_node) {
			//should not happen
			XDX_DEV_ERROR("hantrovcmd: ERROR process_manager_node !!\n");
			spin_unlock_irqrestore(
				&subsys_dev->vcmd_process_manager_lock, flags);
			return -1;
		}
		process_manager_obj = (struct process_manager_obj *)
					      process_manager_node->data;
		if (filp == process_manager_obj->filp)
			break;
		process_manager_node = process_manager_node->next;
		//break;
	}
	spin_unlock_irqrestore(&subsys_dev->vcmd_process_manager_lock, flags);

#if 1 //multiprocess hang
	spin_lock_irqsave(&process_manager_obj->spinlock, flags);
	process_manager_obj->total_exe_time += input_para->executing_time;
	spin_unlock_irqrestore(&process_manager_obj->spinlock, flags);
#endif

	if (wait_event_interruptible(
		    process_manager_obj->wait_queue,
#if 0
			wait_process_resource_rdy(process_manager_obj)))
#else
			process_res_rdy(subsys_dev, input_para->executing_time)))
#endif
		return -1;

	new_cmdbuf_node = pop_cmdbuf_node(subsys_dev);
	if (!new_cmdbuf_node)
		return -1;

	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	cmdbuf_obj->module_type = input_para->module_type;
	cmdbuf_obj->priority = input_para->priority;
	cmdbuf_obj->executing_time = input_para->executing_time;
	cmdbuf_obj->cmdbuf_reserve = input_para->cmdbuf_reserve;
	input_para->cmdbuf_size = cmdbuf_obj->cmdbuf_size;
	cmdbuf_obj->filp = filp;

	cmdbuf_obj->process_manager_obj = process_manager_obj;
	offset = offsetof(struct cmdbuf_obj, executing_status);
	memset((char *)cmdbuf_obj + offset, 0, sizeof(struct cmdbuf_obj) - offset);

	input_para->cmdbuf_id = cmdbuf_obj->cmdbuf_id;
	subsys_dev->global_cmdbuf_node[input_para->cmdbuf_id] = new_cmdbuf_node;

	return 0;
}

static long release_cmdbuf(struct file *filp, vcmd_dev_str *subsys_dev,
			   u16 cmdbuf_id)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *last_cmdbuf_node = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list *list = NULL;
	u32 module_type;
	unsigned long flags;
	vcmd_core_str *subsys_core;

	/*get cmdbuf object according to cmdbuf_id*/
	new_cmdbuf_node = subsys_dev->global_cmdbuf_node[cmdbuf_id];
	if (!new_cmdbuf_node) {
		//should not happen
		XDX_DEV_ERROR("hantrovcmd: ERROR cmdbuf_id !!\n");
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;

#if 1 // multiprocess hang
	if(cmdbuf_obj->filp !=filp)
	{
		XDX_DEV_ERROR("hantrovcmd:ERROR cmdbuf_id with mismatch progress filp!!\n");
		return -1;
	}
#endif

	module_type = cmdbuf_obj->module_type;
	//TODO
	if (down_interruptible(
		    &subsys_dev->vcmd_reserve_cmdbuf_sem[module_type]))
		return -ERESTARTSYS;

	subsys_core = get_dev_of_core(subsys_dev, cmdbuf_obj->core_id);

	list = &subsys_core->list_manager;
	cmdbuf_obj->cmdbuf_need_remove = 1;
	last_cmdbuf_node = new_cmdbuf_node->previous;
	while (1) {
		//remove current node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_need_remove == 1) {
			new_cmdbuf_node = remove_cmdbuf_node_from_list(
				list, new_cmdbuf_node);
			if (new_cmdbuf_node) {
				//free node
				subsys_dev->global_cmdbuf_node
					[cmdbuf_obj->cmdbuf_id] = NULL;
				if (cmdbuf_obj->process_manager_obj) {
					spin_lock_irqsave(
						&cmdbuf_obj->process_manager_obj
							 ->spinlock,
						flags);
					cmdbuf_obj->process_manager_obj
						->total_exe_time -=
						cmdbuf_obj->executing_time;
					spin_unlock_irqrestore(
						&cmdbuf_obj->process_manager_obj
							 ->spinlock,
						flags);
					wake_up_interruptible_all(
						&cmdbuf_obj->process_manager_obj
							 ->wait_queue);
				}
				free_cmdbuf_node(subsys_dev, new_cmdbuf_node);
			}
		}
		if (!last_cmdbuf_node)
			break;
		new_cmdbuf_node = last_cmdbuf_node;
		last_cmdbuf_node = new_cmdbuf_node->previous;
	}
	up(&subsys_dev->vcmd_reserve_cmdbuf_sem[module_type]);
	return 0;
}

static long release_cmdbuf_node_cleanup(vcmd_dev_str *subsys_dev, bi_list *list)
{
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	while (1) {
		new_cmdbuf_node = list->head;
		if (!new_cmdbuf_node)
			return 0;
		//remove node from list
		bi_list_remove_node(list, new_cmdbuf_node);
		//free node
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		subsys_dev->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
		free_cmdbuf_node(subsys_dev, new_cmdbuf_node);
	}
	return 0;
}

static bi_list_node *find_last_linked_cmdbuf(bi_list_node *current_node)
{
	bi_list_node *new_cmdbuf_node = current_node;
	bi_list_node *last_cmdbuf_node;
	struct cmdbuf_obj *cmdbuf_obj = NULL;

	if (!current_node)
		return NULL;
	last_cmdbuf_node = new_cmdbuf_node;
	new_cmdbuf_node = new_cmdbuf_node->previous;
	while (1) {
		if (!new_cmdbuf_node)
			return last_cmdbuf_node;
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->cmdbuf_data_linked)
			return new_cmdbuf_node;
		last_cmdbuf_node = new_cmdbuf_node;
		new_cmdbuf_node = new_cmdbuf_node->previous;
	}
	return NULL;
}

static long link_and_run_cmdbuf(struct file *filp, vcmd_dev_str *subsys_dev,
				struct exchange_parameter *input_para)
{
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *last_cmdbuf_node;
	u32 *jmp_addr;
	u32 op_code;
	//u32 temp_opcode;
	u32 record_last_cmdbuf_rdy_num;
	unsigned long flags;
	int return_value;
	u16 cmdbuf_id = input_para->cmdbuf_id;
	vcmd_core_str *subsys_core = NULL;

	new_cmdbuf_node = subsys_dev->global_cmdbuf_node[cmdbuf_id];
	if (!new_cmdbuf_node) {
		//should not happen
		XDX_DEV_ERROR("hantrovcmd: ERROR cmdbuf_id !!\n");
		return -1;
	}
	cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
	if (cmdbuf_obj->filp != filp) {
		//should not happen
		XDX_DEV_ERROR("hantrovcmd: ERROR cmdbuf_id !!\n");
		return -1;
	}
	cmdbuf_obj->cmdbuf_data_loaded = 1;
	cmdbuf_obj->cmdbuf_size = input_para->cmdbuf_size;
	cmdbuf_obj->waited = 0;

	//test nop and end opcode, then assign value.
	cmdbuf_obj->has_end_cmdbuf = 0; //0: has jmp opcode,1 has end code
	cmdbuf_obj->no_normal_int_cmdbuf = 0; //0: interrupt when JMP,1 not interrupt when JMP
	jmp_addr = cmdbuf_obj->cmdbuf_virtualAddress + (cmdbuf_obj->cmdbuf_size >> 2);
	op_code = *(jmp_addr - 4);
	//op_code = temp_opcode;
	//op_code >>= 27;
	//op_code <<= 27;

	/*we can't identify END opcode or JMP opcode,
	 *so we don't support END opcode in control sw and driver.
	 */
	if ((op_code & 0xF8000000) == OPCODE_JMP) {
		//op_code = temp_opcode;
		//op_code &= 0x02000000;
		if ((op_code & 0x02000000) == JMP_IE_1)
			cmdbuf_obj->no_normal_int_cmdbuf = 0;
		else
			cmdbuf_obj->no_normal_int_cmdbuf = 1;
	} else {
		XDX_DEV_ERROR("didn't support op_code in link_and_run_cmdbuf, re -1 \n ");
		return -1;
	}

	if (down_interruptible(&subsys_dev->vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]))
		return -ERESTARTSYS;

	return_value = select_vcmd(subsys_dev, new_cmdbuf_node);

	if (return_value)
		return return_value;

	subsys_core = get_dev_of_core(subsys_dev, cmdbuf_obj->core_id);

	input_para->core_id = cmdbuf_obj->core_id;
	//set ddr address for vcmd registers copy.
	if (subsys_core->hw_version_id > HW_ID_1_0_C) {
		//This logic depend UMD call CWLCollectReadRegData to read vcmd register
		//read vcmd executing register into ddr memory.
		//now core id is got and output ddr address of
		//vcmd register can be filled in.
		//each core has its own fixed output ddr address
		//of vcmd registers.
		u32 *vcmd_rreg_data = NULL;
		u32 jmp_cmd_size = 4;
		u32 rreg_cmd_size = 4;

		vcmd_rreg_data = cmdbuf_obj->cmdbuf_virtualAddress;

		if (sizeof(size_t) == 8) {
			*(vcmd_rreg_data + 2) = (u32)(
				(u64)(subsys_core->vcmd_reg_mem_busAddress +
				      (EXECUTING_CMDBUF_ID_ADDR + 1) * 4) >>
				32);
		} else {
			*(vcmd_rreg_data + 2) = 0;
		}
		*(vcmd_rreg_data + 1) = (u32)((subsys_core->vcmd_reg_mem_busAddress +
					 (EXECUTING_CMDBUF_ID_ADDR + 1) * 4));
#ifdef HAS_MMU
		*(vcmd_rreg_data + 1) = (u32)((subsys_core->mmu_vcmd_reg_mem_bus_address +
					(EXECUTING_CMDBUF_ID_ADDR + 1) * 4));
		*(vcmd_rreg_data + 2) = 0;
#endif
#ifdef SUPPORT_VPU_SMMU
		if(hantro_dev.asic_type == CHIP_PANGU_B0){
			*(vcmd_rreg_data + 1) = (u32)((u64)(subsys_core->smmu_vcmd_reg_mem_bus_address +
						(EXECUTING_CMDBUF_ID_ADDR + 1) * 4));
			*(vcmd_rreg_data + 2) = (u32)(
					(u64)(subsys_core->smmu_vcmd_reg_mem_bus_address +
						(EXECUTING_CMDBUF_ID_ADDR + 1) * 4) >>
					32);
		}
#endif
		vcmd_rreg_data = cmdbuf_obj->cmdbuf_virtualAddress +
			(cmdbuf_obj->cmdbuf_size / 4) - jmp_cmd_size - rreg_cmd_size;

		//read vcmd all registers into ddr memory.
		//now core id is got and output ddr address
		//of vcmd registers can be filled in.
		//each core has its own fixed output ddr
		//address of vcmd registers.
		if (sizeof(size_t) == 8) {
			*(vcmd_rreg_data + 2) = (u32)(
				(u64)subsys_core->vcmd_reg_mem_busAddress >>
				32);
		} else {
			*(vcmd_rreg_data + 2) = 0;
		}
		*(vcmd_rreg_data + 1) = (u32)(subsys_core->vcmd_reg_mem_busAddress);

#ifdef HAS_MMU

		*(vcmd_rreg_data + 1) = (u32)(subsys_core->mmu_vcmd_reg_mem_bus_address);
		*(vcmd_rreg_data + 2) = 0;
#endif
#ifdef SUPPORT_VPU_SMMU
		if(hantro_dev.asic_type == CHIP_PANGU_B0){
			*(vcmd_rreg_data + 1) = (u32)(subsys_core->smmu_vcmd_reg_mem_bus_address);
			*(vcmd_rreg_data + 2) = (u32)(subsys_core->smmu_vcmd_reg_mem_bus_address>>32);
		}
#endif
	}
	//start to link and/or run
	spin_lock_irqsave(&subsys_core->spinlock, flags);
	last_cmdbuf_node = find_last_linked_cmdbuf(new_cmdbuf_node);
	record_last_cmdbuf_rdy_num = subsys_core->sw_cmdbuf_rdy_num;
	vcmd_link_cmdbuf(subsys_core, last_cmdbuf_node);
	if (subsys_core->working_state == WORKING_STATE_IDLE) {
		//run
		while (last_cmdbuf_node &&
			  ((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done)
			last_cmdbuf_node = last_cmdbuf_node->next;

		if (last_cmdbuf_node && last_cmdbuf_node->data) {
			XDX_DEV_DEBUG("vcmd start for cmdbuf id %d, cmdbuf_run_done = %d\n",
			((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_id,
			((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_run_done);
		}
		vcmd_start(subsys_core, last_cmdbuf_node);
	} else {
		//just update cmdbuf ready number
		XDX_DEV_DEBUG("subsys_core state = %d, cmdbuf rdy num updated %d -> %d\n",
			subsys_core->working_state, record_last_cmdbuf_rdy_num,
			subsys_core->sw_cmdbuf_rdy_num);
		if (record_last_cmdbuf_rdy_num !=
		    subsys_core->sw_cmdbuf_rdy_num)
			vcmd_write_register_value(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_RDY_CMDBUF_COUNT,
				subsys_core->sw_cmdbuf_rdy_num);
	}
	spin_unlock_irqrestore(&subsys_core->spinlock, flags);
	up(&subsys_dev->vcmd_reserve_cmdbuf_sem[cmdbuf_obj->module_type]);

	return 0;
}

/******************************************************************************/
static int check_cmdbuf_irq(vcmd_core_str *subsys_core,
			    struct cmdbuf_obj *cmdbuf_obj, u32 *irq_status_ret)
{
	int rdy = 0;

	if (atomic_read((atomic_t *)&cmdbuf_obj->cmdbuf_run_done)) {
		rdy = 1;
		//need to decide how to assign this variable
		*irq_status_ret = atomic_read((atomic_t *)&cmdbuf_obj->executing_status);
	}
	return rdy;
}

static int check_mc_cmdbuf_irq(struct file *filp, vcmd_dev_str *subsys_dev,
			       struct cmdbuf_obj *cmdbuf_obj,
			       u32 *irq_status_ret)
{
	int k;
	bi_list_node *new_cmdbuf_node = NULL;
	vcmd_core_str *subsys_core;

	for (k = 1; k < TOTAL_DISCRETE_CMDBUF_NUM; k++) {
		new_cmdbuf_node = subsys_dev->global_cmdbuf_node[k];
		if (!new_cmdbuf_node)
			continue;

		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;

		//multiprocess hang
		if (!cmdbuf_obj || cmdbuf_obj->filp !=filp)
			continue;
		if (cmdbuf_obj->cmdbuf_reserve == 1)
			continue;

		subsys_core = get_dev_of_core(subsys_dev, cmdbuf_obj->core_id);
		if (check_cmdbuf_irq(subsys_core, cmdbuf_obj, irq_status_ret) ==
		    1) {
			/* Return cmdbuf_id when ANY_CMDBUF_ID is used. */
			if (!cmdbuf_obj->waited) {
				*irq_status_ret = cmdbuf_obj->cmdbuf_id;
				cmdbuf_obj->waited = 1;
				return 1;
			}
		}
	}
	return 0;
}

static unsigned int wait_cmdbuf_ready(struct file *filp,
				      vcmd_dev_str *subsys_dev, u16 cmdbuf_id,
				      u32 *irq_status_ret)
{
	struct cmdbuf_obj *cmdbuf_obj;
	bi_list_node *new_cmdbuf_node;
	vcmd_core_str *subsys_core;
	unsigned long flags;

	XDX_DEV_DEBUG("%s, %d\n", __func__, __LINE__);
	if (cmdbuf_id != ANY_CMDBUF_ID) {
		new_cmdbuf_node = subsys_dev->global_cmdbuf_node[cmdbuf_id];
		if (!new_cmdbuf_node) {
			//should not happen
			XDX_DEV_ERROR("hantrovcmd: ERROR cmdbuf_id !!\n");
			return (unsigned int)-1;
		}
		cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
		if (cmdbuf_obj->filp != filp) {
			//should not happen
			XDX_DEV_ERROR("hantrovcmd: ERROR cmdbuf_id !!\n");
			return (unsigned int)-1;
		}
		subsys_core = get_dev_of_core(subsys_dev, cmdbuf_obj->core_id);
retry:
		if (wait_event_interruptible_timeout(subsys_core->wait_queue,
								check_cmdbuf_irq(subsys_core, cmdbuf_obj, irq_status_ret),
								msecs_to_jiffies(WAIT_CMDBUF_TIMEOUT)) == 0) {

			if (subsys_core->timeout_cnt-- > 0) {
				XDX_DEV_ERROR("Wait vcmd timeout. irq_sync:%d \n", subsys_core->irq_sync);
				printk_vcmd_register_debug((const void *)subsys_core->hwregs, "Wait vcmd timeout.");
				if (subsys_core->timeout_cnt == 1) {
					if (subsys_core->type == HANTRO_CORE_DEC) {
						struct xdxvpu_dev_info *vpu_dev_info=get_xdxvpu_dev_info(VCMD_DEC_MODULE_TYPE_0);
						XDX_DEV_ERROR("vcd1:0x%x \n",ioread32((void *)vpu_dev_info->submodule_main_vaddr+0x4));
						iowrite32(0x20, vpu_dev_info->submodule_main_vaddr+0x4);
					}
				}
				XDX_DEV_ERROR("id:%d done:%d rdy_num:%d \n",cmdbuf_id,cmdbuf_obj->cmdbuf_run_done,
						subsys_core->sw_cmdbuf_rdy_num);

				goto retry;
			}

			//abort the vcmd
			//vcmd_write_register_value(
			//	(const void *)subsys_core->hwregs,
			//	subsys_core->reg_mirror,
			//	HWIF_VCMD_START_TRIGGER, 0);
			return (unsigned int)(-ERESTARTSYS);
		}

		spin_lock_irqsave(&subsys_core->spinlock, flags);
		subsys_core->irq_sync = 0;
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
	} else {
		if (check_mc_cmdbuf_irq(filp, subsys_dev, cmdbuf_obj,
					irq_status_ret))
			return 0;
		if (wait_event_interruptible_timeout(
			    subsys_dev->mc_wait_queue,
			    check_mc_cmdbuf_irq(filp, subsys_dev, cmdbuf_obj,
						irq_status_ret), msecs_to_jiffies(WAIT_CMDBUF_TIMEOUT)) == 0) {
			XDX_DEV_ERROR("multicore wait queue interrupted\n");
			return (unsigned int)(-ERESTARTSYS);
		}
	}

	return 0;
}

#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
irqreturn_t hantrovcmd_isr(int irq, void *dev_id, struct pt_regs *regs)
#else
irqreturn_t hantrovcmd_isr(int irq, void *dev_id)
#endif
{
	unsigned int handled = 0;
	vcmd_core_str *subsys_core = (vcmd_core_str *)dev_id;
	u32 irq_status = 0;
	unsigned long flags;
	bi_list_node *new_cmdbuf_node = NULL;
	bi_list_node *base_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj = NULL;
	size_t exe_cmdbuf_busAddress;
	u32 cmdbuf_processed_num = 0;
	u32 cmdbuf_id = 0;
	vcmd_dev_str *subsys_dev;

	subsys_dev = subsys_core->parent_dev;

	/*If core is not reserved by any user,
	 *but irq is received, just clean it
	 */
	spin_lock_irqsave(&subsys_core->spinlock, flags);
	subsys_core->irq_sync = 1;
	if (!subsys_core->list_manager.head) {
		XDX_DEV_DEBUG("hantrovcmd_isr:received IRQ but core has nothing to do.\n");
		irq_status = vcmd_read_reg((const void *)subsys_core->hwregs,
					   VCMD_REGISTER_INT_STATUS_OFFSET);
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_INT_STATUS_OFFSET, irq_status);
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_none++;
#endif
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		return IRQ_HANDLED;
	}

	XDX_DEV_DEBUG("hantrovcmd_isr:received IRQ!\n");
	irq_status = vcmd_read_reg((const void *)subsys_core->hwregs,
				   VCMD_REGISTER_INT_STATUS_OFFSET);

	if (!irq_status) {
        if (irq) {
            irq_status = (u32)irq;
        } else {
#ifdef VPU_HW_STAT
		    vpu_irq_stat.irq_noirq++;
#endif
		    spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		    return IRQ_HANDLED;
        }
	}

	XDX_DEV_DEBUG("irq_status of %d is:%x\n", subsys_core->core_id, irq_status);
	vcmd_write_reg((const void *)subsys_core->hwregs,
		       VCMD_REGISTER_INT_STATUS_OFFSET, irq_status);
	subsys_core->reg_mirror[VCMD_REGISTER_INT_STATUS_OFFSET / 4] =
		irq_status;

	if (subsys_core->hw_version_id > HW_ID_1_0_C && (irq_status & 0x3f)) {
		//if error,read from register directly.
		cmdbuf_id = vcmd_get_register_value(
			(const void *)subsys_core->hwregs,
			subsys_core->reg_mirror, HWIF_VCMD_CMDBUF_EXECUTING_ID);
		if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM) {
			XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id(0x%x) greater than the ceiling !!\n",cmdbuf_id);
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			return IRQ_HANDLED;
		}
	} else if (subsys_core->hw_version_id > HW_ID_1_0_C) {
		//read cmdbuf id from ddr
		cmdbuf_id = *(subsys_core->vcmd_reg_mem_virtualAddress +
			      EXECUTING_CMDBUF_ID_ADDR);
		if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM) {
			XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id greater than the ceiling !!\n");
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			return IRQ_HANDLED;
		}
	}
	if(vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					HWIF_VCMD_IRQ_JMPD)) {
		if (cmdbuf_id == subsys_core->pre_cmdbuf_id) {
			spin_unlock_irqrestore(&subsys_core->spinlock, flags); // this cmdbuf's irq has been handled by last polling
			return IRQ_HANDLED;
		}
		subsys_core->pre_cmdbuf_id = cmdbuf_id;
	}

	if (vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					   HWIF_VCMD_IRQ_RESET)) {
		//reset error,all cmdbuf that is not  done will be run again.
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_reset++;
#endif
		new_cmdbuf_node = subsys_core->list_manager.head;
		subsys_core->working_state = WORKING_STATE_IDLE;
		//find the first run_done=0
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (cmdbuf_obj->cmdbuf_run_done == 0)
				break;
			new_cmdbuf_node = new_cmdbuf_node->next;
		}
		base_cmdbuf_node = new_cmdbuf_node;
		vcmd_delink_cmdbuf(subsys_core, base_cmdbuf_node);
		vcmd_write_reg((const void *)subsys_core->hwregs,
						   VCMD_REGISTER_EXE_COUNT_OFFSET, 0);
		vcmd_link_cmdbuf(subsys_core, base_cmdbuf_node);
		if (subsys_core->sw_cmdbuf_rdy_num != 0) {
			//restart new command
			vcmd_start(subsys_core, base_cmdbuf_node);
		}
		handled++;
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		return IRQ_HANDLED;
	}
	if (vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					   HWIF_VCMD_IRQ_ABORT)) {
		//abort error,don't need to reset
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_abort++;
#endif
		new_cmdbuf_node = subsys_core->list_manager.head;
		subsys_core->working_state = WORKING_STATE_IDLE;
		if (subsys_core->hw_version_id > HW_ID_1_0_C) {
			new_cmdbuf_node =
				subsys_dev->global_cmdbuf_node[cmdbuf_id];
			if (!new_cmdbuf_node) {
				XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id !!\n");
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
				return IRQ_HANDLED;
			}
		} else {
			exe_cmdbuf_busAddress = VCMDGetAddrRegisterValue(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR);
			//find the cmderror cmdbuf
			while (1) {
				if (!new_cmdbuf_node) {
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return IRQ_HANDLED;
				}
				cmdbuf_obj = (struct cmdbuf_obj *)
						     new_cmdbuf_node->data;
				if ((((cmdbuf_obj->cmdbuf_busAddress -
				       subsys_dev->base_ddr_addr) <=
				      exe_cmdbuf_busAddress) &&
				     (((cmdbuf_obj->cmdbuf_busAddress -
					subsys_dev->base_ddr_addr +
					cmdbuf_obj->cmdbuf_size) >
				       exe_cmdbuf_busAddress))) &&
				    (cmdbuf_obj->cmdbuf_run_done == 0))
					break;
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
		}
		base_cmdbuf_node = new_cmdbuf_node;
		// this cmdbuf and cmdbufs prior to itself, run_done = 1
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (cmdbuf_obj->cmdbuf_run_done == 0) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_obj->executing_status =
					CMDBUF_EXE_STATUS_OK;
				cmdbuf_processed_num++;
			} else {
				break;
			}
			new_cmdbuf_node = new_cmdbuf_node->previous;
		}
		base_cmdbuf_node = base_cmdbuf_node->next;
		vcmd_delink_cmdbuf(subsys_core, base_cmdbuf_node);
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_EXE_COUNT_OFFSET, 0);
		if (subsys_dev->type == HANTRO_ENCODER) {
			if (subsys_dev->software_triger_abort == 0) {
				//for QCFE
				vcmd_link_cmdbuf(subsys_core, base_cmdbuf_node);
				if (subsys_core->sw_cmdbuf_rdy_num != 0) {
					//restart new command
					vcmd_start(subsys_core,
						   base_cmdbuf_node);
				}
			}
		}
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		if (cmdbuf_processed_num) {
			wake_up_interruptible_all(&subsys_core->wait_queue);
			wake_up_interruptible_all(&subsys_dev->mc_wait_queue);
		}
		//to let high priority cmdbuf be inserted
		wake_up_interruptible_all(&subsys_core->wait_abort_queue);
		handled++;
		return IRQ_HANDLED;
	}
	if (vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					   HWIF_VCMD_IRQ_BUSERR)) {
		//bus error ,don't need to reset, where to record status?
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_buserr++;
#endif
		new_cmdbuf_node = subsys_core->list_manager.head;
		subsys_core->working_state = WORKING_STATE_IDLE;
		if (subsys_core->hw_version_id > HW_ID_1_0_C) {
			new_cmdbuf_node =
				subsys_dev->global_cmdbuf_node[cmdbuf_id];
			if (!new_cmdbuf_node) {
				XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id !!\n");
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
				return IRQ_HANDLED;
			}
		} else {
			exe_cmdbuf_busAddress = VCMDGetAddrRegisterValue(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR);
			//find the buserr cmdbuf
			while (1) {
				if (!new_cmdbuf_node) {
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return IRQ_HANDLED;
				}
				cmdbuf_obj = (struct cmdbuf_obj *)
						     new_cmdbuf_node->data;
				if ((((cmdbuf_obj->cmdbuf_busAddress -
				       subsys_dev->base_ddr_addr) <=
				      exe_cmdbuf_busAddress) &&
				     (((cmdbuf_obj->cmdbuf_busAddress -
					subsys_dev->base_ddr_addr +
					cmdbuf_obj->cmdbuf_size) >
				       exe_cmdbuf_busAddress))) &&
				    (cmdbuf_obj->cmdbuf_run_done == 0))
					break;
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
		}
		base_cmdbuf_node = new_cmdbuf_node;
		// this cmdbuf and cmdbufs prior to itself, run_done = 1
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if ((cmdbuf_obj->cmdbuf_run_done == 0)) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_obj->executing_status =
					CMDBUF_EXE_STATUS_OK;
				cmdbuf_processed_num++;
			} else {
				break;
			}
			new_cmdbuf_node = new_cmdbuf_node->previous;
		}
		new_cmdbuf_node = base_cmdbuf_node;
		if (new_cmdbuf_node) {
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			cmdbuf_obj->executing_status = CMDBUF_EXE_STATUS_BUSERR;
		}
		base_cmdbuf_node = base_cmdbuf_node->next;
		vcmd_delink_cmdbuf(subsys_core, base_cmdbuf_node);
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_EXE_COUNT_OFFSET, 0);
		vcmd_link_cmdbuf(subsys_core, base_cmdbuf_node);
		if (subsys_core->sw_cmdbuf_rdy_num != 0) {
			//restart new command
			vcmd_start(subsys_core, base_cmdbuf_node);
		}
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		if (cmdbuf_processed_num) {
			wake_up_interruptible_all(&subsys_core->wait_queue);
			wake_up_interruptible_all(&subsys_dev->mc_wait_queue);
		}
		handled++;
		return IRQ_HANDLED;
	}
	if (vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					   HWIF_VCMD_IRQ_TIMEOUT)) {
		//time out,need to reset
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_timeout++;
#endif
		new_cmdbuf_node = subsys_core->list_manager.head;
		subsys_core->working_state = WORKING_STATE_IDLE;
		if (subsys_core->hw_version_id > HW_ID_1_0_C) {
			new_cmdbuf_node =
				subsys_dev->global_cmdbuf_node[cmdbuf_id];
			if (!new_cmdbuf_node) {
				XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id !!\n");
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
				return IRQ_HANDLED;
			}
		} else {
			exe_cmdbuf_busAddress = VCMDGetAddrRegisterValue(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR);
			//find the timeout cmdbuf
			while (1) {
				if (!new_cmdbuf_node) {
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return IRQ_HANDLED;
				}
				cmdbuf_obj = (struct cmdbuf_obj *)
						     new_cmdbuf_node->data;
				if ((((cmdbuf_obj->cmdbuf_busAddress -
				       subsys_dev->base_ddr_addr) <=
				      exe_cmdbuf_busAddress) &&
				     (((cmdbuf_obj->cmdbuf_busAddress -
					subsys_dev->base_ddr_addr +
					cmdbuf_obj->cmdbuf_size) >
				       exe_cmdbuf_busAddress))) &&
				    (cmdbuf_obj->cmdbuf_run_done == 0))
					break;
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
		}
		base_cmdbuf_node = new_cmdbuf_node;
		new_cmdbuf_node = new_cmdbuf_node->previous;
		// this cmdbuf and cmdbufs prior to itself, run_done = 1
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (cmdbuf_obj->cmdbuf_run_done == 0) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_obj->executing_status =
					CMDBUF_EXE_STATUS_OK;
				cmdbuf_processed_num++;
			} else {
				break;
			}
			new_cmdbuf_node = new_cmdbuf_node->previous;
		}
		vcmd_delink_cmdbuf(subsys_core, base_cmdbuf_node);
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_EXE_COUNT_OFFSET, 0);
		vcmd_link_cmdbuf(subsys_core, base_cmdbuf_node);
		if (subsys_core->sw_cmdbuf_rdy_num != 0) {
			//reset
			vcmd_reset_current_asic(subsys_core);
			//restart new command
			vcmd_start(subsys_core, base_cmdbuf_node);
		}
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		if (cmdbuf_processed_num) {
			wake_up_interruptible_all(&subsys_core->wait_queue);
			wake_up_interruptible_all(&subsys_dev->mc_wait_queue);
		}
		handled++;
		return IRQ_HANDLED;
	}
	if (vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					   HWIF_VCMD_IRQ_CMDERR)) {
		//command error,don't need to reset
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_cmderr++;
#endif
		new_cmdbuf_node = subsys_core->list_manager.head;
		subsys_core->working_state = WORKING_STATE_IDLE;
		if (subsys_core->hw_version_id > HW_ID_1_0_C) {
			new_cmdbuf_node =
				subsys_dev->global_cmdbuf_node[cmdbuf_id];
			if (!new_cmdbuf_node) {
				XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id !!\n");
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
				return IRQ_HANDLED;
			}
		} else {
			exe_cmdbuf_busAddress = VCMDGetAddrRegisterValue(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR);
			//find the cmderror cmdbuf
			while (1) {
				if (!new_cmdbuf_node) {
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return IRQ_HANDLED;
				}
				cmdbuf_obj = (struct cmdbuf_obj *)
						     new_cmdbuf_node->data;
				if ((((cmdbuf_obj->cmdbuf_busAddress -
				       subsys_dev->base_ddr_addr) <=
				      exe_cmdbuf_busAddress) &&
				     (((cmdbuf_obj->cmdbuf_busAddress -
					subsys_dev->base_ddr_addr +
					cmdbuf_obj->cmdbuf_size) >
				       exe_cmdbuf_busAddress))) &&
				    (cmdbuf_obj->cmdbuf_run_done == 0))
					break;
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
		}
		base_cmdbuf_node = new_cmdbuf_node;

#ifdef VPU_XMEM
		if (xvcmd_debug_start == 1) {

			xvcmd.cmdbuf_id = cmdbuf_id;

			memcpy(xvcmd.buf,(void *)subsys_dev->vcmd_buf_mem_pool->virtualAddress,
				   CMDBUF_POOL_TOTAL_SIZE);
			memcpy(xvcmd.status_buf,(void *)subsys_dev->vcmd_status_buf_mem_pool->virtualAddress,
				   CMDBUF_POOL_TOTAL_SIZE);
			memcpy(xvcmd.registers,(void *)subsys_dev->vcmd_registers_mem_pool->virtualAddress,
				   CMDBUF_VCMD_REGISTER_TOTAL_SIZE);

			smp_mb();
			pr_info("xvcmd.cmdbuf_id:%u\n", xvcmd.cmdbuf_id);

			spin_unlock_irqrestore(&subsys_core->spinlock, flags);

			return IRQ_HANDLED;
		}
#endif
		// this cmdbuf and cmdbufs prior to itself, run_done = 1
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (cmdbuf_obj->cmdbuf_run_done == 0) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_obj->executing_status =
					CMDBUF_EXE_STATUS_OK;
				cmdbuf_processed_num++;
			} else {
				break;
			}
			new_cmdbuf_node = new_cmdbuf_node->previous;
		}
		new_cmdbuf_node = base_cmdbuf_node;
		if (new_cmdbuf_node) {
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			cmdbuf_obj->executing_status =
				CMDBUF_EXE_STATUS_CMDERR; //cmderr
#if 0
		{
			int cmderr_cout;
			size_t current_addr;
			u32 i, fordebug;
			exe_cmdbuf_busAddress = VCMDGetAddrRegisterValue(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_EXECUTING_CMD_ADDR);

			current_addr = (size_t)(char *)cmdbuf_obj->cmdbuf_virtualAddress +
				(exe_cmdbuf_busAddress-(cmdbuf_obj->cmdbuf_busAddress-subsys_dev->base_ddr_addr));

			for(cmderr_cout=0;cmderr_cout<16;cmderr_cout++) {
				pr_info("0x%x ",*((u32 *)current_addr+cmderr_cout));
			}
			pr_info("\n");

			for (i = 0; i < ASIC_VCMD_SWREG_AMOUNT; i++) {
				fordebug = vcmd_read_reg((const void *)subsys_core->hwregs, i * 4);
				pr_info("%d:0x%x\n", i, fordebug);
			}
		}
#endif
		}
		base_cmdbuf_node = base_cmdbuf_node->next;
		vcmd_delink_cmdbuf(subsys_core, base_cmdbuf_node);
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_EXE_COUNT_OFFSET, 0);
		vcmd_link_cmdbuf(subsys_core, base_cmdbuf_node);
		if (subsys_core->sw_cmdbuf_rdy_num != 0) {
			//restart new command
			vcmd_start(subsys_core, base_cmdbuf_node);
		}
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		if (cmdbuf_processed_num) {
			wake_up_interruptible_all(&subsys_core->wait_queue);
			wake_up_interruptible_all(&subsys_dev->mc_wait_queue);
		}
		handled++;
		return IRQ_HANDLED;
	}

	if (vcmd_get_register_mirror_value(subsys_core->reg_mirror,
					   HWIF_VCMD_IRQ_ENDCMD)) {
		//end command interrupt
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_endcmd++;
#endif
		new_cmdbuf_node = subsys_core->list_manager.head;
		subsys_core->working_state = WORKING_STATE_IDLE;
		if (subsys_core->hw_version_id > HW_ID_1_0_C) {
			new_cmdbuf_node =
				subsys_dev->global_cmdbuf_node[cmdbuf_id];
			if (!new_cmdbuf_node) {
				XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id !!\n");
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
				return IRQ_HANDLED;
			}
		} else {
			//find the end cmdbuf
			while (1) {
				if (!new_cmdbuf_node) {
					spin_unlock_irqrestore(
						&subsys_core->spinlock, flags);
					return IRQ_HANDLED;
				}
				cmdbuf_obj = (struct cmdbuf_obj *)
						     new_cmdbuf_node->data;
				if ((cmdbuf_obj->has_end_cmdbuf == 1) &&
				    (cmdbuf_obj->cmdbuf_run_done == 0))
					break;
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
		}
		base_cmdbuf_node = new_cmdbuf_node;
		// this cmdbuf and cmdbufs prior to itself, run_done = 1
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (cmdbuf_obj->cmdbuf_run_done == 0) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_obj->executing_status =
					CMDBUF_EXE_STATUS_OK;
				cmdbuf_processed_num++;
			} else {
				break;
			}
			new_cmdbuf_node = new_cmdbuf_node->previous;
		}
		base_cmdbuf_node = base_cmdbuf_node->next;
		vcmd_delink_cmdbuf(subsys_core, base_cmdbuf_node);
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_EXE_COUNT_OFFSET, 0);
		vcmd_link_cmdbuf(subsys_core, base_cmdbuf_node);
		if (subsys_core->sw_cmdbuf_rdy_num != 0) {
			//restart new command
			vcmd_start(subsys_core, base_cmdbuf_node);
		}
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);
		if (cmdbuf_processed_num) {
			wake_up_interruptible_all(&subsys_core->wait_queue);
			wake_up_interruptible_all(&subsys_dev->mc_wait_queue);
		}
		handled++;
		return IRQ_HANDLED;
	}
	if (subsys_core->hw_version_id <= HW_ID_1_0_C)
		cmdbuf_id = vcmd_get_register_mirror_value(
			subsys_core->reg_mirror, HWIF_VCMD_IRQ_INTCMD);
	if (cmdbuf_id) {
		if (subsys_core->hw_version_id <= HW_ID_1_0_C) {
			if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM) {
				XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id greater than the ceiling !!\n");
				spin_unlock_irqrestore(&subsys_core->spinlock,
						       flags);
				return IRQ_HANDLED;
			}
		}
#ifdef VPU_HW_STAT
		vpu_irq_stat.irq_cmdid++;
#endif
		new_cmdbuf_node = subsys_dev->global_cmdbuf_node[cmdbuf_id];
		if (!new_cmdbuf_node) {
			XDX_DEV_ERROR("hantrovcmd_isr error cmdbuf_id !!\n");
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			return IRQ_HANDLED;
		}
		// interrupt cmdbuf and cmdbufs prior to itself, run_done = 1
		while (1) {
			if (!new_cmdbuf_node)
				break;
			cmdbuf_obj = (struct cmdbuf_obj *)new_cmdbuf_node->data;
			if (cmdbuf_obj->cmdbuf_run_done == 0) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_obj->executing_status =
					CMDBUF_EXE_STATUS_OK;
				cmdbuf_processed_num++;
			} else {
				break;
			}
			new_cmdbuf_node = new_cmdbuf_node->previous;
		}
		handled++;
	}

	spin_unlock_irqrestore(&subsys_core->spinlock, flags);
	if (cmdbuf_processed_num) {
		wake_up_interruptible_all(&subsys_core->wait_queue);
		wake_up_interruptible_all(&subsys_dev->mc_wait_queue);
	}
	if (!handled)
		XDX_DEV_DEBUG("IRQ received, but not hantro's!\n");
	return IRQ_HANDLED;
}

#define VPU_IOCTL(ioctl) \
[DRM_IOCTL_NR(ioctl) - DRM_IOCTL_NR(VCMD_IOCH_GET_CMDBUF_PARAMETER)] = \
	{.name = #ioctl}

const struct vpu_ioctl_desc vcmd_ioctls[] = {
	VPU_IOCTL(VCMD_IOCH_GET_CMDBUF_PARAMETER),
	VPU_IOCTL(VCMD_IOCH_GET_CMDBUF_POOL_SIZE),
	VPU_IOCTL(VCMD_IOCH_SET_CMDBUF_POOL_BASE),
	VPU_IOCTL(VCMD_IOCH_GET_VCMD_PARAMETER),
	VPU_IOCTL(VCMD_IOCH_RESERVE_CMDBUF),
	VPU_IOCTL(VCMD_IOCH_LINK_RUN_CMDBUF),
	VPU_IOCTL(VCMD_IOCH_WAIT_CMDBUF),
	VPU_IOCTL(VCMD_IOCH_RELEASE_CMDBUF),
	VPU_IOCTL(VCMD_IOCH_POLLING_CMDBUF),
};
static void vcmd_ioctl_debug(unsigned int cmd)
{
	unsigned int index = 0;

	index = DRM_IOCTL_NR(cmd)-DRM_IOCTL_NR(VCMD_IOCH_GET_CMDBUF_PARAMETER);
	if (0 <= index &&
	    index <= DRM_IOCTL_NR(VCMD_IOCH_POLLING_CMDBUF) -
				 DRM_IOCTL_NR(VCMD_IOCH_GET_CMDBUF_PARAMETER))
		XDX_DEV_DEBUG("%s\n", vcmd_ioctls[index].name);
	else
		XDX_DEV_DEBUG("Invalid vcmd ioctl\n");
	return ;
}

static void vcmd_ioctl_restore_process(struct file *filp, vcmd_dev_str *subsys_dev)
{
	struct process_manager_obj *p = NULL;

	list_for_each_entry(p, &subsys_dev->pro_mngr, list) {
		if (p->subsys_dev == NULL && p->filp == filp) {
			p->subsys_dev = subsys_dev;
			break;
		}
	}
}
static int vcmd_ioctl_process_check(struct file *filp, vcmd_dev_str *subsys_dev)
{
	int ret = 0;
	struct process_manager_obj *p = NULL;
	struct file *fp = NULL;

	spin_lock(&subsys_dev->vcmd_process_manager_lock);
	list_for_each_entry(p, &subsys_dev->pro_mngr, list) {
		if (p->filp == filp) {
			fp = p->filp;
			break;
		}
	}
	spin_unlock(&subsys_dev->vcmd_process_manager_lock);

	if (fp == NULL) {
		ret = hantrovcmd_open(NULL, filp);
	}

	return ret;
}

long vcmd_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	int ret = 0;
	vcmd_dev_str *subsys_dev;
	vcmd_core_str *subsys_core;
	hantro_ioctl_id ioctl_id_par;
	u32 i = 0;
	int (*check)(struct vcmd_dev *vcmd_dev) = sku_check;

	vcmd_ioctl_debug(cmd);

	switch (cmd) {
	case VCMD_IOCH_GET_CMDBUF_PARAMETER: {
		struct cmdbuf_mem_parameter local_cmdbuf_mem_data;
		ret = copy_from_user(&local_cmdbuf_mem_data,
				     (struct cmdbuf_mem_parameter *)arg,
				     sizeof(struct cmdbuf_mem_parameter));
		if (ret)
			return ret;
		ioctl_id_par.data = local_cmdbuf_mem_data.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);
		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}

#if 0
		if (check(subsys_dev)) {
			XDX_DEV_DEBUG("subsys_dev->type: %d check failed.\n", subsys_dev->type);
			return -1;
		}
#else
		check = NULL;
#endif
		vcmd_ioctl_process_check(filp, subsys_dev);

		local_cmdbuf_mem_data.cmdbuf_unit_size = CMDBUF_MAX_SIZE;
		local_cmdbuf_mem_data.status_cmdbuf_unit_size = CMDBUF_MAX_SIZE;
		local_cmdbuf_mem_data.cmdbuf_total_size =
			CMDBUF_POOL_TOTAL_SIZE;
		local_cmdbuf_mem_data.status_cmdbuf_total_size =
			CMDBUF_POOL_TOTAL_SIZE;
		local_cmdbuf_mem_data.phy_status_cmdbuf_addr =
			subsys_dev->vcmd_status_buf_mem_pool->busAddress;
		local_cmdbuf_mem_data.phy_cmdbuf_addr =
			subsys_dev->vcmd_buf_mem_pool->busAddress;
#ifdef HAS_MMU
		local_cmdbuf_mem_data.mmu_phy_status_cmdbuf_addr =
			subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address;
		local_cmdbuf_mem_data.mmu_phy_cmdbuf_addr =
			subsys_dev->vcmd_buf_mem_pool->mmu_bus_address;
#endif
#ifdef SUPPORT_VPU_SMMU
		if(hantro_dev.asic_type == CHIP_PANGU_B0){
			local_cmdbuf_mem_data.mmu_phy_status_cmdbuf_addr =
				subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address;
			local_cmdbuf_mem_data.mmu_phy_cmdbuf_addr =
				subsys_dev->vcmd_buf_mem_pool->smmu_bus_address;
		}
#endif
#if !defined(HAS_MMU) && !defined(SUPPORT_VPU_SMMU)
		if (hantro_dev.asic_type == CHIP_PANGU_B0) {
			local_cmdbuf_mem_data.mmu_phy_status_cmdbuf_addr =
				subsys_dev->vcmd_status_buf_mem_pool->busAddress - subsys_dev->base_ddr_addr;
			local_cmdbuf_mem_data.mmu_phy_cmdbuf_addr =
				subsys_dev->vcmd_buf_mem_pool->busAddress - subsys_dev->base_ddr_addr;
		}
#endif
		local_cmdbuf_mem_data.base_ddr_addr = subsys_dev->base_ddr_addr;
		ret = copy_to_user((struct cmdbuf_mem_parameter *)arg,
				   &local_cmdbuf_mem_data,
				   sizeof(struct cmdbuf_mem_parameter));
		break;
	}
	case VCMD_IOCH_GET_VCMD_PARAMETER: {
		struct vcmd_cfg_par input_para;

		ret = copy_from_user(&input_para, (struct vcmd_cfg_par *)arg,
				     sizeof(struct vcmd_cfg_par));
		if (ret)
			return ret;
		ioctl_id_par.data = input_para.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);

		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}
		XDX_DEV_DEBUG("subsys_dev->type: %d\n", subsys_dev->type);
		vcmd_ioctl_process_check(filp, subsys_dev);
		if (subsys_dev->vcmd_type_core_num[input_para.module_type]) {
			input_para.submodule_main_addr =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->vcmd_core_cfg.submodule_main_addr;
			input_para.submodule_dec400_addr =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->vcmd_core_cfg.submodule_dec400_addr;
			input_para.submodule_L2Cache_addr =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->vcmd_core_cfg.submodule_L2Cache_addr;
			input_para.submodule_MMU_addr =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->vcmd_core_cfg.submodule_MMU_addr;
			input_para.submodule_MMUWrite_addr =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->vcmd_core_cfg.submodule_MMUWrite_addr;
			for (i = 0; i < 2; i++)
				input_para.submodule_axife_addr[i] =
					subsys_dev
						->vcmd_manager
							[input_para.module_type]
							[0]
						->vcmd_core_cfg
						.submodule_axife_addr[i];
			input_para.config_status_cmdbuf_id =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->status_cmdbuf_id;
			input_para.vcmd_hw_version_id =
				subsys_dev
					->vcmd_manager[input_para.module_type][0]
					->hw_version_id;
			input_para.vcmd_core_num =
				subsys_dev->vcmd_type_core_num
					[input_para.module_type];
		} else {
			input_para.submodule_main_addr = 0xffff;
			input_para.submodule_dec400_addr = 0xffff;
			input_para.submodule_L2Cache_addr = 0xffff;
			input_para.submodule_MMU_addr = 0xffff;
			input_para.submodule_MMUWrite_addr = 0xffff;
			input_para.config_status_cmdbuf_id = 0;
			input_para.vcmd_core_num = 0;
			input_para.vcmd_hw_version_id = HW_ID_1_0_C;
		}
		ret = copy_to_user((struct vcmd_cfg_par *)arg, &input_para,
				   sizeof(struct vcmd_cfg_par));
		break;
	}
	case VCMD_IOCH_RESERVE_CMDBUF: {
		struct exchange_parameter input_para;

		ret = copy_from_user(&input_para,
				     (struct exchange_parameter *)arg,
				     sizeof(struct exchange_parameter));
		if (ret)
			return ret;
		ioctl_id_par.data = input_para.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);

		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}
		vcmd_ioctl_process_check(filp, subsys_dev);
		vcmd_ioctl_restore_process(filp, subsys_dev);
		ret = reserve_cmdbuf(filp, subsys_dev, &input_para);
		if (ret == 0)
			ret = copy_to_user((struct exchange_parameter *)arg,
					   &input_para,
					   sizeof(struct exchange_parameter));
		subsys_dev->vcmd_timer.c++;
		return ret;
	}

	case VCMD_IOCH_LINK_RUN_CMDBUF: {
		struct exchange_parameter input_para;
		long ret_val;

#ifdef VPU_HW_STAT
		vpu_stat.vcmd_stat ++;
#endif
		ret = copy_from_user(&input_para,
				     (struct exchange_parameter *)arg,
				     sizeof(struct exchange_parameter));
		if (ret)
			return ret;
		ioctl_id_par.data = input_para.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);

		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}
		vcmd_ioctl_process_check(filp, subsys_dev);
#ifdef VPU_HW_STAT
		if (subsys_dev->type == HANTRO_ENCODER)
			vpu_stat.vcmd_enc_stat ++;
		else if (subsys_dev->type == HANTRO_DECODER)
			vpu_stat.vcmd_dec_stat ++;
#endif
		ret_val = link_and_run_cmdbuf(filp, subsys_dev, &input_para);
		ret_val = copy_to_user((struct exchange_parameter *)arg,
				       &input_para,
				       sizeof(struct exchange_parameter));
		return ret_val;
	}

	case VCMD_IOCH_WAIT_CMDBUF: {
		struct cmdbuf_id_parameter cmdbuf_par;
		unsigned int tmp;
		u32 irq_status_ret = 0;

		ret = copy_from_user(&cmdbuf_par,
				     (struct cmdbuf_id_parameter *)arg,
				     sizeof(struct cmdbuf_id_parameter));
		if (ret)
			return ret;
		//TODO
		ioctl_id_par.data = cmdbuf_par.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);

		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}
		vcmd_ioctl_process_check(filp, subsys_dev);
		tmp = wait_cmdbuf_ready(filp, subsys_dev, cmdbuf_par.cmdbuf_id,
					&irq_status_ret);
		cmdbuf_par.cmdbuf_id = (u16)irq_status_ret;
		if (tmp == 0) {
			tmp = copy_to_user((struct cmdbuf_id_parameter *)arg,
					   &cmdbuf_par,
					   sizeof(struct cmdbuf_id_parameter));
			return tmp; //return core_id
		} else {
			cmdbuf_par.cmdbuf_id = 0;
			ret = copy_to_user((struct cmdbuf_id_parameter *)arg,
					   &cmdbuf_par,
					   sizeof(struct cmdbuf_id_parameter));
			return -1;
		}
		ret = copy_from_user(&cmdbuf_par,
				     (struct cmdbuf_id_parameter *)arg,
				     sizeof(struct cmdbuf_id_parameter));
		if (ret)
			return ret;
		ioctl_id_par.data = cmdbuf_par.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);
		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}
		release_cmdbuf(filp, subsys_dev, cmdbuf_par.cmdbuf_id);
		break;
	}
	case VCMD_IOCH_RELEASE_CMDBUF: {
		struct cmdbuf_id_parameter cmdbuf_par;

		ret = copy_from_user(&cmdbuf_par,
				     (struct cmdbuf_id_parameter *)arg,
				     sizeof(struct cmdbuf_id_parameter));
		if (ret)
			return ret;
		ioctl_id_par.data = cmdbuf_par.id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);
		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}

		vcmd_ioctl_process_check(filp, subsys_dev);
		release_cmdbuf(filp, subsys_dev, cmdbuf_par.cmdbuf_id);
		break;
	}
	case VCMD_IOCH_POLLING_CMDBUF: {
		unsigned int id;

		ret = copy_from_user(&id, (unsigned int *)arg,
				     sizeof(unsigned int));
		if (ret)
			return ret;
		ioctl_id_par.data = id;
		subsys_dev = get_dev_by_sliceidx(ioctl_id_par.ID_PAR.node_idx,
						 ioctl_id_par.ID_PAR.group_idx);
		if (NULL == subsys_dev) {
			XDX_DEV_ERROR("Err: subsys_dev = NULL");
			return 1;
		}

		vcmd_ioctl_process_check(filp, subsys_dev);
		subsys_core = get_dev_of_core(subsys_dev,
					      ioctl_id_par.ID_PAR.codec_idx);
		hantrovcmd_isr(0, subsys_core);
		break;
	}
	default: {
		break;
	}
	}
	return ret;
}

static void
create_read_all_registers_cmdbuf(vcmd_core_str *subsys_core,
				 struct exchange_parameter *input_para)
{
#if defined(HAS_MMU) || defined(SUPPORT_VPU_SMMU)
	u64 map_status_base_phy_addr;
#endif
	u32 register_range[] = { VCMD_ENCODER_REGISTER_SIZE,
				 VCMD_IM_REGISTER_SIZE,
				 VCMD_DECODER_REGISTER_SIZE,
				 VCMD_JPEG_ENCODER_REGISTER_SIZE,
				 VCMD_JPEG_DECODER_REGISTER_SIZE };
	u32 counter_cmdbuf_size = 0;
	u32 *set_base_addr;
	ptr_t status_base_phy_addr;
	u32 offset_inc = 0;
	u32 offset_inc_dec400 = 0;
	vcmd_dev_str *subsys_dev;
	struct vcmd_config *core_cfg = NULL;
	u32 offset_inc_axife = 0;
	u32 i = 0;

	subsys_dev = subsys_core->parent_dev;
	core_cfg = &subsys_dev->vcmd_manager[input_para->module_type][input_para->core_id]->vcmd_core_cfg;
	set_base_addr = subsys_dev->vcmd_buf_mem_pool->virtualAddress +
			input_para->cmdbuf_id * CMDBUF_MAX_SIZE / 4;

	if (hantro_dev.asic_type == CHIP_PANGU_B0) {
		status_base_phy_addr =
			subsys_dev->vcmd_status_buf_mem_pool->busAddress +
			input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
			(core_cfg->submodule_main_addr / 2 +
			 0);
	} else {
		status_base_phy_addr =
			subsys_dev->vcmd_status_buf_mem_pool->busAddress +
			input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
			(core_cfg->submodule_main_addr +
			 0);
	}

#ifdef HAS_MMU
	if (hantro_dev.asic_type == CHIP_PANGU_B0) {
		map_status_base_phy_addr=subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address +
			input_para->cmdbuf_id* CMDBUF_MAX_SIZE +
			(core_cfg->submodule_main_addr / 2);
	} else {
		map_status_base_phy_addr=subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address +
			input_para->cmdbuf_id* CMDBUF_MAX_SIZE +
			(core_cfg->submodule_main_addr);
	}
#endif

#ifdef SUPPORT_VPU_SMMU
	if(hantro_dev.asic_type == CHIP_PANGU_B0){
		if (hantro_dev.asic_type == CHIP_PANGU_B0) {
			map_status_base_phy_addr=subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address +
				input_para->cmdbuf_id* CMDBUF_MAX_SIZE +
				(core_cfg->submodule_main_addr / 2);
		} else {
			map_status_base_phy_addr=subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address +
				input_para->cmdbuf_id* CMDBUF_MAX_SIZE +
				(core_cfg->submodule_main_addr);
		}
	}
#endif
	if (subsys_dev
		    ->vcmd_manager[input_para->module_type][input_para->core_id]
		    ->hw_version_id > HW_ID_1_0_C) {
		XDX_DEV_DEBUG("vcmd_driver:create cmdbuf data when hw_version_id = 0x%x\n",
			subsys_dev
				->vcmd_manager[input_para->module_type]
					      [input_para->core_id]
				->hw_version_id);

		//read vcmd executing cmdbuf id registers
		//to ddr for balancing core load.
		*(set_base_addr + 0) = (OPCODE_RREG) | (1 << 16) |
				       (EXECUTING_CMDBUF_ID_ADDR * 4);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 1) = (u32)0; //will be changed in link stage
		counter_cmdbuf_size += 4;
		*(set_base_addr + 2) = (u32)0; //will be changed in link stage
		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 3) = 0;
		counter_cmdbuf_size += 4;

		//read main IP all registers
		*(set_base_addr + 4) =
			(OPCODE_RREG) |
			((register_range[input_para->module_type] / 4) << 16) |
			(core_cfg->submodule_main_addr +
			 0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 5) =
			(u32)(status_base_phy_addr - subsys_dev->base_ddr_addr);

#if defined(HAS_MMU) || defined(SUPPORT_VPU_SMMU)
		*(set_base_addr + 5) = (u32)map_status_base_phy_addr;
#endif

		counter_cmdbuf_size += 4;
		if (sizeof(size_t) == 8) {
			*(set_base_addr + 6) =
				(u32)((u64)(status_base_phy_addr -
					    subsys_dev->base_ddr_addr) >>
				      32);
		} else {
			*(set_base_addr + 6) = 0;
		}
		counter_cmdbuf_size += 4;

#if defined(HAS_MMU) || defined(SUPPORT_VPU_SMMU)
		*(set_base_addr+6) = (u32)(map_status_base_phy_addr >> 32);
#endif

		//alignment
		*(set_base_addr + 7) = 0;
		counter_cmdbuf_size += 4;

		if (core_cfg->submodule_L2Cache_addr != 0xffff) {
			//read L2 cache register
			offset_inc = 4;
			if (hantro_dev.asic_type == CHIP_PANGU_B0) {
				status_base_phy_addr =
					subsys_dev->vcmd_status_buf_mem_pool
						->busAddress +
					input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
					(core_cfg->submodule_L2Cache_addr / 2 +
					 0);
			} else {
				status_base_phy_addr =
					subsys_dev->vcmd_status_buf_mem_pool
						->busAddress +
					input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
					(core_cfg->submodule_L2Cache_addr +
					 0);
			}

#ifdef HAS_MMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0) {
				map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address + \
					input_para->cmdbuf_id*CMDBUF_MAX_SIZE + \
					(core_cfg->submodule_L2Cache_addr / 2 + 0);
			} else {
				map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address + \
					input_para->cmdbuf_id*CMDBUF_MAX_SIZE + \
					(core_cfg->submodule_L2Cache_addr + 0);
			}
#endif
#ifdef SUPPORT_VPU_SMMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0){
				if (hantro_dev.asic_type == CHIP_PANGU_B0) {
					map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address + \
						input_para->cmdbuf_id*CMDBUF_MAX_SIZE + \
						(core_cfg->submodule_L2Cache_addr / 2 + 0);
				} else {
					map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address + \
						input_para->cmdbuf_id*CMDBUF_MAX_SIZE + \
						(core_cfg->submodule_L2Cache_addr + 0);
				}
			}
#endif
			//read L2cache IP first register
			*(set_base_addr +
			  8) = (OPCODE_RREG) | (1 << 16) |
			       (core_cfg->submodule_L2Cache_addr +
				0);
			counter_cmdbuf_size += 4;
			*(set_base_addr + 9) = (u32)(status_base_phy_addr -
						     subsys_dev->base_ddr_addr);


#if defined(HAS_MMU) || defined(SUPPORT_VPU_SMMU)
		*(set_base_addr+ 9) = map_status_base_phy_addr;
#endif
			counter_cmdbuf_size += 4;
			if (sizeof(size_t) == 8) {
				*(set_base_addr + 10) = (u32)(
					(u64)(status_base_phy_addr -
					      subsys_dev->base_ddr_addr) >>
					32);
			} else {
				*(set_base_addr + 10) = 0;
			}
#ifdef HAS_MMU
			*(set_base_addr + 10) = 0;
#endif
#ifdef SUPPORT_VPU_SMMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0){
				*(set_base_addr + 10) = (u32)(map_status_base_phy_addr >> 32);
			}
#endif
			counter_cmdbuf_size += 4;
			//alignment
			*(set_base_addr + 11) = 0;
			counter_cmdbuf_size += 4;
		}
		if (core_cfg->submodule_dec400_addr != 0xffff) {
			//read dec400 register
			offset_inc_dec400 = 4;
			if (hantro_dev.asic_type == CHIP_PANGU_B0) {
				status_base_phy_addr =
					subsys_dev->vcmd_status_buf_mem_pool
						->busAddress +
					input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
					(core_cfg->submodule_dec400_addr / 2 +
					 0);
			} else {
				status_base_phy_addr =
					subsys_dev->vcmd_status_buf_mem_pool
						->busAddress +
					input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
					(core_cfg->submodule_dec400_addr +
					 0);
			}

#ifdef HAS_MMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0) {
				map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address +
					input_para->cmdbuf_id*CMDBUF_MAX_SIZE +
					(core_cfg->submodule_dec400_addr / 2 +0);
			} else {
				map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address +
					input_para->cmdbuf_id*CMDBUF_MAX_SIZE +
					(core_cfg->submodule_dec400_addr+0);
			}
#endif
#ifdef SUPPORT_VPU_SMMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0){
				if (hantro_dev.asic_type == CHIP_PANGU_B0) {
					map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address +
						input_para->cmdbuf_id*CMDBUF_MAX_SIZE +
						(core_cfg->submodule_dec400_addr / 2 +0);
				} else {
					map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address +
						input_para->cmdbuf_id*CMDBUF_MAX_SIZE +
						(core_cfg->submodule_dec400_addr+0);
				}
			}
#endif
			//read Dec400 IP first register
			*(set_base_addr + 8 + offset_inc) =
				(OPCODE_RREG) | (0x2b << 16) |
				(core_cfg->submodule_dec400_addr +
				 0);
			counter_cmdbuf_size += 4;
			*(set_base_addr + 9 + offset_inc) =
				(u32)(status_base_phy_addr -
				      subsys_dev->base_ddr_addr);
			counter_cmdbuf_size += 4;
			if (sizeof(size_t) == 8) {
				*(set_base_addr + 10 + offset_inc) =
					(u32)((status_base_phy_addr -
					       subsys_dev->base_ddr_addr) >>
					      32);
			} else {
				*(set_base_addr + 10 + offset_inc) = 0;
			}
#ifdef HAS_MMU
			*(set_base_addr + 9 + offset_inc) = map_status_base_phy_addr;
			*(set_base_addr + 10 + offset_inc) = 0;
#endif
#ifdef SUPPORT_VPU_SMMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0){
				*(set_base_addr + 9 + offset_inc) = map_status_base_phy_addr;
				*(set_base_addr + 10 + offset_inc) = (u32)(map_status_base_phy_addr >> 32);;
			}
#endif
			counter_cmdbuf_size += 4;
			//alignment
			*(set_base_addr + 11 + offset_inc) = 0;
			counter_cmdbuf_size += 4;
		}

		if (hantro_dev.asic_type == CHIP_PANGU_B0) {
			for (i = 0; i < 2; i++) {
				if (core_cfg->submodule_axife_addr[i] != 0xffff) {
					//read axife register
					status_base_phy_addr =
						subsys_dev->vcmd_status_buf_mem_pool
							->busAddress +
								input_para->cmdbuf_id * CMDBUF_MAX_SIZE +
									(core_cfg->submodule_axife_addr[i] / 2 +
										0);
#ifdef HAS_MMU
				map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address +
					input_para->cmdbuf_id*CMDBUF_MAX_SIZE +
						(core_cfg->submodule_axife_addr[i] / 2 +0);
#endif
#ifdef SUPPORT_VPU_SMMU
					if (hantro_dev.asic_type == CHIP_PANGU_B0){
						map_status_base_phy_addr = subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address +
							input_para->cmdbuf_id*CMDBUF_MAX_SIZE +
								(core_cfg->submodule_axife_addr[i] / 2 +0)+subsys_dev->base_ddr_addr;
					}
#endif
					*(set_base_addr + 8 + offset_inc + offset_inc_dec400 + offset_inc_axife) =
						(OPCODE_RREG) | (0x2b << 16) |
							(core_cfg->submodule_axife_addr[i] +
								0);
					counter_cmdbuf_size += 4;
					*(set_base_addr + 9 + offset_inc + offset_inc_dec400 + offset_inc_axife) =
						(u32)(status_base_phy_addr -
							subsys_dev->base_ddr_addr);
					counter_cmdbuf_size += 4;
					if (sizeof(size_t) == 8) {
						*(set_base_addr + 10 + offset_inc + offset_inc_dec400 + offset_inc_axife) =
							(u32)((status_base_phy_addr -
								subsys_dev->base_ddr_addr) >>
									32);
					} else {
						*(set_base_addr + 10 + offset_inc + offset_inc_dec400 + offset_inc_axife) = 0;
					}
#if defined(HAS_MMU) || defined(SUPPORT_VPU_SMMU)
					*(set_base_addr + 9 + offset_inc + offset_inc_dec400 + offset_inc_axife) = map_status_base_phy_addr;
					*(set_base_addr + 10 + offset_inc + offset_inc_dec400 + offset_inc_axife) = 0;
#endif
#ifdef SUPPORT_VPU_SMMU
					if (hantro_dev.asic_type == CHIP_PANGU_B0){
						*(set_base_addr + 9 + offset_inc + offset_inc_dec400 + offset_inc_axife) = (u32)map_status_base_phy_addr;
						*(set_base_addr + 10 + offset_inc + offset_inc_dec400 + offset_inc_axife) = (u32)(map_status_base_phy_addr >> 32);
					}
#endif
					counter_cmdbuf_size += 4;
					//alignment
					*(set_base_addr + 11 + offset_inc + offset_inc_dec400 + offset_inc_axife) = 0;
					counter_cmdbuf_size += 4;
					offset_inc_axife += 4;
				}
			}
			offset_inc_dec400 += offset_inc_axife;
		}

		//read vcmd registers to ddr
		*(set_base_addr + 8 + offset_inc + offset_inc_dec400) =
			(OPCODE_RREG) | (27 << 16) | (0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 9 + offset_inc + offset_inc_dec400) =
			(u32)0; //will be changed in link stage
		counter_cmdbuf_size += 4;
		*(set_base_addr + 10 + offset_inc + offset_inc_dec400) =
			(u32)0; //will be changed in link stage
		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 11 + offset_inc + offset_inc_dec400) = 0;
		counter_cmdbuf_size += 4;
		//JMP RDY = 0
		*(set_base_addr + 12 + offset_inc + offset_inc_dec400) =
			(OPCODE_JMP_RDY0) | 0 | JMP_IE_1 | 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 13 + offset_inc + offset_inc_dec400) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 14 + offset_inc + offset_inc_dec400) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 15 + offset_inc + offset_inc_dec400) =
			input_para->cmdbuf_id;
		//don't add the last alignment DWORD in order
		//to identify END command or JMP command.
		//counter_cmdbuf_size += 4;
		input_para->cmdbuf_size =
			(16 + offset_inc + offset_inc_dec400) * 4;
	} else {
		XDX_DEV_DEBUG("vcmd_driver:create cmdbuf data when hw_version_id = 0x%x\n",
			subsys_dev
				->vcmd_manager[input_para->module_type]
					      [input_para->core_id]
				->hw_version_id);
		//read all registers
		*(set_base_addr + 0) =
			(OPCODE_RREG) |
			((register_range[input_para->module_type] / 4) << 16) |
			(core_cfg->submodule_main_addr +
			 0);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 1) =
			(u32)(status_base_phy_addr - subsys_dev->base_ddr_addr);
		counter_cmdbuf_size += 4;
		*(set_base_addr + 2) = (u32)(
			(status_base_phy_addr - subsys_dev->base_ddr_addr) >>
			32);
		counter_cmdbuf_size += 4;
		//alignment
		*(set_base_addr + 3) = 0;
		counter_cmdbuf_size += 4;

		//JMP RDY = 0
		*(set_base_addr + 4) = (OPCODE_JMP_RDY0) | 0 | JMP_IE_1 | 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 5) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 6) = 0;
		counter_cmdbuf_size += 4;
		*(set_base_addr + 7) = input_para->cmdbuf_id;
		//don't add the last alignment DWORD in order to
		//identify END command or JMP command.
		//counter_cmdbuf_size += 4;
		input_para->cmdbuf_size = 8 * 4;
	}
}

static void read_main_module_all_registers(vcmd_core_str *subsys_core)
{
	int ret;
	struct exchange_parameter input_para;
	u32 irq_status_ret = 0;
	u32 *status_base_virt_addr;
	u32 main_module_type;
	u32 manager_id;
	u32 core_id;
	vcmd_dev_str *subsys_dev;

	if (!subsys_core)
		return;
	subsys_dev = subsys_core->parent_dev;
	core_id = subsys_core->core_id;

	main_module_type = subsys_core->vcmd_core_cfg.sub_module_type;
	manager_id = subsys_core->id_in_type;
	input_para.executing_time = 0;
	input_para.priority = CMDBUF_PRIORITY_NORMAL;
	input_para.module_type = main_module_type;
	input_para.cmdbuf_size = 0;
	input_para.cmdbuf_reserve = 1;
	input_para.core_id = manager_id;
	ret = reserve_cmdbuf(NULL, subsys_dev, &input_para);
	(void)(ret);
	subsys_dev->vcmd_manager[main_module_type][manager_id]
		->status_cmdbuf_id = input_para.cmdbuf_id;
	create_read_all_registers_cmdbuf(subsys_core, &input_para);
	link_and_run_cmdbuf(NULL, subsys_dev, &input_para);
	msleep(10);

	subsys_core = get_dev_of_core(subsys_dev, input_para.core_id);

	hantrovcmd_isr(0, subsys_core);
	wait_cmdbuf_ready(NULL, subsys_dev, input_para.cmdbuf_id,
			  &irq_status_ret);
	if (hantro_dev.asic_type == CHIP_PANGU_B0) {
		status_base_virt_addr =
			subsys_dev->vcmd_status_buf_mem_pool->virtualAddress +
			input_para.cmdbuf_id * CMDBUF_MAX_SIZE / 4 +
			(subsys_dev->vcmd_manager[main_module_type][manager_id]
					->vcmd_core_cfg.submodule_main_addr / 2 / 4 +
			 0);
	} else {
		status_base_virt_addr =
			subsys_dev->vcmd_status_buf_mem_pool->virtualAddress +
			input_para.cmdbuf_id * CMDBUF_MAX_SIZE / 4 +
			(subsys_dev->vcmd_manager[main_module_type][manager_id]
					 ->vcmd_core_cfg.submodule_main_addr / 4 +
			 0);
	}

	if (subsys_dev->type == HANTRO_ENCODER) {
		XDX_DEV_INFO("vcmd_driver: main module register 0:0x%x\n",
			*status_base_virt_addr);
		XDX_DEV_INFO("vcmd_driver: main module register 80:0x%x\n",
			*(status_base_virt_addr + 80));
		XDX_DEV_INFO("vcmd_driver: main module register 214:0x%x\n",
			*(status_base_virt_addr + 214));
		XDX_DEV_INFO("vcmd_driver: main module register 226:0x%x\n",
			*(status_base_virt_addr + 226));
		XDX_DEV_INFO("vcmd_driver: main module register 287:0x%x\n",
			*(status_base_virt_addr + 287));
		XDX_DEV_INFO("vcmd_driver: main module register 509:0x%x\n",
			*(status_base_virt_addr + 509));
	} else if (subsys_dev->type == HANTRO_DECODER) {
		XDX_DEV_INFO("vcmd_driver: main module register 0:0x%x\n",
			*status_base_virt_addr);
		XDX_DEV_INFO("vcmd_driver: main module register 50:0x%08x\n",
			*(status_base_virt_addr + 50));
		XDX_DEV_INFO("vcmd_driver: main module register 54:0x%08x\n",
			*(status_base_virt_addr + 54));
		XDX_DEV_INFO("vcmd_driver: main module register 56:0x%08x\n",
			*(status_base_virt_addr + 56));
		XDX_DEV_INFO("vcmd_driver: main module register 309:0x%x\n",
			*(status_base_virt_addr + 309));
	} else {
		XDX_DEV_ERROR("%s,%d unkonwn subsys_dev->type %d\n\n", __func__, __LINE__,subsys_dev->type);
	}
}

static void hantro_vcmd_mem_pool_init(struct xdx_device* xdev, vcmd_dev_str *subsys_dev)
{
#ifndef VSI_FPGA
	dma_addr_t dma_handle;
#endif
#ifdef HAS_MMU
	int result;
	u32 id;
	unsigned long bus_address;
	int tmp = 0;
#endif
#ifdef SUPPORT_VPU_SMMU
	unsigned long long smmu_bus_address;
#endif
	struct noncache_mem *mem_temp;
	unsigned long vcmd_base_ddr_hw;

	vcmd_base_ddr_hw = subsys_dev->base_ddr_addr;

#ifndef VSI_FPGA
	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	mem_temp->virtualAddress = dma_alloc_coherent(
		subsys_dev->dev,
		CMDBUF_POOL_TOTAL_SIZE + CMDBUF_POOL_TOTAL_SIZE +
			CMDBUF_VCMD_REGISTER_TOTAL_SIZE,
		&dma_handle, GFP_KERNEL | GFP_DMA);
	mem_temp->busAddress = (uint64_t)dma_handle;
	mem_temp->size = CMDBUF_POOL_TOTAL_SIZE;
	subsys_dev->vcmd_buf_mem_pool = mem_temp;

	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	mem_temp->virtualAddress =
		subsys_dev->vcmd_buf_mem_pool->virtualAddress +
		CMDBUF_POOL_TOTAL_SIZE / 4;
	mem_temp->busAddress =
		(uint64_t)(subsys_dev->vcmd_buf_mem_pool->busAddress) +
		CMDBUF_POOL_TOTAL_SIZE;
	mem_temp->size = CMDBUF_POOL_TOTAL_SIZE;
	subsys_dev->vcmd_status_buf_mem_pool = mem_temp;

	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	mem_temp->virtualAddress =
		subsys_dev->vcmd_status_buf_mem_pool->virtualAddress +
		CMDBUF_POOL_TOTAL_SIZE / 4;
	mem_temp->busAddress =
		(uint64_t)(subsys_dev->vcmd_status_buf_mem_pool->busAddress) +
		CMDBUF_POOL_TOTAL_SIZE;
	mem_temp->size = CMDBUF_VCMD_REGISTER_TOTAL_SIZE;
	subsys_dev->vcmd_registers_mem_pool = mem_temp;
	subsys_dev->base_ddr_addr = 0;
#else

#ifdef HAS_MMU
#if 1
	tmp = 0;
	if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0){
		if (subsys_dev->vcmd_core->type == HANTRO_CORE_DEC)
			id = NODE_TYPE_DEC;
		else if (subsys_dev->vcmd_core->type == HANTRO_CORE_ENC)
			id = NODE_TYPE_ENC;

		result = hantroMMUInit(xdev, subsys_dev->vcmd_core->type);
		if (result != MMU_STATUS_OK) {
			pr_err("%s,%d hantroMMUprobe failed\n", __func__, __LINE__);
		}
	}
#else
#if VPU_SHARE_PAGETABLE
	id = NODE_TYPE_DEC;
#else
	if (subsys_dev->vcmd_core->type == HANTRO_CORE_DEC)
		id = NODE_TYPE_DEC;
	else if (subsys_dev->vcmd_core->type == HANTRO_CORE_ENC)
		id = NODE_TYPE_ENC;
#endif
	result = vpu_mmu_probe(NULL, subsys_dev->vcmd_core->type, NULL, xdev, vcmd_base_ddr_hw, xdev->rmmio_base);
	if (result != MMU_STATUS_OK) {
		pr_err("%s,%d vpu_mmu_probe failed\n", __func__, __LINE__);
	}

/*
	result = hantroMMUprobe(NULL, 0, NULL, xdev, vcmd_base_ddr_hw, xdev->rmmio_base);
	if (result != MMU_STATUS_OK) {
		pr_err("%s,%d hantroMMUprobe failed\n", __func__, __LINE__);
	}
*/
#endif
#endif
	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	subsys_dev->vcmd_buf_mem_pool = mem_temp;
	subsys_dev->vcmd_buf_mem_pool->busAddress = vpu_alloc_gddr(xdev, &subsys_dev->vcmd_bo, CMDBUF_POOL_TOTAL_SIZE);
	subsys_dev->vcmd_buf_mem_pool->size = CMDBUF_POOL_TOTAL_SIZE;
	if (!request_mem_region(subsys_dev->vcmd_buf_mem_pool->busAddress,
				subsys_dev->vcmd_buf_mem_pool->size,
				"hantro_vcmd")) {
		pr_info("Init: failed to request hw region.\n");
		return;
	}
	pr_info("Init: vcmd_buf_mem_pool.busAddress=0x%llx.\n",
		(unsigned long long)subsys_dev->vcmd_buf_mem_pool->busAddress);
	subsys_dev->vcmd_buf_mem_pool->virtualAddress =
		(u32 *)ioremap_wc(subsys_dev->vcmd_buf_mem_pool->busAddress,
			       subsys_dev->vcmd_buf_mem_pool->size);

	if (!subsys_dev->vcmd_buf_mem_pool->virtualAddress) {
		pr_info("Init: failed to ioremap_wc.\n");
		return;
	}
	pr_info("Init: vcmd_buf_mem_pool.virtualAddress=0x%llx.\n",
		(unsigned long long)
			subsys_dev->vcmd_buf_mem_pool->virtualAddress);

#ifdef HAS_MMU
#if 1
	tmp = 0;
	if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0){
		hantroMMUMap(xdev, subsys_dev->vcmd_bo, &bus_address, subsys_dev->vcmd_core->type);
		subsys_dev->vcmd_buf_mem_pool->mmu_bus_address = bus_address;
	}
	// subsys_dev->vcmd_buf_mem_pool->mmu_bus_address = bus_address;
#else
	addr.virtual_address = 0;
	addr.bus_address = subsys_dev->vcmd_buf_mem_pool->busAddress - vcmd_base_ddr_hw;
	addr.size = subsys_dev->vcmd_buf_mem_pool->size;
	addr.id = id << 8;
	hantro_mmu_map(&addr, 1);
	subsys_dev->vcmd_buf_mem_pool->mmu_bus_address = addr.bus_address;
#endif
#endif
#ifdef SUPPORT_VPU_SMMU
	if (hantro_dev.asic_type == CHIP_PANGU_B0){
		vpu_smmu_map(xdev, subsys_dev->vcmd_bo, &smmu_bus_address);
		subsys_dev->vcmd_buf_mem_pool->smmu_bus_address = smmu_bus_address;
		pr_info("Init: vcmd_buf_mem_pool.smmu_bus_address=0x%llx.\n",
			(unsigned long long)subsys_dev->vcmd_buf_mem_pool->smmu_bus_address);
	}
#endif
	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	subsys_dev->vcmd_status_buf_mem_pool = mem_temp;
	subsys_dev->vcmd_status_buf_mem_pool->busAddress =
		vpu_alloc_gddr(xdev, &subsys_dev->vcmd_status_bo, CMDBUF_POOL_TOTAL_SIZE);
	subsys_dev->vcmd_status_buf_mem_pool->size = CMDBUF_POOL_TOTAL_SIZE;
	if (!request_mem_region(subsys_dev->vcmd_status_buf_mem_pool->busAddress,
				subsys_dev->vcmd_status_buf_mem_pool->size,
				"hantro_vcmd")) {
		pr_info("Init: failed to request hw region.\n");
		return;
	}
	pr_info("Init: vcmd_status_buf_mem_pool.busAddress=0x%llx.\n",
		(unsigned long long)
			subsys_dev->vcmd_status_buf_mem_pool->busAddress);
	subsys_dev->vcmd_status_buf_mem_pool->virtualAddress =
		(u32 *)ioremap_wc(subsys_dev->vcmd_status_buf_mem_pool->busAddress,
			       subsys_dev->vcmd_status_buf_mem_pool->size);

	if (!subsys_dev->vcmd_status_buf_mem_pool->virtualAddress) {
		pr_info("Init: failed to ioremap_wc.\n");
		return;
	}
	pr_info("Init: vcmd_status_buf_mem_pool.virtualAddress=0x%llx.\n",
		(unsigned long long)
			subsys_dev->vcmd_status_buf_mem_pool->virtualAddress);

#ifdef HAS_MMU
#if 1
	tmp = 0;
	if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0){
		hantroMMUMap(xdev, subsys_dev->vcmd_status_bo, &bus_address, subsys_dev->vcmd_core->type);
		subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address = bus_address;
	}
#else
	addr.virtual_address = 0;
	addr.bus_address = subsys_dev->vcmd_status_buf_mem_pool->busAddress- vcmd_base_ddr_hw;
	addr.size = subsys_dev->vcmd_status_buf_mem_pool->size;
	addr.id = id << 8;
	hantro_mmu_map(&addr, 1);
	subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address = addr.bus_address;
#endif
#endif
#ifdef SUPPORT_VPU_SMMU
	if (hantro_dev.asic_type == CHIP_PANGU_B0){
		vpu_smmu_map(xdev, subsys_dev->vcmd_status_bo, &smmu_bus_address);
		subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address = smmu_bus_address;
		pr_info("Init: vcmd_status_buf_mem_pool.smmu_bus_address=0x%llx.\n",
			(unsigned long long)
				subsys_dev->vcmd_status_buf_mem_pool->smmu_bus_address);
	}
#endif
	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	subsys_dev->vcmd_registers_mem_pool = mem_temp;
	subsys_dev->vcmd_registers_mem_pool->busAddress =
		vpu_alloc_gddr(xdev, &subsys_dev->vcmd_reg_bo, CMDBUF_VCMD_REGISTER_TOTAL_SIZE);
	subsys_dev->vcmd_registers_mem_pool->size =
		CMDBUF_VCMD_REGISTER_TOTAL_SIZE;
	if (!request_mem_region(subsys_dev->vcmd_registers_mem_pool->busAddress,
				subsys_dev->vcmd_registers_mem_pool->size,
				"hantro_vcmd")) {
		pr_info("Init: failed to request hw region.\n");
		return;
	}
	pr_info("Init: vcmd_registers_mem_pool.busAddress=0x%llx.\n",
		(unsigned long long)
			subsys_dev->vcmd_registers_mem_pool->busAddress);
	subsys_dev->vcmd_registers_mem_pool->virtualAddress =
		(u32 *)ioremap_wc(subsys_dev->vcmd_registers_mem_pool->busAddress,
			       subsys_dev->vcmd_registers_mem_pool->size);

	if (!subsys_dev->vcmd_registers_mem_pool->virtualAddress) {
		pr_info("Init: failed to ioremap_wc.\n");
		return;
	}
	pr_info("Init: vcmd_registers_mem_pool.virtualAddress=0x%llx.\n",
		(unsigned long long)
			subsys_dev->vcmd_registers_mem_pool->virtualAddress);
#ifdef HAS_MMU
#if 1
	tmp = 0;
	if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0){
		hantroMMUMap(xdev, subsys_dev->vcmd_reg_bo, &bus_address, subsys_dev->vcmd_core->type);
		subsys_dev->vcmd_registers_mem_pool->mmu_bus_address = bus_address;
	}
#else
	addr.virtual_address = 0;
	addr.bus_address = subsys_dev->vcmd_registers_mem_pool->busAddress- vcmd_base_ddr_hw;
	addr.size = subsys_dev->vcmd_registers_mem_pool->size;
	addr.id = id << 8;
	hantro_mmu_map(&addr, 1);
	subsys_dev->vcmd_registers_mem_pool->mmu_bus_address = addr.bus_address;
#endif
#endif
#ifdef SUPPORT_VPU_SMMU
	if (hantro_dev.asic_type == CHIP_PANGU_B0){
		vpu_smmu_map(xdev, subsys_dev->vcmd_reg_bo, &smmu_bus_address);
		subsys_dev->vcmd_registers_mem_pool->smmu_bus_address = smmu_bus_address;
		pr_info("Init: vcmd_registers_mem_pool.smmu_bus_address=0x%llx.\n",
			(unsigned long long)subsys_dev->vcmd_registers_mem_pool->smmu_bus_address);
	}
#endif
#endif
}


#ifdef VSI_FPGA

#define PCI_VENDOR_ID_HANTRO 0x10ee // 0x16c3
#define PCI_DEVICE_ID_HANTRO_PCI 0x8014 // 0x7011
/* Base address got control register */
#define PCI_H2_BAR 4
/* Base address DDR register */
#define PCI_DDR_BAR 0

#define MMU_REG_CONTROL             (226*4)	//0x388
#define MMU_REG_PDENTRY0            (237 * 4)	//0x3b4
#define MMU_REG_SIZE                0x430
#define VCMD_REG_SIZE				0x200

void set_mmu_bypass(struct xdx_device *xdev)
{
	unsigned long mmu_reg_base_addr;
	volatile u8 *base_hwreg;
	unsigned int i;

	for (i = 0; i < ARRAY_SIZE(vcmd_core_array[0]); i++) {
		if (vcmd_core_array[0][i].submodule_main_addr == 0)
				continue;

		mmu_reg_base_addr = xdev->rmmio_base + vcmd_core_array[0][i].vcmd_base_addr +
							vcmd_core_array[0][i].submodule_MMU_addr;

		if (!request_mem_region(mmu_reg_base_addr, MMU_REG_SIZE, "hantrommu")) {
			XDX_DEV_ERROR("hantrommu: failed to reserve mmu HW regs\n");
			return ;
		}
		base_hwreg = (u8 *)ioremap_wc(mmu_reg_base_addr, MMU_REG_SIZE);

		iowrite32(0x21, (void *)(base_hwreg + MMU_REG_CONTROL));
#ifdef SUPPORT_VPU_SMMU
		if (hantro_dev.asic_type == CHIP_PANGU_B0)
			iowrite32(0xF3000002, (void *)(base_hwreg + MMU_REG_PDENTRY0));
		else
			iowrite32(0x1000002, (void *)(base_hwreg + MMU_REG_PDENTRY0));
#else
		iowrite32(0x1000002, (void *)(base_hwreg + MMU_REG_PDENTRY0));
#endif
		iounmap((void *)base_hwreg);
		release_mem_region(mmu_reg_base_addr, MMU_REG_SIZE);
	}
	return ;
}
struct xdxvpu_dev_info xdxvpu_dev[3]={{0, 0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0, 0}};
struct xdxvpu_dev_info *get_xdxvpu_dev_info(u32 type)
{
	unsigned int i;
	for(i=0; i<ARRAY_SIZE(xdxvpu_dev);i++)
	{
		if (xdxvpu_dev[i].sub_module_type == type)
			return &xdxvpu_dev[i];
	}
	return NULL;
}

void xdxvpu_dev_reg_map(struct xdx_device *xdev)
{
	unsigned int i;
	u32 iosize = 0;

	for (i = 0; i < ARRAY_SIZE(vcmd_core_array[0]); i++) {
		if (vcmd_core_array[0][i].submodule_main_addr == 0)
				continue;

		XDX_DEV_DEBUG("i:%d \n",i);
		xdxvpu_dev[i].sub_module_type = vcmd_core_array[0][i].sub_module_type;

		/*vcmd reg map*/
		xdxvpu_dev[i].vcmd_reg_base = xdev->rmmio_base + vcmd_core_array[0][i].vcmd_base_addr;

		if (!request_mem_region(xdxvpu_dev[i].vcmd_reg_base, vcmd_core_array[0][i].vcmd_iosize, "hantrovcmd")) {
			XDX_DEV_ERROR("hantrovcmd: failed to reserve vcmd HW regs\n");
			return ;
		}
		xdxvpu_dev[i].vcmd_reg_vaddr = (u8 *)ioremap_wc(xdxvpu_dev[i].vcmd_reg_base, vcmd_core_array[0][i].vcmd_iosize);
		xdxvpu_dev[i].vcmd_reg_size = vcmd_core_array[0][i].vcmd_iosize;
		XDX_DEV_DEBUG("i:%d vcmd_v:0x%p vcmd_p:0x%llx \n",i,xdxvpu_dev[i].vcmd_reg_vaddr, xdxvpu_dev[i].vcmd_reg_base);

		/* dec/enc reg map*/
		xdxvpu_dev[i].submodule_main_addr = xdev->rmmio_base + vcmd_core_array[0][i].vcmd_base_addr+
						vcmd_core_array[0][i].submodule_main_addr;

		if (xdxvpu_dev[i].sub_module_type == VCMD_DEC_MODULE_TYPE_0)
		{
			iosize = DEC_IO_SIZE_MAX;
			if (!request_mem_region(xdxvpu_dev[i].submodule_main_addr, iosize, "hantrodec")) {
				XDX_DEV_ERROR("hantrovcmd: failed to reserve vcmd HW regs\n");
				return ;
			}
		}
		else if (xdxvpu_dev[i].sub_module_type == VCMD_ENC_MODULE_TYPE_0)
		{
			iosize = CORE_0_0_IO_SIZE;
			if (!request_mem_region(xdxvpu_dev[i].submodule_main_addr, iosize, "hantroenc")) {
				XDX_DEV_ERROR("hantrovcmd: failed to reserve vcmd HW regs\n");
				return ;
			}
		}

		xdxvpu_dev[i].submodule_main_vaddr = (u8 *)ioremap_wc(xdxvpu_dev[i].submodule_main_addr, iosize);
		if (xdxvpu_dev[i].submodule_main_vaddr == NULL)
		{
			XDX_DEV_ERROR("Failed to ioremap_wc submodule register!\n");
			release_mem_region(xdxvpu_dev[i].submodule_main_addr, iosize);
			return ;
		}
		xdxvpu_dev[i].submodule_main_size = iosize;
		XDX_DEV_DEBUG("i:%d main_vaddr:0x%p main_paddr:0x%llx \n",i, xdxvpu_dev[i].submodule_main_vaddr, xdxvpu_dev[i].submodule_main_addr);
	}

	return ;
}
void xdxvpu_dev_reg_ummap(void)
{
	unsigned int i;
	for(i=0;i<ARRAY_SIZE(xdxvpu_dev);i++)
	{
		if (!xdxvpu_dev[i].vcmd_reg_vaddr)
			continue;

//		iounmap((void *)xdxvpu_dev[i].vcmd_reg_vaddr);
//		release_mem_region(xdxvpu_dev[i].vcmd_reg_base, xdxvpu_dev[i].vcmd_reg_size);

		iounmap((void *)xdxvpu_dev[i].submodule_main_vaddr);
		release_mem_region(xdxvpu_dev[i].submodule_main_addr, xdxvpu_dev[i].submodule_main_size);
	}
	return ;
}

void irq_vcmd_init(struct xdxvpu_dev_info *xdxvpu_dev)
{
	unsigned int i;

	XDX_DEV_DEBUG();
	for (i = 0; i < ARRAY_SIZE(vcmd_core_array[0]); i++) {
		if (vcmd_core_array[0][i].submodule_main_addr == 0)
				continue;

		/*vcmd irq init*/
		iowrite32(0x1f, (void *)(xdxvpu_dev[i].vcmd_reg_vaddr + VCMD_REG_IRQ_CONTROL)); //Enable vcmd intr except jmp
		iowrite32(0x0, (void *)(xdxvpu_dev[i].vcmd_reg_vaddr + VCMD_REG_IRQ_CPU_GATE)); //Enable submodule ip intr to CPU
		iowrite32(0xffffffff, (void *)(xdxvpu_dev[i].vcmd_reg_vaddr + VCMD_REG_IRQ_STATUS)); //Clear vcmd irq status

		/* dec/enc irq init */
		if (vcmd_core_array[0][i].sub_module_type == VCMD_DEC_MODULE_TYPE_0)
		{
			iowrite32(0x0, (void *)(xdxvpu_dev[i].submodule_main_vaddr + VDEC_REG_IRQ_STATUS));	//Clear dec irq status
		}
		else if (vcmd_core_array[0][i].sub_module_type == VCMD_ENC_MODULE_TYPE_0)
		{
			iowrite32(0x3ff, (void *)(xdxvpu_dev[i].submodule_main_vaddr + VENC_REG_IRQ_STATUS)); //Clear enc irq status
		}
		else
		{
			XDX_DEV_ERROR("Bad submodule type!\n");
		}

	}
	return ;
}

#if 0
/* Function name   : vcmd_pcie_init */
/* Description     : Initialize PCI Hw access */
/* Return type     : int */
static int vcmd_pcie_init(struct xdx_device* xdev, vcmd_dev_str *subsys_dev)
{
	int i = 0;
	/* PCI base register address (Hardware address) */
	unsigned long vcmd_base_reg_hw;
	/* PCI base register address (memalloc) */
	unsigned long vcmd_base_ddr_hw;
	/* Base register address Length */
	u32 vcmd_base_len;
	/* Base register address Length */
	u32 vcmd_ddr_len;
	struct noncache_mem *mem_temp;
	vcmd_core_str *subsys_core;

	xdxvpu_dev_reg_map(xdev);
#ifndef HAS_MMU
	set_mmu_bypass(xdev);
#endif
	irq_vcmd_init(xdxvpu_dev);

	vcmd_base_reg_hw = xdev->rmmio_base;
	vcmd_base_ddr_hw = xdev->aper_base; // + 0x80000000;//temporarily set to GDRR 2G as start
	vcmd_base_len = xdev->rmmio_size;
	vcmd_ddr_len =  xdev->aper_size;

	printk("vcmd_pcie_init-- VPU vcmd_base_reg_hw=0x%llx, vcmd_base_ddr_hw=0x%llx",vcmd_base_reg_hw,vcmd_base_ddr_hw);
	subsys_core = subsys_dev->vcmd_core;
	for (i = 0; i < subsys_dev->subsys_num; i++) {
		subsys_core->vcmd_core_cfg.vcmd_base_addr =
			vcmd_base_reg_hw + subsys_core->vcmd_core_cfg.vcmd_base_addr;
		subsys_core = subsys_core->core_next;
	}

	subsys_dev->base_ddr_addr = vcmd_base_ddr_hw;

#ifdef HAS_MMU
	int result;
	result = hantroMMUprobe(NULL, 0, NULL, xdev, vcmd_base_ddr_hw,vcmd_base_reg_hw);
	if (result != MMU_STATUS_OK) {
		XDX_DEV_ERROR("%s,%d hantroMMUprobe failed\n", __func__, __LINE__);
	}
#endif
	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	subsys_dev->vcmd_buf_mem_pool = mem_temp;
	subsys_dev->vcmd_buf_mem_pool->busAddress = vpu_alloc_gddr(xdev, &subsys_dev->vcmd_bo, CMDBUF_POOL_TOTAL_SIZE);
	subsys_dev->vcmd_buf_mem_pool->size = CMDBUF_POOL_TOTAL_SIZE;
	if (!request_mem_region(subsys_dev->vcmd_buf_mem_pool->busAddress,
				subsys_dev->vcmd_buf_mem_pool->size,
				"hantro_vcmd")) {
		XDX_DEV_ERROR("Init: failed to request hw region.\n");
		return -1;
	}
	subsys_dev->vcmd_buf_mem_pool->virtualAddress =
		(u32 *)ioremap_wc(subsys_dev->vcmd_buf_mem_pool->busAddress,
			       subsys_dev->vcmd_buf_mem_pool->size);

	if (!subsys_dev->vcmd_buf_mem_pool->virtualAddress) {
		XDX_DEV_ERROR("Init: failed to ioremap_wc.\n");
		return -1;
	}

#ifdef HAS_MMU
	struct addr_desc addr = {0};
	addr.virtual_address = 0;
	addr.bus_address = subsys_dev->vcmd_buf_mem_pool->busAddress - vcmd_base_ddr_hw;
	addr.size = subsys_dev->vcmd_buf_mem_pool->size;
	addr.id = NODE_TYPE_VCMD << 8;
	hantro_mmu_map(&addr, 1);
	subsys_dev->vcmd_buf_mem_pool->mmu_bus_address = addr.bus_address;
#endif

	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	subsys_dev->vcmd_status_buf_mem_pool = mem_temp;
	subsys_dev->vcmd_status_buf_mem_pool->busAddress =
		vpu_alloc_gddr(xdev, &subsys_dev->vcmd_status_bo, CMDBUF_POOL_TOTAL_SIZE);
	subsys_dev->vcmd_status_buf_mem_pool->size = CMDBUF_POOL_TOTAL_SIZE;
	if (!request_mem_region(subsys_dev->vcmd_status_buf_mem_pool->busAddress,
				subsys_dev->vcmd_status_buf_mem_pool->size,
				"hantro_vcmd")) {
		XDX_DEV_ERROR("Init: failed to request hw region.\n");
		return -1;
	}
	subsys_dev->vcmd_status_buf_mem_pool->virtualAddress =
		(u32 *)ioremap_wc(subsys_dev->vcmd_status_buf_mem_pool->busAddress,
			       subsys_dev->vcmd_status_buf_mem_pool->size);

	if (!subsys_dev->vcmd_status_buf_mem_pool->virtualAddress) {
		XDX_DEV_ERROR("Init: failed to ioremap_wc.\n");
		return -1;
	}

#ifdef HAS_MMU
	addr.virtual_address = 0;
	addr.bus_address = subsys_dev->vcmd_status_buf_mem_pool->busAddress- vcmd_base_ddr_hw;
	addr.size = subsys_dev->vcmd_status_buf_mem_pool->size;
	addr.id = NODE_TYPE_VCMD << 8;
	hantro_mmu_map(&addr, 1);
	subsys_dev->vcmd_status_buf_mem_pool->mmu_bus_address = addr.bus_address;
#endif

	mem_temp = kzalloc(sizeof(*mem_temp), GFP_KERNEL);
	subsys_dev->vcmd_registers_mem_pool = mem_temp;
	subsys_dev->vcmd_registers_mem_pool->busAddress =
		vpu_alloc_gddr(xdev, &subsys_dev->vcmd_reg_bo, CMDBUF_VCMD_REGISTER_TOTAL_SIZE);
	subsys_dev->vcmd_registers_mem_pool->size =
		CMDBUF_VCMD_REGISTER_TOTAL_SIZE;
	if (!request_mem_region(subsys_dev->vcmd_registers_mem_pool->busAddress,
				subsys_dev->vcmd_registers_mem_pool->size,
				"hantro_vcmd")) {
		XDX_DEV_ERROR("Init: failed to request hw region.\n");
		return -1;
	}
	subsys_dev->vcmd_registers_mem_pool->virtualAddress =
		(u32 *)ioremap_wc(subsys_dev->vcmd_registers_mem_pool->busAddress,
			       subsys_dev->vcmd_registers_mem_pool->size);

	if (!subsys_dev->vcmd_registers_mem_pool->virtualAddress) {
		XDX_DEV_ERROR("Init: failed to ioremap_wc.\n");
		return -1;
	}
#ifdef HAS_MMU
	addr.virtual_address = 0;
	addr.bus_address = subsys_dev->vcmd_registers_mem_pool->busAddress- vcmd_base_ddr_hw;
	addr.size = subsys_dev->vcmd_registers_mem_pool->size;
	addr.id = NODE_TYPE_VCMD << 8;
	hantro_mmu_map(&addr, 1);
	subsys_dev->vcmd_registers_mem_pool->mmu_bus_address = addr.bus_address;
#endif

	return 0;

out:
	return -1;
}
#endif

/* Function name   : vcmd_store_pcie_info */
/* Description     : Store PCI Hw info */
/* Return type     : int */
static int vcmd_store_pcie_info(vcmd_dev_str *subsys_dev,
					     struct hantro_pci_t *vcmd_pci_par)
{
	unsigned int i = 0;
	/* PCI base register address (Hardware address) */
	unsigned long vcmd_base_reg_hw;
	/* PCI base register address (memalloc) */
	unsigned long vcmd_base_ddr_hw;
	vcmd_core_str *subsys_core;

	vcmd_base_reg_hw = vcmd_pci_par->pci_base_reg_hw;
	vcmd_base_ddr_hw = vcmd_pci_par->pci_base_ddr_hw;

	subsys_core = subsys_dev->vcmd_core;
	for (i = 0; i < subsys_dev->subsys_num; i++) {
		subsys_core->vcmd_core_cfg.vcmd_base_addr =
			vcmd_base_reg_hw +
			subsys_core->vcmd_core_cfg.vcmd_base_addr;
		subsys_core = subsys_core->core_next;
	}

	subsys_dev->base_ddr_addr = vcmd_base_ddr_hw;

	return 0;
}
#endif

static int hantro_vcmd_subsys_probe(struct xdx_device* xdev, vcmd_dev_str *subsys_dev)
{
	vcmd_core_str *subsys_core;
	u32 i, k;
	int result;
	bi_list_node *new_cmdbuf_node = NULL;
	unsigned long flags;

	for (k = 0; k < MAX_VCMD_TYPE; k++) {
		subsys_dev->vcmd_type_core_num[k] = 0;
		subsys_dev->vcmd_position[k] = 0;
		for (i = 0; i < MAX_VCMD_NUMBER; i++)
			subsys_dev->vcmd_manager[k][i] = NULL;
	}

	init_bi_list(&subsys_dev->global_process_manager);
	init_bi_list(&subsys_dev->global_vcmd_res);
	
	init_waitqueue_head(&subsys_dev->vcmd_cmdbuf_memory_wait);
	spin_lock_init(&subsys_dev->vcmd_cmdbuf_alloc_lock);
	spin_lock_init(&subsys_dev->vcmd_process_manager_lock);

	subsys_core = subsys_dev->vcmd_core;
	for (i = 0; i < subsys_dev->subsys_num; i++) {
		subsys_core->working_state = WORKING_STATE_IDLE;
		subsys_core->sw_cmdbuf_rdy_num = 0;
		spin_lock_init(&subsys_core->spinlock);
		init_waitqueue_head(&subsys_core->wait_queue);
		init_waitqueue_head(&subsys_core->wait_abort_queue);
		init_bi_list(&subsys_core->list_manager);
		subsys_core->irq_sync = 0;
		subsys_core->timeout_cnt = 3;
		subsys_core->duration_without_int = 0;
		subsys_core->id_in_type =
			subsys_dev->vcmd_type_core_num
				[subsys_core->vcmd_core_cfg.sub_module_type];
		subsys_dev->vcmd_manager
			[subsys_core->vcmd_core_cfg.sub_module_type]
			[subsys_dev->vcmd_type_core_num
				 [subsys_core->vcmd_core_cfg.sub_module_type]] =
			subsys_core;
		subsys_dev->vcmd_type_core_num[subsys_core->vcmd_core_cfg
						       .sub_module_type]++;
		subsys_core->vcmd_reg_mem_busAddress =
			subsys_dev->vcmd_registers_mem_pool->busAddress +
			i * VCMD_REGISTER_SIZE - subsys_dev->base_ddr_addr;
#ifdef HAS_MMU
		subsys_core->mmu_vcmd_reg_mem_bus_address =
			subsys_dev->vcmd_registers_mem_pool->mmu_bus_address +
			i * VCMD_REGISTER_SIZE;
#endif
#ifdef SUPPORT_VPU_SMMU
		if (hantro_dev.asic_type == CHIP_PANGU_B0){
			subsys_core->smmu_vcmd_reg_mem_bus_address =
				subsys_dev->vcmd_registers_mem_pool->smmu_bus_address +
				i * VCMD_REGISTER_SIZE;
		}
#endif
		subsys_core->vcmd_reg_mem_virtualAddress =
			subsys_dev->vcmd_registers_mem_pool->virtualAddress +
			i * VCMD_REGISTER_SIZE / 4;
		subsys_core->vcmd_reg_mem_size = VCMD_REGISTER_SIZE;
		OSDeviceMemSetIO(subsys_core->vcmd_reg_mem_virtualAddress, 0,
		       VCMD_REGISTER_SIZE);
		subsys_core = subsys_core->core_next;
	}
	init_waitqueue_head(&subsys_dev->mc_wait_queue);
	result = vcmd_reserve_IO(subsys_dev);
	if (result < 0)
		goto err;
	vcmd_reset_asic(subsys_dev);
	/* get the IRQ line */

	subsys_core = subsys_dev->vcmd_core;
	for (i = 0; i < subsys_dev->subsys_num; i++) {
		if (!subsys_core->hwregs)
			continue;
		if (subsys_core->vcmd_core_cfg.vcmd_irq != -1) {
			result =
				request_irq(subsys_core->vcmd_core_cfg.vcmd_irq,
					    hantrovcmd_isr,
#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
					    SA_INTERRUPT | SA_SHIRQ,
#else
					    IRQF_SHARED,
#endif
					    "hantro_vcmd", (void *)subsys_core);
			if (result == -EINVAL) {
				XDX_DEV_ERROR("vcmd_driver: Bad vcmd_irq number or handler，core_id=%d\n",
				       i);
				vcmd_release_IO(subsys_dev);
				goto err;
			} else if (result == -EBUSY) {
				XDX_DEV_ERROR("vcmd_driver: IRQ <%d> busy, change your config.，core_id=%d\n",
				       subsys_core->vcmd_core_cfg.vcmd_irq, i);
				vcmd_release_IO(subsys_dev);
				goto err;
			}
		} else {
			XDX_DEV_INFO("vcmd_driver: IRQ not in use!\n");
		}
		subsys_core = subsys_core->core_next;
	}
	//subsys_dev->cmdbuf_used_pos = 0;
	for (k = 1; k < TOTAL_DISCRETE_CMDBUF_NUM; k++) {
		new_cmdbuf_node = create_cmdbuf_node(subsys_dev, k);
		if(new_cmdbuf_node)
		{
			spin_lock_irqsave(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);
			bi_list_insert_node_tail(&subsys_dev->global_vcmd_res, new_cmdbuf_node);
			spin_unlock_irqrestore(&subsys_dev->vcmd_cmdbuf_alloc_lock, flags);
		}
	}
	
	create_kernel_process_manager(subsys_dev);
	for (i = 0; i < MAX_VCMD_TYPE; i++) {
		if (subsys_dev->vcmd_type_core_num[i] == 0)
			continue;
		sema_init(&subsys_dev->vcmd_reserve_cmdbuf_sem[i], 1);
	}
	/* read all registers for each type of module
	 *for analyzing configuration in cwl
	 */
	subsys_core = subsys_dev->vcmd_core;
	if(subsys_core == NULL) {
		goto err;
	}
	for (i = 0; i < subsys_dev->subsys_num; i++) {
		read_main_module_all_registers(subsys_core);
		subsys_core = subsys_core->core_next;
		if(subsys_core == NULL)
			break;
	}
	return 0;
err:
	return result;
}
static void hantro_vcmd_mem_pool_release(vcmd_dev_str *subsys_dev)
{
#ifndef VSI_FPGA
	dma_addr_t dma_handle;

	dma_handle = (dma_addr_t)subsys_dev->vcmd_buf_mem_pool->busAddress;
	dma_free_coherent(subsys_dev->dev,
			  CMDBUF_POOL_TOTAL_SIZE + CMDBUF_POOL_TOTAL_SIZE +
				  CMDBUF_VCMD_REGISTER_TOTAL_SIZE,
			  (void *)subsys_dev->vcmd_buf_mem_pool->virtualAddress,
			  dma_handle);
#else
	iounmap((void *)subsys_dev->vcmd_buf_mem_pool->virtualAddress);
	release_mem_region(subsys_dev->vcmd_buf_mem_pool->busAddress,
			   subsys_dev->vcmd_buf_mem_pool->size);
	iounmap((void *)subsys_dev->vcmd_status_buf_mem_pool->virtualAddress);
	release_mem_region(subsys_dev->vcmd_status_buf_mem_pool->busAddress,
			   subsys_dev->vcmd_status_buf_mem_pool->size);
	iounmap((void *)subsys_dev->vcmd_registers_mem_pool->virtualAddress);
	release_mem_region(subsys_dev->vcmd_registers_mem_pool->busAddress,
			   subsys_dev->vcmd_registers_mem_pool->size);
#endif
	kfree(subsys_dev->vcmd_buf_mem_pool);
	kfree(subsys_dev->vcmd_status_buf_mem_pool);
	kfree(subsys_dev->vcmd_registers_mem_pool);
}

static int hantro_vcmd_subsys_cleanup(vcmd_dev_str *subsys_dev)
{
	u32 i;
	u32 result;
	vcmd_core_str *subsys_core;
	vcmd_core_str *back_core;

	subsys_core = subsys_dev->vcmd_core;

	for (i = 0; i < subsys_dev->subsys_num; i++) {
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
		//disable HW
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_CONTROL_OFFSET, 0x0000);
		//read status register
		result = vcmd_read_reg((const void *)subsys_core->hwregs,
				       VCMD_REGISTER_INT_STATUS_OFFSET);
		//clean status register
		vcmd_write_reg((const void *)subsys_core->hwregs,
			       VCMD_REGISTER_INT_STATUS_OFFSET, result);

		/* free the vcmd IRQ */
		if (subsys_core->vcmd_core_cfg.vcmd_irq != -1)
			free_irq(subsys_core->vcmd_core_cfg.vcmd_irq,
				 (void *)subsys_core);
		release_cmdbuf_node_cleanup(subsys_dev,
					    &subsys_core->list_manager);
		subsys_core = subsys_core->core_next;
	}
	release_process_node_cleanup(&subsys_dev->global_process_manager);
	destroy_cmdbuf_node_cleanup(&subsys_dev->global_vcmd_res);

	vcmd_release_IO(subsys_dev);

	subsys_core = subsys_dev->vcmd_core;

	for (i = 0; i < subsys_dev->subsys_num; i++) {
		back_core = subsys_core;
		subsys_core = subsys_core->core_next;
		kfree(back_core);
	}

	subsys_dev->subsys_num = 0;
	subsys_dev->vcmd_core = NULL;
	if (subsys_dev->vcmd_bo)
		xdx_gem_object_put(&subsys_dev->vcmd_bo->base);
	if (subsys_dev->vcmd_bo)
		xdx_gem_object_put(&subsys_dev->vcmd_status_bo->base);
	if (subsys_dev->vcmd_bo)
		xdx_gem_object_put(&subsys_dev->vcmd_reg_bo->base);

	return 0;
}

#ifdef USE_DTB_PROBE

int hantro_vcmd_init(u32 sliceidx)
{
	vcmd_slice_str *cur_slice;

	cur_slice = get_slice_by_sliceidx(sliceidx);
	if (cur_slice->dec_vcmd.subsys_num != 0)
		hantro_vcmd_subsys_probe(&cur_slice->dec_vcmd);
	if (cur_slice->enc_vcmd.subsys_num != 0)
		hantro_vcmd_subsys_probe(&cur_slice->enc_vcmd);
	return 0;
}

int hantro_vcmd_probe(struct device *dev, struct device_node *slice)
{
	vcmd_slice_str *cur_slice;

	cur_slice = hantro_vcmd_analyze_subnode(dev, slice);
	return cur_slice->sliceidx;
}
#else

static void hantro_vcmd_hw_restart(struct vcmd_dev *subsys_dev)
{
	unsigned int i;
	vcmd_core_str *subsys_core = subsys_dev->vcmd_core;
	u32 cmdbuf_id;

	for (i = 0; i < subsys_dev->subsys_num; i++) {
		// read_main_module_all_registers(subsys_core);
		if (subsys_core->hw_version_id > HW_ID_1_0_C) {
			cmdbuf_id = *(subsys_core->vcmd_reg_mem_virtualAddress +
				      EXECUTING_CMDBUF_ID_ADDR);
			if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM)
				cmdbuf_id = 1;
			vcmd_write_register_value(
				(const void *)subsys_core->hwregs,
				subsys_core->reg_mirror,
				HWIF_VCMD_CMDBUF_EXECUTING_ID,
				cmdbuf_id);
		}
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x40,
				subsys_core->reg_mirror[0x40 / 4]);
		vcmd_write_reg(
			(const void *)subsys_core->hwregs, 0x44,
			vcmd_read_reg((const void *)subsys_core->hwregs,
					0x44));
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x48,
				subsys_core->reg_mirror[0x48 / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x4c,
				subsys_core->reg_mirror[0x4c / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x50,
				subsys_core->reg_mirror[0x50 / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x54,
				subsys_core->reg_mirror[0x54 / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x58,
				subsys_core->reg_mirror[0x58 / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x5c,
				subsys_core->reg_mirror[0x5c / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x60,
				subsys_core->reg_mirror[0x60 / 4]);
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x64,
				0xffffffff); //not interrupt cpu
		vcmd_write_reg((const void *)subsys_core->hwregs, 0x40,
				subsys_core->reg_mirror[0x40 / 4]);
		subsys_core = subsys_core->core_next;
		if(subsys_core == NULL)
			break;
	}
}

int xdx_vcmd_hw_reset(struct xdx_device* xdev)
{
	int slice_num, i;
	vcmd_slice_str *cur_slice;

#ifdef VSI_FPGA
	struct hantro_pci_t vcmd_pci_par;

	if (pcie_init(xdev,&vcmd_pci_par) < 0) {
		pr_err("%s,%d err pci init failed\n", __func__, __LINE__);
		return -1;
	}
#ifndef HAS_MMU
	set_mmu_bypass(xdev);
#endif
#if defined(SUPPORT_VPU_SMMU) && defined(HAS_MMU)
	if(xdev->asic_type == CHIP_PANGU_B0){
		set_mmu_bypass(xdev);
	}
#endif
	irq_vcmd_init(xdxvpu_dev);
#endif
	slice_num = get_vcmd_slice_num();
	for (i = 0; i < slice_num; i++) {
		cur_slice = get_slice_by_sliceidx(i);
		if (!cur_slice) {
			pr_err("%s,%d slice num %d is null\n", __func__, __LINE__, i);
			continue;
		}
#ifdef HAS_VC8000D
		if (cur_slice->dec_vcmd.subsys_num != 0) {
			vcmd_reset_asic(&cur_slice->dec_vcmd);
			hantro_vcmd_hw_restart(&cur_slice->dec_vcmd);
		}
#endif
#ifdef HAS_VC8000E
		if (cur_slice->enc_vcmd.subsys_num != 0) {
			vcmd_reset_asic(&cur_slice->enc_vcmd);
			hantro_vcmd_hw_restart(&cur_slice->enc_vcmd);
		}
#endif
	}
	return 0;
}

int hantro_vcmd_init(struct xdx_device* xdev)
{
	int i;
	int slice_num = 0;
	vcmd_slice_str *cur_slice;

#ifdef VSI_FPGA
	struct hantro_pci_t vcmd_pci_par;

	if (pcie_init(xdev,&vcmd_pci_par) < 0) {
		pr_err("%s,%d err pci init failed\n", __func__, __LINE__);
		return -1;
	}

#if 0
	hantro_fpga_init((unsigned long)(vcmd_pci_par.pci_base_ddr_hw),
				 HLINA_TRANSL_VCMD_SIZE + 0x800000);
#endif

	xdxvpu_dev_reg_map(xdev);
#ifndef HAS_MMU
	set_mmu_bypass(xdev);
#endif
#if defined(SUPPORT_VPU_SMMU) && defined(HAS_MMU)
	if(xdev->asic_type == CHIP_PANGU_B0){
		set_mmu_bypass(xdev);
	}
#endif
	irq_vcmd_init(xdxvpu_dev);
#endif

	slice_num = get_vcmd_slice_num();
	for (i = 0; i < slice_num; i++) {
		cur_slice = get_slice_by_sliceidx(i);
		if (!cur_slice) {
			pr_err("%s,%d slice num %d is null\n", __func__, __LINE__, i);
			continue;
		}
#ifdef HAS_VC8000D
		if (cur_slice->dec_vcmd.subsys_num != 0) {
#ifdef VSI_FPGA
			vcmd_store_pcie_info(&cur_slice->dec_vcmd, &vcmd_pci_par);
#endif
			hantro_vcmd_mem_pool_init(xdev,&cur_slice->dec_vcmd);
			hantro_vcmd_subsys_probe(xdev, &cur_slice->dec_vcmd);
		}
#endif

#ifdef HAS_VC8000E
		if (cur_slice->enc_vcmd.subsys_num != 0) {
#ifdef VSI_FPGA
			vcmd_store_pcie_info(&cur_slice->enc_vcmd, &vcmd_pci_par);
#endif
			hantro_vcmd_mem_pool_init(xdev,&cur_slice->enc_vcmd);
			hantro_vcmd_subsys_probe(xdev, &cur_slice->enc_vcmd);
		}
#endif
	}
	return 0;
}

static int hantro_get_core_type(u32 sub_module_type)
{
	int core_type;

	switch (sub_module_type) {
#ifdef HAS_VC8000E
	case VCMD_TYPE_ENCODER:
	case VCMD_TYPE_CUTREE:
		core_type = HANTRO_CORE_ENC;
		break;
	case VCMD_TYPE_JPEG_ENCODER:
		core_type = HANTRO_CORE_ENCJPG;
		break;
#endif
#ifdef HAS_VC8000D
	case VCMD_TYPE_DECODER:
		core_type = HANTRO_CORE_DEC;
		break;
	case VCMD_TYPE_JPEG_DECODER:
		core_type = HANTRO_CORE_DECJPG;
		break;
#endif
	default:
		core_type = -1;
		break;
	}
	return core_type;
}

static void sku_init(struct xdx_device *xdev,	vcmd_core_str *vcmd_core)
{
	u32 sku = xdev->pdev->device;
	struct vcmd_dev *vcmd = vcmd_core->parent_dev;

	vcmd_timer_init(vcmd);
	switch (sku) {
	case 0x1160:
	case 0x1170:
		if (vcmd_core->type == HANTRO_CORE_DEC ||
		    vcmd_core->type == HANTRO_CORE_DECJPG) {
			vcmd->sku_size = 4096*2160*70;
		} else if (vcmd_core->type == HANTRO_CORE_ENC ||
			   vcmd_core->type == HANTRO_CORE_ENCJPG) {
			vcmd->sku_size = 1920*1088*70;
		} else {
			break;
		}

		break;
	case 0x10a4:
	case 0x1140:
	case 0x1330:
	case 0x1333:
	case 0x1340:
	case 0x1350:
	case 0x1360:
	case 0x13c0:
	case 0x13d3:
		if (vcmd_core->type == HANTRO_CORE_DEC ||
		    vcmd_core->type == HANTRO_CORE_DECJPG) {
			vcmd->sku_size = 1920*1088*(16*30 + 10);//16 channels for 1080p decode
		} else if (vcmd_core->type == HANTRO_CORE_ENC ||
			   vcmd_core->type == HANTRO_CORE_ENCJPG) {
			vcmd->sku_size = 1920*1088*(4*30 + 10);//4 channels for 1080p encode
		} else {
			break;
		}

		break;
	case 0x11e0:
	case 0x1500:
	case 0x15a0:
	case 0x1324:
	case 0x15a3:
	case 0x15a4:
		if (vcmd_core->type == HANTRO_CORE_DEC ||
		    vcmd_core->type == HANTRO_CORE_DECJPG) {

			vcmd->sku_size = 1920*1088*(16*30 + 10);//16 channels for 1080p decode
		} else if (vcmd_core->type == HANTRO_CORE_ENC ||
			   vcmd_core->type == HANTRO_CORE_ENCJPG) {
			vcmd->sku_size = 1920*1088*(8*30 + 10);//8 channels for 1080p encode
		} else {
			break;
		}

		break;
	default:
		vcmd->sku_size = 1920*1088*130;
		break;
	}
}

void vcmd_core_array_init(void)
{
	int i, j;
	int slice_num = 0;
	int core_num = 0;

	slice_num = ARRAY_SIZE(vcmd_core_array);
	for (i = 0; i < slice_num; i++) {
		core_num = ARRAY_SIZE(vcmd_core_array[i]);
		for (j = 0; j < core_num; j++) {
			if (vcmd_core_array[i][j].submodule_main_addr == 0)
				continue;
			if (hantro_get_core_type(vcmd_core_array[i][j].sub_module_type) == VCMD_DEC_MODULE_TYPE_0) {
				if (hantro_dev.asic_type == CHIP_PANGU_B0) {
					vcmd_core_array[i][j].submodule_L2Cache_addr = VCMD_DEC_MODULE_L2CACHE_ADDR_0_B0;
					vcmd_core_array[i][j].submodule_axife_addr[0] = VCMD_DEC_MODULE_AXIFE0_ADDR_0_B0;
				}
			}
		}
	}
	return ;
}

int hantro_vcmd_probe(struct xdx_device *xdev, struct device_node *slice)
{
	int i, j;
	int slice_num = 0;
	int core_num = 0;
	vcmd_slice_str *cur_slice;
	vcmd_core_str *vcmd_core;
	struct vcmd_config *vcmd_cfg_p;
	struct device *dev = xdev->dev;

	slice_num = ARRAY_SIZE(vcmd_core_array);
	for (i = 0; i < slice_num; i++) {
		cur_slice = kzalloc(sizeof(*cur_slice), GFP_KERNEL);
		add_vcmd_slice(dev, cur_slice);
		list_add_tail(&cur_slice->list, &xdev->vpu.vcmd_slice);
		cur_slice->vpu = &xdev->vpu;
		INIT_LIST_HEAD(&cur_slice->dec_vcmd.pro_mngr);
		INIT_LIST_HEAD(&cur_slice->enc_vcmd.pro_mngr);
		core_num = ARRAY_SIZE(vcmd_core_array[i]);
		for (j = 0; j < core_num; j++) {
			if (vcmd_core_array[i][j].submodule_main_addr == 0)
				continue;
			vcmd_core = kzalloc(sizeof(*vcmd_core), GFP_KERNEL);
			vcmd_cfg_p = &vcmd_core->vcmd_core_cfg;
			memcpy(vcmd_cfg_p, &vcmd_core_array[i][j],
			       sizeof(struct vcmd_config));
			vcmd_core->type = hantro_get_core_type(
				vcmd_cfg_p->sub_module_type);
			if (vcmd_core->type == -1) {
				XDX_DEV_ERROR("error! %s,%d sub_module_type is not correct\n",
				       __func__, __LINE__);
				kfree(vcmd_core);
				continue;
			}
			pr_info("[%s]%d sku:%x \n", __func__, __LINE__,
				    xdev->pdev->device);
			add_vcmd_core(cur_slice, vcmd_core);
			sku_init(xdev, vcmd_core);
#ifndef HAS_AXIFE
			vcmd_cfg_p->submodule_axife_addr[0] = 0xffff;
			vcmd_cfg_p->submodule_axife_addr[1] = 0xffff;
#endif
#ifndef HAS_CACHECORE
			vcmd_cfg_p->submodule_L2Cache_addr = 0xffff;
#endif
#ifndef HAS_DEC400
			vcmd_cfg_p->submodule_dec400_addr = 0xffff;
#endif
#ifndef HAS_MMU
			vcmd_cfg_p->submodule_MMU_addr = 0xffff;
			vcmd_cfg_p->submodule_MMUWrite_addr = 0xffff;
#endif
#ifdef SUPPORT_VPU_SMMU
			if(xdev->asic_type == CHIP_PANGU_B0){
				vcmd_cfg_p->submodule_MMU_addr = 0xffff;
				vcmd_cfg_p->submodule_MMUWrite_addr = 0xffff;
			}
#endif
#ifdef SUPPORT_VPU_SMMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0){
				cur_slice->config |= CONFIG_VPU_SMMU;
			}
#endif
			cur_slice->config |= CONFIG_VCMD;
			if (vcmd_cfg_p->submodule_dec400_addr != 0xffff)
				cur_slice->config |= CONFIG_DEC400;
			if (vcmd_cfg_p->submodule_L2Cache_addr != 0xffff)
				cur_slice->config |= CONFIG_L2CACHE;
			if (vcmd_cfg_p->submodule_axife_addr[0] != 0xffff)
				cur_slice->config |= CONFIG_AXIFE;
			if (vcmd_cfg_p->submodule_MMU_addr != 0xffff)
				cur_slice->config |= CONFIG_HANTROMMU;
			if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
				cur_slice->config &= (~CONFIG_HANTROMMU);
#endif
			}
		}
	}
	return slice_num;
}

#endif

int hantro_vcmd_cleanup(void)
{
	vcmd_slice_str *cur_slice;
	vcmd_slice_str *back_slice;

	cur_slice = get_vcmd_slice_head();
	while (cur_slice) {
		if (cur_slice->dec_vcmd.subsys_num != 0) {
			vcmd_timer_deinit(&cur_slice->dec_vcmd);
			hantro_vcmd_subsys_cleanup(&cur_slice->dec_vcmd);
			hantro_vcmd_mem_pool_release(&cur_slice->dec_vcmd);
		}
		if (cur_slice->enc_vcmd.subsys_num != 0) {
			vcmd_timer_deinit(&cur_slice->enc_vcmd);
			hantro_vcmd_subsys_cleanup(&cur_slice->enc_vcmd);
			hantro_vcmd_mem_pool_release(&cur_slice->enc_vcmd);
		}
		back_slice = cur_slice;
		cur_slice = cur_slice->slice_next;
		kfree(back_slice);
	}
	xdxvpu_dev_reg_ummap();
	set_vcmd_slice_listnull();
#ifdef VSI_FPGA
	hantro_fpga_release();
#endif
	return 1;
}

/* Update the last JMP cmd in cmdbuf_ojb in order to jump to next_cmdbuf_obj. */
static void cmdbuf_update_jmp_cmd(int hw_version_id,
					struct cmdbuf_obj *cmdbuf_obj,
					struct cmdbuf_obj *next_cmdbuf_obj,
					int jmp_IE_1,
					struct vcmd_dev *subsys_dev)
{
	u32 *jmp_addr;
	u32 vcmd_array[3];
	u64 next_cmd_buf_addr;
	u32 operation_code;

	if (!cmdbuf_obj)
		return;

	if (cmdbuf_obj->has_end_cmdbuf == 0) {
		//need to link, current cmdbuf link to next cmdbuf
		jmp_addr = cmdbuf_obj->cmdbuf_virtualAddress + (cmdbuf_obj->cmdbuf_size >> 2);
		if (!next_cmdbuf_obj) {
		// If next cmdbuf is not available, set the RDY to 0.
			operation_code = *(jmp_addr - 4);
			//operation_code >>= 16;
			//operation_code <<= 16;
			operation_code &= 0xFFFF0000;
			
			*(jmp_addr - 4) = (u32)(operation_code & ~JMP_RDY_1);
		} else {
			if (hw_version_id > HW_ID_1_0_C) {
				//set next cmdbuf id
				//*(jmp_addr - 1) = next_cmdbuf_obj->cmdbuf_id;
				vcmd_array[2] = next_cmdbuf_obj->cmdbuf_id;
			}
			else
			{
				vcmd_array[2] = *(jmp_addr - 1);
			}
			next_cmd_buf_addr = (u64)(next_cmdbuf_obj->cmdbuf_busAddress -
					subsys_dev->base_ddr_addr);

#ifdef HAS_MMU
			next_cmd_buf_addr = (u64)(next_cmdbuf_obj->mmu_cmdbuf_bus_address);
#endif
#ifdef SUPPORT_VPU_SMMU
			if (hantro_dev.asic_type == CHIP_PANGU_B0){
				next_cmd_buf_addr = (u64)(next_cmdbuf_obj->smmu_cmdbuf_bus_address);
			}
#endif
			if (sizeof(size_t) == 8) {
				//*(jmp_addr - 2) = (u32)(next_cmd_buf_addr >> 32);
				vcmd_array[1] = (u32)(next_cmd_buf_addr >> 32);
			} else {
				//*(jmp_addr - 2) = 0;
				vcmd_array[1] = 0;
			}
			//*(jmp_addr - 3) = (u32)(next_cmd_buf_addr);
			vcmd_array[0] = (u32)(next_cmd_buf_addr);
			memcpy((jmp_addr - 3), vcmd_array, sizeof(vcmd_array)); 

			operation_code = *(jmp_addr - 4);
			// operation_code >>= 16;
			// operation_code <<= 16;
			operation_code &= 0xFFFF0000;
			//*(jmp_addr - 4) = (u32)(operation_code | JMP_RDY_1 | jmp_IE_1 |
			//	((next_cmdbuf_obj->cmdbuf_size + 7) / 8));
			*(jmp_addr - 4) = (u32)(operation_code | JMP_RDY_1 | jmp_IE_1 | ((next_cmdbuf_obj->cmdbuf_size + 7) >> 3));
			
			// guarantee gddr write is complete to avoid cmd_err added by liyao
			OSWriteMemoryBarrier(jmp_addr - 4);
		}

#ifdef VCMD_DEBUG_INTERNAL
		{
		u32 i;

		XDX_DEV_INFO("vcmd link, last cmdbuf content\n");
		for (i = cmdbuf_obj->cmdbuf_size / 4 - 8; i < cmdbuf_obj->cmdbuf_size / 4; i++)
			XDX_DEV_INFO("current linked cmdbuf data %d =0x%x\n",
				i, *(cmdbuf_obj->cmdbuf_virtualAddress + i));
		}
#endif
	}
}

/* delink given cmd buffer (cmdbuf_node) and remove it from list.
 * Also modify the last JMP of buf P to point to cmdbuf N.
 * Used when a process is terminated but there are pending cmd bufs in vmcd list.
 * E.g.,
 * before:

 *   L->L->...->P->X->N->        ...        ->L
 *   ^             ^                          ^
 *   head          cmdbuf_node                tail

 * end:

 *   L->L->...->P->N->        ...        ->L
 *   ^                                     ^
 *   head                                  tail

 * Return: pointer to N or NULL if N doesn't exist.
 */

static void vcmd_delink_rm_cmdbuf(vcmd_core_str *subsys_core, bi_list_node *cmdbuf_node)
{
	struct vcmd_dev *subsys_dev = subsys_core->parent_dev;
	bi_list *list = &subsys_core->list_manager;
	struct cmdbuf_obj *cmdbuf_obj = (struct cmdbuf_obj *)cmdbuf_node->data;
	bi_list_node *prev = cmdbuf_node->previous;
	bi_list_node *next = cmdbuf_node->next;

	XDX_DEV_DEBUG("Delink and remove cmdbuf [%d] from vcmd list.\n",
	       cmdbuf_obj->cmdbuf_id);
	//#ifdef HANTRO_VCMD_DRIVER_DEBUG
	if (prev) {
		XDX_DEV_DEBUG("prev cmdbuf [%d].\n",
		       ((struct cmdbuf_obj *)prev->data)->cmdbuf_id);
	} else {
		XDX_DEV_DEBUG("NO prev cmdbuf.\n");
	}
	if (next) {
		XDX_DEV_DEBUG("next cmdbuf [%d].\n",
		       ((struct cmdbuf_obj *)next->data)->cmdbuf_id);
	} else {
		XDX_DEV_DEBUG("NO next cmdbuf.\n");
	}
	//#endif

	bi_list_remove_node(list, cmdbuf_node);
	subsys_dev->global_cmdbuf_node[cmdbuf_obj->cmdbuf_id] = NULL;
	free_cmdbuf_node(subsys_dev, cmdbuf_node);

	cmdbuf_update_jmp_cmd(subsys_core->hw_version_id, prev ? prev->data : NULL, next ? next->data : NULL,
			subsys_core->duration_without_int > INT_MIN_SUM_OF_IMAGE_SIZE, subsys_dev);
}
#define SUBS_DEV_COUNT (2)

int hantrovcmd_open(struct inode *inode, struct file *filp)
{
	int result = 0;
	int i = 0;
	vcmd_slice_str *cur_slice = get_vcmd_slice_head();
//TODO: refine enc/dec in one platform

	struct vcmd_dev *all_subsys_dev[SUBS_DEV_COUNT] = {NULL};

	if (cur_slice->enc_vcmd.subsys_num > 0)
		all_subsys_dev[0] = &cur_slice->enc_vcmd;
	if (cur_slice->dec_vcmd.subsys_num > 0)
		all_subsys_dev[1] = &cur_slice->dec_vcmd;

	for (i = 0; i < SUBS_DEV_COUNT; i++)
	{
		vcmd_core_str *subsys_core = NULL;
		unsigned long flags;
		struct process_manager_obj *process_manager_obj = NULL;
		struct vcmd_dev *subsys_dev = all_subsys_dev[i];
		bi_list_node *process_manager_node;

		if (subsys_dev == NULL)
			continue;
		subsys_core = subsys_dev->vcmd_core;
		//filp->private_data = (void *) subsys_core;
		subsys_core->pre_cmdbuf_id = -1;
		process_manager_node = create_process_manager_node();
		if (!process_manager_node)
			return -1;
		process_manager_obj = (struct process_manager_obj *)process_manager_node->data;
		process_manager_obj->filp = filp;//#h264bsdA

		spin_lock_irqsave(&subsys_dev->vcmd_process_manager_lock, flags);
		bi_list_insert_node_tail(&subsys_dev->global_process_manager, process_manager_node);
		list_add_tail(&process_manager_obj->list, &subsys_dev->pro_mngr);

		spin_unlock_irqrestore(&subsys_dev->vcmd_process_manager_lock, flags);

		XDX_DEV_DEBUG("dev opened\n");
		XDX_DEV_DEBUG("process node %p for filp opened %p\n",
			(void *)process_manager_node, (void *)filp);
	}

	return result;
}

int hantrovcmd_release(struct inode *inode, struct file *filp)
{
	//vcmd_core_str *subsys_core = (vcmd_core_str *) filp->private_data;
	//struct vcmd_dev *subsys_dev = subsys_core->parent_dev;
	vcmd_slice_str *cur_slice = NULL;

	u32 core_id = 0;
	u32 i = 0;
	unsigned int total_vcmd_core_num = 0;
	u32 release_cmdbuf_num = 0;
	bi_list_node *new_cmdbuf_node = NULL;
	struct cmdbuf_obj *cmdbuf_obj_temp = NULL;
	bi_list_node *process_manager_node = NULL;
	struct process_manager_obj *process_manager_obj = NULL;
	int vcmd_aborted = 0;  // vcmd is aborted in this function
	struct cmdbuf_obj *restart_cmdbuf = NULL;

	unsigned long flags;
	long retVal = 0;
	struct vcmd_dev *all_subsys_dev[SUBS_DEV_COUNT] = {NULL};

	if (!filp)
		XDX_DEV_DEBUG("%s,%d\n", __func__, __LINE__);

	cur_slice = get_vcmd_slice_head();
	total_vcmd_core_num = sizeof(vcmd_core_array) / sizeof(struct vcmd_config);

//TODO: refine enc/dec in one platform
	if (cur_slice->enc_vcmd.subsys_num > 0)
		all_subsys_dev[0] = &cur_slice->enc_vcmd;
	if (cur_slice->dec_vcmd.subsys_num > 0)
		all_subsys_dev[1] = &cur_slice->dec_vcmd;

	for (i = 0; i < SUBS_DEV_COUNT; i++)
	{
		vcmd_core_str *subsys_core = NULL;
		struct vcmd_dev *subsys_dev = all_subsys_dev[i];
		if (subsys_dev == NULL)
			continue;
		subsys_core = subsys_dev->vcmd_core;

		if (down_interruptible(&subsys_dev->vcmd_reserve_cmdbuf_sem[subsys_dev->vcmd_core->vcmd_core_cfg.sub_module_type])) {
			return -ERESTARTSYS;
		}
		for (core_id = 0; core_id < total_vcmd_core_num; core_id++) {
			if ((subsys_core) == NULL)
				continue;
			spin_lock_irqsave(&subsys_core->spinlock, flags);
			new_cmdbuf_node = subsys_core->list_manager.head;
	#if 0
			while (1) {
				if (!new_cmdbuf_node)
					break;
				XDX_DEV_DEBUG("Before: new_cmdbuf_node->data->filp = %p with current filp: %p\n", ((struct cmdbuf_obj *)new_cmdbuf_node->data)->filp, filp);
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
			new_cmdbuf_node = subsys_core->list_manager.head;
	#endif
			while (1) {
				if (!new_cmdbuf_node)
					break;
				cmdbuf_obj_temp = (struct cmdbuf_obj *)new_cmdbuf_node->data;
				if (subsys_core->hwregs && (cmdbuf_obj_temp->filp == filp)) {
					if (cmdbuf_obj_temp->cmdbuf_run_done) {
						cmdbuf_obj_temp->cmdbuf_need_remove = 1;
						retVal = release_cmdbuf_node(subsys_dev,
							&subsys_core->list_manager, new_cmdbuf_node);
						if (retVal == 1)
							cmdbuf_obj_temp->process_manager_obj = NULL;
					} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 0) {
						cmdbuf_obj_temp->cmdbuf_data_linked = 1;
						cmdbuf_obj_temp->cmdbuf_run_done    = 1;
						cmdbuf_obj_temp->cmdbuf_need_remove = 1;
						retVal = release_cmdbuf_node(subsys_dev,
							&subsys_core->list_manager, new_cmdbuf_node);
						if (retVal == 1)
							cmdbuf_obj_temp->process_manager_obj = NULL;
					} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 1 &&
						subsys_core->working_state == WORKING_STATE_IDLE) {
						vcmd_delink_rm_cmdbuf(subsys_core, new_cmdbuf_node);
						if (restart_cmdbuf == cmdbuf_obj_temp)
							restart_cmdbuf = new_cmdbuf_node->next ?
								new_cmdbuf_node->next->data : NULL;
						if (restart_cmdbuf) {
							XDX_DEV_DEBUG("Set restart cmdbuf [%d].\n",
								restart_cmdbuf->cmdbuf_id);
						} else {
							XDX_DEV_DEBUG("Set restart cmdbuf to NULL.\n");
						}
					} else if (cmdbuf_obj_temp->cmdbuf_data_linked == 1 &&
						subsys_core->working_state == WORKING_STATE_WORKING) {
						bi_list_node *last_cmdbuf_node = NULL;
						bi_list_node *done_cmdbuf_node = NULL;
						int abort_cmdbuf_id;
						int loop_count = 0;

						//abort the vcmd and wait
						XDX_DEV_DEBUG("Abort due to linked cmdbuf %d of current process.\n",
							cmdbuf_obj_temp->cmdbuf_id);
	#ifdef VCMD_DEBUG_INTERNAL
						//printk_vcmd_register_debug((const void *)subsys_core->hwregs, "Before trigger to 0");
	#endif
						// disable abort interrupt
						//vcmd_write_register_value((const void *)subsys_core->hwregs,subsys_core->reg_mirror,HWIF_VCMD_IRQ_ABORT_EN,0);
						vcmd_write_register_value((const void *)subsys_core->hwregs,
							subsys_core->reg_mirror, HWIF_VCMD_START_TRIGGER, 0);
						vcmd_aborted = 1;
						subsys_dev->software_triger_abort = 1;
	#ifdef VCMD_DEBUG_INTERNAL
						//printk_vcmd_register_debug((const void *)subsys_core->hwregs,"After trigger to 0");
	#endif
						// Wait vcmd core aborted and vcmd enters IDLE mode.
						//while (subsys_core->working_state != WORKING_STATE_IDLE) {
						while (vcmd_get_register_value((const void *)subsys_core->hwregs,
							subsys_core->reg_mirror, HWIF_VCMD_WORK_STATE)) {
							loop_count++;

#if 0
							if (!(loop_count % 10)) {
								u32 irq_status = vcmd_read_reg((const void *)subsys_core->hwregs,
									VCMD_REGISTER_INT_STATUS_OFFSET);
							}
#endif
							mdelay(10);  // wait 10ms
							if (loop_count > 100) { // too long
								XDX_DEV_ERROR("hantrovcmd: too long before vcmd core to IDLE state\n");
								spin_unlock_irqrestore(&subsys_core->spinlock, flags);
								up(&subsys_dev->vcmd_reserve_cmdbuf_sem[subsys_dev->vcmd_core->vcmd_core_cfg.sub_module_type]);
								return -ERESTARTSYS;
							}
						}

						subsys_core->working_state = WORKING_STATE_IDLE;
						// clear interrupt & restore abort_e
						if (vcmd_get_register_value((const void *)subsys_core->hwregs,
							subsys_core->reg_mirror, HWIF_VCMD_IRQ_ABORT)) {
							XDX_DEV_DEBUG("Abort interrupt triggered, now clear it to avoid abort int...\n");

							vcmd_write_reg((const void *)(subsys_core->hwregs),
								VCMD_REGISTER_INT_STATUS_OFFSET, 0x1 << 4);

							XDX_DEV_DEBUG("Now irq status = 0x%08x.\n",
								vcmd_read_reg((const void *)subsys_core->hwregs, VCMD_REGISTER_INT_STATUS_OFFSET));
						}

	#ifdef VCMD_DEBUG_INTERNAL
						//printk_vcmd_register_debug((const void *)subsys_core->hwregs, "vcmd status to IDLE");
	#endif
						abort_cmdbuf_id = vcmd_get_register_value((const void *)subsys_core->hwregs,
							subsys_core->reg_mirror, HWIF_VCMD_CMDBUF_EXECUTING_ID);
						XDX_DEV_DEBUG("Abort when executing cmd buf %d.\n", abort_cmdbuf_id);
						subsys_core->sw_cmdbuf_rdy_num = 0;
						subsys_core->duration_without_int = 0;
						vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
							HWIF_VCMD_EXE_CMDBUF_COUNT, 0);
						vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
							HWIF_VCMD_RDY_CMDBUF_COUNT, 0);

						/* Mark cmdbuf_run_done to 1 for all the cmd buf executed. */
						done_cmdbuf_node = subsys_core->list_manager.head;
						while (done_cmdbuf_node) {
							if (!((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_run_done) {
								((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_run_done = 1;
								((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_data_linked = 0;
								XDX_DEV_DEBUG("Set cmdbuf [%d] cmdbuf_run_done to 1.\n",
									((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_id);
							}
							if (((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_id == abort_cmdbuf_id)
								break;
							done_cmdbuf_node = done_cmdbuf_node->next;
						}
						if (cmdbuf_obj_temp->cmdbuf_run_done) {
							/* current cmdbuf is in fact has been executed, but due to interrupt is not triggered,
							* the status is not updated.
							* Just delink and remove it from the list.
							*/
							if (done_cmdbuf_node && done_cmdbuf_node->data) {
								XDX_DEV_DEBUG("done_cmdbuf_node is cmdbuf [%d].\n",
									((struct cmdbuf_obj *)done_cmdbuf_node->data)->cmdbuf_id);
							}
							done_cmdbuf_node = done_cmdbuf_node->next;
							if (done_cmdbuf_node)
								restart_cmdbuf = (struct cmdbuf_obj *)done_cmdbuf_node->data;
							if (restart_cmdbuf) {
								XDX_DEV_DEBUG("Set restart cmdbuf [%d] via if.\n", restart_cmdbuf->cmdbuf_id);
							}
						} else {
							last_cmdbuf_node = new_cmdbuf_node;
							/* cmd buf num from aborted cmd buf to current cmdbuf_obj_temp */
							if (cmdbuf_obj_temp->cmdbuf_id != abort_cmdbuf_id) {
								last_cmdbuf_node  = new_cmdbuf_node->previous;

								while (last_cmdbuf_node &&
									((struct cmdbuf_obj *)last_cmdbuf_node->data)->cmdbuf_id != abort_cmdbuf_id) {
									restart_cmdbuf = (struct cmdbuf_obj *)last_cmdbuf_node->data;
									last_cmdbuf_node = last_cmdbuf_node->previous;
									subsys_core->sw_cmdbuf_rdy_num++;
									subsys_core->duration_without_int += restart_cmdbuf->executing_time;
									XDX_DEV_DEBUG("Keep valid cmdbuf [%d] in the list.\n", restart_cmdbuf->cmdbuf_id);
								}
							}
							if (restart_cmdbuf) {
								XDX_DEV_DEBUG("Set restart cmdbuf [%d] via else.\n", restart_cmdbuf->cmdbuf_id);
							}
						}
						// remove first linked cmdbuf from list
						vcmd_delink_rm_cmdbuf(subsys_core, new_cmdbuf_node);
					}
					subsys_dev->software_triger_abort = 0;
					release_cmdbuf_num++;
					XDX_DEV_DEBUG("release reserved cmdbuf\n");
				} else if (vcmd_aborted && !cmdbuf_obj_temp->cmdbuf_run_done) {
					/* VCMD is aborted, need to re-calculate the duration_without_int */
					if (!restart_cmdbuf)
						restart_cmdbuf = cmdbuf_obj_temp; /* first cmdbuf to be restarted */
					subsys_core->duration_without_int += cmdbuf_obj_temp->executing_time;
					subsys_core->sw_cmdbuf_rdy_num++;
				}

				if (!new_cmdbuf_node)
					break;

				/* Fix Jira PANGUA0POS-909, multi process hang */
				if (!bi_list_search_node(&subsys_core->list_manager, \
				new_cmdbuf_node))
					new_cmdbuf_node = subsys_core->list_manager.head;
				new_cmdbuf_node = new_cmdbuf_node->next;
			}
	#if 0
			new_cmdbuf_node = subsys_core->list_manager.head;
			while (1) {
				if (!new_cmdbuf_node)
					break;
				XDX_DEV_ERROR("After: new_cmdbuf_node->data->filp = %p with current filp: %p\n", ((struct cmdbuf_obj *)new_cmdbuf_node->data)->filp, filp);
				new_cmdbuf_node = new_cmdbuf_node->next;
		}
	#endif
			if (restart_cmdbuf && restart_cmdbuf->core_id == core_id) {
				u32 irq_status1, irq_status2;
				u64 restart_cmd_buf_addr;

				XDX_DEV_DEBUG("Restart from cmdbuf [%d] after aborting.\n", restart_cmdbuf->cmdbuf_id);

				irq_status1 = vcmd_read_reg((const void *)subsys_core->hwregs, VCMD_REGISTER_INT_STATUS_OFFSET);

				vcmd_write_reg((const void *)subsys_core->hwregs, VCMD_REGISTER_INT_STATUS_OFFSET, irq_status1);

				irq_status2 = vcmd_read_reg((const void *)subsys_core->hwregs, VCMD_REGISTER_INT_STATUS_OFFSET);

				XDX_DEV_DEBUG("Clear irq status from 0x%0x -> 0x%0x\n", irq_status1, irq_status2);

				restart_cmd_buf_addr = (u64)(restart_cmdbuf->cmdbuf_busAddress - subsys_dev->base_ddr_addr);
#ifdef HAS_MMU
				restart_cmd_buf_addr = (u64)(restart_cmdbuf->mmu_cmdbuf_bus_address);
#endif
#ifdef SUPPORT_VPU_SMMU
				if (hantro_dev.asic_type == CHIP_PANGU_B0){
					restart_cmd_buf_addr = (u64)(restart_cmdbuf->smmu_cmdbuf_bus_address);
				}
#endif
				vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
					HWIF_VCMD_EXECUTING_CMD_ADDR, (u32)(restart_cmd_buf_addr));
				if (sizeof(size_t) == 8) {
					vcmd_write_register_value((const void *)subsys_core->hwregs,
						subsys_core->reg_mirror, HWIF_VCMD_EXECUTING_CMD_ADDR_MSB,
						(u32)((u64)(restart_cmd_buf_addr) >> 32));
				} else {
					vcmd_write_register_value((const void *)subsys_core->hwregs,
						subsys_core->reg_mirror, HWIF_VCMD_EXECUTING_CMD_ADDR_MSB, 0);
				}
				vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
					HWIF_VCMD_EXE_CMDBUF_COUNT, 0);

				vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
					HWIF_VCMD_EXE_CMDBUF_LENGTH, (u32)((restart_cmdbuf->cmdbuf_size + 7) / 8));

				vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
					HWIF_VCMD_CMDBUF_EXECUTING_ID, restart_cmdbuf->cmdbuf_id);

				XDX_DEV_DEBUG("====subsys_core->sw_cmdbuf_rdy_num is %d\n", subsys_core->sw_cmdbuf_rdy_num);
				vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
					HWIF_VCMD_RDY_CMDBUF_COUNT, subsys_dev->vcmd_core->sw_cmdbuf_rdy_num);

	#ifdef VCMD_DEBUG_INTERNAL
				printk_vcmd_register_debug((const void *)subsys_core->hwregs, "before restart");
	#endif
				vcmd_write_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
					HWIF_VCMD_START_TRIGGER, 1);

				XDX_DEV_DEBUG("Restart from cmdbuf [%d] after aborting: start trigger = %d.\n", restart_cmdbuf->cmdbuf_id,
				vcmd_get_register_value((const void *)subsys_core->hwregs, subsys_core->reg_mirror,
				HWIF_VCMD_START_TRIGGER));
				XDX_DEV_DEBUG("dev state from %d -> WORKING.\n", subsys_core->working_state);
				subsys_core->working_state = WORKING_STATE_WORKING;
	#ifdef VCMD_DEBUG_INTERNAL
				printk_vcmd_register_debug((const void *)subsys_core->hwregs, "after restart");
	#endif
			}
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);

			// VCMD aborted but not restarted, need to wake up

			if (vcmd_aborted && !restart_cmdbuf)
				wake_up_interruptible_all(&subsys_core->wait_queue);
			subsys_core = subsys_core->core_next;// To next core in loop
		}

		if (release_cmdbuf_num)
			wake_up_interruptible_all(&subsys_dev->vcmd_cmdbuf_memory_wait);

		spin_lock_irqsave(&subsys_dev->vcmd_process_manager_lock, flags);
		process_manager_node = subsys_dev->global_process_manager.head;
		while (1) {
			if (!process_manager_node)
				break;
			process_manager_obj = (struct process_manager_obj *)process_manager_node->data;

			if (process_manager_obj->filp == filp)
				break;
			process_manager_node = process_manager_node->next;
		}
		//remove node from list
		if (process_manager_node && process_manager_obj) {
			XDX_DEV_DEBUG("process node %p for filp to be removed: %p\n",
				(void *)process_manager_node, (void *)process_manager_obj->filp);
			bi_list_remove_node(&subsys_dev->global_process_manager, process_manager_node);
			list_del(&process_manager_obj->list);
		}
		spin_unlock_irqrestore(&subsys_dev->vcmd_process_manager_lock, flags);

		free_process_manager_node(process_manager_node);
		up(&subsys_dev->vcmd_reserve_cmdbuf_sem[subsys_dev->vcmd_core->vcmd_core_cfg.sub_module_type]);

	}

	return 0;
}

void vcmd_dev_suspend(struct vcmd_dev *subsys_dev)
{
	vcmd_core_str *subsys_core = subsys_dev->vcmd_core;
	int loop_count = 0;
	unsigned long flags;

	/* wait vcmd cores enter IDLE state */
	spin_lock_irqsave(&subsys_core->spinlock, flags);

	vcmd_write_register_value((const void *)subsys_core->hwregs,
				  subsys_core->reg_mirror,
				  HWIF_VCMD_START_TRIGGER,
				  0);
	spin_unlock_irqrestore(&subsys_core->spinlock, flags);

	while (1) {
		spin_lock_irqsave(&subsys_core->spinlock, flags);
		if (subsys_core->working_state == WORKING_STATE_IDLE && \
		    vcmd_get_register_value((const void *)subsys_core->hwregs,
					    subsys_core->reg_mirror,
					    HWIF_VCMD_WORK_STATE) == WORKING_STATE_IDLE) {
			spin_unlock_irqrestore(&subsys_core->spinlock, flags);
			break;
		}
		/*
		 * release the spinlock frequently so that vpu irq workqueue
		 * could have the chance to handle cmd idle
		 */
		spin_unlock_irqrestore(&subsys_core->spinlock, flags);

		loop_count++;
		mdelay(10);
		if (loop_count > 100) {
			XDX_DEV_ERROR("hantrovcmd: too long before vcmd core to IDLE state\n");
			break;
		}
	}
	/* disable interruption and clear irq abort status*/
	spin_lock_irqsave(&subsys_core->spinlock, flags);
	vcmd_write_reg((const void *)subsys_core->hwregs,
		       VCMD_REGISTER_INT_CTL_OFFSET, 0x0000);
	spin_unlock_irqrestore(&subsys_core->spinlock, flags);
}

static void printk_vcmd_register_debug(const void *hwregs, char *info)
{
	u32 i, fordebug;

	pr_info("[xdxgpu]%s,%d\n", __func__, __LINE__);
	for (i = 0; i < ASIC_VCMD_SWREG_AMOUNT; i++) {
		fordebug = vcmd_read_reg((const void *)hwregs, i * 4);
		pr_info("%s vcmd register %d:0x%x\n", info, i, fordebug);
	}
}

